/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * @summary Settings which specify how to run a multi-instance Task.
  *
 * Multi-instance Tasks are commonly used to support MPI Tasks. In the MPI
 * case, if any of the subtasks fail (for example due to exiting with a
 * non-zero exit code) the entire multi-instance Task fails. The multi-instance
 * Task is then terminated and retried, up to its retry limit.
 *
 */
class MultiInstanceSettings {
  /**
   * Create a MultiInstanceSettings.
   * @property {number} [numberOfInstances] The number of Compute Nodes
   * required by the Task. If omitted, the default is 1.
   * @property {string} coordinationCommandLine The command line to run on all
   * the Compute Nodes to enable them to coordinate when the primary runs the
   * main Task command. A typical coordination command line launches a
   * background service and verifies that the service is ready to process
   * inter-node messages.
   * @property {array} [commonResourceFiles] A list of files that the Batch
   * service will download before running the coordination command line. The
   * difference between common resource files and Task resource files is that
   * common resource files are downloaded for all subtasks including the
   * primary, whereas Task resource files are downloaded only for the primary.
   * Also note that these resource files are not downloaded to the Task working
   * directory, but instead are downloaded to the Task root directory (one
   * directory above the working directory).  There is a maximum size for the
   * list of resource files.  When the max size is exceeded, the request will
   * fail and the response error code will be RequestEntityTooLarge. If this
   * occurs, the collection of ResourceFiles must be reduced in size. This can
   * be achieved using .zip files, Application Packages, or Docker Containers.
   */
  constructor() {
  }

  /**
   * Defines the metadata of MultiInstanceSettings
   *
   * @returns {object} metadata of MultiInstanceSettings
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'MultiInstanceSettings',
      type: {
        name: 'Composite',
        className: 'MultiInstanceSettings',
        modelProperties: {
          numberOfInstances: {
            required: false,
            serializedName: 'numberOfInstances',
            type: {
              name: 'Number'
            }
          },
          coordinationCommandLine: {
            required: true,
            serializedName: 'coordinationCommandLine',
            type: {
              name: 'String'
            }
          },
          commonResourceFiles: {
            required: false,
            serializedName: 'commonResourceFiles',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ResourceFileElementType',
                  type: {
                    name: 'Composite',
                    className: 'ResourceFile'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = MultiInstanceSettings;
