/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * @summary Information about a Task running on a Compute Node.
  *
 */
class TaskInformation {
  /**
   * Create a TaskInformation.
   * @property {string} [taskUrl] The URL of the Task.
   * @property {string} [jobId] The ID of the Job to which the Task belongs.
   * @property {string} [taskId] The ID of the Task.
   * @property {number} [subtaskId] The ID of the subtask if the Task is a
   * multi-instance Task.
   * @property {string} taskState The current state of the Task. Possible
   * values include: 'active', 'preparing', 'running', 'completed'
   * @property {object} [executionInfo] Information about the execution of the
   * Task.
   * @property {date} [executionInfo.startTime] 'Running' corresponds to the
   * running state, so if the Task specifies resource files or Packages, then
   * the start time reflects the time at which the Task started downloading or
   * deploying these. If the Task has been restarted or retried, this is the
   * most recent time at which the Task started running. This property is
   * present only for Tasks that are in the running or completed state.
   * @property {date} [executionInfo.endTime] This property is set only if the
   * Task is in the Completed state.
   * @property {number} [executionInfo.exitCode] This property is set only if
   * the Task is in the completed state. In general, the exit code for a
   * process reflects the specific convention implemented by the application
   * developer for that process. If you use the exit code value to make
   * decisions in your code, be sure that you know the exit code convention
   * used by the application process. However, if the Batch service terminates
   * the Task (due to timeout, or user termination via the API) you may see an
   * operating system-defined exit code.
   * @property {object} [executionInfo.containerInfo] This property is set only
   * if the Task runs in a container context.
   * @property {string} [executionInfo.containerInfo.containerId]
   * @property {string} [executionInfo.containerInfo.state] This is the state
   * of the container according to the Docker service. It is equivalent to the
   * status field returned by "docker inspect".
   * @property {string} [executionInfo.containerInfo.error] This is the
   * detailed error string from the Docker service, if available. It is
   * equivalent to the error field returned by "docker inspect".
   * @property {object} [executionInfo.failureInfo] This property is set only
   * if the Task is in the completed state and encountered a failure.
   * @property {string} [executionInfo.failureInfo.category] Possible values
   * include: 'userError', 'serverError'
   * @property {string} [executionInfo.failureInfo.code]
   * @property {string} [executionInfo.failureInfo.message]
   * @property {array} [executionInfo.failureInfo.details]
   * @property {number} [executionInfo.retryCount] Task application failures
   * (non-zero exit code) are retried, pre-processing errors (the Task could
   * not be run) and file upload errors are not retried. The Batch service will
   * retry the Task up to the limit specified by the constraints.
   * @property {date} [executionInfo.lastRetryTime] This element is present
   * only if the Task was retried (i.e. retryCount is nonzero). If present,
   * this is typically the same as startTime, but may be different if the Task
   * has been restarted for reasons other than retry; for example, if the
   * Compute Node was rebooted during a retry, then the startTime is updated
   * but the lastRetryTime is not.
   * @property {number} [executionInfo.requeueCount] When the user removes
   * Compute Nodes from a Pool (by resizing/shrinking the pool) or when the Job
   * is being disabled, the user can specify that running Tasks on the Compute
   * Nodes be requeued for execution. This count tracks how many times the Task
   * has been requeued for these reasons.
   * @property {date} [executionInfo.lastRequeueTime] This property is set only
   * if the requeueCount is nonzero.
   * @property {string} [executionInfo.result] If the value is 'failed', then
   * the details of the failure can be found in the failureInfo property.
   * Possible values include: 'success', 'failure'
   */
  constructor() {
  }

  /**
   * Defines the metadata of TaskInformation
   *
   * @returns {object} metadata of TaskInformation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'TaskInformation',
      type: {
        name: 'Composite',
        className: 'TaskInformation',
        modelProperties: {
          taskUrl: {
            required: false,
            serializedName: 'taskUrl',
            type: {
              name: 'String'
            }
          },
          jobId: {
            required: false,
            serializedName: 'jobId',
            type: {
              name: 'String'
            }
          },
          taskId: {
            required: false,
            serializedName: 'taskId',
            type: {
              name: 'String'
            }
          },
          subtaskId: {
            required: false,
            serializedName: 'subtaskId',
            type: {
              name: 'Number'
            }
          },
          taskState: {
            required: true,
            serializedName: 'taskState',
            type: {
              name: 'Enum',
              allowedValues: [ 'active', 'preparing', 'running', 'completed' ]
            }
          },
          executionInfo: {
            required: false,
            serializedName: 'executionInfo',
            type: {
              name: 'Composite',
              className: 'TaskExecutionInformation'
            }
          }
        }
      }
    };
  }
}

module.exports = TaskInformation;
