/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Defines a thing.
 *
 * @extends models['Response']
 */
class Thing extends models['Response'] {
  /**
   * Create a Thing.
   * @property {string} [name] The name of the thing represented by this
   * object.
   * @property {string} [url] The URL to get more information about the thing
   * represented by this object.
   * @property {object} [entityPresentationInfo] Additional information about
   * the entity such as hints that you can use to determine the entity's type.
   * To determine the entity's type, use the entityScenario and entityTypeHint
   * fields.
   * @property {string} [entityPresentationInfo.entityScenario] The supported
   * scenario. Possible values include: 'DominantEntity', 'DisambiguationItem',
   * 'ListItem'
   * @property {array} [entityPresentationInfo.entityTypeHints] A list of hints
   * that indicate the entity's type. The list could contain a single hint such
   * as Movie or a list of hints such as Place, LocalBusiness, Restaurant. Each
   * successive hint in the array narrows the entity's type.
   * @property {string} [entityPresentationInfo.entityTypeDisplayHint] A
   * display version of the entity hint. For example, if entityTypeHints is
   * Artist, this field may be set to American Singer.
   * @property {string} [entityPresentationInfo.query]
   * @property {array} [entityPresentationInfo.entitySubTypeHints]
   * @property {string} [entityPresentationInfo._type] Polymorphic
   * Discriminator
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of Thing
   *
   * @returns {object} metadata of Thing
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Thing',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: '_type',
          clientName: '_type'
        },
        uberParent: 'ResponseBase',
        className: 'Thing',
        modelProperties: {
          _type: {
            required: true,
            serializedName: '_type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          readLink: {
            required: false,
            readOnly: true,
            serializedName: 'readLink',
            type: {
              name: 'String'
            }
          },
          webSearchUrl: {
            required: false,
            readOnly: true,
            serializedName: 'webSearchUrl',
            type: {
              name: 'String'
            }
          },
          potentialAction: {
            required: false,
            readOnly: true,
            serializedName: 'potentialAction',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ActionElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '_type',
                      clientName: '_type'
                    },
                    uberParent: 'ResponseBase',
                    className: 'Action'
                  }
              }
            }
          },
          immediateAction: {
            required: false,
            readOnly: true,
            serializedName: 'immediateAction',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ActionElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '_type',
                      clientName: '_type'
                    },
                    uberParent: 'ResponseBase',
                    className: 'Action'
                  }
              }
            }
          },
          preferredClickthroughUrl: {
            required: false,
            readOnly: true,
            serializedName: 'preferredClickthroughUrl',
            type: {
              name: 'String'
            }
          },
          adaptiveCard: {
            required: false,
            readOnly: true,
            serializedName: 'adaptiveCard',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          url: {
            required: false,
            readOnly: true,
            serializedName: 'url',
            type: {
              name: 'String'
            }
          },
          entityPresentationInfo: {
            required: false,
            readOnly: true,
            serializedName: 'entityPresentationInfo',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'EntitiesEntityPresentationInfo',
              className: 'EntitiesEntityPresentationInfo'
            }
          }
        }
      }
    };
  }
}

module.exports = Thing;
