/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The parameters for updating a build task.
 *
 */
class BuildTaskUpdateParameters {
  /**
   * Create a BuildTaskUpdateParameters.
   * @member {string} [alias] The alternative updatable name for a build task.
   * @member {string} [status] The current status of build task. Possible
   * values include: 'Disabled', 'Enabled'
   * @member {object} [platform] The platform properties against which the
   * build has to happen.
   * @member {string} [platform.osType] The operating system type required for
   * the build. Possible values include: 'Windows', 'Linux'
   * @member {number} [platform.cpu] The CPU configuration in terms of number
   * of cores required for the build.
   * @member {number} [timeout] Build timeout in seconds.
   * @member {object} [sourceRepository] The properties that describes the
   * source(code) for the build task.
   * @member {object} [sourceRepository.sourceControlAuthProperties] The
   * authorization properties for accessing the source code repository.
   * @member {string} [sourceRepository.sourceControlAuthProperties.tokenType]
   * The type of Auth token. Possible values include: 'PAT', 'OAuth'
   * @member {string} [sourceRepository.sourceControlAuthProperties.token] The
   * access token used to access the source control provider.
   * @member {string}
   * [sourceRepository.sourceControlAuthProperties.refreshToken] The refresh
   * token used to refresh the access token.
   * @member {string} [sourceRepository.sourceControlAuthProperties.scope] The
   * scope of the access token.
   * @member {number} [sourceRepository.sourceControlAuthProperties.expiresIn]
   * Time in seconds that the token remains valid
   * @member {boolean} [sourceRepository.isCommitTriggerEnabled] The value of
   * this property indicates whether the source control commit trigger is
   * enabled or not.
   * @member {object} [tags] The ARM resource tags.
   */
  constructor() {
  }

  /**
   * Defines the metadata of BuildTaskUpdateParameters
   *
   * @returns {object} metadata of BuildTaskUpdateParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'BuildTaskUpdateParameters',
      type: {
        name: 'Composite',
        className: 'BuildTaskUpdateParameters',
        modelProperties: {
          alias: {
            required: false,
            serializedName: 'properties.alias',
            type: {
              name: 'String'
            }
          },
          status: {
            required: false,
            serializedName: 'properties.status',
            type: {
              name: 'String'
            }
          },
          platform: {
            required: false,
            serializedName: 'properties.platform',
            type: {
              name: 'Composite',
              className: 'PlatformProperties'
            }
          },
          timeout: {
            required: false,
            serializedName: 'properties.timeout',
            constraints: {
              InclusiveMaximum: 28800,
              InclusiveMinimum: 300
            },
            type: {
              name: 'Number'
            }
          },
          sourceRepository: {
            required: false,
            serializedName: 'properties.sourceRepository',
            type: {
              name: 'Composite',
              className: 'SourceRepositoryUpdateParameters'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = BuildTaskUpdateParameters;
