/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Class representing a ImportImageParameters.
 */
class ImportImageParameters {
  /**
   * Create a ImportImageParameters.
   * @property {object} source The source of the image.
   * @property {string} [source.resourceId] The resource identifier of the
   * source Azure Container Registry.
   * @property {string} [source.registryUri] The address of the source registry
   * (e.g. 'mcr.microsoft.com').
   * @property {object} [source.credentials] Credentials used when importing
   * from a registry uri.
   * @property {string} [source.credentials.username] The username to
   * authenticate with the source registry.
   * @property {string} [source.credentials.password] The password used to
   * authenticate with the source registry.
   * @property {string} [source.sourceImage] Repository name of the source
   * image.
   * Specify an image by repository ('hello-world'). This will use the 'latest'
   * tag.
   * Specify an image by tag ('hello-world:latest').
   * Specify an image by sha256-based manifest digest
   * ('hello-world@sha256:abc123').
   * @property {array} [targetTags] List of strings of the form repo[:tag].
   * When tag is omitted the source will be used (or 'latest' if source tag is
   * also omitted).
   * @property {array} [untaggedTargetRepositories] List of strings of
   * repository names to do a manifest only copy. No tag will be created.
   * @property {string} [mode] When Force, any existing target tags will be
   * overwritten. When NoForce, any existing target tags will fail the
   * operation before any copying begins. Possible values include: 'NoForce',
   * 'Force'. Default value: 'NoForce' .
   */
  constructor() {
  }

  /**
   * Defines the metadata of ImportImageParameters
   *
   * @returns {object} metadata of ImportImageParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ImportImageParameters',
      type: {
        name: 'Composite',
        className: 'ImportImageParameters',
        modelProperties: {
          source: {
            required: true,
            serializedName: 'source',
            type: {
              name: 'Composite',
              className: 'ImportSource'
            }
          },
          targetTags: {
            required: false,
            serializedName: 'targetTags',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          untaggedTargetRepositories: {
            required: false,
            serializedName: 'untaggedTargetRepositories',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          mode: {
            required: false,
            serializedName: 'mode',
            defaultValue: 'NoForce',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ImportImageParameters;
