/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Amazon Marketplace Web Service linked service.
 *
 * @extends models['LinkedService']
 */
class AmazonMWSLinkedService extends models['LinkedService'] {
  /**
   * Create a AmazonMWSLinkedService.
   * @member {object} endpoint The endpoint of the Amazon MWS server, (i.e.
   * mws.amazonservices.com)
   * @member {object} marketplaceID The Amazon Marketplace ID you want to
   * retrieve data from. To retrive data from multiple Marketplace IDs,
   * seperate them with a comma (,). (i.e. A2EUQ1WTGCTBG2)
   * @member {object} sellerID The Amazon seller ID.
   * @member {object} [mwsAuthToken] The Amazon MWS authentication token.
   * @member {string} [mwsAuthToken.type] Polymorphic Discriminator
   * @member {object} accessKeyId The access key id used to access data.
   * @member {object} [secretKey] The secret key used to access data.
   * @member {string} [secretKey.type] Polymorphic Discriminator
   * @member {object} [useEncryptedEndpoints] Specifies whether the data source
   * endpoints are encrypted using HTTPS. The default value is true.
   * @member {object} [useHostVerification] Specifies whether to require the
   * host name in the server's certificate to match the host name of the server
   * when connecting over SSL. The default value is true.
   * @member {object} [usePeerVerification] Specifies whether to verify the
   * identity of the server when connecting over SSL. The default value is
   * true.
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of AmazonMWSLinkedService
   *
   * @returns {object} metadata of AmazonMWSLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'AmazonMWS',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'AmazonMWSLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          endpoint: {
            required: true,
            serializedName: 'typeProperties.endpoint',
            type: {
              name: 'Object'
            }
          },
          marketplaceID: {
            required: true,
            serializedName: 'typeProperties.marketplaceID',
            type: {
              name: 'Object'
            }
          },
          sellerID: {
            required: true,
            serializedName: 'typeProperties.sellerID',
            type: {
              name: 'Object'
            }
          },
          mwsAuthToken: {
            required: false,
            serializedName: 'typeProperties.mwsAuthToken',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          accessKeyId: {
            required: true,
            serializedName: 'typeProperties.accessKeyId',
            type: {
              name: 'Object'
            }
          },
          secretKey: {
            required: false,
            serializedName: 'typeProperties.secretKey',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          useEncryptedEndpoints: {
            required: false,
            serializedName: 'typeProperties.useEncryptedEndpoints',
            type: {
              name: 'Object'
            }
          },
          useHostVerification: {
            required: false,
            serializedName: 'typeProperties.useHostVerification',
            type: {
              name: 'Object'
            }
          },
          usePeerVerification: {
            required: false,
            serializedName: 'typeProperties.usePeerVerification',
            type: {
              name: 'Object'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = AmazonMWSLinkedService;
