/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * MariaDB server linked service.
 *
 * @extends models['LinkedService']
 */
class MariaDBLinkedService extends models['LinkedService'] {
  /**
   * Create a MariaDBLinkedService.
   * @member {object} [connectionString] An ODBC connection string. Type:
   * string, SecureString or AzureKeyVaultSecretReference.
   * @member {object} [pwd] The Azure key vault secret reference of password in
   * connection string.
   * @member {object} [pwd.store] The Azure Key Vault linked service reference.
   * @member {string} [pwd.store.referenceName] Reference LinkedService name.
   * @member {object} [pwd.store.parameters] Arguments for LinkedService.
   * @member {object} [pwd.secretName] The name of the secret in Azure Key
   * Vault. Type: string (or Expression with resultType string).
   * @member {object} [pwd.secretVersion] The version of the secret in Azure
   * Key Vault. The default value is the latest version of the secret. Type:
   * string (or Expression with resultType string).
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of MariaDBLinkedService
   *
   * @returns {object} metadata of MariaDBLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'MariaDB',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'MariaDBLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          connectionString: {
            required: false,
            serializedName: 'typeProperties.connectionString',
            type: {
              name: 'Object'
            }
          },
          pwd: {
            required: false,
            serializedName: 'typeProperties.pwd',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'AzureKeyVaultSecretReference'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = MariaDBLinkedService;
