/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Open Database Connectivity (ODBC) linked service.
 *
 * @extends models['LinkedService']
 */
class OdbcLinkedService extends models['LinkedService'] {
  /**
   * Create a OdbcLinkedService.
   * @member {object} connectionString The non-access credential portion of the
   * connection string as well as an optional encrypted credential. Type:
   * string, SecureString or AzureKeyVaultSecretReference.
   * @member {object} [authenticationType] Type of authentication used to
   * connect to the ODBC data store. Possible values are: Anonymous and Basic.
   * Type: string (or Expression with resultType string).
   * @member {object} [credential] The access credential portion of the
   * connection string specified in driver-specific property-value format.
   * @member {string} [credential.type] Polymorphic Discriminator
   * @member {object} [userName] User name for Basic authentication. Type:
   * string (or Expression with resultType string).
   * @member {object} [password] Password for Basic authentication.
   * @member {string} [password.type] Polymorphic Discriminator
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of OdbcLinkedService
   *
   * @returns {object} metadata of OdbcLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Odbc',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'OdbcLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          connectionString: {
            required: true,
            serializedName: 'typeProperties.connectionString',
            type: {
              name: 'Object'
            }
          },
          authenticationType: {
            required: false,
            serializedName: 'typeProperties.authenticationType',
            type: {
              name: 'Object'
            }
          },
          credential: {
            required: false,
            serializedName: 'typeProperties.credential',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          userName: {
            required: false,
            serializedName: 'typeProperties.userName',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: false,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = OdbcLinkedService;
