/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Sku class.
 * @constructor
 * Model representing SKU for Azure Dev Spaces Controller.
 *
 * @member {string} [tier] The tier of the SKU for Azure Dev Spaces Controller.
 * Possible values include: 'Standard'
 */
export interface Sku {
  tier?: string;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * An Azure resource.
 *
 * @member {string} [id] Fully qualified resource Id for the resource.
 * @member {string} [name] The name of the resource.
 * @member {string} [type] The type of the resource.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
}

/**
 * @class
 * Initializes a new instance of the TrackedResource class.
 * @constructor
 * The resource model definition for a ARM tracked top level resource.
 *
 * @member {object} [tags] Tags for the Azure resource.
 * @member {string} [location] Region where the Azure resource is located.
 */
export interface TrackedResource extends Resource {
  tags?: { [propertyName: string]: string };
  location?: string;
}

/**
 * @class
 * Initializes a new instance of the Controller class.
 * @constructor
 * @member {string} [provisioningState] Provisioning state of the Azure Dev
 * Spaces Controller. Possible values include: 'Succeeded', 'Failed',
 * 'Canceled', 'Updating', 'Creating', 'Deleting'
 * @member {string} hostSuffix DNS suffix for public endpoints running in the
 * Azure Dev Spaces Controller.
 * @member {string} [dataPlaneFqdn] DNS name for accessing DataPlane services
 * @member {string} targetContainerHostResourceId Resource ID of the target
 * container host
 * @member {string} targetContainerHostCredentialsBase64 Credentials of the
 * target container host (base64).
 * @member {object} sku
 * @member {string} [sku.tier] The tier of the SKU for Azure Dev Spaces
 * Controller. Possible values include: 'Standard'
 */
export interface Controller extends TrackedResource {
  readonly provisioningState?: string;
  hostSuffix: string;
  readonly dataPlaneFqdn?: string;
  targetContainerHostResourceId: string;
  targetContainerHostCredentialsBase64: string;
  sku: Sku;
}

/**
 * @class
 * Initializes a new instance of the ControllerUpdateParameters class.
 * @constructor
 * Parameters for updating an Azure Dev Spaces Controller.
 *
 * @member {object} [tags] Tags for the Azure Dev Spaces Controller.
 */
export interface ControllerUpdateParameters {
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the OrchestratorSpecificConnectionDetails class.
 * @constructor
 * @member {string} instanceType Polymorphic Discriminator
 */
export interface OrchestratorSpecificConnectionDetails {
  instanceType: string;
}

/**
 * @class
 * Initializes a new instance of the ControllerConnectionDetails class.
 * @constructor
 * @member {string} [authKey] Authentication key for communicating with
 * services.
 * @member {string} [workspaceStorageAccountName] Workspace storage account
 * name.
 * @member {string} [workspaceStorageSasToken] Workspace storage account SAS
 * token.
 * @member {object} [orchestratorSpecificConnectionDetails]
 * @member {string} [orchestratorSpecificConnectionDetails.instanceType]
 * Polymorphic Discriminator
 */
export interface ControllerConnectionDetails {
  readonly authKey?: string;
  readonly workspaceStorageAccountName?: string;
  readonly workspaceStorageSasToken?: string;
  orchestratorSpecificConnectionDetails?: OrchestratorSpecificConnectionDetails;
}

/**
 * @class
 * Initializes a new instance of the ControllerConnectionDetailsList class.
 * @constructor
 * @member {array} [connectionDetailsList] List of Azure Dev Spaces Controller
 * connection details.
 */
export interface ControllerConnectionDetailsList {
  connectionDetailsList?: ControllerConnectionDetails[];
}

/**
 * @class
 * Initializes a new instance of the ResourceProviderOperationDisplay class.
 * @constructor
 * @member {string} [provider] Name of the resource provider.
 * @member {string} [resource] Name of the resource type.
 * @member {string} [operation] Name of the resource provider operation.
 * @member {string} [description] Description of the resource provider
 * operation.
 */
export interface ResourceProviderOperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceProviderOperationDefinition class.
 * @constructor
 * @member {string} [name] Resource provider operation name.
 * @member {object} [display]
 * @member {string} [display.provider] Name of the resource provider.
 * @member {string} [display.resource] Name of the resource type.
 * @member {string} [display.operation] Name of the resource provider
 * operation.
 * @member {string} [display.description] Description of the resource provider
 * operation.
 */
export interface ResourceProviderOperationDefinition {
  name?: string;
  display?: ResourceProviderOperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the KubernetesConnectionDetails class.
 * @constructor
 * @member {string} [kubeConfig] Gets the kubeconfig for the cluster.
 */
export interface KubernetesConnectionDetails extends OrchestratorSpecificConnectionDetails {
  kubeConfig?: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorDetails class.
 * @constructor
 * @member {string} [code] Status code for the error.
 * @member {string} [message] Error message describing the error in detail.
 * @member {string} [target] The target of the particular error.
 */
export interface ErrorDetails {
  readonly code?: string;
  readonly message?: string;
  readonly target?: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * Error response indicates that the service is not able to process the
 * incoming request. The reason is provided in the error message.
 *
 * @member {object} [error] The details of the error.
 * @member {string} [error.code] Status code for the error.
 * @member {string} [error.message] Error message describing the error in
 * detail.
 * @member {string} [error.target] The target of the particular error.
 */
export interface ErrorResponse {
  error?: ErrorDetails;
}


/**
 * @class
 * Initializes a new instance of the ControllerList class.
 * @constructor
 * @member {string} [nextLink] The URI that can be used to request the next
 * page for list of Azure Dev Spaces Controllers.
 */
export interface ControllerList extends Array<Controller> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceProviderOperationList class.
 * @constructor
 * @member {string} [nextLink] The URI that can be used to request the next
 * page for list of Azure operations.
 */
export interface ResourceProviderOperationList extends Array<ResourceProviderOperationDefinition> {
  readonly nextLink?: string;
}
