/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;


/**
 * Removes a content from Front Door.
 *
 * @param {string} resourceGroupName Name of the Resource group within the
 * Azure subscription.
 *
 * @param {string} frontDoorName Name of the Front Door which is globally
 * unique.
 *
 * @param {object} contentFilePaths The path to the content to be purged. Path
 * can be a full URL, e.g. '/pictures/city.png' which removes a single file, or
 * a directory with a wildcard, e.g. '/pictures/*' which removes all folders
 * and files in the directory.
 *
 * @param {array} contentFilePaths.contentPaths The path to the content to be
 * purged. Can describe a file path or a wild card directory.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _purgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginPurgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}

/**
 * Removes a content from Front Door.
 *
 * @param {string} resourceGroupName Name of the Resource group within the
 * Azure subscription.
 *
 * @param {string} frontDoorName Name of the Front Door which is globally
 * unique.
 *
 * @param {object} contentFilePaths The path to the content to be purged. Path
 * can be a full URL, e.g. '/pictures/city.png' which removes a single file, or
 * a directory with a wildcard, e.g. '/pictures/*' which removes all folders
 * and files in the directory.
 *
 * @param {array} contentFilePaths.contentPaths The path to the content to be
 * purged. Can describe a file path or a wild card directory.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginPurgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-08-01';
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName !== null && resourceGroupName !== undefined) {
      if (resourceGroupName.length > 80)
      {
        throw new Error('"resourceGroupName" should satisfy the constraint - "MaxLength": 80');
      }
      if (resourceGroupName.length < 1)
      {
        throw new Error('"resourceGroupName" should satisfy the constraint - "MinLength": 1');
      }
      if (resourceGroupName.match(/^[a-zA-Z0-9_\-\(\)\.]*[^\.]$/) === null)
      {
        throw new Error('"resourceGroupName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9_\-\(\)\.]*[^\.]$/');
      }
    }
    if (frontDoorName === null || frontDoorName === undefined || typeof frontDoorName.valueOf() !== 'string') {
      throw new Error('frontDoorName cannot be null or undefined and it must be of type string.');
    }
    if (frontDoorName !== null && frontDoorName !== undefined) {
      if (frontDoorName.length > 64)
      {
        throw new Error('"frontDoorName" should satisfy the constraint - "MaxLength": 64');
      }
      if (frontDoorName.length < 5)
      {
        throw new Error('"frontDoorName" should satisfy the constraint - "MinLength": 5');
      }
      if (frontDoorName.match(/^[a-zA-Z0-9]+([-a-zA-Z0-9]?[a-zA-Z0-9])*$/) === null)
      {
        throw new Error('"frontDoorName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9]+([-a-zA-Z0-9]?[a-zA-Z0-9])*$/');
      }
    }
    if (contentFilePaths === null || contentFilePaths === undefined) {
      throw new Error('contentFilePaths cannot be null or undefined.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/frontDoors/{frontDoorName}/purge';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{frontDoorName}', encodeURIComponent(frontDoorName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (contentFilePaths !== null && contentFilePaths !== undefined) {
      let requestModelMapper = new client.models['PurgeParameters']().mapper();
      requestModel = client.serialize(requestModelMapper, contentFilePaths, 'contentFilePaths');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(contentFilePaths, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          let internalError = null;
          if (parsedErrorResponse.error) internalError = parsedErrorResponse.error;
          error.code = internalError ? internalError.code : parsedErrorResponse.code;
          error.message = internalError ? internalError.message : parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['ErrorResponse']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a Endpoints. */
class Endpoints {
  /**
   * Create a Endpoints.
   * @param {FrontDoorManagementClient} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._purgeContent = _purgeContent;
    this._beginPurgeContent = _beginPurgeContent;
  }

  /**
   * Removes a content from Front Door.
   *
   * @param {string} resourceGroupName Name of the Resource group within the
   * Azure subscription.
   *
   * @param {string} frontDoorName Name of the Front Door which is globally
   * unique.
   *
   * @param {object} contentFilePaths The path to the content to be purged. Path
   * can be a full URL, e.g. '/pictures/city.png' which removes a single file, or
   * a directory with a wildcard, e.g. '/pictures/*' which removes all folders
   * and files in the directory.
   *
   * @param {array} contentFilePaths.contentPaths The path to the content to be
   * purged. Can describe a file path or a wild card directory.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  purgeContentWithHttpOperationResponse(resourceGroupName, frontDoorName, contentFilePaths, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._purgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Removes a content from Front Door.
   *
   * @param {string} resourceGroupName Name of the Resource group within the
   * Azure subscription.
   *
   * @param {string} frontDoorName Name of the Front Door which is globally
   * unique.
   *
   * @param {object} contentFilePaths The path to the content to be purged. Path
   * can be a full URL, e.g. '/pictures/city.png' which removes a single file, or
   * a directory with a wildcard, e.g. '/pictures/*' which removes all folders
   * and files in the directory.
   *
   * @param {array} contentFilePaths.contentPaths The path to the content to be
   * purged. Can describe a file path or a wild card directory.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  purgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._purgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._purgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, optionalCallback);
    }
  }

  /**
   * Removes a content from Front Door.
   *
   * @param {string} resourceGroupName Name of the Resource group within the
   * Azure subscription.
   *
   * @param {string} frontDoorName Name of the Front Door which is globally
   * unique.
   *
   * @param {object} contentFilePaths The path to the content to be purged. Path
   * can be a full URL, e.g. '/pictures/city.png' which removes a single file, or
   * a directory with a wildcard, e.g. '/pictures/*' which removes all folders
   * and files in the directory.
   *
   * @param {array} contentFilePaths.contentPaths The path to the content to be
   * purged. Can describe a file path or a wild card directory.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginPurgeContentWithHttpOperationResponse(resourceGroupName, frontDoorName, contentFilePaths, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginPurgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Removes a content from Front Door.
   *
   * @param {string} resourceGroupName Name of the Resource group within the
   * Azure subscription.
   *
   * @param {string} frontDoorName Name of the Front Door which is globally
   * unique.
   *
   * @param {object} contentFilePaths The path to the content to be purged. Path
   * can be a full URL, e.g. '/pictures/city.png' which removes a single file, or
   * a directory with a wildcard, e.g. '/pictures/*' which removes all folders
   * and files in the directory.
   *
   * @param {array} contentFilePaths.contentPaths The path to the content to be
   * purged. Can describe a file path or a wild card directory.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginPurgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginPurgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginPurgeContent(resourceGroupName, frontDoorName, contentFilePaths, options, optionalCallback);
    }
  }

}

module.exports = Endpoints;
