/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents an environment instance
 *
 * @extends models['Resource']
 */
class Environment extends models['Resource'] {
  /**
   * Create a Environment.
   * @property {object} [resourceSets] The set of a VM and the setting id it
   * was created for
   * @property {string} [resourceSets.vmResourceId] VM resource Id for the
   * environment
   * @property {string} [resourceSets.resourceSettingId] resourceSettingId for
   * the environment
   * @property {string} [claimedByUserObjectId] The AAD object Id of the user
   * who has claimed the environment
   * @property {string} [claimedByUserPrincipalId] The user principal Id of the
   * user who has claimed the environment
   * @property {string} [claimedByUserName] The name or email address of the
   * user who has claimed the environment
   * @property {boolean} [isClaimed] Is the environment claimed or not
   * @property {string} [lastKnownPowerState] Last known power state of the
   * environment
   * @property {object} [networkInterface] Network details of the environment
   * @property {string} [networkInterface.privateIpAddress] PrivateIp address
   * of the Compute VM
   * @property {string} [networkInterface.sshAuthority] Connection information
   * for Linux
   * @property {string} [networkInterface.rdpAuthority] Connection information
   * for Windows
   * @property {string} [networkInterface.username] Username of the VM
   * @property {moment.duration} [totalUsage] How long the environment has been
   * used by a lab user
   * @property {date} [passwordLastReset] When the password was last reset on
   * the environment.
   * @property {string} [provisioningState] The provisioning status of the
   * resource.
   * @property {string} [uniqueIdentifier] The unique immutable identifier of a
   * resource (Guid).
   * @property {object} [latestOperationResult] The details of the latest
   * operation. ex: status, error
   * @property {string} [latestOperationResult.status] The current status of
   * the operation.
   * @property {string} [latestOperationResult.errorCode] Error code on
   * failure.
   * @property {string} [latestOperationResult.errorMessage] The error message.
   * @property {string} [latestOperationResult.requestUri] Request URI of the
   * operation.
   * @property {string} [latestOperationResult.httpMethod] The HttpMethod -
   * PUT/POST/DELETE for the operation.
   * @property {string} [latestOperationResult.operationUrl] The URL to use to
   * check long-running operation status
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of Environment
   *
   * @returns {object} metadata of Environment
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Environment',
      type: {
        name: 'Composite',
        className: 'Environment',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          resourceSets: {
            required: false,
            serializedName: 'properties.resourceSets',
            type: {
              name: 'Composite',
              className: 'ResourceSet'
            }
          },
          claimedByUserObjectId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.claimedByUserObjectId',
            type: {
              name: 'String'
            }
          },
          claimedByUserPrincipalId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.claimedByUserPrincipalId',
            type: {
              name: 'String'
            }
          },
          claimedByUserName: {
            required: false,
            readOnly: true,
            serializedName: 'properties.claimedByUserName',
            type: {
              name: 'String'
            }
          },
          isClaimed: {
            required: false,
            readOnly: true,
            serializedName: 'properties.isClaimed',
            type: {
              name: 'Boolean'
            }
          },
          lastKnownPowerState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.lastKnownPowerState',
            type: {
              name: 'String'
            }
          },
          networkInterface: {
            required: false,
            readOnly: true,
            serializedName: 'properties.networkInterface',
            type: {
              name: 'Composite',
              className: 'NetworkInterface'
            }
          },
          totalUsage: {
            required: false,
            readOnly: true,
            serializedName: 'properties.totalUsage',
            type: {
              name: 'TimeSpan'
            }
          },
          passwordLastReset: {
            required: false,
            readOnly: true,
            serializedName: 'properties.passwordLastReset',
            type: {
              name: 'DateTime'
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          uniqueIdentifier: {
            required: false,
            serializedName: 'properties.uniqueIdentifier',
            type: {
              name: 'String'
            }
          },
          latestOperationResult: {
            required: false,
            readOnly: true,
            serializedName: 'properties.latestOperationResult',
            type: {
              name: 'Composite',
              className: 'LatestOperationResult'
            }
          }
        }
      }
    };
  }
}

module.exports = Environment;
