/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The workflow run properties.
 *
 */
class WorkflowRunProperties {
  /**
   * Create a WorkflowRunProperties.
   * @member {date} [startTime] Gets the start time.
   * @member {date} [endTime] Gets the end time.
   * @member {string} [status] Gets the status. Possible values include:
   * 'NotSpecified', 'Paused', 'Running', 'Waiting', 'Succeeded', 'Skipped',
   * 'Suspended', 'Cancelled', 'Failed', 'Faulted', 'TimedOut', 'Aborted',
   * 'Ignored'
   * @member {string} [code] Gets the code.
   * @member {object} [error] Gets the error.
   * @member {string} [correlationId] Gets the correlation id.
   * @member {object} [correlation] The run correlation.
   * @member {string} [correlation.clientTrackingId] The client tracking id.
   * @member {object} [workflow] Gets the reference to workflow version.
   * @member {string} [workflow.id] The resource id.
   * @member {string} [workflow.name] Gets the resource name.
   * @member {string} [workflow.type] Gets the resource type.
   * @member {object} [trigger] Gets the fired trigger.
   * @member {string} [trigger.name] Gets the name.
   * @member {object} [trigger.inputs] Gets the inputs.
   * @member {object} [trigger.inputsLink] Gets the link to inputs.
   * @member {string} [trigger.inputsLink.uri] The content link URI.
   * @member {string} [trigger.inputsLink.contentVersion] The content version.
   * @member {number} [trigger.inputsLink.contentSize] The content size.
   * @member {object} [trigger.inputsLink.contentHash] The content hash.
   * @member {string} [trigger.inputsLink.contentHash.algorithm] The algorithm
   * of the content hash.
   * @member {string} [trigger.inputsLink.contentHash.value] The value of the
   * content hash.
   * @member {object} [trigger.inputsLink.metadata] The metadata.
   * @member {object} [trigger.outputs] Gets the outputs.
   * @member {object} [trigger.outputsLink] Gets the link to outputs.
   * @member {string} [trigger.outputsLink.uri] The content link URI.
   * @member {string} [trigger.outputsLink.contentVersion] The content version.
   * @member {number} [trigger.outputsLink.contentSize] The content size.
   * @member {object} [trigger.outputsLink.contentHash] The content hash.
   * @member {string} [trigger.outputsLink.contentHash.algorithm] The algorithm
   * of the content hash.
   * @member {string} [trigger.outputsLink.contentHash.value] The value of the
   * content hash.
   * @member {object} [trigger.outputsLink.metadata] The metadata.
   * @member {date} [trigger.startTime] Gets the start time.
   * @member {date} [trigger.endTime] Gets the end time.
   * @member {string} [trigger.trackingId] Gets the tracking id.
   * @member {object} [trigger.correlation] The run correlation.
   * @member {string} [trigger.correlation.clientTrackingId] The client
   * tracking id.
   * @member {string} [trigger.code] Gets the code.
   * @member {string} [trigger.status] Gets the status. Possible values
   * include: 'NotSpecified', 'Paused', 'Running', 'Waiting', 'Succeeded',
   * 'Skipped', 'Suspended', 'Cancelled', 'Failed', 'Faulted', 'TimedOut',
   * 'Aborted', 'Ignored'
   * @member {object} [trigger.error] Gets the error.
   * @member {object} [trigger.trackedProperties] Gets the tracked properties.
   * @member {object} [outputs] Gets the outputs.
   * @member {object} [response] Gets the response of the flow run.
   * @member {string} [response.name] Gets the name.
   * @member {object} [response.inputs] Gets the inputs.
   * @member {object} [response.inputsLink] Gets the link to inputs.
   * @member {string} [response.inputsLink.uri] The content link URI.
   * @member {string} [response.inputsLink.contentVersion] The content version.
   * @member {number} [response.inputsLink.contentSize] The content size.
   * @member {object} [response.inputsLink.contentHash] The content hash.
   * @member {string} [response.inputsLink.contentHash.algorithm] The algorithm
   * of the content hash.
   * @member {string} [response.inputsLink.contentHash.value] The value of the
   * content hash.
   * @member {object} [response.inputsLink.metadata] The metadata.
   * @member {object} [response.outputs] Gets the outputs.
   * @member {object} [response.outputsLink] Gets the link to outputs.
   * @member {string} [response.outputsLink.uri] The content link URI.
   * @member {string} [response.outputsLink.contentVersion] The content
   * version.
   * @member {number} [response.outputsLink.contentSize] The content size.
   * @member {object} [response.outputsLink.contentHash] The content hash.
   * @member {string} [response.outputsLink.contentHash.algorithm] The
   * algorithm of the content hash.
   * @member {string} [response.outputsLink.contentHash.value] The value of the
   * content hash.
   * @member {object} [response.outputsLink.metadata] The metadata.
   * @member {date} [response.startTime] Gets the start time.
   * @member {date} [response.endTime] Gets the end time.
   * @member {string} [response.trackingId] Gets the tracking id.
   * @member {object} [response.correlation] The run correlation.
   * @member {string} [response.correlation.clientTrackingId] The client
   * tracking id.
   * @member {string} [response.code] Gets the code.
   * @member {string} [response.status] Gets the status. Possible values
   * include: 'NotSpecified', 'Paused', 'Running', 'Waiting', 'Succeeded',
   * 'Skipped', 'Suspended', 'Cancelled', 'Failed', 'Faulted', 'TimedOut',
   * 'Aborted', 'Ignored'
   * @member {object} [response.error] Gets the error.
   * @member {object} [response.trackedProperties] Gets the tracked properties.
   */
  constructor() {
  }

  /**
   * Defines the metadata of WorkflowRunProperties
   *
   * @returns {object} metadata of WorkflowRunProperties
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'WorkflowRunProperties',
      type: {
        name: 'Composite',
        className: 'WorkflowRunProperties',
        modelProperties: {
          startTime: {
            required: false,
            readOnly: true,
            serializedName: 'startTime',
            type: {
              name: 'DateTime'
            }
          },
          endTime: {
            required: false,
            readOnly: true,
            serializedName: 'endTime',
            type: {
              name: 'DateTime'
            }
          },
          status: {
            required: false,
            readOnly: true,
            serializedName: 'status',
            type: {
              name: 'Enum',
              allowedValues: [ 'NotSpecified', 'Paused', 'Running', 'Waiting', 'Succeeded', 'Skipped', 'Suspended', 'Cancelled', 'Failed', 'Faulted', 'TimedOut', 'Aborted', 'Ignored' ]
            }
          },
          code: {
            required: false,
            readOnly: true,
            serializedName: 'code',
            type: {
              name: 'String'
            }
          },
          error: {
            required: false,
            readOnly: true,
            serializedName: 'error',
            type: {
              name: 'Object'
            }
          },
          correlationId: {
            required: false,
            readOnly: true,
            serializedName: 'correlationId',
            type: {
              name: 'String'
            }
          },
          correlation: {
            required: false,
            serializedName: 'correlation',
            type: {
              name: 'Composite',
              className: 'Correlation'
            }
          },
          workflow: {
            required: false,
            readOnly: true,
            serializedName: 'workflow',
            type: {
              name: 'Composite',
              className: 'ResourceReference'
            }
          },
          trigger: {
            required: false,
            readOnly: true,
            serializedName: 'trigger',
            type: {
              name: 'Composite',
              className: 'WorkflowRunTrigger'
            }
          },
          outputs: {
            required: false,
            readOnly: true,
            serializedName: 'outputs',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'WorkflowOutputParameterElementType',
                  type: {
                    name: 'Composite',
                    className: 'WorkflowOutputParameter'
                  }
              }
            }
          },
          response: {
            required: false,
            readOnly: true,
            serializedName: 'response',
            type: {
              name: 'Composite',
              className: 'WorkflowRunTrigger'
            }
          }
        }
      }
    };
  }
}

module.exports = WorkflowRunProperties;
