/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A Machine Learning compute based on AKS.
 *
 * @extends models['Compute']
 */
class AKS extends models['Compute'] {
  /**
   * Create a AKS.
   * @member {object} [properties] AKS properties
   * @member {string} [properties.clusterFqdn] Cluster full qualified domain
   * name
   * @member {array} [properties.systemServices] System services
   * @member {number} [properties.agentCount] Number of agents
   * @member {string} [properties.agentVMSize] Agent virtual machine size
   * @member {object} [properties.sslConfiguration] SSL configuration
   * @member {string} [properties.sslConfiguration.status] Enable or disable
   * ssl for scoring. Possible values include: 'Disabled', 'Enabled'
   * @member {string} [properties.sslConfiguration.cert] Cert data
   * @member {string} [properties.sslConfiguration.key] Key data
   * @member {string} [properties.sslConfiguration.cname] CNAME of the cert
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of AKS
   *
   * @returns {object} metadata of AKS
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'AKS',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'computeType',
          clientName: 'computeType'
        },
        uberParent: 'Compute',
        className: 'AKS',
        modelProperties: {
          computeLocation: {
            required: false,
            serializedName: 'computeLocation',
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'provisioningState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          createdOn: {
            required: false,
            readOnly: true,
            serializedName: 'createdOn',
            type: {
              name: 'DateTime'
            }
          },
          modifiedOn: {
            required: false,
            readOnly: true,
            serializedName: 'modifiedOn',
            type: {
              name: 'DateTime'
            }
          },
          resourceId: {
            required: false,
            serializedName: 'resourceId',
            type: {
              name: 'String'
            }
          },
          provisioningErrors: {
            required: false,
            readOnly: true,
            serializedName: 'provisioningErrors',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'MachineLearningServiceErrorElementType',
                  type: {
                    name: 'Composite',
                    className: 'MachineLearningServiceError'
                  }
              }
            }
          },
          computeType: {
            required: true,
            serializedName: 'computeType',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          properties: {
            required: false,
            serializedName: 'properties',
            type: {
              name: 'Composite',
              className: 'AKSProperties'
            }
          }
        }
      }
    };
  }
}

module.exports = AKS;
