/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Base type for all overlays - image, audio or video.
 *
 */
class Overlay {
  /**
   * Create a Overlay.
   * @property {string} inputLabel The label of the job input which is to be
   * used as an overlay. The Input must specify exactly one file. You can
   * specify an image file in JPG or PNG formats, or an audio file (such as a
   * WAV, MP3, WMA or M4A file), or a video file. See https://aka.ms/mesformats
   * for the complete list of supported audio and video file formats.
   * @property {moment.duration} [start] The start position, with reference to
   * the input video, at which the overlay starts. The value should be in ISO
   * 8601 format. For example, PT05S to start the overlay at 5 seconds in to
   * the input video. If not specified the overlay starts from the beginning of
   * the input video.
   * @property {moment.duration} [end] The position in the input video at which
   * the overlay ends. The value should be in ISO 8601 duration format. For
   * example, PT30S to end the overlay at 30 seconds in to the input video. If
   * not specified the overlay will be applied until the end of the input video
   * if inputLoop is true. Else, if inputLoop is false, then overlay will last
   * as long as the duration of the overlay media.
   * @property {moment.duration} [fadeInDuration] The duration over which the
   * overlay fades in onto the input video. The value should be in ISO 8601
   * duration format. If not specified the default behavior is to have no fade
   * in (same as PT0S).
   * @property {moment.duration} [fadeOutDuration] The duration over which the
   * overlay fades out of the input video. The value should be in ISO 8601
   * duration format. If not specified the default behavior is to have no fade
   * out (same as PT0S).
   * @property {number} [audioGainLevel] The gain level of audio in the
   * overlay. The value should be in the range [0, 1.0]. The default is 1.0.
   * @property {string} odatatype Polymorphic Discriminator
   */
  constructor() {
  }

  /**
   * Defines the metadata of Overlay
   *
   * @returns {object} metadata of Overlay
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Overlay',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: '@odata.type',
          clientName: 'odatatype'
        },
        uberParent: 'Overlay',
        className: 'Overlay',
        modelProperties: {
          inputLabel: {
            required: true,
            serializedName: 'inputLabel',
            type: {
              name: 'String'
            }
          },
          start: {
            required: false,
            serializedName: 'start',
            type: {
              name: 'TimeSpan'
            }
          },
          end: {
            required: false,
            serializedName: 'end',
            type: {
              name: 'TimeSpan'
            }
          },
          fadeInDuration: {
            required: false,
            serializedName: 'fadeInDuration',
            type: {
              name: 'TimeSpan'
            }
          },
          fadeOutDuration: {
            required: false,
            serializedName: 'fadeOutDuration',
            type: {
              name: 'TimeSpan'
            }
          },
          audioGainLevel: {
            required: false,
            serializedName: 'audioGainLevel',
            type: {
              name: 'Number'
            }
          },
          odatatype: {
            required: true,
            serializedName: '@odata\\.type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = Overlay;
