/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;

/**
 * @summary Get assessed machines for assessment.
 *
 * Get list of machines that assessed as part of the specified assessment.
 * Returns a json array of objects of type 'assessedMachine' as specified in
 * the Models section.
 *
 * Whenever an assessment is created or updated, it goes under computation.
 * During this phase, the 'status' field of Assessment object reports
 * 'Computing'.
 * During the period when the assessment is under computation, the list of
 * assessed machines is empty and no assessed machines are returned by this
 * call.
 *
 *
 * @param {string} resourceGroupName Name of the Azure Resource Group that
 * project is part of.
 *
 * @param {string} projectName Name of the Azure Migrate project.
 *
 * @param {string} groupName Unique name of a group within a project.
 *
 * @param {string} assessmentName Unique name of an assessment within a
 * project.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link AssessedMachineResultList} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _listByAssessment(resourceGroupName, projectName, groupName, assessmentName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (projectName === null || projectName === undefined || typeof projectName.valueOf() !== 'string') {
      throw new Error('projectName cannot be null or undefined and it must be of type string.');
    }
    if (groupName === null || groupName === undefined || typeof groupName.valueOf() !== 'string') {
      throw new Error('groupName cannot be null or undefined and it must be of type string.');
    }
    if (assessmentName === null || assessmentName === undefined || typeof assessmentName.valueOf() !== 'string') {
      throw new Error('assessmentName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Migrate/projects/{projectName}/groups/{groupName}/assessments/{assessmentName}/assessedMachines';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{projectName}', encodeURIComponent(projectName));
  requestUrl = requestUrl.replace('{groupName}', encodeURIComponent(groupName));
  requestUrl = requestUrl.replace('{assessmentName}', encodeURIComponent(assessmentName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['Accept-Language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 401 && statusCode !== 404 && statusCode !== 500 && statusCode !== 503) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['AssessedMachineResultList']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * @summary Get an assessed machine.
 *
 * Get an assessed machine with its size & cost estimnate that was evaluated in
 * the specified assessment.
 *
 * @param {string} resourceGroupName Name of the Azure Resource Group that
 * project is part of.
 *
 * @param {string} projectName Name of the Azure Migrate project.
 *
 * @param {string} groupName Unique name of a group within a project.
 *
 * @param {string} assessmentName Unique name of an assessment within a
 * project.
 *
 * @param {string} assessedMachineName Unique name of an assessed machine
 * evaluated as part of an assessment.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link AssessedMachine} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _get(resourceGroupName, projectName, groupName, assessmentName, assessedMachineName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (projectName === null || projectName === undefined || typeof projectName.valueOf() !== 'string') {
      throw new Error('projectName cannot be null or undefined and it must be of type string.');
    }
    if (groupName === null || groupName === undefined || typeof groupName.valueOf() !== 'string') {
      throw new Error('groupName cannot be null or undefined and it must be of type string.');
    }
    if (assessmentName === null || assessmentName === undefined || typeof assessmentName.valueOf() !== 'string') {
      throw new Error('assessmentName cannot be null or undefined and it must be of type string.');
    }
    if (assessedMachineName === null || assessedMachineName === undefined || typeof assessedMachineName.valueOf() !== 'string') {
      throw new Error('assessedMachineName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Migrate/projects/{projectName}/groups/{groupName}/assessments/{assessmentName}/assessedMachines/{assessedMachineName}';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{projectName}', encodeURIComponent(projectName));
  requestUrl = requestUrl.replace('{groupName}', encodeURIComponent(groupName));
  requestUrl = requestUrl.replace('{assessmentName}', encodeURIComponent(assessmentName));
  requestUrl = requestUrl.replace('{assessedMachineName}', encodeURIComponent(assessedMachineName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['Accept-Language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 401 && statusCode !== 404 && statusCode !== 500 && statusCode !== 503) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['AssessedMachine']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a AssessedMachines. */
class AssessedMachines {
  /**
   * Create a AssessedMachines.
   * @param {AzureMigrate} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._listByAssessment = _listByAssessment;
    this._get = _get;
  }

  /**
   * @summary Get assessed machines for assessment.
   *
   * Get list of machines that assessed as part of the specified assessment.
   * Returns a json array of objects of type 'assessedMachine' as specified in
   * the Models section.
   *
   * Whenever an assessment is created or updated, it goes under computation.
   * During this phase, the 'status' field of Assessment object reports
   * 'Computing'.
   * During the period when the assessment is under computation, the list of
   * assessed machines is empty and no assessed machines are returned by this
   * call.
   *
   *
   * @param {string} resourceGroupName Name of the Azure Resource Group that
   * project is part of.
   *
   * @param {string} projectName Name of the Azure Migrate project.
   *
   * @param {string} groupName Unique name of a group within a project.
   *
   * @param {string} assessmentName Unique name of an assessment within a
   * project.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<AssessedMachineResultList>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listByAssessmentWithHttpOperationResponse(resourceGroupName, projectName, groupName, assessmentName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._listByAssessment(resourceGroupName, projectName, groupName, assessmentName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * @summary Get assessed machines for assessment.
   *
   * Get list of machines that assessed as part of the specified assessment.
   * Returns a json array of objects of type 'assessedMachine' as specified in
   * the Models section.
   *
   * Whenever an assessment is created or updated, it goes under computation.
   * During this phase, the 'status' field of Assessment object reports
   * 'Computing'.
   * During the period when the assessment is under computation, the list of
   * assessed machines is empty and no assessed machines are returned by this
   * call.
   *
   *
   * @param {string} resourceGroupName Name of the Azure Resource Group that
   * project is part of.
   *
   * @param {string} projectName Name of the Azure Migrate project.
   *
   * @param {string} groupName Unique name of a group within a project.
   *
   * @param {string} assessmentName Unique name of an assessment within a
   * project.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {AssessedMachineResultList} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link AssessedMachineResultList} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  listByAssessment(resourceGroupName, projectName, groupName, assessmentName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._listByAssessment(resourceGroupName, projectName, groupName, assessmentName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._listByAssessment(resourceGroupName, projectName, groupName, assessmentName, options, optionalCallback);
    }
  }

  /**
   * @summary Get an assessed machine.
   *
   * Get an assessed machine with its size & cost estimnate that was evaluated in
   * the specified assessment.
   *
   * @param {string} resourceGroupName Name of the Azure Resource Group that
   * project is part of.
   *
   * @param {string} projectName Name of the Azure Migrate project.
   *
   * @param {string} groupName Unique name of a group within a project.
   *
   * @param {string} assessmentName Unique name of an assessment within a
   * project.
   *
   * @param {string} assessedMachineName Unique name of an assessed machine
   * evaluated as part of an assessment.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<AssessedMachine>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getWithHttpOperationResponse(resourceGroupName, projectName, groupName, assessmentName, assessedMachineName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._get(resourceGroupName, projectName, groupName, assessmentName, assessedMachineName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * @summary Get an assessed machine.
   *
   * Get an assessed machine with its size & cost estimnate that was evaluated in
   * the specified assessment.
   *
   * @param {string} resourceGroupName Name of the Azure Resource Group that
   * project is part of.
   *
   * @param {string} projectName Name of the Azure Migrate project.
   *
   * @param {string} groupName Unique name of a group within a project.
   *
   * @param {string} assessmentName Unique name of an assessment within a
   * project.
   *
   * @param {string} assessedMachineName Unique name of an assessed machine
   * evaluated as part of an assessment.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {AssessedMachine} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link AssessedMachine} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  get(resourceGroupName, projectName, groupName, assessmentName, assessedMachineName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._get(resourceGroupName, projectName, groupName, assessmentName, assessedMachineName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._get(resourceGroupName, projectName, groupName, assessmentName, assessedMachineName, options, optionalCallback);
    }
  }

}

module.exports = AssessedMachines;
