/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Projects
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMigrate.
 */
export interface Projects {


    /**
     * @summary Get all projects.
     *
     * Get all the projects in the resource group.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ProjectResultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ProjectResultList>>;

    /**
     * @summary Get all projects.
     *
     * Get all the projects in the resource group.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ProjectResultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ProjectResultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ProjectResultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ProjectResultList>;
    list(resourceGroupName: string, callback: ServiceCallback<models.ProjectResultList>): void;
    list(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ProjectResultList>): void;


    /**
     * @summary Get the specified project.
     *
     * Get the project with the specified name.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Project>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Project>>;

    /**
     * @summary Get the specified project.
     *
     * Get the project with the specified name.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Project} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Project} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Project} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Project>;
    get(resourceGroupName: string, projectName: string, callback: ServiceCallback<models.Project>): void;
    get(resourceGroupName: string, projectName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Project>): void;


    /**
     * @summary Create or update project.
     *
     * Create a project with specified name. If a project already exists, update
     * it.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.project] New or Updated project object.
     *
     * @param {string} [options.project.eTag] For optimistic concurrency control.
     *
     * @param {string} [options.project.location] Azure location in which project
     * is created.
     *
     * @param {object} [options.project.tags] Tags provided by Azure Tagging
     * service.
     *
     * @param {string} [options.project.customerWorkspaceId] ARM ID of the Service
     * Map workspace created by user.
     *
     * @param {string} [options.project.customerWorkspaceLocation] Location of the
     * Service Map workspace created by user.
     *
     * @param {string} [options.project.provisioningState] Provisioning state of
     * the project. Possible values include: 'Accepted', 'Creating', 'Deleting',
     * 'Failed', 'Moving', 'Succeeded'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Project>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, projectName: string, options?: { project? : models.Project, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Project>>;

    /**
     * @summary Create or update project.
     *
     * Create a project with specified name. If a project already exists, update
     * it.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.project] New or Updated project object.
     *
     * @param {string} [options.project.eTag] For optimistic concurrency control.
     *
     * @param {string} [options.project.location] Azure location in which project
     * is created.
     *
     * @param {object} [options.project.tags] Tags provided by Azure Tagging
     * service.
     *
     * @param {string} [options.project.customerWorkspaceId] ARM ID of the Service
     * Map workspace created by user.
     *
     * @param {string} [options.project.customerWorkspaceLocation] Location of the
     * Service Map workspace created by user.
     *
     * @param {string} [options.project.provisioningState] Provisioning state of
     * the project. Possible values include: 'Accepted', 'Creating', 'Deleting',
     * 'Failed', 'Moving', 'Succeeded'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Project} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Project} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Project} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, projectName: string, options?: { project? : models.Project, customHeaders? : { [headerName: string]: string; } }): Promise<models.Project>;
    create(resourceGroupName: string, projectName: string, callback: ServiceCallback<models.Project>): void;
    create(resourceGroupName: string, projectName: string, options: { project? : models.Project, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Project>): void;


    /**
     * @summary Update project.
     *
     * Update a project with specified name. Supports partial updates, for example
     * only tags can be provided.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.project] Updated project object.
     *
     * @param {string} [options.project.eTag] For optimistic concurrency control.
     *
     * @param {string} [options.project.location] Azure location in which project
     * is created.
     *
     * @param {object} [options.project.tags] Tags provided by Azure Tagging
     * service.
     *
     * @param {string} [options.project.customerWorkspaceId] ARM ID of the Service
     * Map workspace created by user.
     *
     * @param {string} [options.project.customerWorkspaceLocation] Location of the
     * Service Map workspace created by user.
     *
     * @param {string} [options.project.provisioningState] Provisioning state of
     * the project. Possible values include: 'Accepted', 'Creating', 'Deleting',
     * 'Failed', 'Moving', 'Succeeded'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Project>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, projectName: string, options?: { project? : models.Project, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Project>>;

    /**
     * @summary Update project.
     *
     * Update a project with specified name. Supports partial updates, for example
     * only tags can be provided.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.project] Updated project object.
     *
     * @param {string} [options.project.eTag] For optimistic concurrency control.
     *
     * @param {string} [options.project.location] Azure location in which project
     * is created.
     *
     * @param {object} [options.project.tags] Tags provided by Azure Tagging
     * service.
     *
     * @param {string} [options.project.customerWorkspaceId] ARM ID of the Service
     * Map workspace created by user.
     *
     * @param {string} [options.project.customerWorkspaceLocation] Location of the
     * Service Map workspace created by user.
     *
     * @param {string} [options.project.provisioningState] Provisioning state of
     * the project. Possible values include: 'Accepted', 'Creating', 'Deleting',
     * 'Failed', 'Moving', 'Succeeded'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Project} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Project} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Project} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, projectName: string, options?: { project? : models.Project, customHeaders? : { [headerName: string]: string; } }): Promise<models.Project>;
    update(resourceGroupName: string, projectName: string, callback: ServiceCallback<models.Project>): void;
    update(resourceGroupName: string, projectName: string, options: { project? : models.Project, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Project>): void;


    /**
     * @summary Delete the project
     *
     * Delete the project. Deleting non-existent project is a no-operation.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Delete the project
     *
     * Delete the project. Deleting non-existent project is a no-operation.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, projectName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, projectName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * @summary Get shared keys for the project.
     *
     * Gets the Log Analytics Workspace ID and Primary Key for the specified
     * project.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ProjectKey>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getKeysWithHttpOperationResponse(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ProjectKey>>;

    /**
     * @summary Get shared keys for the project.
     *
     * Gets the Log Analytics Workspace ID and Primary Key for the specified
     * project.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ProjectKey} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ProjectKey} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ProjectKey} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getKeys(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ProjectKey>;
    getKeys(resourceGroupName: string, projectName: string, callback: ServiceCallback<models.ProjectKey>): void;
    getKeys(resourceGroupName: string, projectName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ProjectKey>): void;
}

/**
 * @class
 * Machines
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMigrate.
 */
export interface Machines {


    /**
     * @summary Get all machines in the project
     *
     * Get data of all the machines available in the project. Returns a json array
     * of objects of type 'machine' defined in Models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<MachineResultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByProjectWithHttpOperationResponse(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.MachineResultList>>;

    /**
     * @summary Get all machines in the project
     *
     * Get data of all the machines available in the project. Returns a json array
     * of objects of type 'machine' defined in Models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {MachineResultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {MachineResultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link MachineResultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByProject(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.MachineResultList>;
    listByProject(resourceGroupName: string, projectName: string, callback: ServiceCallback<models.MachineResultList>): void;
    listByProject(resourceGroupName: string, projectName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.MachineResultList>): void;


    /**
     * @summary Get a specific machine.
     *
     * Get the machine with the specified name. Returns a json object of type
     * 'machine' defined in Models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} machineName Unique name of a machine in private datacenter.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Machine>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, projectName: string, machineName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Machine>>;

    /**
     * @summary Get a specific machine.
     *
     * Get the machine with the specified name. Returns a json object of type
     * 'machine' defined in Models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} machineName Unique name of a machine in private datacenter.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Machine} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Machine} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Machine} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, projectName: string, machineName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Machine>;
    get(resourceGroupName: string, projectName: string, machineName: string, callback: ServiceCallback<models.Machine>): void;
    get(resourceGroupName: string, projectName: string, machineName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Machine>): void;
}

/**
 * @class
 * Groups
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMigrate.
 */
export interface Groups {


    /**
     * @summary Get all groups
     *
     * Get all groups created in the project. Returns a json array of objects of
     * type 'group' as specified in the Models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<GroupResultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByProjectWithHttpOperationResponse(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.GroupResultList>>;

    /**
     * @summary Get all groups
     *
     * Get all groups created in the project. Returns a json array of objects of
     * type 'group' as specified in the Models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {GroupResultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {GroupResultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link GroupResultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByProject(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.GroupResultList>;
    listByProject(resourceGroupName: string, projectName: string, callback: ServiceCallback<models.GroupResultList>): void;
    listByProject(resourceGroupName: string, projectName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.GroupResultList>): void;


    /**
     * @summary Get a specific group.
     *
     * Get information related to a specific group in the project. Returns a json
     * object of type 'group' as specified in the models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Group>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Group>>;

    /**
     * @summary Get a specific group.
     *
     * Get information related to a specific group in the project. Returns a json
     * object of type 'group' as specified in the models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Group} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Group} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Group} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, projectName: string, groupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Group>;
    get(resourceGroupName: string, projectName: string, groupName: string, callback: ServiceCallback<models.Group>): void;
    get(resourceGroupName: string, projectName: string, groupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Group>): void;


    /**
     * @summary Create a new group with specified settings. If group with the name
     * provided already exists, then the existing group is updated.
     *
     * Create a new group by sending a json object of type 'group' as given in
     * Models section as part of the Request Body. The group name in a project is
     * unique. Labels can be applied on a group as part of creation.
     *
     * If a group with the groupName specified in the URL already exists, then this
     * call acts as an update.
     *
     * This operation is Idempotent.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.group] New or Updated Group object.
     *
     * @param {string} [options.group.eTag] For optimistic concurrency control.
     *
     * @param {array} options.group.machines List of machine names that are part of
     * this group.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Group>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, options?: { group? : models.Group, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Group>>;

    /**
     * @summary Create a new group with specified settings. If group with the name
     * provided already exists, then the existing group is updated.
     *
     * Create a new group by sending a json object of type 'group' as given in
     * Models section as part of the Request Body. The group name in a project is
     * unique. Labels can be applied on a group as part of creation.
     *
     * If a group with the groupName specified in the URL already exists, then this
     * call acts as an update.
     *
     * This operation is Idempotent.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.group] New or Updated Group object.
     *
     * @param {string} [options.group.eTag] For optimistic concurrency control.
     *
     * @param {array} options.group.machines List of machine names that are part of
     * this group.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Group} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Group} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Group} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, projectName: string, groupName: string, options?: { group? : models.Group, customHeaders? : { [headerName: string]: string; } }): Promise<models.Group>;
    create(resourceGroupName: string, projectName: string, groupName: string, callback: ServiceCallback<models.Group>): void;
    create(resourceGroupName: string, projectName: string, groupName: string, options: { group? : models.Group, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Group>): void;


    /**
     * @summary Delete the group
     *
     * Delete the group from the project. The machines remain in the project.
     * Deleting a non-existent group results in a no-operation.
     *
     * A group is an aggregation mechanism for machines in a project. Therefore,
     * deleting group does not delete machines in it.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Delete the group
     *
     * Delete the group from the project. The machines remain in the project.
     * Deleting a non-existent group results in a no-operation.
     *
     * A group is an aggregation mechanism for machines in a project. Therefore,
     * deleting group does not delete machines in it.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, projectName: string, groupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, projectName: string, groupName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, projectName: string, groupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
}

/**
 * @class
 * Assessments
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMigrate.
 */
export interface Assessments {


    /**
     * @summary Get all assessments created for the specified group.
     *
     * Get all assessments created for the specified group.
     *
     * Returns a json array of objects of type 'assessment' as specified in Models
     * section.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<AssessmentResultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByGroupWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AssessmentResultList>>;

    /**
     * @summary Get all assessments created for the specified group.
     *
     * Get all assessments created for the specified group.
     *
     * Returns a json array of objects of type 'assessment' as specified in Models
     * section.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {AssessmentResultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {AssessmentResultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link AssessmentResultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByGroup(resourceGroupName: string, projectName: string, groupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.AssessmentResultList>;
    listByGroup(resourceGroupName: string, projectName: string, groupName: string, callback: ServiceCallback<models.AssessmentResultList>): void;
    listByGroup(resourceGroupName: string, projectName: string, groupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AssessmentResultList>): void;


    /**
     * @summary Get all assessments created in the project.
     *
     * Get all assessments created in the project.
     *
     * Returns a json array of objects of type 'assessment' as specified in Models
     * section.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<AssessmentResultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByProjectWithHttpOperationResponse(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AssessmentResultList>>;

    /**
     * @summary Get all assessments created in the project.
     *
     * Get all assessments created in the project.
     *
     * Returns a json array of objects of type 'assessment' as specified in Models
     * section.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {AssessmentResultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {AssessmentResultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link AssessmentResultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByProject(resourceGroupName: string, projectName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.AssessmentResultList>;
    listByProject(resourceGroupName: string, projectName: string, callback: ServiceCallback<models.AssessmentResultList>): void;
    listByProject(resourceGroupName: string, projectName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AssessmentResultList>): void;


    /**
     * @summary Get an assessment.
     *
     * Get an existing assessment with the specified name. Returns a json object of
     * type 'assessment' as specified in Models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Assessment>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Assessment>>;

    /**
     * @summary Get an assessment.
     *
     * Get an existing assessment with the specified name. Returns a json object of
     * type 'assessment' as specified in Models section.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Assessment} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Assessment} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Assessment} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Assessment>;
    get(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, callback: ServiceCallback<models.Assessment>): void;
    get(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Assessment>): void;


    /**
     * @summary Create or Update assessment.
     *
     * Create a new assessment with the given name and the specified settings.
     * Since name of an assessment in a project is a unique identiefier, if an
     * assessment with the name provided already exists, then the existing
     * assessment is updated.
     *
     * Any PUT operation, resulting in either create or update on an assessment,
     * will cause the assessment to go in a "InProgress" state. This will be
     * indicated by the field 'computationState' on the Assessment object. During
     * this time no other PUT operation will be allowed on that assessment object,
     * nor will a Delete operation. Once the computation for the assessment is
     * complete, the field 'computationState' will be updated to 'Ready', and then
     * other PUT or DELETE operations can happen on the assessment.
     *
     * When assessment is under computation, any PUT will lead to a 400 - Bad
     * Request error.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.assessment] New or Updated Assessment object.
     *
     * @param {string} [options.assessment.eTag] For optimistic concurrency
     * control.
     *
     * @param {string} options.assessment.azureLocation Target Azure location for
     * which the machines should be assessed. These enums are the same as used by
     * Compute API. Possible values include: 'Unknown', 'EastAsia',
     * 'SoutheastAsia', 'AustraliaEast', 'AustraliaSoutheast', 'BrazilSouth',
     * 'CanadaCentral', 'CanadaEast', 'WestEurope', 'NorthEurope', 'CentralIndia',
     * 'SouthIndia', 'WestIndia', 'JapanEast', 'JapanWest', 'KoreaCentral',
     * 'KoreaSouth', 'UkWest', 'UkSouth', 'NorthCentralUs', 'EastUs', 'WestUs2',
     * 'SouthCentralUs', 'CentralUs', 'EastUs2', 'WestUs', 'WestCentralUs',
     * 'GermanyCentral', 'GermanyNortheast', 'ChinaNorth', 'ChinaEast'
     *
     * @param {string} options.assessment.azureOfferCode Offer code according to
     * which cost estimation is done. Possible values include: 'Unknown',
     * 'MSAZR0003P', 'MSAZR0044P', 'MSAZR0059P', 'MSAZR0060P', 'MSAZR0062P',
     * 'MSAZR0063P', 'MSAZR0064P', 'MSAZR0029P', 'MSAZR0022P', 'MSAZR0023P',
     * 'MSAZR0148P', 'MSAZR0025P', 'MSAZR0036P', 'MSAZR0120P', 'MSAZR0121P',
     * 'MSAZR0122P', 'MSAZR0123P', 'MSAZR0124P', 'MSAZR0125P', 'MSAZR0126P',
     * 'MSAZR0127P', 'MSAZR0128P', 'MSAZR0129P', 'MSAZR0130P', 'MSAZR0111P',
     * 'MSAZR0144P', 'MSAZR0149P', 'MSMCAZR0044P', 'MSMCAZR0059P', 'MSMCAZR0060P',
     * 'MSMCAZR0063P', 'MSMCAZR0120P', 'MSMCAZR0121P', 'MSMCAZR0125P',
     * 'MSMCAZR0128P', 'MSAZRDE0003P', 'MSAZRDE0044P'
     *
     * @param {string} options.assessment.azurePricingTier Pricing tier for Size
     * evaluation. Possible values include: 'Standard', 'Basic'
     *
     * @param {string} options.assessment.azureStorageRedundancy Storage Redundancy
     * type offered by Azure. Possible values include: 'Unknown',
     * 'LocallyRedundant', 'ZoneRedundant', 'GeoRedundant',
     * 'ReadAccessGeoRedundant'
     *
     * @param {number} options.assessment.scalingFactor Scaling factor used over
     * utilization data to add a performance buffer for new machines to be created
     * in Azure. Min Value = 1.0, Max value = 1.9, Default = 1.3.
     *
     * @param {string} options.assessment.percentile Percentile of performance data
     * used to recommend Azure size. Possible values include: 'Percentile50',
     * 'Percentile90', 'Percentile95', 'Percentile99'
     *
     * @param {string} options.assessment.timeRange Time range of performance data
     * used to recommend a size. Possible values include: 'Day', 'Week', 'Month'
     *
     * @param {string} options.assessment.stage User configurable setting that
     * describes the status of the assessment. Possible values include:
     * 'InProgress', 'UnderReview', 'Approved'
     *
     * @param {string} options.assessment.currency Currency to report prices in.
     * Possible values include: 'Unknown', 'USD', 'DKK', 'CAD', 'IDR', 'JPY',
     * 'KRW', 'NZD', 'NOK', 'RUB', 'SAR', 'ZAR', 'SEK', 'TRY', 'GBP', 'MXN', 'MYR',
     * 'INR', 'HKD', 'BRL', 'TWD', 'EUR', 'CHF', 'ARS', 'AUD', 'CNY'
     *
     * @param {string} options.assessment.azureHybridUseBenefit AHUB discount on
     * windows virtual machines. Possible values include: 'Unknown', 'Yes', 'No'
     *
     * @param {number} options.assessment.discountPercentage Custom discount
     * percentage to be applied on final costs. Can be in the range [0, 100].
     *
     * @param {string} options.assessment.sizingCriterion Assessment sizing
     * criterion. Possible values include: 'PerformanceBased', 'AsOnPremises'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Assessment>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { assessment? : models.Assessment, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Assessment>>;

    /**
     * @summary Create or Update assessment.
     *
     * Create a new assessment with the given name and the specified settings.
     * Since name of an assessment in a project is a unique identiefier, if an
     * assessment with the name provided already exists, then the existing
     * assessment is updated.
     *
     * Any PUT operation, resulting in either create or update on an assessment,
     * will cause the assessment to go in a "InProgress" state. This will be
     * indicated by the field 'computationState' on the Assessment object. During
     * this time no other PUT operation will be allowed on that assessment object,
     * nor will a Delete operation. Once the computation for the assessment is
     * complete, the field 'computationState' will be updated to 'Ready', and then
     * other PUT or DELETE operations can happen on the assessment.
     *
     * When assessment is under computation, any PUT will lead to a 400 - Bad
     * Request error.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.assessment] New or Updated Assessment object.
     *
     * @param {string} [options.assessment.eTag] For optimistic concurrency
     * control.
     *
     * @param {string} options.assessment.azureLocation Target Azure location for
     * which the machines should be assessed. These enums are the same as used by
     * Compute API. Possible values include: 'Unknown', 'EastAsia',
     * 'SoutheastAsia', 'AustraliaEast', 'AustraliaSoutheast', 'BrazilSouth',
     * 'CanadaCentral', 'CanadaEast', 'WestEurope', 'NorthEurope', 'CentralIndia',
     * 'SouthIndia', 'WestIndia', 'JapanEast', 'JapanWest', 'KoreaCentral',
     * 'KoreaSouth', 'UkWest', 'UkSouth', 'NorthCentralUs', 'EastUs', 'WestUs2',
     * 'SouthCentralUs', 'CentralUs', 'EastUs2', 'WestUs', 'WestCentralUs',
     * 'GermanyCentral', 'GermanyNortheast', 'ChinaNorth', 'ChinaEast'
     *
     * @param {string} options.assessment.azureOfferCode Offer code according to
     * which cost estimation is done. Possible values include: 'Unknown',
     * 'MSAZR0003P', 'MSAZR0044P', 'MSAZR0059P', 'MSAZR0060P', 'MSAZR0062P',
     * 'MSAZR0063P', 'MSAZR0064P', 'MSAZR0029P', 'MSAZR0022P', 'MSAZR0023P',
     * 'MSAZR0148P', 'MSAZR0025P', 'MSAZR0036P', 'MSAZR0120P', 'MSAZR0121P',
     * 'MSAZR0122P', 'MSAZR0123P', 'MSAZR0124P', 'MSAZR0125P', 'MSAZR0126P',
     * 'MSAZR0127P', 'MSAZR0128P', 'MSAZR0129P', 'MSAZR0130P', 'MSAZR0111P',
     * 'MSAZR0144P', 'MSAZR0149P', 'MSMCAZR0044P', 'MSMCAZR0059P', 'MSMCAZR0060P',
     * 'MSMCAZR0063P', 'MSMCAZR0120P', 'MSMCAZR0121P', 'MSMCAZR0125P',
     * 'MSMCAZR0128P', 'MSAZRDE0003P', 'MSAZRDE0044P'
     *
     * @param {string} options.assessment.azurePricingTier Pricing tier for Size
     * evaluation. Possible values include: 'Standard', 'Basic'
     *
     * @param {string} options.assessment.azureStorageRedundancy Storage Redundancy
     * type offered by Azure. Possible values include: 'Unknown',
     * 'LocallyRedundant', 'ZoneRedundant', 'GeoRedundant',
     * 'ReadAccessGeoRedundant'
     *
     * @param {number} options.assessment.scalingFactor Scaling factor used over
     * utilization data to add a performance buffer for new machines to be created
     * in Azure. Min Value = 1.0, Max value = 1.9, Default = 1.3.
     *
     * @param {string} options.assessment.percentile Percentile of performance data
     * used to recommend Azure size. Possible values include: 'Percentile50',
     * 'Percentile90', 'Percentile95', 'Percentile99'
     *
     * @param {string} options.assessment.timeRange Time range of performance data
     * used to recommend a size. Possible values include: 'Day', 'Week', 'Month'
     *
     * @param {string} options.assessment.stage User configurable setting that
     * describes the status of the assessment. Possible values include:
     * 'InProgress', 'UnderReview', 'Approved'
     *
     * @param {string} options.assessment.currency Currency to report prices in.
     * Possible values include: 'Unknown', 'USD', 'DKK', 'CAD', 'IDR', 'JPY',
     * 'KRW', 'NZD', 'NOK', 'RUB', 'SAR', 'ZAR', 'SEK', 'TRY', 'GBP', 'MXN', 'MYR',
     * 'INR', 'HKD', 'BRL', 'TWD', 'EUR', 'CHF', 'ARS', 'AUD', 'CNY'
     *
     * @param {string} options.assessment.azureHybridUseBenefit AHUB discount on
     * windows virtual machines. Possible values include: 'Unknown', 'Yes', 'No'
     *
     * @param {number} options.assessment.discountPercentage Custom discount
     * percentage to be applied on final costs. Can be in the range [0, 100].
     *
     * @param {string} options.assessment.sizingCriterion Assessment sizing
     * criterion. Possible values include: 'PerformanceBased', 'AsOnPremises'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Assessment} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Assessment} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Assessment} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { assessment? : models.Assessment, customHeaders? : { [headerName: string]: string; } }): Promise<models.Assessment>;
    create(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, callback: ServiceCallback<models.Assessment>): void;
    create(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options: { assessment? : models.Assessment, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Assessment>): void;


    /**
     * @summary Deletes an assessment from the project.
     *
     * Delete an assessment from the project. The machines remain in the
     * assessment. Deleting a non-existent assessment results in a no-operation.
     *
     * When an assessment is under computation, as indicated by the
     * 'computationState' field, it cannot be deleted. Any such attempt will return
     * a 400 - Bad Request.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Deletes an assessment from the project.
     *
     * Delete an assessment from the project. The machines remain in the
     * assessment. Deleting a non-existent assessment results in a no-operation.
     *
     * When an assessment is under computation, as indicated by the
     * 'computationState' field, it cannot be deleted. Any such attempt will return
     * a 400 - Bad Request.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * @summary Get download URL for the assessment report.
     *
     * Get the URL for downloading the assessment in a report format.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DownloadUrl>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getReportDownloadUrlWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DownloadUrl>>;

    /**
     * @summary Get download URL for the assessment report.
     *
     * Get the URL for downloading the assessment in a report format.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DownloadUrl} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DownloadUrl} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DownloadUrl} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getReportDownloadUrl(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DownloadUrl>;
    getReportDownloadUrl(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, callback: ServiceCallback<models.DownloadUrl>): void;
    getReportDownloadUrl(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DownloadUrl>): void;
}

/**
 * @class
 * AssessedMachines
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMigrate.
 */
export interface AssessedMachines {


    /**
     * @summary Get assessed machines for assessment.
     *
     * Get list of machines that assessed as part of the specified assessment.
     * Returns a json array of objects of type 'assessedMachine' as specified in
     * the Models section.
     *
     * Whenever an assessment is created or updated, it goes under computation.
     * During this phase, the 'status' field of Assessment object reports
     * 'Computing'.
     * During the period when the assessment is under computation, the list of
     * assessed machines is empty and no assessed machines are returned by this
     * call.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<AssessedMachineResultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByAssessmentWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AssessedMachineResultList>>;

    /**
     * @summary Get assessed machines for assessment.
     *
     * Get list of machines that assessed as part of the specified assessment.
     * Returns a json array of objects of type 'assessedMachine' as specified in
     * the Models section.
     *
     * Whenever an assessment is created or updated, it goes under computation.
     * During this phase, the 'status' field of Assessment object reports
     * 'Computing'.
     * During the period when the assessment is under computation, the list of
     * assessed machines is empty and no assessed machines are returned by this
     * call.
     *
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {AssessedMachineResultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {AssessedMachineResultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link AssessedMachineResultList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByAssessment(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.AssessedMachineResultList>;
    listByAssessment(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, callback: ServiceCallback<models.AssessedMachineResultList>): void;
    listByAssessment(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AssessedMachineResultList>): void;


    /**
     * @summary Get an assessed machine.
     *
     * Get an assessed machine with its size & cost estimnate that was evaluated in
     * the specified assessment.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {string} assessedMachineName Unique name of an assessed machine
     * evaluated as part of an assessment.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<AssessedMachine>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, assessedMachineName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AssessedMachine>>;

    /**
     * @summary Get an assessed machine.
     *
     * Get an assessed machine with its size & cost estimnate that was evaluated in
     * the specified assessment.
     *
     * @param {string} resourceGroupName Name of the Azure Resource Group that
     * project is part of.
     *
     * @param {string} projectName Name of the Azure Migrate project.
     *
     * @param {string} groupName Unique name of a group within a project.
     *
     * @param {string} assessmentName Unique name of an assessment within a
     * project.
     *
     * @param {string} assessedMachineName Unique name of an assessed machine
     * evaluated as part of an assessment.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {AssessedMachine} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {AssessedMachine} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link AssessedMachine} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, assessedMachineName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.AssessedMachine>;
    get(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, assessedMachineName: string, callback: ServiceCallback<models.AssessedMachine>): void;
    get(resourceGroupName: string, projectName: string, groupName: string, assessmentName: string, assessedMachineName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AssessedMachine>): void;
}

/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMigrate.
 */
export interface Operations {


    /**
     * @summary Get list of operations supported in the API.
     *
     * Get a list of REST API supported by Microsoft.Migrate provider.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationResultList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationResultList>>;

    /**
     * @summary Get list of operations supported in the API.
     *
     * Get a list of REST API supported by Microsoft.Migrate provider.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationResultList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationResultList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationResultList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationResultList>;
    list(callback: ServiceCallback<models.OperationResultList>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationResultList>): void;
}
