/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * AzureStorage backup policy.
 *
 * @extends models['ProtectionPolicy']
 */
class AzureFileShareProtectionPolicy extends models['ProtectionPolicy'] {
  /**
   * Create a AzureFileShareProtectionPolicy.
   * @member {string} [workLoadType] Type of workload for the backup
   * management. Possible values include: 'Invalid', 'VM', 'FileFolder',
   * 'AzureSqlDb', 'SQLDB', 'Exchange', 'Sharepoint', 'VMwareVM',
   * 'SystemState', 'Client', 'GenericDataSource', 'SQLDataBase',
   * 'AzureFileShare', 'SAPHanaDatabase'
   * @member {object} [schedulePolicy] Backup schedule specified as part of
   * backup policy.
   * @member {string} [schedulePolicy.schedulePolicyType] Polymorphic
   * Discriminator
   * @member {object} [retentionPolicy] Retention policy with the details on
   * backup copy retention ranges.
   * @member {string} [retentionPolicy.retentionPolicyType] Polymorphic
   * Discriminator
   * @member {string} [timeZone] TimeZone optional input as string. For
   * example: TimeZone = "Pacific Standard Time".
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of AzureFileShareProtectionPolicy
   *
   * @returns {object} metadata of AzureFileShareProtectionPolicy
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'AzureStorage',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'backupManagementType',
          clientName: 'backupManagementType'
        },
        uberParent: 'ProtectionPolicy',
        className: 'AzureFileShareProtectionPolicy',
        modelProperties: {
          protectedItemsCount: {
            required: false,
            serializedName: 'protectedItemsCount',
            type: {
              name: 'Number'
            }
          },
          backupManagementType: {
            required: true,
            serializedName: 'backupManagementType',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          workLoadType: {
            required: false,
            serializedName: 'workLoadType',
            type: {
              name: 'String'
            }
          },
          schedulePolicy: {
            required: false,
            serializedName: 'schedulePolicy',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'schedulePolicyType',
                clientName: 'schedulePolicyType'
              },
              uberParent: 'SchedulePolicy',
              className: 'SchedulePolicy'
            }
          },
          retentionPolicy: {
            required: false,
            serializedName: 'retentionPolicy',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'retentionPolicyType',
                clientName: 'retentionPolicyType'
              },
              uberParent: 'RetentionPolicy',
              className: 'RetentionPolicy'
            }
          },
          timeZone: {
            required: false,
            serializedName: 'timeZone',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = AzureFileShareProtectionPolicy;
