/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Azure File Share workload-specific backup item.
 *
 * @extends models['ProtectedItem']
 */
class AzureFileshareProtectedItem extends models['ProtectedItem'] {
  /**
   * Create a AzureFileshareProtectedItem.
   * @member {string} [friendlyName] Friendly name of the fileshare represented
   * by this backup item.
   * @member {string} [protectionStatus] Backup status of this backup item.
   * @member {string} [protectionState] Backup state of this backup item.
   * Possible values include: 'Invalid', 'IRPending', 'Protected',
   * 'ProtectionError', 'ProtectionStopped', 'ProtectionPaused'
   * @member {string} [healthStatus] backups running status for this backup
   * item. Possible values include: 'Passed', 'ActionRequired',
   * 'ActionSuggested', 'Invalid'
   * @member {string} [lastBackupStatus] Last backup operation status. Possible
   * values: Healthy, Unhealthy.
   * @member {date} [lastBackupTime] Timestamp of the last backup operation on
   * this backup item.
   * @member {object} [extendedInfo] Additional information with this backup
   * item.
   * @member {date} [extendedInfo.oldestRecoveryPoint] The oldest backup copy
   * available for this item in the service.
   * @member {number} [extendedInfo.recoveryPointCount] Number of available
   * backup copies associated with this backup item.
   * @member {string} [extendedInfo.policyState] Indicates consistency of
   * policy object and policy applied to this backup item.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of AzureFileshareProtectedItem
   *
   * @returns {object} metadata of AzureFileshareProtectedItem
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'AzureFileShareProtectedItem',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'protectedItemType',
          clientName: 'protectedItemType'
        },
        uberParent: 'ProtectedItem',
        className: 'AzureFileshareProtectedItem',
        modelProperties: {
          backupManagementType: {
            required: false,
            serializedName: 'backupManagementType',
            type: {
              name: 'String'
            }
          },
          workloadType: {
            required: false,
            serializedName: 'workloadType',
            type: {
              name: 'String'
            }
          },
          containerName: {
            required: false,
            serializedName: 'containerName',
            type: {
              name: 'String'
            }
          },
          sourceResourceId: {
            required: false,
            serializedName: 'sourceResourceId',
            type: {
              name: 'String'
            }
          },
          policyId: {
            required: false,
            serializedName: 'policyId',
            type: {
              name: 'String'
            }
          },
          lastRecoveryPoint: {
            required: false,
            serializedName: 'lastRecoveryPoint',
            type: {
              name: 'DateTime'
            }
          },
          backupSetName: {
            required: false,
            serializedName: 'backupSetName',
            type: {
              name: 'String'
            }
          },
          createMode: {
            required: false,
            serializedName: 'createMode',
            type: {
              name: 'String'
            }
          },
          protectedItemType: {
            required: true,
            serializedName: 'protectedItemType',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          friendlyName: {
            required: false,
            serializedName: 'friendlyName',
            type: {
              name: 'String'
            }
          },
          protectionStatus: {
            required: false,
            serializedName: 'protectionStatus',
            type: {
              name: 'String'
            }
          },
          protectionState: {
            required: false,
            serializedName: 'protectionState',
            type: {
              name: 'String'
            }
          },
          healthStatus: {
            required: false,
            serializedName: 'healthStatus',
            type: {
              name: 'String'
            }
          },
          lastBackupStatus: {
            required: false,
            serializedName: 'lastBackupStatus',
            type: {
              name: 'String'
            }
          },
          lastBackupTime: {
            required: false,
            serializedName: 'lastBackupTime',
            type: {
              name: 'DateTime'
            }
          },
          extendedInfo: {
            required: false,
            serializedName: 'extendedInfo',
            type: {
              name: 'Composite',
              className: 'AzureFileshareProtectedItemExtendedInfo'
            }
          }
        }
      }
    };
  }
}

module.exports = AzureFileshareProtectedItem;
