/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Compares the Sequence Number of a property with the SequenceNumber argument.
 * A property's sequence number can be thought of as that property's version.
 * Every time the property is modified, its sequence number is increased.
 * The sequence number can be found in a property's metadata.
 * The comparison fails if the sequence numbers are not equal.
 * CheckSequencePropertyBatchOperation is generally used as a precondition for
 * the write operations in the batch.
 * Note that if one PropertyBatchOperation in a PropertyBatch fails,
 * the entire batch fails and cannot be committed in a transactional manner.
 *
 * @extends models['PropertyBatchOperation']
 */
class CheckSequencePropertyBatchOperation extends models['PropertyBatchOperation'] {
  /**
   * Create a CheckSequencePropertyBatchOperation.
   * @property {string} sequenceNumber The expected sequence number.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of CheckSequencePropertyBatchOperation
   *
   * @returns {object} metadata of CheckSequencePropertyBatchOperation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'CheckSequence',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'Kind',
          clientName: 'kind'
        },
        uberParent: 'PropertyBatchOperation',
        className: 'CheckSequencePropertyBatchOperation',
        modelProperties: {
          propertyName: {
            required: true,
            serializedName: 'PropertyName',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          sequenceNumber: {
            required: true,
            serializedName: 'SequenceNumber',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = CheckSequencePropertyBatchOperation;
