/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Information about a stateless instance running in a code package. Note that
 * DeployedServiceReplicaQueryResult will contain duplicate data like
 * ServiceKind, ServiceName, PartitionId and InstanceId.
 *
 * @extends models['DeployedServiceReplicaDetailInfo']
 */
class DeployedStatelessServiceInstanceDetailInfo extends models['DeployedServiceReplicaDetailInfo'] {
  /**
   * Create a DeployedStatelessServiceInstanceDetailInfo.
   * @property {string} [instanceId] Id of a stateless service instance.
   * InstanceId is used by Service Fabric to uniquely identify an instance of a
   * partition of a stateless service. It is unique within a partition and does
   * not change for the lifetime of the instance. If the instance has failed
   * over on the same or different node, it will get a different value for the
   * InstanceId.
   * @property {object} [deployedServiceReplicaQueryResult] Information about a
   * stateless service instance deployed on a node.
   * @property {string} [deployedServiceReplicaQueryResult.instanceId] Id of a
   * stateless service instance. InstanceId is used by Service Fabric to
   * uniquely identify an instance of a partition of a stateless service. It is
   * unique within a partition and does not change for the lifetime of the
   * instance. If the instance has failed over on the same or different node,
   * it will get a different value for the InstanceId.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of DeployedStatelessServiceInstanceDetailInfo
   *
   * @returns {object} metadata of DeployedStatelessServiceInstanceDetailInfo
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Stateless',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'ServiceKind',
          clientName: 'serviceKind'
        },
        uberParent: 'DeployedServiceReplicaDetailInfo',
        className: 'DeployedStatelessServiceInstanceDetailInfo',
        modelProperties: {
          serviceName: {
            required: false,
            serializedName: 'ServiceName',
            type: {
              name: 'String'
            }
          },
          partitionId: {
            required: false,
            serializedName: 'PartitionId',
            type: {
              name: 'String'
            }
          },
          currentServiceOperation: {
            required: false,
            serializedName: 'CurrentServiceOperation',
            type: {
              name: 'String'
            }
          },
          currentServiceOperationStartTimeUtc: {
            required: false,
            serializedName: 'CurrentServiceOperationStartTimeUtc',
            type: {
              name: 'DateTime'
            }
          },
          reportedLoad: {
            required: false,
            serializedName: 'ReportedLoad',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'LoadMetricReportInfoElementType',
                  type: {
                    name: 'Composite',
                    className: 'LoadMetricReportInfo'
                  }
              }
            }
          },
          serviceKind: {
            required: true,
            serializedName: 'ServiceKind',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          instanceId: {
            required: false,
            serializedName: 'InstanceId',
            type: {
              name: 'String'
            }
          },
          deployedServiceReplicaQueryResult: {
            required: false,
            serializedName: 'DeployedServiceReplicaQueryResult',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'ServiceKind',
                clientName: 'serviceKind'
              },
              uberParent: 'DeployedServiceReplicaInfo',
              className: 'DeployedStatelessServiceInstanceInfo'
            }
          }
        }
      }
    };
  }
}

module.exports = DeployedStatelessServiceInstanceDetailInfo;
