/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

/* jshint latedef:false */
/* jshint forin:false */
/* jshint noempty:false */

'use strict';

const msRest = require('ms-rest');
const ServiceClient = msRest.ServiceClient;
const WebResource = msRest.WebResource;

const models = require('./models');


/**
 * @summary The Bing Spell Check API lets you perform contextual grammar and
 * spell checking. Bing has developed a web-based spell-checker that leverages
 * machine learning and statistical machine translation to dynamically train a
 * constantly evolving and highly contextual algorithm. The spell-checker is
 * based on a massive corpus of web searches and documents.
 *
 * @param {string} text The text string to check for spelling and grammar
 * errors. The combined length of the text string, preContextText string, and
 * postContextText string may not exceed 10,000 characters. You may specify
 * this parameter in the query string of a GET request or in the body of a POST
 * request. Because of the query string length limit, you'll typically use a
 * POST request unless you're checking only short strings.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {string} [options.acceptLanguage] A comma-delimited list of one or
 * more languages to use for user interface strings. The list is in decreasing
 * order of preference. For additional information, including expected format,
 * see [RFC2616](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html). This
 * header and the setLang query parameter are mutually exclusive; do not
 * specify both. If you set this header, you must also specify the cc query
 * parameter. Bing will use the first supported language it finds from the
 * list, and combine that language with the cc parameter value to determine the
 * market to return results for. If the list does not include a supported
 * language, Bing will find the closest language and market that supports the
 * request, and may use an aggregated or default market for the results instead
 * of a specified one. You should use this header and the cc query parameter
 * only if you specify multiple languages; otherwise, you should use the mkt
 * and setLang query parameters. A user interface string is a string that's
 * used as a label in a user interface. There are very few user interface
 * strings in the JSON response objects. Any links in the response objects to
 * Bing.com properties will apply the specified language.
 *
 * @param {string} [options.pragma] By default, Bing returns cached content, if
 * available. To prevent Bing from returning cached content, set the Pragma
 * header to no-cache (for example, Pragma: no-cache).
 *
 * @param {string} [options.userAgent] The user agent originating the request.
 * Bing uses the user agent to provide mobile users with an optimized
 * experience. Although optional, you are strongly encouraged to always specify
 * this header. The user-agent should be the same string that any commonly used
 * browser would send. For information about user agents, see [RFC
 * 2616](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html).
 *
 * @param {string} [options.clientId] Bing uses this header to provide users
 * with consistent behavior across Bing API calls. Bing often flights new
 * features and improvements, and it uses the client ID as a key for assigning
 * traffic on different flights. If you do not use the same client ID for a
 * user across multiple requests, then Bing may assign the user to multiple
 * conflicting flights. Being assigned to multiple conflicting flights can lead
 * to an inconsistent user experience. For example, if the second request has a
 * different flight assignment than the first, the experience may be
 * unexpected. Also, Bing can use the client ID to tailor web results to that
 * client ID’s search history, providing a richer experience for the user. Bing
 * also uses this header to help improve result rankings by analyzing the
 * activity generated by a client ID. The relevance improvements help with
 * better quality of results delivered by Bing APIs and in turn enables higher
 * click-through rates for the API consumer. IMPORTANT: Although optional, you
 * should consider this header required. Persisting the client ID across
 * multiple requests for the same end user and device combination enables 1)
 * the API consumer to receive a consistent user experience, and 2) higher
 * click-through rates via better quality of results from the Bing APIs. Each
 * user that uses your application on the device must have a unique, Bing
 * generated client ID. If you do not include this header in the request, Bing
 * generates an ID and returns it in the X-MSEdge-ClientID response header. The
 * only time that you should NOT include this header in a request is the first
 * time the user uses your app on that device. Use the client ID for each Bing
 * API request that your app makes for this user on the device. Persist the
 * client ID. To persist the ID in a browser app, use a persistent HTTP cookie
 * to ensure the ID is used across all sessions. Do not use a session cookie.
 * For other apps such as mobile apps, use the device's persistent storage to
 * persist the ID. The next time the user uses your app on that device, get the
 * client ID that you persisted. Bing responses may or may not include this
 * header. If the response includes this header, capture the client ID and use
 * it for all subsequent Bing requests for the user on that device. If you
 * include the X-MSEdge-ClientID, you must not include cookies in the request.
 *
 * @param {string} [options.clientIp] The IPv4 or IPv6 address of the client
 * device. The IP address is used to discover the user's location. Bing uses
 * the location information to determine safe search behavior. Although
 * optional, you are encouraged to always specify this header and the
 * X-Search-Location header. Do not obfuscate the address (for example, by
 * changing the last octet to 0). Obfuscating the address results in the
 * location not being anywhere near the device's actual location, which may
 * result in Bing serving erroneous results.
 *
 * @param {string} [options.location] A semicolon-delimited list of key/value
 * pairs that describe the client's geographical location. Bing uses the
 * location information to determine safe search behavior and to return
 * relevant local content. Specify the key/value pair as <key>:<value>. The
 * following are the keys that you use to specify the user's location. lat
 * (required): The latitude of the client's location, in degrees. The latitude
 * must be greater than or equal to -90.0 and less than or equal to +90.0.
 * Negative values indicate southern latitudes and positive values indicate
 * northern latitudes. long (required): The longitude of the client's location,
 * in degrees. The longitude must be greater than or equal to -180.0 and less
 * than or equal to +180.0. Negative values indicate western longitudes and
 * positive values indicate eastern longitudes. re (required): The radius, in
 * meters, which specifies the horizontal accuracy of the coordinates. Pass the
 * value returned by the device's location service. Typical values might be 22m
 * for GPS/Wi-Fi, 380m for cell tower triangulation, and 18,000m for reverse IP
 * lookup. ts (optional): The UTC UNIX timestamp of when the client was at the
 * location. (The UNIX timestamp is the number of seconds since January 1,
 * 1970.) head (optional): The client's relative heading or direction of
 * travel. Specify the direction of travel as degrees from 0 through 360,
 * counting clockwise relative to true north. Specify this key only if the sp
 * key is nonzero. sp (optional): The horizontal velocity (speed), in meters
 * per second, that the client device is traveling. alt (optional): The
 * altitude of the client device, in meters. are (optional): The radius, in
 * meters, that specifies the vertical accuracy of the coordinates. Specify
 * this key only if you specify the alt key. Although many of the keys are
 * optional, the more information that you provide, the more accurate the
 * location results are. Although optional, you are encouraged to always
 * specify the user's geographical location. Providing the location is
 * especially important if the client's IP address does not accurately reflect
 * the user's physical location (for example, if the client uses VPN). For
 * optimal results, you should include this header and the  X-Search-ClientIP
 * header, but at a minimum, you should include this header.
 *
 * @param {string} [options.actionType] A string that's used by logging to
 * determine whether the request is coming from an interactive session or a
 * page load. The following are the possible values. 1) Edit—The request is
 * from an interactive session 2) Load—The request is from a page load.
 * Possible values include: 'Edit', 'Load'
 *
 * @param {string} [options.appName] The unique name of your app. The name must
 * be known by Bing. Do not include this parameter unless you have previously
 * contacted Bing to get a unique app name. To get a unique name, contact your
 * Bing Business Development manager.
 *
 * @param {string} [options.countryCode] A 2-character country code of the
 * country where the results come from. This API supports only the United
 * States market. If you specify this query parameter, it must be set to us. If
 * you set this parameter, you must also specify the Accept-Language header.
 * Bing uses the first supported language it finds from the languages list, and
 * combine that language with the country code that you specify to determine
 * the market to return results for. If the languages list does not include a
 * supported language, Bing finds the closest language and market that supports
 * the request, or it may use an aggregated or default market for the results
 * instead of a specified one. You should use this query parameter and the
 * Accept-Language query parameter only if you specify multiple languages;
 * otherwise, you should use the mkt and setLang query parameters. This
 * parameter and the mkt query parameter are mutually exclusive—do not specify
 * both.
 *
 * @param {string} [options.clientMachineName] A unique name of the device that
 * the request is being made from. Generate a unique value for each device (the
 * value is unimportant). The service uses the ID to help debug issues and
 * improve the quality of corrections.
 *
 * @param {string} [options.docId] A unique ID that identifies the document
 * that the text belongs to. Generate a unique value for each document (the
 * value is unimportant). The service uses the ID to help debug issues and
 * improve the quality of corrections.
 *
 * @param {string} [options.market] The market where the results come from. You
 * are strongly encouraged to always specify the market, if known. Specifying
 * the market helps Bing route the request and return an appropriate and
 * optimal response. This parameter and the cc query parameter are mutually
 * exclusive—do not specify both.
 *
 * @param {string} [options.sessionId] A unique ID that identifies this user
 * session. Generate a unique value for each user session (the value is
 * unimportant). The service uses the ID to help debug issues and improve the
 * quality of corrections
 *
 * @param {string} [options.setLang] The language to use for user interface
 * strings. Specify the language using the ISO 639-1 2-letter language code.
 * For example, the language code for English is EN. The default is EN
 * (English). Although optional, you should always specify the language.
 * Typically, you set setLang to the same language specified by mkt unless the
 * user wants the user interface strings displayed in a different language.
 * This parameter and the Accept-Language header are mutually exclusive—do not
 * specify both. A user interface string is a string that's used as a label in
 * a user interface. There are few user interface strings in the JSON response
 * objects. Also, any links to Bing.com properties in the response objects
 * apply the specified language.
 *
 * @param {string} [options.userId] A unique ID that identifies the user.
 * Generate a unique value for each user (the value is unimportant). The
 * service uses the ID to help debug issues and improve the quality of
 * corrections.
 *
 * @param {string} [options.mode] The type of spelling and grammar checks to
 * perform. The following are the possible values (the values are case
 * insensitive). The default is Proof. 1) Proof—Finds most spelling and grammar
 * mistakes. 2) Spell—Finds most spelling mistakes but does not find some of
 * the grammar errors that Proof catches (for example, capitalization and
 * repeated words). Possible values include: 'proof', 'spell'
 *
 * @param {string} [options.preContextText] A string that gives context to the
 * text string. For example, the text string petal is valid. However, if you
 * set preContextText to bike, the context changes and the text string becomes
 * not valid. In this case, the API suggests that you change petal to pedal (as
 * in bike pedal). This text is not checked for grammar or spelling errors. The
 * combined length of the text string, preContextText string, and
 * postContextText string may not exceed 10,000 characters. You may specify
 * this parameter in the query string of a GET request or in the body of a POST
 * request.
 *
 * @param {string} [options.postContextText] A string that gives context to the
 * text string. For example, the text string read is valid. However, if you set
 * postContextText to carpet, the context changes and the text string becomes
 * not valid. In this case, the API suggests that you change read to red (as in
 * red carpet). This text is not checked for grammar or spelling errors. The
 * combined length of the text string, preContextText string, and
 * postContextText string may not exceed 10,000 characters. You may specify
 * this parameter in the query string of a GET request or in the body of a POST
 * request.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link SpellCheck} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _spellChecker(text, options, callback) {
   /* jshint validthis: true */
  let client = this;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let acceptLanguage = (options && options.acceptLanguage !== undefined) ? options.acceptLanguage : undefined;
  let pragma = (options && options.pragma !== undefined) ? options.pragma : undefined;
  let userAgent = (options && options.userAgent !== undefined) ? options.userAgent : undefined;
  let clientId = (options && options.clientId !== undefined) ? options.clientId : undefined;
  let clientIp = (options && options.clientIp !== undefined) ? options.clientIp : undefined;
  let location = (options && options.location !== undefined) ? options.location : undefined;
  let actionType = (options && options.actionType !== undefined) ? options.actionType : undefined;
  let appName = (options && options.appName !== undefined) ? options.appName : undefined;
  let countryCode = (options && options.countryCode !== undefined) ? options.countryCode : undefined;
  let clientMachineName = (options && options.clientMachineName !== undefined) ? options.clientMachineName : undefined;
  let docId = (options && options.docId !== undefined) ? options.docId : undefined;
  let market = (options && options.market !== undefined) ? options.market : undefined;
  let sessionId = (options && options.sessionId !== undefined) ? options.sessionId : undefined;
  let setLang = (options && options.setLang !== undefined) ? options.setLang : undefined;
  let userId = (options && options.userId !== undefined) ? options.userId : undefined;
  let mode = (options && options.mode !== undefined) ? options.mode : undefined;
  let preContextText = (options && options.preContextText !== undefined) ? options.preContextText : undefined;
  let postContextText = (options && options.postContextText !== undefined) ? options.postContextText : undefined;
  let xBingApisSDK = 'true';
  // Validate
  try {
    if (this.endpoint === null || this.endpoint === undefined || typeof this.endpoint.valueOf() !== 'string') {
      throw new Error('this.endpoint cannot be null or undefined and it must be of type string.');
    }
    if (acceptLanguage !== null && acceptLanguage !== undefined && typeof acceptLanguage.valueOf() !== 'string') {
      throw new Error('acceptLanguage must be of type string.');
    }
    if (pragma !== null && pragma !== undefined && typeof pragma.valueOf() !== 'string') {
      throw new Error('pragma must be of type string.');
    }
    if (userAgent !== null && userAgent !== undefined && typeof userAgent.valueOf() !== 'string') {
      throw new Error('userAgent must be of type string.');
    }
    if (clientId !== null && clientId !== undefined && typeof clientId.valueOf() !== 'string') {
      throw new Error('clientId must be of type string.');
    }
    if (clientIp !== null && clientIp !== undefined && typeof clientIp.valueOf() !== 'string') {
      throw new Error('clientIp must be of type string.');
    }
    if (location !== null && location !== undefined && typeof location.valueOf() !== 'string') {
      throw new Error('location must be of type string.');
    }
    if (actionType !== null && actionType !== undefined && typeof actionType.valueOf() !== 'string') {
      throw new Error('actionType must be of type string.');
    }
    if (appName !== null && appName !== undefined && typeof appName.valueOf() !== 'string') {
      throw new Error('appName must be of type string.');
    }
    if (countryCode !== null && countryCode !== undefined && typeof countryCode.valueOf() !== 'string') {
      throw new Error('countryCode must be of type string.');
    }
    if (clientMachineName !== null && clientMachineName !== undefined && typeof clientMachineName.valueOf() !== 'string') {
      throw new Error('clientMachineName must be of type string.');
    }
    if (docId !== null && docId !== undefined && typeof docId.valueOf() !== 'string') {
      throw new Error('docId must be of type string.');
    }
    if (market !== null && market !== undefined && typeof market.valueOf() !== 'string') {
      throw new Error('market must be of type string.');
    }
    if (sessionId !== null && sessionId !== undefined && typeof sessionId.valueOf() !== 'string') {
      throw new Error('sessionId must be of type string.');
    }
    if (setLang !== null && setLang !== undefined && typeof setLang.valueOf() !== 'string') {
      throw new Error('setLang must be of type string.');
    }
    if (userId !== null && userId !== undefined && typeof userId.valueOf() !== 'string') {
      throw new Error('userId must be of type string.');
    }
    if (mode !== null && mode !== undefined && typeof mode.valueOf() !== 'string') {
      throw new Error('mode must be of type string.');
    }
    if (preContextText !== null && preContextText !== undefined && typeof preContextText.valueOf() !== 'string') {
      throw new Error('preContextText must be of type string.');
    }
    if (postContextText !== null && postContextText !== undefined && typeof postContextText.valueOf() !== 'string') {
      throw new Error('postContextText must be of type string.');
    }
    if (text === null || text === undefined || typeof text.valueOf() !== 'string') {
      throw new Error('text cannot be null or undefined and it must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'spellcheck';
  requestUrl = requestUrl.replace('{Endpoint}', this.endpoint);
  let queryParameters = [];
  if (actionType !== null && actionType !== undefined) {
    queryParameters.push('ActionType=' + encodeURIComponent(actionType));
  }
  if (appName !== null && appName !== undefined) {
    queryParameters.push('AppName=' + encodeURIComponent(appName));
  }
  if (countryCode !== null && countryCode !== undefined) {
    queryParameters.push('cc=' + encodeURIComponent(countryCode));
  }
  if (clientMachineName !== null && clientMachineName !== undefined) {
    queryParameters.push('ClientMachineName=' + encodeURIComponent(clientMachineName));
  }
  if (docId !== null && docId !== undefined) {
    queryParameters.push('DocId=' + encodeURIComponent(docId));
  }
  if (market !== null && market !== undefined) {
    queryParameters.push('mkt=' + encodeURIComponent(market));
  }
  if (sessionId !== null && sessionId !== undefined) {
    queryParameters.push('SessionId=' + encodeURIComponent(sessionId));
  }
  if (setLang !== null && setLang !== undefined) {
    queryParameters.push('SetLang=' + encodeURIComponent(setLang));
  }
  if (userId !== null && userId !== undefined) {
    queryParameters.push('UserId=' + encodeURIComponent(userId));
  }
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/x-www-form-urlencoded';
  if (xBingApisSDK !== undefined && xBingApisSDK !== null) {
    httpRequest.headers['X-BingApis-SDK'] = xBingApisSDK;
  }
  if (acceptLanguage !== undefined && acceptLanguage !== null) {
    httpRequest.headers['Accept-Language'] = acceptLanguage;
  }
  if (pragma !== undefined && pragma !== null) {
    httpRequest.headers['Pragma'] = pragma;
  }
  if (userAgent !== undefined && userAgent !== null) {
    httpRequest.headers['User-Agent'] = userAgent;
  }
  if (clientId !== undefined && clientId !== null) {
    httpRequest.headers['X-MSEdge-ClientID'] = clientId;
  }
  if (clientIp !== undefined && clientIp !== null) {
    httpRequest.headers['X-MSEdge-ClientIP'] = clientIp;
  }
  if (location !== undefined && location !== null) {
    httpRequest.headers['X-Search-Location'] = location;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let formData = {};
  if (mode !== undefined && mode !== null) {
    formData['Mode'] = mode;
  }
  if (preContextText !== undefined && preContextText !== null) {
    formData['PreContextText'] = preContextText;
  }
  if (postContextText !== undefined && postContextText !== null) {
    formData['PostContextText'] = postContextText;
  }
  if (text !== undefined && text !== null) {
    formData['Text'] = text;
  }
  httpRequest.formData = formData;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          let internalError = null;
          if (parsedErrorResponse.error) internalError = parsedErrorResponse.error;
          error.code = internalError ? internalError.code : parsedErrorResponse.code;
          error.message = internalError ? internalError.message : parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['ErrorResponse']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['SpellCheck']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a SpellCheckClient. */
class SpellCheckClient extends ServiceClient {
  /**
   * Create a SpellCheckClient.
   * @param {credentials} credentials - Subscription credentials which uniquely identify client subscription.
   * @param {object} [options] - The parameter options
   * @param {Array} [options.filters] - Filters to be added to the request pipeline
   * @param {object} [options.requestOptions] - Options for the underlying request object
   * {@link https://github.com/request/request#requestoptions-callback Options doc}
   * @param {boolean} [options.noRetryPolicy] - If set to true, turn off default retry policy
   * @param {string} [options.endpoint] - Supported Cognitive Services endpoints (protocol and hostname, for example: "https://westus.api.cognitive.microsoft.com", "https://api.cognitive.microsoft.com").
   */
  constructor(credentials, options) {
    if (credentials === null || credentials === undefined) {
      throw new Error('\'credentials\' cannot be null.');
    }

    if (!options) options = {};

    super(credentials, options);

    this.endpoint = 'https://api.cognitive.microsoft.com';
    this.baseUri = '{Endpoint}/bing/v7.0';
    this.credentials = credentials;

    let packageInfo = this.getPackageJsonInfo(__dirname);
    this.addUserAgentInfo(`${packageInfo.name}/${packageInfo.version}`);
    if(options.endpoint !== null && options.endpoint !== undefined) {
      this.endpoint = options.endpoint;
    }
    this.models = models;
    this._spellChecker = _spellChecker;
    msRest.addSerializationMixin(this);
  }

  /**
   * @summary The Bing Spell Check API lets you perform contextual grammar and
   * spell checking. Bing has developed a web-based spell-checker that leverages
   * machine learning and statistical machine translation to dynamically train a
   * constantly evolving and highly contextual algorithm. The spell-checker is
   * based on a massive corpus of web searches and documents.
   *
   * @param {string} text The text string to check for spelling and grammar
   * errors. The combined length of the text string, preContextText string, and
   * postContextText string may not exceed 10,000 characters. You may specify
   * this parameter in the query string of a GET request or in the body of a POST
   * request. Because of the query string length limit, you'll typically use a
   * POST request unless you're checking only short strings.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.acceptLanguage] A comma-delimited list of one or
   * more languages to use for user interface strings. The list is in decreasing
   * order of preference. For additional information, including expected format,
   * see [RFC2616](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html). This
   * header and the setLang query parameter are mutually exclusive; do not
   * specify both. If you set this header, you must also specify the cc query
   * parameter. Bing will use the first supported language it finds from the
   * list, and combine that language with the cc parameter value to determine the
   * market to return results for. If the list does not include a supported
   * language, Bing will find the closest language and market that supports the
   * request, and may use an aggregated or default market for the results instead
   * of a specified one. You should use this header and the cc query parameter
   * only if you specify multiple languages; otherwise, you should use the mkt
   * and setLang query parameters. A user interface string is a string that's
   * used as a label in a user interface. There are very few user interface
   * strings in the JSON response objects. Any links in the response objects to
   * Bing.com properties will apply the specified language.
   *
   * @param {string} [options.pragma] By default, Bing returns cached content, if
   * available. To prevent Bing from returning cached content, set the Pragma
   * header to no-cache (for example, Pragma: no-cache).
   *
   * @param {string} [options.userAgent] The user agent originating the request.
   * Bing uses the user agent to provide mobile users with an optimized
   * experience. Although optional, you are strongly encouraged to always specify
   * this header. The user-agent should be the same string that any commonly used
   * browser would send. For information about user agents, see [RFC
   * 2616](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html).
   *
   * @param {string} [options.clientId] Bing uses this header to provide users
   * with consistent behavior across Bing API calls. Bing often flights new
   * features and improvements, and it uses the client ID as a key for assigning
   * traffic on different flights. If you do not use the same client ID for a
   * user across multiple requests, then Bing may assign the user to multiple
   * conflicting flights. Being assigned to multiple conflicting flights can lead
   * to an inconsistent user experience. For example, if the second request has a
   * different flight assignment than the first, the experience may be
   * unexpected. Also, Bing can use the client ID to tailor web results to that
   * client ID’s search history, providing a richer experience for the user. Bing
   * also uses this header to help improve result rankings by analyzing the
   * activity generated by a client ID. The relevance improvements help with
   * better quality of results delivered by Bing APIs and in turn enables higher
   * click-through rates for the API consumer. IMPORTANT: Although optional, you
   * should consider this header required. Persisting the client ID across
   * multiple requests for the same end user and device combination enables 1)
   * the API consumer to receive a consistent user experience, and 2) higher
   * click-through rates via better quality of results from the Bing APIs. Each
   * user that uses your application on the device must have a unique, Bing
   * generated client ID. If you do not include this header in the request, Bing
   * generates an ID and returns it in the X-MSEdge-ClientID response header. The
   * only time that you should NOT include this header in a request is the first
   * time the user uses your app on that device. Use the client ID for each Bing
   * API request that your app makes for this user on the device. Persist the
   * client ID. To persist the ID in a browser app, use a persistent HTTP cookie
   * to ensure the ID is used across all sessions. Do not use a session cookie.
   * For other apps such as mobile apps, use the device's persistent storage to
   * persist the ID. The next time the user uses your app on that device, get the
   * client ID that you persisted. Bing responses may or may not include this
   * header. If the response includes this header, capture the client ID and use
   * it for all subsequent Bing requests for the user on that device. If you
   * include the X-MSEdge-ClientID, you must not include cookies in the request.
   *
   * @param {string} [options.clientIp] The IPv4 or IPv6 address of the client
   * device. The IP address is used to discover the user's location. Bing uses
   * the location information to determine safe search behavior. Although
   * optional, you are encouraged to always specify this header and the
   * X-Search-Location header. Do not obfuscate the address (for example, by
   * changing the last octet to 0). Obfuscating the address results in the
   * location not being anywhere near the device's actual location, which may
   * result in Bing serving erroneous results.
   *
   * @param {string} [options.location] A semicolon-delimited list of key/value
   * pairs that describe the client's geographical location. Bing uses the
   * location information to determine safe search behavior and to return
   * relevant local content. Specify the key/value pair as <key>:<value>. The
   * following are the keys that you use to specify the user's location. lat
   * (required): The latitude of the client's location, in degrees. The latitude
   * must be greater than or equal to -90.0 and less than or equal to +90.0.
   * Negative values indicate southern latitudes and positive values indicate
   * northern latitudes. long (required): The longitude of the client's location,
   * in degrees. The longitude must be greater than or equal to -180.0 and less
   * than or equal to +180.0. Negative values indicate western longitudes and
   * positive values indicate eastern longitudes. re (required): The radius, in
   * meters, which specifies the horizontal accuracy of the coordinates. Pass the
   * value returned by the device's location service. Typical values might be 22m
   * for GPS/Wi-Fi, 380m for cell tower triangulation, and 18,000m for reverse IP
   * lookup. ts (optional): The UTC UNIX timestamp of when the client was at the
   * location. (The UNIX timestamp is the number of seconds since January 1,
   * 1970.) head (optional): The client's relative heading or direction of
   * travel. Specify the direction of travel as degrees from 0 through 360,
   * counting clockwise relative to true north. Specify this key only if the sp
   * key is nonzero. sp (optional): The horizontal velocity (speed), in meters
   * per second, that the client device is traveling. alt (optional): The
   * altitude of the client device, in meters. are (optional): The radius, in
   * meters, that specifies the vertical accuracy of the coordinates. Specify
   * this key only if you specify the alt key. Although many of the keys are
   * optional, the more information that you provide, the more accurate the
   * location results are. Although optional, you are encouraged to always
   * specify the user's geographical location. Providing the location is
   * especially important if the client's IP address does not accurately reflect
   * the user's physical location (for example, if the client uses VPN). For
   * optimal results, you should include this header and the  X-Search-ClientIP
   * header, but at a minimum, you should include this header.
   *
   * @param {string} [options.actionType] A string that's used by logging to
   * determine whether the request is coming from an interactive session or a
   * page load. The following are the possible values. 1) Edit—The request is
   * from an interactive session 2) Load—The request is from a page load.
   * Possible values include: 'Edit', 'Load'
   *
   * @param {string} [options.appName] The unique name of your app. The name must
   * be known by Bing. Do not include this parameter unless you have previously
   * contacted Bing to get a unique app name. To get a unique name, contact your
   * Bing Business Development manager.
   *
   * @param {string} [options.countryCode] A 2-character country code of the
   * country where the results come from. This API supports only the United
   * States market. If you specify this query parameter, it must be set to us. If
   * you set this parameter, you must also specify the Accept-Language header.
   * Bing uses the first supported language it finds from the languages list, and
   * combine that language with the country code that you specify to determine
   * the market to return results for. If the languages list does not include a
   * supported language, Bing finds the closest language and market that supports
   * the request, or it may use an aggregated or default market for the results
   * instead of a specified one. You should use this query parameter and the
   * Accept-Language query parameter only if you specify multiple languages;
   * otherwise, you should use the mkt and setLang query parameters. This
   * parameter and the mkt query parameter are mutually exclusive—do not specify
   * both.
   *
   * @param {string} [options.clientMachineName] A unique name of the device that
   * the request is being made from. Generate a unique value for each device (the
   * value is unimportant). The service uses the ID to help debug issues and
   * improve the quality of corrections.
   *
   * @param {string} [options.docId] A unique ID that identifies the document
   * that the text belongs to. Generate a unique value for each document (the
   * value is unimportant). The service uses the ID to help debug issues and
   * improve the quality of corrections.
   *
   * @param {string} [options.market] The market where the results come from. You
   * are strongly encouraged to always specify the market, if known. Specifying
   * the market helps Bing route the request and return an appropriate and
   * optimal response. This parameter and the cc query parameter are mutually
   * exclusive—do not specify both.
   *
   * @param {string} [options.sessionId] A unique ID that identifies this user
   * session. Generate a unique value for each user session (the value is
   * unimportant). The service uses the ID to help debug issues and improve the
   * quality of corrections
   *
   * @param {string} [options.setLang] The language to use for user interface
   * strings. Specify the language using the ISO 639-1 2-letter language code.
   * For example, the language code for English is EN. The default is EN
   * (English). Although optional, you should always specify the language.
   * Typically, you set setLang to the same language specified by mkt unless the
   * user wants the user interface strings displayed in a different language.
   * This parameter and the Accept-Language header are mutually exclusive—do not
   * specify both. A user interface string is a string that's used as a label in
   * a user interface. There are few user interface strings in the JSON response
   * objects. Also, any links to Bing.com properties in the response objects
   * apply the specified language.
   *
   * @param {string} [options.userId] A unique ID that identifies the user.
   * Generate a unique value for each user (the value is unimportant). The
   * service uses the ID to help debug issues and improve the quality of
   * corrections.
   *
   * @param {string} [options.mode] The type of spelling and grammar checks to
   * perform. The following are the possible values (the values are case
   * insensitive). The default is Proof. 1) Proof—Finds most spelling and grammar
   * mistakes. 2) Spell—Finds most spelling mistakes but does not find some of
   * the grammar errors that Proof catches (for example, capitalization and
   * repeated words). Possible values include: 'proof', 'spell'
   *
   * @param {string} [options.preContextText] A string that gives context to the
   * text string. For example, the text string petal is valid. However, if you
   * set preContextText to bike, the context changes and the text string becomes
   * not valid. In this case, the API suggests that you change petal to pedal (as
   * in bike pedal). This text is not checked for grammar or spelling errors. The
   * combined length of the text string, preContextText string, and
   * postContextText string may not exceed 10,000 characters. You may specify
   * this parameter in the query string of a GET request or in the body of a POST
   * request.
   *
   * @param {string} [options.postContextText] A string that gives context to the
   * text string. For example, the text string read is valid. However, if you set
   * postContextText to carpet, the context changes and the text string becomes
   * not valid. In this case, the API suggests that you change read to red (as in
   * red carpet). This text is not checked for grammar or spelling errors. The
   * combined length of the text string, preContextText string, and
   * postContextText string may not exceed 10,000 characters. You may specify
   * this parameter in the query string of a GET request or in the body of a POST
   * request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<SpellCheck>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  spellCheckerWithHttpOperationResponse(text, options) {
    let client = this;
    let self = this;
    return new Promise((resolve, reject) => {
      self._spellChecker(text, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * @summary The Bing Spell Check API lets you perform contextual grammar and
   * spell checking. Bing has developed a web-based spell-checker that leverages
   * machine learning and statistical machine translation to dynamically train a
   * constantly evolving and highly contextual algorithm. The spell-checker is
   * based on a massive corpus of web searches and documents.
   *
   * @param {string} text The text string to check for spelling and grammar
   * errors. The combined length of the text string, preContextText string, and
   * postContextText string may not exceed 10,000 characters. You may specify
   * this parameter in the query string of a GET request or in the body of a POST
   * request. Because of the query string length limit, you'll typically use a
   * POST request unless you're checking only short strings.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.acceptLanguage] A comma-delimited list of one or
   * more languages to use for user interface strings. The list is in decreasing
   * order of preference. For additional information, including expected format,
   * see [RFC2616](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html). This
   * header and the setLang query parameter are mutually exclusive; do not
   * specify both. If you set this header, you must also specify the cc query
   * parameter. Bing will use the first supported language it finds from the
   * list, and combine that language with the cc parameter value to determine the
   * market to return results for. If the list does not include a supported
   * language, Bing will find the closest language and market that supports the
   * request, and may use an aggregated or default market for the results instead
   * of a specified one. You should use this header and the cc query parameter
   * only if you specify multiple languages; otherwise, you should use the mkt
   * and setLang query parameters. A user interface string is a string that's
   * used as a label in a user interface. There are very few user interface
   * strings in the JSON response objects. Any links in the response objects to
   * Bing.com properties will apply the specified language.
   *
   * @param {string} [options.pragma] By default, Bing returns cached content, if
   * available. To prevent Bing from returning cached content, set the Pragma
   * header to no-cache (for example, Pragma: no-cache).
   *
   * @param {string} [options.userAgent] The user agent originating the request.
   * Bing uses the user agent to provide mobile users with an optimized
   * experience. Although optional, you are strongly encouraged to always specify
   * this header. The user-agent should be the same string that any commonly used
   * browser would send. For information about user agents, see [RFC
   * 2616](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html).
   *
   * @param {string} [options.clientId] Bing uses this header to provide users
   * with consistent behavior across Bing API calls. Bing often flights new
   * features and improvements, and it uses the client ID as a key for assigning
   * traffic on different flights. If you do not use the same client ID for a
   * user across multiple requests, then Bing may assign the user to multiple
   * conflicting flights. Being assigned to multiple conflicting flights can lead
   * to an inconsistent user experience. For example, if the second request has a
   * different flight assignment than the first, the experience may be
   * unexpected. Also, Bing can use the client ID to tailor web results to that
   * client ID’s search history, providing a richer experience for the user. Bing
   * also uses this header to help improve result rankings by analyzing the
   * activity generated by a client ID. The relevance improvements help with
   * better quality of results delivered by Bing APIs and in turn enables higher
   * click-through rates for the API consumer. IMPORTANT: Although optional, you
   * should consider this header required. Persisting the client ID across
   * multiple requests for the same end user and device combination enables 1)
   * the API consumer to receive a consistent user experience, and 2) higher
   * click-through rates via better quality of results from the Bing APIs. Each
   * user that uses your application on the device must have a unique, Bing
   * generated client ID. If you do not include this header in the request, Bing
   * generates an ID and returns it in the X-MSEdge-ClientID response header. The
   * only time that you should NOT include this header in a request is the first
   * time the user uses your app on that device. Use the client ID for each Bing
   * API request that your app makes for this user on the device. Persist the
   * client ID. To persist the ID in a browser app, use a persistent HTTP cookie
   * to ensure the ID is used across all sessions. Do not use a session cookie.
   * For other apps such as mobile apps, use the device's persistent storage to
   * persist the ID. The next time the user uses your app on that device, get the
   * client ID that you persisted. Bing responses may or may not include this
   * header. If the response includes this header, capture the client ID and use
   * it for all subsequent Bing requests for the user on that device. If you
   * include the X-MSEdge-ClientID, you must not include cookies in the request.
   *
   * @param {string} [options.clientIp] The IPv4 or IPv6 address of the client
   * device. The IP address is used to discover the user's location. Bing uses
   * the location information to determine safe search behavior. Although
   * optional, you are encouraged to always specify this header and the
   * X-Search-Location header. Do not obfuscate the address (for example, by
   * changing the last octet to 0). Obfuscating the address results in the
   * location not being anywhere near the device's actual location, which may
   * result in Bing serving erroneous results.
   *
   * @param {string} [options.location] A semicolon-delimited list of key/value
   * pairs that describe the client's geographical location. Bing uses the
   * location information to determine safe search behavior and to return
   * relevant local content. Specify the key/value pair as <key>:<value>. The
   * following are the keys that you use to specify the user's location. lat
   * (required): The latitude of the client's location, in degrees. The latitude
   * must be greater than or equal to -90.0 and less than or equal to +90.0.
   * Negative values indicate southern latitudes and positive values indicate
   * northern latitudes. long (required): The longitude of the client's location,
   * in degrees. The longitude must be greater than or equal to -180.0 and less
   * than or equal to +180.0. Negative values indicate western longitudes and
   * positive values indicate eastern longitudes. re (required): The radius, in
   * meters, which specifies the horizontal accuracy of the coordinates. Pass the
   * value returned by the device's location service. Typical values might be 22m
   * for GPS/Wi-Fi, 380m for cell tower triangulation, and 18,000m for reverse IP
   * lookup. ts (optional): The UTC UNIX timestamp of when the client was at the
   * location. (The UNIX timestamp is the number of seconds since January 1,
   * 1970.) head (optional): The client's relative heading or direction of
   * travel. Specify the direction of travel as degrees from 0 through 360,
   * counting clockwise relative to true north. Specify this key only if the sp
   * key is nonzero. sp (optional): The horizontal velocity (speed), in meters
   * per second, that the client device is traveling. alt (optional): The
   * altitude of the client device, in meters. are (optional): The radius, in
   * meters, that specifies the vertical accuracy of the coordinates. Specify
   * this key only if you specify the alt key. Although many of the keys are
   * optional, the more information that you provide, the more accurate the
   * location results are. Although optional, you are encouraged to always
   * specify the user's geographical location. Providing the location is
   * especially important if the client's IP address does not accurately reflect
   * the user's physical location (for example, if the client uses VPN). For
   * optimal results, you should include this header and the  X-Search-ClientIP
   * header, but at a minimum, you should include this header.
   *
   * @param {string} [options.actionType] A string that's used by logging to
   * determine whether the request is coming from an interactive session or a
   * page load. The following are the possible values. 1) Edit—The request is
   * from an interactive session 2) Load—The request is from a page load.
   * Possible values include: 'Edit', 'Load'
   *
   * @param {string} [options.appName] The unique name of your app. The name must
   * be known by Bing. Do not include this parameter unless you have previously
   * contacted Bing to get a unique app name. To get a unique name, contact your
   * Bing Business Development manager.
   *
   * @param {string} [options.countryCode] A 2-character country code of the
   * country where the results come from. This API supports only the United
   * States market. If you specify this query parameter, it must be set to us. If
   * you set this parameter, you must also specify the Accept-Language header.
   * Bing uses the first supported language it finds from the languages list, and
   * combine that language with the country code that you specify to determine
   * the market to return results for. If the languages list does not include a
   * supported language, Bing finds the closest language and market that supports
   * the request, or it may use an aggregated or default market for the results
   * instead of a specified one. You should use this query parameter and the
   * Accept-Language query parameter only if you specify multiple languages;
   * otherwise, you should use the mkt and setLang query parameters. This
   * parameter and the mkt query parameter are mutually exclusive—do not specify
   * both.
   *
   * @param {string} [options.clientMachineName] A unique name of the device that
   * the request is being made from. Generate a unique value for each device (the
   * value is unimportant). The service uses the ID to help debug issues and
   * improve the quality of corrections.
   *
   * @param {string} [options.docId] A unique ID that identifies the document
   * that the text belongs to. Generate a unique value for each document (the
   * value is unimportant). The service uses the ID to help debug issues and
   * improve the quality of corrections.
   *
   * @param {string} [options.market] The market where the results come from. You
   * are strongly encouraged to always specify the market, if known. Specifying
   * the market helps Bing route the request and return an appropriate and
   * optimal response. This parameter and the cc query parameter are mutually
   * exclusive—do not specify both.
   *
   * @param {string} [options.sessionId] A unique ID that identifies this user
   * session. Generate a unique value for each user session (the value is
   * unimportant). The service uses the ID to help debug issues and improve the
   * quality of corrections
   *
   * @param {string} [options.setLang] The language to use for user interface
   * strings. Specify the language using the ISO 639-1 2-letter language code.
   * For example, the language code for English is EN. The default is EN
   * (English). Although optional, you should always specify the language.
   * Typically, you set setLang to the same language specified by mkt unless the
   * user wants the user interface strings displayed in a different language.
   * This parameter and the Accept-Language header are mutually exclusive—do not
   * specify both. A user interface string is a string that's used as a label in
   * a user interface. There are few user interface strings in the JSON response
   * objects. Also, any links to Bing.com properties in the response objects
   * apply the specified language.
   *
   * @param {string} [options.userId] A unique ID that identifies the user.
   * Generate a unique value for each user (the value is unimportant). The
   * service uses the ID to help debug issues and improve the quality of
   * corrections.
   *
   * @param {string} [options.mode] The type of spelling and grammar checks to
   * perform. The following are the possible values (the values are case
   * insensitive). The default is Proof. 1) Proof—Finds most spelling and grammar
   * mistakes. 2) Spell—Finds most spelling mistakes but does not find some of
   * the grammar errors that Proof catches (for example, capitalization and
   * repeated words). Possible values include: 'proof', 'spell'
   *
   * @param {string} [options.preContextText] A string that gives context to the
   * text string. For example, the text string petal is valid. However, if you
   * set preContextText to bike, the context changes and the text string becomes
   * not valid. In this case, the API suggests that you change petal to pedal (as
   * in bike pedal). This text is not checked for grammar or spelling errors. The
   * combined length of the text string, preContextText string, and
   * postContextText string may not exceed 10,000 characters. You may specify
   * this parameter in the query string of a GET request or in the body of a POST
   * request.
   *
   * @param {string} [options.postContextText] A string that gives context to the
   * text string. For example, the text string read is valid. However, if you set
   * postContextText to carpet, the context changes and the text string becomes
   * not valid. In this case, the API suggests that you change read to red (as in
   * red carpet). This text is not checked for grammar or spelling errors. The
   * combined length of the text string, preContextText string, and
   * postContextText string may not exceed 10,000 characters. You may specify
   * this parameter in the query string of a GET request or in the body of a POST
   * request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {SpellCheck} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link SpellCheck} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  spellChecker(text, options, optionalCallback) {
    let client = this;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._spellChecker(text, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._spellChecker(text, options, optionalCallback);
    }
  }

}

module.exports = SpellCheckClient;
module.exports['default'] = SpellCheckClient;
module.exports.SpellCheckClient = SpellCheckClient;
module.exports.SpellCheckModels = models;
