/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the SubscriptionCreationResult class.
 * @constructor
 * The created subscription object.
 *
 * @property {string} [subscriptionLink] The link to the new subscription.
 */
export interface SubscriptionCreationResult {
  subscriptionLink?: string;
}

/**
 * @class
 * Initializes a new instance of the AdPrincipal class.
 * @constructor
 * Active Directory Principal for subscription creation delegated permission
 *
 * @property {string} objectId Object id of the Principal
 */
export interface AdPrincipal {
  objectId: string;
}

/**
 * @class
 * Initializes a new instance of the SubscriptionCreationParameters class.
 * @constructor
 * Subscription Creation Parameters required to create a new Azure
 * subscription.
 *
 * @property {string} [displayName] The display name of the subscription.
 * @property {array} [owners] The list of principals that should be granted
 * Owner access on the subscription. Principals should be of type User, Service
 * Principal or Security Group.
 * @property {string} [offerType] The offer type of the subscription. For
 * example, MS-AZR-0017P (EnterpriseAgreement) and MS-AZR-0148P
 * (EnterpriseAgreement devTest) are available. Only valid when creating a
 * subscription in a enrollment account scope. Possible values include:
 * 'MS-AZR-0017P', 'MS-AZR-0148P'
 * @property {object} [additionalParameters] Additional, untyped parameters to
 * support custom subscription creation scenarios.
 */
export interface SubscriptionCreationParameters {
  displayName?: string;
  owners?: AdPrincipal[];
  offerType?: string;
  additionalParameters?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * Describes the format of Error response.
 *
 * @property {string} [code] Error code
 * @property {string} [message] Error message indicating why the operation
 * failed.
 */
export interface ErrorResponse {
  code?: string;
  message?: string;
}

/**
 * @class
 * Initializes a new instance of the SubscriptionOperation class.
 * @constructor
 * status of the subscription POST operation.
 *
 * @property {string} [id] The operation Id.
 * @property {string} [status] Status of the pending subscription
 * @property {string} [statusDetail] Status Detail of the pending subscription
 */
export interface SubscriptionOperation {
  readonly id?: string;
  status?: string;
  statusDetail?: string;
}

/**
 * @class
 * Initializes a new instance of the SubscriptionOperationListResult class.
 * @constructor
 * A list of pending subscription operations.
 *
 * @property {array} [value] A list of pending SubscriptionOperations
 */
export interface SubscriptionOperationListResult {
  value?: SubscriptionOperation[];
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * The object that represents the operation.
 *
 * @property {string} [provider] Service provider: Microsoft.Subscription
 * @property {string} [resource] Resource on which the operation is performed:
 * Profile, endpoint, etc.
 * @property {string} [operation] Operation type: Read, write, delete, etc.
 */
export interface OperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * REST API operation
 *
 * @property {string} [name] Operation name: {provider}/{resource}/{operation}
 * @property {object} [display] The object that represents the operation.
 * @property {string} [display.provider] Service provider:
 * Microsoft.Subscription
 * @property {string} [display.resource] Resource on which the operation is
 * performed: Profile, endpoint, etc.
 * @property {string} [display.operation] Operation type: Read, write, delete,
 * etc.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Result of the request to list operations. It contains a list of operations
 * and a URL link to get the next set of results.
 *
 * @property {array} [value] List of operations.
 * @property {string} [nextLink] URL to get the next set of operation list
 * results if there are any.
 */
export interface OperationListResult {
  value?: Operation[];
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the Location class.
 * @constructor
 * Location information.
 *
 * @property {string} [id] The fully qualified ID of the location. For example,
 * /subscriptions/00000000-0000-0000-0000-000000000000/locations/westus.
 * @property {string} [subscriptionId] The subscription ID.
 * @property {string} [name] The location name.
 * @property {string} [displayName] The display name of the location.
 * @property {string} [latitude] The latitude of the location.
 * @property {string} [longitude] The longitude of the location.
 */
export interface Location {
  readonly id?: string;
  readonly subscriptionId?: string;
  readonly name?: string;
  readonly displayName?: string;
  readonly latitude?: string;
  readonly longitude?: string;
}

/**
 * @class
 * Initializes a new instance of the SubscriptionPolicies class.
 * @constructor
 * Subscription policies.
 *
 * @property {string} [locationPlacementId] The subscription location placement
 * ID. The ID indicates which regions are visible for a subscription. For
 * example, a subscription with a location placement Id of Public_2014-09-01
 * has access to Azure public regions.
 * @property {string} [quotaId] The subscription quota ID.
 * @property {string} [spendingLimit] The subscription spending limit. Possible
 * values include: 'On', 'Off', 'CurrentPeriodOff'
 */
export interface SubscriptionPolicies {
  readonly locationPlacementId?: string;
  readonly quotaId?: string;
  readonly spendingLimit?: string;
}

/**
 * @class
 * Initializes a new instance of the Subscription class.
 * @constructor
 * Subscription information.
 *
 * @property {string} [id] The fully qualified ID for the subscription. For
 * example, /subscriptions/00000000-0000-0000-0000-000000000000.
 * @property {string} [subscriptionId] The subscription ID.
 * @property {string} [displayName] The subscription display name.
 * @property {string} [state] The subscription state. Possible values are
 * Enabled, Warned, PastDue, Disabled, and Deleted. Possible values include:
 * 'Enabled', 'Warned', 'PastDue', 'Disabled', 'Deleted'
 * @property {object} [subscriptionPolicies] The subscription policies.
 * @property {string} [subscriptionPolicies.locationPlacementId] The
 * subscription location placement ID. The ID indicates which regions are
 * visible for a subscription. For example, a subscription with a location
 * placement Id of Public_2014-09-01 has access to Azure public regions.
 * @property {string} [subscriptionPolicies.quotaId] The subscription quota ID.
 * @property {string} [subscriptionPolicies.spendingLimit] The subscription
 * spending limit. Possible values include: 'On', 'Off', 'CurrentPeriodOff'
 * @property {string} [authorizationSource] The authorization source of the
 * request. Valid values are one or more combinations of Legacy, RoleBased,
 * Bypassed, Direct and Management. For example, 'Legacy, RoleBased'.
 */
export interface Subscription {
  readonly id?: string;
  readonly subscriptionId?: string;
  readonly displayName?: string;
  readonly state?: string;
  subscriptionPolicies?: SubscriptionPolicies;
  authorizationSource?: string;
}

/**
 * @class
 * Initializes a new instance of the TenantIdDescription class.
 * @constructor
 * Tenant Id information.
 *
 * @property {string} [id] The fully qualified ID of the tenant. For example,
 * /tenants/00000000-0000-0000-0000-000000000000.
 * @property {string} [tenantId] The tenant ID. For example,
 * 00000000-0000-0000-0000-000000000000.
 */
export interface TenantIdDescription {
  readonly id?: string;
  readonly tenantId?: string;
}


/**
 * @class
 * Initializes a new instance of the LocationListResult class.
 * @constructor
 * Location list operation response.
 *
 */
export interface LocationListResult extends Array<Location> {
}

/**
 * @class
 * Initializes a new instance of the SubscriptionListResult class.
 * @constructor
 * Subscription list operation response.
 *
 * @property {string} nextLink The URL to get the next set of results.
 */
export interface SubscriptionListResult extends Array<Subscription> {
  nextLink: string;
}

/**
 * @class
 * Initializes a new instance of the TenantListResult class.
 * @constructor
 * Tenant Ids information.
 *
 * @property {string} nextLink The URL to use for getting the next set of
 * results.
 */
export interface TenantListResult extends Array<TenantIdDescription> {
  nextLink: string;
}
