/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Registries
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Registries {


    /**
     * Copies an image to this container registry from the specified container
     * registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} parameters The parameters specifying the image to copy and
     * the source container registry.
     *
     * @param {object} parameters.source The source of the image.
     *
     * @param {string} [parameters.source.resourceId] The resource identifier of
     * the source Azure Container Registry.
     *
     * @param {string} [parameters.source.registryUri] The address of the source
     * registry (e.g. 'mcr.microsoft.com').
     *
     * @param {object} [parameters.source.credentials] Credentials used when
     * importing from a registry uri.
     *
     * @param {string} [parameters.source.credentials.username] The username to
     * authenticate with the source registry.
     *
     * @param {string} parameters.source.credentials.password The password used to
     * authenticate with the source registry.
     *
     * @param {string} parameters.source.sourceImage Repository name of the source
     * image.
     * Specify an image by repository ('hello-world'). This will use the 'latest'
     * tag.
     * Specify an image by tag ('hello-world:latest').
     * Specify an image by sha256-based manifest digest
     * ('hello-world@sha256:abc123').
     *
     * @param {array} [parameters.targetTags] List of strings of the form
     * repo[:tag]. When tag is omitted the source will be used (or 'latest' if
     * source tag is also omitted).
     *
     * @param {array} [parameters.untaggedTargetRepositories] List of strings of
     * repository names to do a manifest only copy. No tag will be created.
     *
     * @param {string} [parameters.mode] When Force, any existing target tags will
     * be overwritten. When NoForce, any existing target tags will fail the
     * operation before any copying begins. Possible values include: 'NoForce',
     * 'Force'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    importImageWithHttpOperationResponse(resourceGroupName: string, registryName: string, parameters: models.ImportImageParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Copies an image to this container registry from the specified container
     * registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} parameters The parameters specifying the image to copy and
     * the source container registry.
     *
     * @param {object} parameters.source The source of the image.
     *
     * @param {string} [parameters.source.resourceId] The resource identifier of
     * the source Azure Container Registry.
     *
     * @param {string} [parameters.source.registryUri] The address of the source
     * registry (e.g. 'mcr.microsoft.com').
     *
     * @param {object} [parameters.source.credentials] Credentials used when
     * importing from a registry uri.
     *
     * @param {string} [parameters.source.credentials.username] The username to
     * authenticate with the source registry.
     *
     * @param {string} parameters.source.credentials.password The password used to
     * authenticate with the source registry.
     *
     * @param {string} parameters.source.sourceImage Repository name of the source
     * image.
     * Specify an image by repository ('hello-world'). This will use the 'latest'
     * tag.
     * Specify an image by tag ('hello-world:latest').
     * Specify an image by sha256-based manifest digest
     * ('hello-world@sha256:abc123').
     *
     * @param {array} [parameters.targetTags] List of strings of the form
     * repo[:tag]. When tag is omitted the source will be used (or 'latest' if
     * source tag is also omitted).
     *
     * @param {array} [parameters.untaggedTargetRepositories] List of strings of
     * repository names to do a manifest only copy. No tag will be created.
     *
     * @param {string} [parameters.mode] When Force, any existing target tags will
     * be overwritten. When NoForce, any existing target tags will fail the
     * operation before any copying begins. Possible values include: 'NoForce',
     * 'Force'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    importImage(resourceGroupName: string, registryName: string, parameters: models.ImportImageParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    importImage(resourceGroupName: string, registryName: string, parameters: models.ImportImageParameters, callback: ServiceCallback<void>): void;
    importImage(resourceGroupName: string, registryName: string, parameters: models.ImportImageParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Checks whether the container registry name is available for use. The name
     * must contain only alphanumeric characters, be globally unique, and between 5
     * and 50 characters in length.
     *
     * @param {object} registryNameCheckRequest The object containing information
     * for the availability request.
     *
     * @param {string} registryNameCheckRequest.name The name of the container
     * registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryNameStatus>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    checkNameAvailabilityWithHttpOperationResponse(registryNameCheckRequest: models.RegistryNameCheckRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryNameStatus>>;

    /**
     * Checks whether the container registry name is available for use. The name
     * must contain only alphanumeric characters, be globally unique, and between 5
     * and 50 characters in length.
     *
     * @param {object} registryNameCheckRequest The object containing information
     * for the availability request.
     *
     * @param {string} registryNameCheckRequest.name The name of the container
     * registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryNameStatus} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryNameStatus} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryNameStatus} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    checkNameAvailability(registryNameCheckRequest: models.RegistryNameCheckRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryNameStatus>;
    checkNameAvailability(registryNameCheckRequest: models.RegistryNameCheckRequest, callback: ServiceCallback<models.RegistryNameStatus>): void;
    checkNameAvailability(registryNameCheckRequest: models.RegistryNameCheckRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryNameStatus>): void;


    /**
     * Gets the properties of the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Gets the properties of the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    get(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.Registry>): void;
    get(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Creates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registry The parameters for creating a container registry.
     *
     * @param {object} registry.sku The SKU of the container registry.
     *
     * @param {string} registry.sku.name The SKU name of the container registry.
     * Required for registry creation. Possible values include: 'Classic', 'Basic',
     * 'Standard', 'Premium'
     *
     * @param {boolean} [registry.adminUserEnabled] The value that indicates
     * whether the admin user is enabled.
     *
     * @param {object} [registry.storageAccount] The properties of the storage
     * account for the container registry. Only applicable to Classic SKU.
     *
     * @param {string} registry.storageAccount.id The resource ID of the storage
     * account.
     *
     * @param {object} [registry.networkRuleSet] The network rule set for a
     * container registry.
     *
     * @param {string} registry.networkRuleSet.defaultAction The default action of
     * allow or deny when no other rules match. Possible values include: 'Allow',
     * 'Deny'
     *
     * @param {array} [registry.networkRuleSet.virtualNetworkRules] The virtual
     * network rules.
     *
     * @param {array} [registry.networkRuleSet.ipRules] The IP ACL rules.
     *
     * @param {string} registry.location The location of the resource. This cannot
     * be changed after the resource is created.
     *
     * @param {object} [registry.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, registryName: string, registry: models.Registry, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Creates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registry The parameters for creating a container registry.
     *
     * @param {object} registry.sku The SKU of the container registry.
     *
     * @param {string} registry.sku.name The SKU name of the container registry.
     * Required for registry creation. Possible values include: 'Classic', 'Basic',
     * 'Standard', 'Premium'
     *
     * @param {boolean} [registry.adminUserEnabled] The value that indicates
     * whether the admin user is enabled.
     *
     * @param {object} [registry.storageAccount] The properties of the storage
     * account for the container registry. Only applicable to Classic SKU.
     *
     * @param {string} registry.storageAccount.id The resource ID of the storage
     * account.
     *
     * @param {object} [registry.networkRuleSet] The network rule set for a
     * container registry.
     *
     * @param {string} registry.networkRuleSet.defaultAction The default action of
     * allow or deny when no other rules match. Possible values include: 'Allow',
     * 'Deny'
     *
     * @param {array} [registry.networkRuleSet.virtualNetworkRules] The virtual
     * network rules.
     *
     * @param {array} [registry.networkRuleSet.ipRules] The IP ACL rules.
     *
     * @param {string} registry.location The location of the resource. This cannot
     * be changed after the resource is created.
     *
     * @param {object} [registry.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, registryName: string, registry: models.Registry, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    create(resourceGroupName: string, registryName: string, registry: models.Registry, callback: ServiceCallback<models.Registry>): void;
    create(resourceGroupName: string, registryName: string, registry: models.Registry, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Deletes a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, registryName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryUpdateParameters The parameters for updating a
     * container registry.
     *
     * @param {object} [registryUpdateParameters.tags] The tags for the container
     * registry.
     *
     * @param {object} [registryUpdateParameters.sku] The SKU of the container
     * registry.
     *
     * @param {string} registryUpdateParameters.sku.name The SKU name of the
     * container registry. Required for registry creation. Possible values include:
     * 'Classic', 'Basic', 'Standard', 'Premium'
     *
     * @param {boolean} [registryUpdateParameters.adminUserEnabled] The value that
     * indicates whether the admin user is enabled.
     *
     * @param {object} [registryUpdateParameters.storageAccount] The parameters of
     * a storage account for the container registry. Only applicable to Classic
     * SKU. If specified, the storage account must be in the same physical location
     * as the container registry.
     *
     * @param {string} registryUpdateParameters.storageAccount.id The resource ID
     * of the storage account.
     *
     * @param {object} [registryUpdateParameters.networkRuleSet] The network rule
     * set for a container registry.
     *
     * @param {string} registryUpdateParameters.networkRuleSet.defaultAction The
     * default action of allow or deny when no other rules match. Possible values
     * include: 'Allow', 'Deny'
     *
     * @param {array} [registryUpdateParameters.networkRuleSet.virtualNetworkRules]
     * The virtual network rules.
     *
     * @param {array} [registryUpdateParameters.networkRuleSet.ipRules] The IP ACL
     * rules.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Updates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryUpdateParameters The parameters for updating a
     * container registry.
     *
     * @param {object} [registryUpdateParameters.tags] The tags for the container
     * registry.
     *
     * @param {object} [registryUpdateParameters.sku] The SKU of the container
     * registry.
     *
     * @param {string} registryUpdateParameters.sku.name The SKU name of the
     * container registry. Required for registry creation. Possible values include:
     * 'Classic', 'Basic', 'Standard', 'Premium'
     *
     * @param {boolean} [registryUpdateParameters.adminUserEnabled] The value that
     * indicates whether the admin user is enabled.
     *
     * @param {object} [registryUpdateParameters.storageAccount] The parameters of
     * a storage account for the container registry. Only applicable to Classic
     * SKU. If specified, the storage account must be in the same physical location
     * as the container registry.
     *
     * @param {string} registryUpdateParameters.storageAccount.id The resource ID
     * of the storage account.
     *
     * @param {object} [registryUpdateParameters.networkRuleSet] The network rule
     * set for a container registry.
     *
     * @param {string} registryUpdateParameters.networkRuleSet.defaultAction The
     * default action of allow or deny when no other rules match. Possible values
     * include: 'Allow', 'Deny'
     *
     * @param {array} [registryUpdateParameters.networkRuleSet.virtualNetworkRules]
     * The virtual network rules.
     *
     * @param {array} [registryUpdateParameters.networkRuleSet.ipRules] The IP ACL
     * rules.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    update(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, callback: ServiceCallback<models.Registry>): void;
    update(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Lists all the container registries under the specified resource group.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListResult>>;

    /**
     * Lists all the container registries under the specified resource group.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListResult>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.RegistryListResult>): void;
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListResult>): void;


    /**
     * Lists all the container registries under the specified subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListResult>>;

    /**
     * Lists all the container registries under the specified subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListResult>;
    list(callback: ServiceCallback<models.RegistryListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListResult>): void;


    /**
     * Lists the login credentials for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListCredentialsResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listCredentialsWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListCredentialsResult>>;

    /**
     * Lists the login credentials for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListCredentialsResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListCredentialsResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListCredentialsResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listCredentials(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListCredentialsResult>;
    listCredentials(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.RegistryListCredentialsResult>): void;
    listCredentials(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListCredentialsResult>): void;


    /**
     * Regenerates one of the login credentials for the specified container
     * registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} regenerateCredentialParameters Specifies name of the
     * password which should be regenerated -- password or password2.
     *
     * @param {string} regenerateCredentialParameters.name Specifies name of the
     * password which should be regenerated -- password or password2. Possible
     * values include: 'password', 'password2'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListCredentialsResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    regenerateCredentialWithHttpOperationResponse(resourceGroupName: string, registryName: string, regenerateCredentialParameters: models.RegenerateCredentialParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListCredentialsResult>>;

    /**
     * Regenerates one of the login credentials for the specified container
     * registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} regenerateCredentialParameters Specifies name of the
     * password which should be regenerated -- password or password2.
     *
     * @param {string} regenerateCredentialParameters.name Specifies name of the
     * password which should be regenerated -- password or password2. Possible
     * values include: 'password', 'password2'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListCredentialsResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListCredentialsResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListCredentialsResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    regenerateCredential(resourceGroupName: string, registryName: string, regenerateCredentialParameters: models.RegenerateCredentialParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListCredentialsResult>;
    regenerateCredential(resourceGroupName: string, registryName: string, regenerateCredentialParameters: models.RegenerateCredentialParameters, callback: ServiceCallback<models.RegistryListCredentialsResult>): void;
    regenerateCredential(resourceGroupName: string, registryName: string, regenerateCredentialParameters: models.RegenerateCredentialParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListCredentialsResult>): void;


    /**
     * Gets the quota usages for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryUsageListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listUsagesWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryUsageListResult>>;

    /**
     * Gets the quota usages for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryUsageListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryUsageListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryUsageListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listUsages(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryUsageListResult>;
    listUsages(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.RegistryUsageListResult>): void;
    listUsages(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryUsageListResult>): void;


    /**
     * Lists the policies for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryPolicies>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listPoliciesWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryPolicies>>;

    /**
     * Lists the policies for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryPolicies} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryPolicies} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryPolicies} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listPolicies(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryPolicies>;
    listPolicies(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.RegistryPolicies>): void;
    listPolicies(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryPolicies>): void;


    /**
     * Updates the policies for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryPoliciesUpdateParameters The parameters for updating
     * policies of a container registry.
     *
     * @param {object} [registryPoliciesUpdateParameters.quarantinePolicy] An
     * object that represents quarantine policy for a container registry.
     *
     * @param {string} [registryPoliciesUpdateParameters.quarantinePolicy.status]
     * The value that indicates whether the policy is enabled or not. Possible
     * values include: 'enabled', 'disabled'
     *
     * @param {object} [registryPoliciesUpdateParameters.trustPolicy] An object
     * that represents content trust policy for a container registry.
     *
     * @param {string} [registryPoliciesUpdateParameters.trustPolicy.type] The type
     * of trust policy. Possible values include: 'Notary'
     *
     * @param {string} [registryPoliciesUpdateParameters.trustPolicy.status] The
     * value that indicates whether the policy is enabled or not. Possible values
     * include: 'enabled', 'disabled'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryPolicies>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updatePoliciesWithHttpOperationResponse(resourceGroupName: string, registryName: string, registryPoliciesUpdateParameters: models.RegistryPolicies, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryPolicies>>;

    /**
     * Updates the policies for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryPoliciesUpdateParameters The parameters for updating
     * policies of a container registry.
     *
     * @param {object} [registryPoliciesUpdateParameters.quarantinePolicy] An
     * object that represents quarantine policy for a container registry.
     *
     * @param {string} [registryPoliciesUpdateParameters.quarantinePolicy.status]
     * The value that indicates whether the policy is enabled or not. Possible
     * values include: 'enabled', 'disabled'
     *
     * @param {object} [registryPoliciesUpdateParameters.trustPolicy] An object
     * that represents content trust policy for a container registry.
     *
     * @param {string} [registryPoliciesUpdateParameters.trustPolicy.type] The type
     * of trust policy. Possible values include: 'Notary'
     *
     * @param {string} [registryPoliciesUpdateParameters.trustPolicy.status] The
     * value that indicates whether the policy is enabled or not. Possible values
     * include: 'enabled', 'disabled'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryPolicies} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryPolicies} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryPolicies} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    updatePolicies(resourceGroupName: string, registryName: string, registryPoliciesUpdateParameters: models.RegistryPolicies, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryPolicies>;
    updatePolicies(resourceGroupName: string, registryName: string, registryPoliciesUpdateParameters: models.RegistryPolicies, callback: ServiceCallback<models.RegistryPolicies>): void;
    updatePolicies(resourceGroupName: string, registryName: string, registryPoliciesUpdateParameters: models.RegistryPolicies, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryPolicies>): void;


    /**
     * Schedules a new run based on the request parameters and add it to the run
     * queue.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} runRequest The parameters of a run that needs to scheduled.
     *
     * @param {boolean} [runRequest.isArchiveEnabled] The value that indicates
     * whether archiving is enabled for the run or not.
     *
     * @param {string} runRequest.type Polymorphic Discriminator
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Run>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    scheduleRunWithHttpOperationResponse(resourceGroupName: string, registryName: string, runRequest: models.RunRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Run>>;

    /**
     * Schedules a new run based on the request parameters and add it to the run
     * queue.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} runRequest The parameters of a run that needs to scheduled.
     *
     * @param {boolean} [runRequest.isArchiveEnabled] The value that indicates
     * whether archiving is enabled for the run or not.
     *
     * @param {string} runRequest.type Polymorphic Discriminator
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Run} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Run} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Run} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    scheduleRun(resourceGroupName: string, registryName: string, runRequest: models.RunRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Run>;
    scheduleRun(resourceGroupName: string, registryName: string, runRequest: models.RunRequest, callback: ServiceCallback<models.Run>): void;
    scheduleRun(resourceGroupName: string, registryName: string, runRequest: models.RunRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Run>): void;


    /**
     * Get the upload location for the user to be able to upload the source.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SourceUploadDefinition>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getBuildSourceUploadUrlWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SourceUploadDefinition>>;

    /**
     * Get the upload location for the user to be able to upload the source.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SourceUploadDefinition} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SourceUploadDefinition} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link SourceUploadDefinition} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getBuildSourceUploadUrl(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.SourceUploadDefinition>;
    getBuildSourceUploadUrl(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.SourceUploadDefinition>): void;
    getBuildSourceUploadUrl(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SourceUploadDefinition>): void;


    /**
     * Copies an image to this container registry from the specified container
     * registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} parameters The parameters specifying the image to copy and
     * the source container registry.
     *
     * @param {object} parameters.source The source of the image.
     *
     * @param {string} [parameters.source.resourceId] The resource identifier of
     * the source Azure Container Registry.
     *
     * @param {string} [parameters.source.registryUri] The address of the source
     * registry (e.g. 'mcr.microsoft.com').
     *
     * @param {object} [parameters.source.credentials] Credentials used when
     * importing from a registry uri.
     *
     * @param {string} [parameters.source.credentials.username] The username to
     * authenticate with the source registry.
     *
     * @param {string} parameters.source.credentials.password The password used to
     * authenticate with the source registry.
     *
     * @param {string} parameters.source.sourceImage Repository name of the source
     * image.
     * Specify an image by repository ('hello-world'). This will use the 'latest'
     * tag.
     * Specify an image by tag ('hello-world:latest').
     * Specify an image by sha256-based manifest digest
     * ('hello-world@sha256:abc123').
     *
     * @param {array} [parameters.targetTags] List of strings of the form
     * repo[:tag]. When tag is omitted the source will be used (or 'latest' if
     * source tag is also omitted).
     *
     * @param {array} [parameters.untaggedTargetRepositories] List of strings of
     * repository names to do a manifest only copy. No tag will be created.
     *
     * @param {string} [parameters.mode] When Force, any existing target tags will
     * be overwritten. When NoForce, any existing target tags will fail the
     * operation before any copying begins. Possible values include: 'NoForce',
     * 'Force'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginImportImageWithHttpOperationResponse(resourceGroupName: string, registryName: string, parameters: models.ImportImageParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Copies an image to this container registry from the specified container
     * registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} parameters The parameters specifying the image to copy and
     * the source container registry.
     *
     * @param {object} parameters.source The source of the image.
     *
     * @param {string} [parameters.source.resourceId] The resource identifier of
     * the source Azure Container Registry.
     *
     * @param {string} [parameters.source.registryUri] The address of the source
     * registry (e.g. 'mcr.microsoft.com').
     *
     * @param {object} [parameters.source.credentials] Credentials used when
     * importing from a registry uri.
     *
     * @param {string} [parameters.source.credentials.username] The username to
     * authenticate with the source registry.
     *
     * @param {string} parameters.source.credentials.password The password used to
     * authenticate with the source registry.
     *
     * @param {string} parameters.source.sourceImage Repository name of the source
     * image.
     * Specify an image by repository ('hello-world'). This will use the 'latest'
     * tag.
     * Specify an image by tag ('hello-world:latest').
     * Specify an image by sha256-based manifest digest
     * ('hello-world@sha256:abc123').
     *
     * @param {array} [parameters.targetTags] List of strings of the form
     * repo[:tag]. When tag is omitted the source will be used (or 'latest' if
     * source tag is also omitted).
     *
     * @param {array} [parameters.untaggedTargetRepositories] List of strings of
     * repository names to do a manifest only copy. No tag will be created.
     *
     * @param {string} [parameters.mode] When Force, any existing target tags will
     * be overwritten. When NoForce, any existing target tags will fail the
     * operation before any copying begins. Possible values include: 'NoForce',
     * 'Force'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginImportImage(resourceGroupName: string, registryName: string, parameters: models.ImportImageParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginImportImage(resourceGroupName: string, registryName: string, parameters: models.ImportImageParameters, callback: ServiceCallback<void>): void;
    beginImportImage(resourceGroupName: string, registryName: string, parameters: models.ImportImageParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Creates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registry The parameters for creating a container registry.
     *
     * @param {object} registry.sku The SKU of the container registry.
     *
     * @param {string} registry.sku.name The SKU name of the container registry.
     * Required for registry creation. Possible values include: 'Classic', 'Basic',
     * 'Standard', 'Premium'
     *
     * @param {boolean} [registry.adminUserEnabled] The value that indicates
     * whether the admin user is enabled.
     *
     * @param {object} [registry.storageAccount] The properties of the storage
     * account for the container registry. Only applicable to Classic SKU.
     *
     * @param {string} registry.storageAccount.id The resource ID of the storage
     * account.
     *
     * @param {object} [registry.networkRuleSet] The network rule set for a
     * container registry.
     *
     * @param {string} registry.networkRuleSet.defaultAction The default action of
     * allow or deny when no other rules match. Possible values include: 'Allow',
     * 'Deny'
     *
     * @param {array} [registry.networkRuleSet.virtualNetworkRules] The virtual
     * network rules.
     *
     * @param {array} [registry.networkRuleSet.ipRules] The IP ACL rules.
     *
     * @param {string} registry.location The location of the resource. This cannot
     * be changed after the resource is created.
     *
     * @param {object} [registry.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, registryName: string, registry: models.Registry, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Creates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registry The parameters for creating a container registry.
     *
     * @param {object} registry.sku The SKU of the container registry.
     *
     * @param {string} registry.sku.name The SKU name of the container registry.
     * Required for registry creation. Possible values include: 'Classic', 'Basic',
     * 'Standard', 'Premium'
     *
     * @param {boolean} [registry.adminUserEnabled] The value that indicates
     * whether the admin user is enabled.
     *
     * @param {object} [registry.storageAccount] The properties of the storage
     * account for the container registry. Only applicable to Classic SKU.
     *
     * @param {string} registry.storageAccount.id The resource ID of the storage
     * account.
     *
     * @param {object} [registry.networkRuleSet] The network rule set for a
     * container registry.
     *
     * @param {string} registry.networkRuleSet.defaultAction The default action of
     * allow or deny when no other rules match. Possible values include: 'Allow',
     * 'Deny'
     *
     * @param {array} [registry.networkRuleSet.virtualNetworkRules] The virtual
     * network rules.
     *
     * @param {array} [registry.networkRuleSet.ipRules] The IP ACL rules.
     *
     * @param {string} registry.location The location of the resource. This cannot
     * be changed after the resource is created.
     *
     * @param {object} [registry.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, registryName: string, registry: models.Registry, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    beginCreate(resourceGroupName: string, registryName: string, registry: models.Registry, callback: ServiceCallback<models.Registry>): void;
    beginCreate(resourceGroupName: string, registryName: string, registry: models.Registry, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Deletes a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, registryName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryUpdateParameters The parameters for updating a
     * container registry.
     *
     * @param {object} [registryUpdateParameters.tags] The tags for the container
     * registry.
     *
     * @param {object} [registryUpdateParameters.sku] The SKU of the container
     * registry.
     *
     * @param {string} registryUpdateParameters.sku.name The SKU name of the
     * container registry. Required for registry creation. Possible values include:
     * 'Classic', 'Basic', 'Standard', 'Premium'
     *
     * @param {boolean} [registryUpdateParameters.adminUserEnabled] The value that
     * indicates whether the admin user is enabled.
     *
     * @param {object} [registryUpdateParameters.storageAccount] The parameters of
     * a storage account for the container registry. Only applicable to Classic
     * SKU. If specified, the storage account must be in the same physical location
     * as the container registry.
     *
     * @param {string} registryUpdateParameters.storageAccount.id The resource ID
     * of the storage account.
     *
     * @param {object} [registryUpdateParameters.networkRuleSet] The network rule
     * set for a container registry.
     *
     * @param {string} registryUpdateParameters.networkRuleSet.defaultAction The
     * default action of allow or deny when no other rules match. Possible values
     * include: 'Allow', 'Deny'
     *
     * @param {array} [registryUpdateParameters.networkRuleSet.virtualNetworkRules]
     * The virtual network rules.
     *
     * @param {array} [registryUpdateParameters.networkRuleSet.ipRules] The IP ACL
     * rules.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Updates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryUpdateParameters The parameters for updating a
     * container registry.
     *
     * @param {object} [registryUpdateParameters.tags] The tags for the container
     * registry.
     *
     * @param {object} [registryUpdateParameters.sku] The SKU of the container
     * registry.
     *
     * @param {string} registryUpdateParameters.sku.name The SKU name of the
     * container registry. Required for registry creation. Possible values include:
     * 'Classic', 'Basic', 'Standard', 'Premium'
     *
     * @param {boolean} [registryUpdateParameters.adminUserEnabled] The value that
     * indicates whether the admin user is enabled.
     *
     * @param {object} [registryUpdateParameters.storageAccount] The parameters of
     * a storage account for the container registry. Only applicable to Classic
     * SKU. If specified, the storage account must be in the same physical location
     * as the container registry.
     *
     * @param {string} registryUpdateParameters.storageAccount.id The resource ID
     * of the storage account.
     *
     * @param {object} [registryUpdateParameters.networkRuleSet] The network rule
     * set for a container registry.
     *
     * @param {string} registryUpdateParameters.networkRuleSet.defaultAction The
     * default action of allow or deny when no other rules match. Possible values
     * include: 'Allow', 'Deny'
     *
     * @param {array} [registryUpdateParameters.networkRuleSet.virtualNetworkRules]
     * The virtual network rules.
     *
     * @param {array} [registryUpdateParameters.networkRuleSet.ipRules] The IP ACL
     * rules.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    beginUpdate(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, callback: ServiceCallback<models.Registry>): void;
    beginUpdate(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Updates the policies for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryPoliciesUpdateParameters The parameters for updating
     * policies of a container registry.
     *
     * @param {object} [registryPoliciesUpdateParameters.quarantinePolicy] An
     * object that represents quarantine policy for a container registry.
     *
     * @param {string} [registryPoliciesUpdateParameters.quarantinePolicy.status]
     * The value that indicates whether the policy is enabled or not. Possible
     * values include: 'enabled', 'disabled'
     *
     * @param {object} [registryPoliciesUpdateParameters.trustPolicy] An object
     * that represents content trust policy for a container registry.
     *
     * @param {string} [registryPoliciesUpdateParameters.trustPolicy.type] The type
     * of trust policy. Possible values include: 'Notary'
     *
     * @param {string} [registryPoliciesUpdateParameters.trustPolicy.status] The
     * value that indicates whether the policy is enabled or not. Possible values
     * include: 'enabled', 'disabled'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryPolicies>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdatePoliciesWithHttpOperationResponse(resourceGroupName: string, registryName: string, registryPoliciesUpdateParameters: models.RegistryPolicies, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryPolicies>>;

    /**
     * Updates the policies for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryPoliciesUpdateParameters The parameters for updating
     * policies of a container registry.
     *
     * @param {object} [registryPoliciesUpdateParameters.quarantinePolicy] An
     * object that represents quarantine policy for a container registry.
     *
     * @param {string} [registryPoliciesUpdateParameters.quarantinePolicy.status]
     * The value that indicates whether the policy is enabled or not. Possible
     * values include: 'enabled', 'disabled'
     *
     * @param {object} [registryPoliciesUpdateParameters.trustPolicy] An object
     * that represents content trust policy for a container registry.
     *
     * @param {string} [registryPoliciesUpdateParameters.trustPolicy.type] The type
     * of trust policy. Possible values include: 'Notary'
     *
     * @param {string} [registryPoliciesUpdateParameters.trustPolicy.status] The
     * value that indicates whether the policy is enabled or not. Possible values
     * include: 'enabled', 'disabled'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryPolicies} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryPolicies} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryPolicies} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdatePolicies(resourceGroupName: string, registryName: string, registryPoliciesUpdateParameters: models.RegistryPolicies, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryPolicies>;
    beginUpdatePolicies(resourceGroupName: string, registryName: string, registryPoliciesUpdateParameters: models.RegistryPolicies, callback: ServiceCallback<models.RegistryPolicies>): void;
    beginUpdatePolicies(resourceGroupName: string, registryName: string, registryPoliciesUpdateParameters: models.RegistryPolicies, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryPolicies>): void;


    /**
     * Schedules a new run based on the request parameters and add it to the run
     * queue.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} runRequest The parameters of a run that needs to scheduled.
     *
     * @param {boolean} [runRequest.isArchiveEnabled] The value that indicates
     * whether archiving is enabled for the run or not.
     *
     * @param {string} runRequest.type Polymorphic Discriminator
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Run>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginScheduleRunWithHttpOperationResponse(resourceGroupName: string, registryName: string, runRequest: models.RunRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Run>>;

    /**
     * Schedules a new run based on the request parameters and add it to the run
     * queue.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} runRequest The parameters of a run that needs to scheduled.
     *
     * @param {boolean} [runRequest.isArchiveEnabled] The value that indicates
     * whether archiving is enabled for the run or not.
     *
     * @param {string} runRequest.type Polymorphic Discriminator
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Run} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Run} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Run} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginScheduleRun(resourceGroupName: string, registryName: string, runRequest: models.RunRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Run>;
    beginScheduleRun(resourceGroupName: string, registryName: string, runRequest: models.RunRequest, callback: ServiceCallback<models.Run>): void;
    beginScheduleRun(resourceGroupName: string, registryName: string, runRequest: models.RunRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Run>): void;


    /**
     * Lists all the container registries under the specified resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListResult>>;

    /**
     * Lists all the container registries under the specified resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListResult>;
    listByResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.RegistryListResult>): void;
    listByResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListResult>): void;


    /**
     * Lists all the container registries under the specified subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListResult>>;

    /**
     * Lists all the container registries under the specified subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.RegistryListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListResult>): void;
}

/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Operations {


    /**
     * Lists all of the available Azure Container Registry REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available Azure Container Registry REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    list(callback: ServiceCallback<models.OperationListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;


    /**
     * Lists all of the available Azure Container Registry REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available Azure Container Registry REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.OperationListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;
}

/**
 * @class
 * Replications
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Replications {


    /**
     * Gets the properties of the specified replication.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Gets the properties of the specified replication.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    get(resourceGroupName: string, registryName: string, replicationName: string, callback: ServiceCallback<models.Replication>): void;
    get(resourceGroupName: string, registryName: string, replicationName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Creates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replication The parameters for creating a replication.
     *
     * @param {string} replication.location The location of the resource. This
     * cannot be changed after the resource is created.
     *
     * @param {object} [replication.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Creates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replication The parameters for creating a replication.
     *
     * @param {string} replication.location The location of the resource. This
     * cannot be changed after the resource is created.
     *
     * @param {object} [replication.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    create(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, callback: ServiceCallback<models.Replication>): void;
    create(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Deletes a replication from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a replication from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, registryName: string, replicationName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, registryName: string, replicationName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replicationUpdateParameters The parameters for updating a
     * replication.
     *
     * @param {object} [replicationUpdateParameters.tags] The tags for the
     * replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Updates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replicationUpdateParameters The parameters for updating a
     * replication.
     *
     * @param {object} [replicationUpdateParameters.tags] The tags for the
     * replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    update(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, callback: ServiceCallback<models.Replication>): void;
    update(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Lists all the replications for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ReplicationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ReplicationListResult>>;

    /**
     * Lists all the replications for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ReplicationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ReplicationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ReplicationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ReplicationListResult>;
    list(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.ReplicationListResult>): void;
    list(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ReplicationListResult>): void;


    /**
     * Creates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replication The parameters for creating a replication.
     *
     * @param {string} replication.location The location of the resource. This
     * cannot be changed after the resource is created.
     *
     * @param {object} [replication.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Creates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replication The parameters for creating a replication.
     *
     * @param {string} replication.location The location of the resource. This
     * cannot be changed after the resource is created.
     *
     * @param {object} [replication.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    beginCreate(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, callback: ServiceCallback<models.Replication>): void;
    beginCreate(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Deletes a replication from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a replication from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, registryName: string, replicationName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, registryName: string, replicationName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replicationUpdateParameters The parameters for updating a
     * replication.
     *
     * @param {object} [replicationUpdateParameters.tags] The tags for the
     * replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Updates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replicationUpdateParameters The parameters for updating a
     * replication.
     *
     * @param {object} [replicationUpdateParameters.tags] The tags for the
     * replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    beginUpdate(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, callback: ServiceCallback<models.Replication>): void;
    beginUpdate(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Lists all the replications for the specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ReplicationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ReplicationListResult>>;

    /**
     * Lists all the replications for the specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ReplicationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ReplicationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ReplicationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ReplicationListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ReplicationListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ReplicationListResult>): void;
}

/**
 * @class
 * Webhooks
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Webhooks {


    /**
     * Gets the properties of the specified webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Gets the properties of the specified webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    get(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<models.Webhook>): void;
    get(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Creates a webhook for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookCreateParameters The parameters for creating a
     * webhook.
     *
     * @param {object} [webhookCreateParameters.tags] The tags for the webhook.
     *
     * @param {string} webhookCreateParameters.location The location of the
     * webhook. This cannot be changed after the resource is created.
     *
     * @param {string} webhookCreateParameters.serviceUri The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookCreateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookCreateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} webhookCreateParameters.actions The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookCreateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Creates a webhook for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookCreateParameters The parameters for creating a
     * webhook.
     *
     * @param {object} [webhookCreateParameters.tags] The tags for the webhook.
     *
     * @param {string} webhookCreateParameters.location The location of the
     * webhook. This cannot be changed after the resource is created.
     *
     * @param {string} webhookCreateParameters.serviceUri The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookCreateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookCreateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} webhookCreateParameters.actions The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookCreateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    create(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, callback: ServiceCallback<models.Webhook>): void;
    create(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Deletes a webhook from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a webhook from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a webhook with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookUpdateParameters The parameters for updating a
     * webhook.
     *
     * @param {object} [webhookUpdateParameters.tags] The tags for the webhook.
     *
     * @param {string} [webhookUpdateParameters.serviceUri] The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookUpdateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookUpdateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} [webhookUpdateParameters.actions] The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookUpdateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Updates a webhook with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookUpdateParameters The parameters for updating a
     * webhook.
     *
     * @param {object} [webhookUpdateParameters.tags] The tags for the webhook.
     *
     * @param {string} [webhookUpdateParameters.serviceUri] The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookUpdateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookUpdateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} [webhookUpdateParameters.actions] The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookUpdateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    update(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, callback: ServiceCallback<models.Webhook>): void;
    update(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Lists all the webhooks for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WebhookListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WebhookListResult>>;

    /**
     * Lists all the webhooks for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WebhookListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WebhookListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WebhookListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WebhookListResult>;
    list(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.WebhookListResult>): void;
    list(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WebhookListResult>): void;


    /**
     * Triggers a ping event to be sent to the webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventInfo>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    pingWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventInfo>>;

    /**
     * Triggers a ping event to be sent to the webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventInfo} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventInfo} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventInfo} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    ping(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventInfo>;
    ping(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<models.EventInfo>): void;
    ping(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventInfo>): void;


    /**
     * Gets the configuration of service URI and custom headers for the webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CallbackConfig>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getCallbackConfigWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CallbackConfig>>;

    /**
     * Gets the configuration of service URI and custom headers for the webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CallbackConfig} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CallbackConfig} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CallbackConfig} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getCallbackConfig(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CallbackConfig>;
    getCallbackConfig(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<models.CallbackConfig>): void;
    getCallbackConfig(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CallbackConfig>): void;


    /**
     * Lists recent events for the specified webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listEventsWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventListResult>>;

    /**
     * Lists recent events for the specified webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listEvents(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventListResult>;
    listEvents(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<models.EventListResult>): void;
    listEvents(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventListResult>): void;


    /**
     * Creates a webhook for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookCreateParameters The parameters for creating a
     * webhook.
     *
     * @param {object} [webhookCreateParameters.tags] The tags for the webhook.
     *
     * @param {string} webhookCreateParameters.location The location of the
     * webhook. This cannot be changed after the resource is created.
     *
     * @param {string} webhookCreateParameters.serviceUri The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookCreateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookCreateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} webhookCreateParameters.actions The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookCreateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Creates a webhook for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookCreateParameters The parameters for creating a
     * webhook.
     *
     * @param {object} [webhookCreateParameters.tags] The tags for the webhook.
     *
     * @param {string} webhookCreateParameters.location The location of the
     * webhook. This cannot be changed after the resource is created.
     *
     * @param {string} webhookCreateParameters.serviceUri The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookCreateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookCreateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} webhookCreateParameters.actions The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookCreateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    beginCreate(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, callback: ServiceCallback<models.Webhook>): void;
    beginCreate(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Deletes a webhook from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a webhook from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a webhook with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookUpdateParameters The parameters for updating a
     * webhook.
     *
     * @param {object} [webhookUpdateParameters.tags] The tags for the webhook.
     *
     * @param {string} [webhookUpdateParameters.serviceUri] The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookUpdateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookUpdateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} [webhookUpdateParameters.actions] The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookUpdateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Updates a webhook with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookUpdateParameters The parameters for updating a
     * webhook.
     *
     * @param {object} [webhookUpdateParameters.tags] The tags for the webhook.
     *
     * @param {string} [webhookUpdateParameters.serviceUri] The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookUpdateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookUpdateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} [webhookUpdateParameters.actions] The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookUpdateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    beginUpdate(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, callback: ServiceCallback<models.Webhook>): void;
    beginUpdate(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Lists all the webhooks for the specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WebhookListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WebhookListResult>>;

    /**
     * Lists all the webhooks for the specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WebhookListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WebhookListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WebhookListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WebhookListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.WebhookListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WebhookListResult>): void;


    /**
     * Lists recent events for the specified webhook.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listEventsNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventListResult>>;

    /**
     * Lists recent events for the specified webhook.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listEventsNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventListResult>;
    listEventsNext(nextPageLink: string, callback: ServiceCallback<models.EventListResult>): void;
    listEventsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventListResult>): void;
}

/**
 * @class
 * Runs
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Runs {


    /**
     * Gets all the runs for a registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The runs filter to apply on the operation.
     * Arithmetic operators are not supported. The allowed string function is
     * 'contains'. All logical operators except 'Not', 'Has', 'All' are allowed.
     *
     * @param {number} [options.top] $top is supported for get list of runs, which
     * limits the maximum number of runs to return.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RunListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { filter? : string, top? : number, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RunListResult>>;

    /**
     * Gets all the runs for a registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The runs filter to apply on the operation.
     * Arithmetic operators are not supported. The allowed string function is
     * 'contains'. All logical operators except 'Not', 'Has', 'All' are allowed.
     *
     * @param {number} [options.top] $top is supported for get list of runs, which
     * limits the maximum number of runs to return.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RunListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RunListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RunListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, registryName: string, options?: { filter? : string, top? : number, customHeaders? : { [headerName: string]: string; } }): Promise<models.RunListResult>;
    list(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.RunListResult>): void;
    list(resourceGroupName: string, registryName: string, options: { filter? : string, top? : number, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RunListResult>): void;


    /**
     * Gets the detailed information for a given run.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Run>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, registryName: string, runId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Run>>;

    /**
     * Gets the detailed information for a given run.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Run} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Run} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Run} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, registryName: string, runId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Run>;
    get(resourceGroupName: string, registryName: string, runId: string, callback: ServiceCallback<models.Run>): void;
    get(resourceGroupName: string, registryName: string, runId: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Run>): void;


    /**
     * Patch the run properties.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} runUpdateParameters The run update properties.
     *
     * @param {boolean} [runUpdateParameters.isArchiveEnabled] The value that
     * indicates whether archiving is enabled or not.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Run>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, registryName: string, runId: string, runUpdateParameters: models.RunUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Run>>;

    /**
     * Patch the run properties.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} runUpdateParameters The run update properties.
     *
     * @param {boolean} [runUpdateParameters.isArchiveEnabled] The value that
     * indicates whether archiving is enabled or not.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Run} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Run} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Run} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, registryName: string, runId: string, runUpdateParameters: models.RunUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Run>;
    update(resourceGroupName: string, registryName: string, runId: string, runUpdateParameters: models.RunUpdateParameters, callback: ServiceCallback<models.Run>): void;
    update(resourceGroupName: string, registryName: string, runId: string, runUpdateParameters: models.RunUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Run>): void;


    /**
     * Gets a link to download the run logs.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RunGetLogResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getLogSasUrlWithHttpOperationResponse(resourceGroupName: string, registryName: string, runId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RunGetLogResult>>;

    /**
     * Gets a link to download the run logs.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RunGetLogResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RunGetLogResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RunGetLogResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getLogSasUrl(resourceGroupName: string, registryName: string, runId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RunGetLogResult>;
    getLogSasUrl(resourceGroupName: string, registryName: string, runId: string, callback: ServiceCallback<models.RunGetLogResult>): void;
    getLogSasUrl(resourceGroupName: string, registryName: string, runId: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RunGetLogResult>): void;


    /**
     * Cancel an existing run.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    cancelWithHttpOperationResponse(resourceGroupName: string, registryName: string, runId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Cancel an existing run.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    cancel(resourceGroupName: string, registryName: string, runId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    cancel(resourceGroupName: string, registryName: string, runId: string, callback: ServiceCallback<void>): void;
    cancel(resourceGroupName: string, registryName: string, runId: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Patch the run properties.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} runUpdateParameters The run update properties.
     *
     * @param {boolean} [runUpdateParameters.isArchiveEnabled] The value that
     * indicates whether archiving is enabled or not.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Run>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, registryName: string, runId: string, runUpdateParameters: models.RunUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Run>>;

    /**
     * Patch the run properties.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} runUpdateParameters The run update properties.
     *
     * @param {boolean} [runUpdateParameters.isArchiveEnabled] The value that
     * indicates whether archiving is enabled or not.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Run} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Run} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Run} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, registryName: string, runId: string, runUpdateParameters: models.RunUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Run>;
    beginUpdate(resourceGroupName: string, registryName: string, runId: string, runUpdateParameters: models.RunUpdateParameters, callback: ServiceCallback<models.Run>): void;
    beginUpdate(resourceGroupName: string, registryName: string, runId: string, runUpdateParameters: models.RunUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Run>): void;


    /**
     * Cancel an existing run.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCancelWithHttpOperationResponse(resourceGroupName: string, registryName: string, runId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Cancel an existing run.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} runId The run ID.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCancel(resourceGroupName: string, registryName: string, runId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginCancel(resourceGroupName: string, registryName: string, runId: string, callback: ServiceCallback<void>): void;
    beginCancel(resourceGroupName: string, registryName: string, runId: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Gets all the runs for a registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RunListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RunListResult>>;

    /**
     * Gets all the runs for a registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RunListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RunListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RunListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RunListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.RunListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RunListResult>): void;
}

/**
 * @class
 * Tasks
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Tasks {


    /**
     * Lists all the tasks for a specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<TaskListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.TaskListResult>>;

    /**
     * Lists all the tasks for a specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {TaskListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {TaskListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link TaskListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.TaskListResult>;
    list(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.TaskListResult>): void;
    list(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.TaskListResult>): void;


    /**
     * Get the properties of a specified task.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, registryName: string, taskName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Task>>;

    /**
     * Get the properties of a specified task.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Task} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Task} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Task} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, registryName: string, taskName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Task>;
    get(resourceGroupName: string, registryName: string, taskName: string, callback: ServiceCallback<models.Task>): void;
    get(resourceGroupName: string, registryName: string, taskName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Task>): void;


    /**
     * Creates a task for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} taskCreateParameters The parameters for creating a task.
     *
     * @param {object} [taskCreateParameters.identity] Identity for the resource.
     *
     * @param {string} [taskCreateParameters.identity.principalId] The principal ID
     * of resource identity.
     *
     * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
     * resource.
     *
     * @param {string} [taskCreateParameters.identity.type] The identity type.
     * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
     * UserAssigned', 'None'
     *
     * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
     * list of user identities associated with the resource. The user identity
     * dictionary key references will be ARM resource ids in the form:
     * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
     * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
     *
     * @param {string} [taskCreateParameters.status] The current status of task.
     * Possible values include: 'Disabled', 'Enabled'
     *
     * @param {object} taskCreateParameters.platform The platform properties
     * against which the run has to happen.
     *
     * @param {string} taskCreateParameters.platform.os The operating system type
     * required for the run. Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [taskCreateParameters.platform.architecture] The OS
     * architecture. Possible values include: 'amd64', 'x86', 'arm'
     *
     * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
     * Possible values include: 'v6', 'v7', 'v8'
     *
     * @param {object} [taskCreateParameters.agentConfiguration] The machine
     * configuration of the run agent.
     *
     * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
     * configuration in terms of number of cores required for the run.
     *
     * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
     *
     * @param {object} taskCreateParameters.step The properties of a task step.
     *
     * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
     * relative) of the source context for the task step.
     *
     * @param {string} [taskCreateParameters.step.contextAccessToken] The token
     * (git PAT or SAS token of storage account blob) associated with the context
     * for a step.
     *
     * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
     *
     * @param {object} [taskCreateParameters.trigger] The properties that describe
     * all triggers for the task.
     *
     * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
     * of timer triggers.
     *
     * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
     * of triggers based on source code repository.
     *
     * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
     * based on base image dependencies.
     *
     * @param {string}
     * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
     * of the auto trigger for base image dependency updates. Possible values
     * include: 'All', 'Runtime'
     *
     * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
     * current status of trigger. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
     * of the trigger.
     *
     * @param {object} [taskCreateParameters.credentials] The properties that
     * describes a set of credentials that will be used when this run is invoked.
     *
     * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
     * the credential parameters for accessing the source registry.
     *
     * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
     * The authentication mode which determines the source registry login scope.
     * The credentials for the source registry
     * will be generated using the given scope. These credentials will be used to
     * login to
     * the source registry during the run. Possible values include: 'None',
     * 'Default'
     *
     * @param {object} [taskCreateParameters.credentials.customRegistries]
     * Describes the credential parameters for accessing other custom registries.
     * The key
     * for the dictionary item will be the registry login server
     * (myregistry.azurecr.io) and
     * the value of the item will be the registry credentials for accessing the
     * registry.
     *
     * @param {string} taskCreateParameters.location The location of the resource.
     * This cannot be changed after the resource is created.
     *
     * @param {object} [taskCreateParameters.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, registryName: string, taskName: string, taskCreateParameters: models.Task, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Task>>;

    /**
     * Creates a task for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} taskCreateParameters The parameters for creating a task.
     *
     * @param {object} [taskCreateParameters.identity] Identity for the resource.
     *
     * @param {string} [taskCreateParameters.identity.principalId] The principal ID
     * of resource identity.
     *
     * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
     * resource.
     *
     * @param {string} [taskCreateParameters.identity.type] The identity type.
     * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
     * UserAssigned', 'None'
     *
     * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
     * list of user identities associated with the resource. The user identity
     * dictionary key references will be ARM resource ids in the form:
     * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
     * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
     *
     * @param {string} [taskCreateParameters.status] The current status of task.
     * Possible values include: 'Disabled', 'Enabled'
     *
     * @param {object} taskCreateParameters.platform The platform properties
     * against which the run has to happen.
     *
     * @param {string} taskCreateParameters.platform.os The operating system type
     * required for the run. Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [taskCreateParameters.platform.architecture] The OS
     * architecture. Possible values include: 'amd64', 'x86', 'arm'
     *
     * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
     * Possible values include: 'v6', 'v7', 'v8'
     *
     * @param {object} [taskCreateParameters.agentConfiguration] The machine
     * configuration of the run agent.
     *
     * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
     * configuration in terms of number of cores required for the run.
     *
     * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
     *
     * @param {object} taskCreateParameters.step The properties of a task step.
     *
     * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
     * relative) of the source context for the task step.
     *
     * @param {string} [taskCreateParameters.step.contextAccessToken] The token
     * (git PAT or SAS token of storage account blob) associated with the context
     * for a step.
     *
     * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
     *
     * @param {object} [taskCreateParameters.trigger] The properties that describe
     * all triggers for the task.
     *
     * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
     * of timer triggers.
     *
     * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
     * of triggers based on source code repository.
     *
     * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
     * based on base image dependencies.
     *
     * @param {string}
     * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
     * of the auto trigger for base image dependency updates. Possible values
     * include: 'All', 'Runtime'
     *
     * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
     * current status of trigger. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
     * of the trigger.
     *
     * @param {object} [taskCreateParameters.credentials] The properties that
     * describes a set of credentials that will be used when this run is invoked.
     *
     * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
     * the credential parameters for accessing the source registry.
     *
     * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
     * The authentication mode which determines the source registry login scope.
     * The credentials for the source registry
     * will be generated using the given scope. These credentials will be used to
     * login to
     * the source registry during the run. Possible values include: 'None',
     * 'Default'
     *
     * @param {object} [taskCreateParameters.credentials.customRegistries]
     * Describes the credential parameters for accessing other custom registries.
     * The key
     * for the dictionary item will be the registry login server
     * (myregistry.azurecr.io) and
     * the value of the item will be the registry credentials for accessing the
     * registry.
     *
     * @param {string} taskCreateParameters.location The location of the resource.
     * This cannot be changed after the resource is created.
     *
     * @param {object} [taskCreateParameters.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Task} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Task} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Task} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, registryName: string, taskName: string, taskCreateParameters: models.Task, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Task>;
    create(resourceGroupName: string, registryName: string, taskName: string, taskCreateParameters: models.Task, callback: ServiceCallback<models.Task>): void;
    create(resourceGroupName: string, registryName: string, taskName: string, taskCreateParameters: models.Task, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Task>): void;


    /**
     * Deletes a specified task.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, taskName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a specified task.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, registryName: string, taskName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, registryName: string, taskName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, registryName: string, taskName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a task with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} taskUpdateParameters The parameters for updating a task.
     *
     * @param {object} [taskUpdateParameters.identity] Identity for the resource.
     *
     * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
     * of resource identity.
     *
     * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
     * resource.
     *
     * @param {string} [taskUpdateParameters.identity.type] The identity type.
     * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
     * UserAssigned', 'None'
     *
     * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
     * list of user identities associated with the resource. The user identity
     * dictionary key references will be ARM resource ids in the form:
     * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
     * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
     *
     * @param {string} [taskUpdateParameters.status] The current status of task.
     * Possible values include: 'Disabled', 'Enabled'
     *
     * @param {object} [taskUpdateParameters.platform] The platform properties
     * against which the run has to happen.
     *
     * @param {string} [taskUpdateParameters.platform.os] The operating system type
     * required for the run. Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [taskUpdateParameters.platform.architecture] The OS
     * architecture. Possible values include: 'amd64', 'x86', 'arm'
     *
     * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
     * Possible values include: 'v6', 'v7', 'v8'
     *
     * @param {object} [taskUpdateParameters.agentConfiguration] The machine
     * configuration of the run agent.
     *
     * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
     * configuration in terms of number of cores required for the run.
     *
     * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
     *
     * @param {object} [taskUpdateParameters.step] The properties for updating a
     * task step.
     *
     * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
     * relative) of the source context for the task step.
     *
     * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
     * (git PAT or SAS token of storage account blob) associated with the context
     * for a step.
     *
     * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
     *
     * @param {object} [taskUpdateParameters.trigger] The properties for updating
     * trigger properties.
     *
     * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
     * of timer triggers.
     *
     * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
     * of triggers based on source code repository.
     *
     * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
     * based on base image dependencies.
     *
     * @param {string}
     * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
     * type of the auto trigger for base image dependency updates. Possible values
     * include: 'All', 'Runtime'
     *
     * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
     * current status of trigger. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
     * of the trigger.
     *
     * @param {object} [taskUpdateParameters.credentials] The parameters that
     * describes a set of credentials that will be used when this run is invoked.
     *
     * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
     * the credential parameters for accessing the source registry.
     *
     * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
     * The authentication mode which determines the source registry login scope.
     * The credentials for the source registry
     * will be generated using the given scope. These credentials will be used to
     * login to
     * the source registry during the run. Possible values include: 'None',
     * 'Default'
     *
     * @param {object} [taskUpdateParameters.credentials.customRegistries]
     * Describes the credential parameters for accessing other custom registries.
     * The key
     * for the dictionary item will be the registry login server
     * (myregistry.azurecr.io) and
     * the value of the item will be the registry credentials for accessing the
     * registry.
     *
     * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, registryName: string, taskName: string, taskUpdateParameters: models.TaskUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Task>>;

    /**
     * Updates a task with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} taskUpdateParameters The parameters for updating a task.
     *
     * @param {object} [taskUpdateParameters.identity] Identity for the resource.
     *
     * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
     * of resource identity.
     *
     * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
     * resource.
     *
     * @param {string} [taskUpdateParameters.identity.type] The identity type.
     * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
     * UserAssigned', 'None'
     *
     * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
     * list of user identities associated with the resource. The user identity
     * dictionary key references will be ARM resource ids in the form:
     * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
     * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
     *
     * @param {string} [taskUpdateParameters.status] The current status of task.
     * Possible values include: 'Disabled', 'Enabled'
     *
     * @param {object} [taskUpdateParameters.platform] The platform properties
     * against which the run has to happen.
     *
     * @param {string} [taskUpdateParameters.platform.os] The operating system type
     * required for the run. Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [taskUpdateParameters.platform.architecture] The OS
     * architecture. Possible values include: 'amd64', 'x86', 'arm'
     *
     * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
     * Possible values include: 'v6', 'v7', 'v8'
     *
     * @param {object} [taskUpdateParameters.agentConfiguration] The machine
     * configuration of the run agent.
     *
     * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
     * configuration in terms of number of cores required for the run.
     *
     * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
     *
     * @param {object} [taskUpdateParameters.step] The properties for updating a
     * task step.
     *
     * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
     * relative) of the source context for the task step.
     *
     * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
     * (git PAT or SAS token of storage account blob) associated with the context
     * for a step.
     *
     * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
     *
     * @param {object} [taskUpdateParameters.trigger] The properties for updating
     * trigger properties.
     *
     * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
     * of timer triggers.
     *
     * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
     * of triggers based on source code repository.
     *
     * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
     * based on base image dependencies.
     *
     * @param {string}
     * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
     * type of the auto trigger for base image dependency updates. Possible values
     * include: 'All', 'Runtime'
     *
     * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
     * current status of trigger. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
     * of the trigger.
     *
     * @param {object} [taskUpdateParameters.credentials] The parameters that
     * describes a set of credentials that will be used when this run is invoked.
     *
     * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
     * the credential parameters for accessing the source registry.
     *
     * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
     * The authentication mode which determines the source registry login scope.
     * The credentials for the source registry
     * will be generated using the given scope. These credentials will be used to
     * login to
     * the source registry during the run. Possible values include: 'None',
     * 'Default'
     *
     * @param {object} [taskUpdateParameters.credentials.customRegistries]
     * Describes the credential parameters for accessing other custom registries.
     * The key
     * for the dictionary item will be the registry login server
     * (myregistry.azurecr.io) and
     * the value of the item will be the registry credentials for accessing the
     * registry.
     *
     * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Task} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Task} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Task} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, registryName: string, taskName: string, taskUpdateParameters: models.TaskUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Task>;
    update(resourceGroupName: string, registryName: string, taskName: string, taskUpdateParameters: models.TaskUpdateParameters, callback: ServiceCallback<models.Task>): void;
    update(resourceGroupName: string, registryName: string, taskName: string, taskUpdateParameters: models.TaskUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Task>): void;


    /**
     * Returns a task with extended information that includes all secrets.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getDetailsWithHttpOperationResponse(resourceGroupName: string, registryName: string, taskName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Task>>;

    /**
     * Returns a task with extended information that includes all secrets.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Task} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Task} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Task} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getDetails(resourceGroupName: string, registryName: string, taskName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Task>;
    getDetails(resourceGroupName: string, registryName: string, taskName: string, callback: ServiceCallback<models.Task>): void;
    getDetails(resourceGroupName: string, registryName: string, taskName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Task>): void;


    /**
     * Creates a task for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} taskCreateParameters The parameters for creating a task.
     *
     * @param {object} [taskCreateParameters.identity] Identity for the resource.
     *
     * @param {string} [taskCreateParameters.identity.principalId] The principal ID
     * of resource identity.
     *
     * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
     * resource.
     *
     * @param {string} [taskCreateParameters.identity.type] The identity type.
     * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
     * UserAssigned', 'None'
     *
     * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
     * list of user identities associated with the resource. The user identity
     * dictionary key references will be ARM resource ids in the form:
     * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
     * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
     *
     * @param {string} [taskCreateParameters.status] The current status of task.
     * Possible values include: 'Disabled', 'Enabled'
     *
     * @param {object} taskCreateParameters.platform The platform properties
     * against which the run has to happen.
     *
     * @param {string} taskCreateParameters.platform.os The operating system type
     * required for the run. Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [taskCreateParameters.platform.architecture] The OS
     * architecture. Possible values include: 'amd64', 'x86', 'arm'
     *
     * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
     * Possible values include: 'v6', 'v7', 'v8'
     *
     * @param {object} [taskCreateParameters.agentConfiguration] The machine
     * configuration of the run agent.
     *
     * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
     * configuration in terms of number of cores required for the run.
     *
     * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
     *
     * @param {object} taskCreateParameters.step The properties of a task step.
     *
     * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
     * relative) of the source context for the task step.
     *
     * @param {string} [taskCreateParameters.step.contextAccessToken] The token
     * (git PAT or SAS token of storage account blob) associated with the context
     * for a step.
     *
     * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
     *
     * @param {object} [taskCreateParameters.trigger] The properties that describe
     * all triggers for the task.
     *
     * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
     * of timer triggers.
     *
     * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
     * of triggers based on source code repository.
     *
     * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
     * based on base image dependencies.
     *
     * @param {string}
     * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
     * of the auto trigger for base image dependency updates. Possible values
     * include: 'All', 'Runtime'
     *
     * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
     * current status of trigger. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
     * of the trigger.
     *
     * @param {object} [taskCreateParameters.credentials] The properties that
     * describes a set of credentials that will be used when this run is invoked.
     *
     * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
     * the credential parameters for accessing the source registry.
     *
     * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
     * The authentication mode which determines the source registry login scope.
     * The credentials for the source registry
     * will be generated using the given scope. These credentials will be used to
     * login to
     * the source registry during the run. Possible values include: 'None',
     * 'Default'
     *
     * @param {object} [taskCreateParameters.credentials.customRegistries]
     * Describes the credential parameters for accessing other custom registries.
     * The key
     * for the dictionary item will be the registry login server
     * (myregistry.azurecr.io) and
     * the value of the item will be the registry credentials for accessing the
     * registry.
     *
     * @param {string} taskCreateParameters.location The location of the resource.
     * This cannot be changed after the resource is created.
     *
     * @param {object} [taskCreateParameters.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, registryName: string, taskName: string, taskCreateParameters: models.Task, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Task>>;

    /**
     * Creates a task for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} taskCreateParameters The parameters for creating a task.
     *
     * @param {object} [taskCreateParameters.identity] Identity for the resource.
     *
     * @param {string} [taskCreateParameters.identity.principalId] The principal ID
     * of resource identity.
     *
     * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
     * resource.
     *
     * @param {string} [taskCreateParameters.identity.type] The identity type.
     * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
     * UserAssigned', 'None'
     *
     * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
     * list of user identities associated with the resource. The user identity
     * dictionary key references will be ARM resource ids in the form:
     * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
     * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
     *
     * @param {string} [taskCreateParameters.status] The current status of task.
     * Possible values include: 'Disabled', 'Enabled'
     *
     * @param {object} taskCreateParameters.platform The platform properties
     * against which the run has to happen.
     *
     * @param {string} taskCreateParameters.platform.os The operating system type
     * required for the run. Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [taskCreateParameters.platform.architecture] The OS
     * architecture. Possible values include: 'amd64', 'x86', 'arm'
     *
     * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
     * Possible values include: 'v6', 'v7', 'v8'
     *
     * @param {object} [taskCreateParameters.agentConfiguration] The machine
     * configuration of the run agent.
     *
     * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
     * configuration in terms of number of cores required for the run.
     *
     * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
     *
     * @param {object} taskCreateParameters.step The properties of a task step.
     *
     * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
     * relative) of the source context for the task step.
     *
     * @param {string} [taskCreateParameters.step.contextAccessToken] The token
     * (git PAT or SAS token of storage account blob) associated with the context
     * for a step.
     *
     * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
     *
     * @param {object} [taskCreateParameters.trigger] The properties that describe
     * all triggers for the task.
     *
     * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
     * of timer triggers.
     *
     * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
     * of triggers based on source code repository.
     *
     * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
     * based on base image dependencies.
     *
     * @param {string}
     * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
     * of the auto trigger for base image dependency updates. Possible values
     * include: 'All', 'Runtime'
     *
     * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
     * current status of trigger. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
     * of the trigger.
     *
     * @param {object} [taskCreateParameters.credentials] The properties that
     * describes a set of credentials that will be used when this run is invoked.
     *
     * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
     * the credential parameters for accessing the source registry.
     *
     * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
     * The authentication mode which determines the source registry login scope.
     * The credentials for the source registry
     * will be generated using the given scope. These credentials will be used to
     * login to
     * the source registry during the run. Possible values include: 'None',
     * 'Default'
     *
     * @param {object} [taskCreateParameters.credentials.customRegistries]
     * Describes the credential parameters for accessing other custom registries.
     * The key
     * for the dictionary item will be the registry login server
     * (myregistry.azurecr.io) and
     * the value of the item will be the registry credentials for accessing the
     * registry.
     *
     * @param {string} taskCreateParameters.location The location of the resource.
     * This cannot be changed after the resource is created.
     *
     * @param {object} [taskCreateParameters.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Task} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Task} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Task} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, registryName: string, taskName: string, taskCreateParameters: models.Task, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Task>;
    beginCreate(resourceGroupName: string, registryName: string, taskName: string, taskCreateParameters: models.Task, callback: ServiceCallback<models.Task>): void;
    beginCreate(resourceGroupName: string, registryName: string, taskName: string, taskCreateParameters: models.Task, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Task>): void;


    /**
     * Deletes a specified task.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, taskName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a specified task.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, registryName: string, taskName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, registryName: string, taskName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, registryName: string, taskName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a task with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} taskUpdateParameters The parameters for updating a task.
     *
     * @param {object} [taskUpdateParameters.identity] Identity for the resource.
     *
     * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
     * of resource identity.
     *
     * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
     * resource.
     *
     * @param {string} [taskUpdateParameters.identity.type] The identity type.
     * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
     * UserAssigned', 'None'
     *
     * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
     * list of user identities associated with the resource. The user identity
     * dictionary key references will be ARM resource ids in the form:
     * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
     * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
     *
     * @param {string} [taskUpdateParameters.status] The current status of task.
     * Possible values include: 'Disabled', 'Enabled'
     *
     * @param {object} [taskUpdateParameters.platform] The platform properties
     * against which the run has to happen.
     *
     * @param {string} [taskUpdateParameters.platform.os] The operating system type
     * required for the run. Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [taskUpdateParameters.platform.architecture] The OS
     * architecture. Possible values include: 'amd64', 'x86', 'arm'
     *
     * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
     * Possible values include: 'v6', 'v7', 'v8'
     *
     * @param {object} [taskUpdateParameters.agentConfiguration] The machine
     * configuration of the run agent.
     *
     * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
     * configuration in terms of number of cores required for the run.
     *
     * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
     *
     * @param {object} [taskUpdateParameters.step] The properties for updating a
     * task step.
     *
     * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
     * relative) of the source context for the task step.
     *
     * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
     * (git PAT or SAS token of storage account blob) associated with the context
     * for a step.
     *
     * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
     *
     * @param {object} [taskUpdateParameters.trigger] The properties for updating
     * trigger properties.
     *
     * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
     * of timer triggers.
     *
     * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
     * of triggers based on source code repository.
     *
     * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
     * based on base image dependencies.
     *
     * @param {string}
     * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
     * type of the auto trigger for base image dependency updates. Possible values
     * include: 'All', 'Runtime'
     *
     * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
     * current status of trigger. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
     * of the trigger.
     *
     * @param {object} [taskUpdateParameters.credentials] The parameters that
     * describes a set of credentials that will be used when this run is invoked.
     *
     * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
     * the credential parameters for accessing the source registry.
     *
     * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
     * The authentication mode which determines the source registry login scope.
     * The credentials for the source registry
     * will be generated using the given scope. These credentials will be used to
     * login to
     * the source registry during the run. Possible values include: 'None',
     * 'Default'
     *
     * @param {object} [taskUpdateParameters.credentials.customRegistries]
     * Describes the credential parameters for accessing other custom registries.
     * The key
     * for the dictionary item will be the registry login server
     * (myregistry.azurecr.io) and
     * the value of the item will be the registry credentials for accessing the
     * registry.
     *
     * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, registryName: string, taskName: string, taskUpdateParameters: models.TaskUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Task>>;

    /**
     * Updates a task with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} taskName The name of the container registry task.
     *
     * @param {object} taskUpdateParameters The parameters for updating a task.
     *
     * @param {object} [taskUpdateParameters.identity] Identity for the resource.
     *
     * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
     * of resource identity.
     *
     * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
     * resource.
     *
     * @param {string} [taskUpdateParameters.identity.type] The identity type.
     * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
     * UserAssigned', 'None'
     *
     * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
     * list of user identities associated with the resource. The user identity
     * dictionary key references will be ARM resource ids in the form:
     * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
     * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
     *
     * @param {string} [taskUpdateParameters.status] The current status of task.
     * Possible values include: 'Disabled', 'Enabled'
     *
     * @param {object} [taskUpdateParameters.platform] The platform properties
     * against which the run has to happen.
     *
     * @param {string} [taskUpdateParameters.platform.os] The operating system type
     * required for the run. Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [taskUpdateParameters.platform.architecture] The OS
     * architecture. Possible values include: 'amd64', 'x86', 'arm'
     *
     * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
     * Possible values include: 'v6', 'v7', 'v8'
     *
     * @param {object} [taskUpdateParameters.agentConfiguration] The machine
     * configuration of the run agent.
     *
     * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
     * configuration in terms of number of cores required for the run.
     *
     * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
     *
     * @param {object} [taskUpdateParameters.step] The properties for updating a
     * task step.
     *
     * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
     * relative) of the source context for the task step.
     *
     * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
     * (git PAT or SAS token of storage account blob) associated with the context
     * for a step.
     *
     * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
     *
     * @param {object} [taskUpdateParameters.trigger] The properties for updating
     * trigger properties.
     *
     * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
     * of timer triggers.
     *
     * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
     * of triggers based on source code repository.
     *
     * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
     * based on base image dependencies.
     *
     * @param {string}
     * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
     * type of the auto trigger for base image dependency updates. Possible values
     * include: 'All', 'Runtime'
     *
     * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
     * current status of trigger. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
     * of the trigger.
     *
     * @param {object} [taskUpdateParameters.credentials] The parameters that
     * describes a set of credentials that will be used when this run is invoked.
     *
     * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
     * the credential parameters for accessing the source registry.
     *
     * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
     * The authentication mode which determines the source registry login scope.
     * The credentials for the source registry
     * will be generated using the given scope. These credentials will be used to
     * login to
     * the source registry during the run. Possible values include: 'None',
     * 'Default'
     *
     * @param {object} [taskUpdateParameters.credentials.customRegistries]
     * Describes the credential parameters for accessing other custom registries.
     * The key
     * for the dictionary item will be the registry login server
     * (myregistry.azurecr.io) and
     * the value of the item will be the registry credentials for accessing the
     * registry.
     *
     * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Task} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Task} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Task} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, registryName: string, taskName: string, taskUpdateParameters: models.TaskUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Task>;
    beginUpdate(resourceGroupName: string, registryName: string, taskName: string, taskUpdateParameters: models.TaskUpdateParameters, callback: ServiceCallback<models.Task>): void;
    beginUpdate(resourceGroupName: string, registryName: string, taskName: string, taskUpdateParameters: models.TaskUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Task>): void;


    /**
     * Lists all the tasks for a specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<TaskListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.TaskListResult>>;

    /**
     * Lists all the tasks for a specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {TaskListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {TaskListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link TaskListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.TaskListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.TaskListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.TaskListResult>): void;
}
