/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * ProviderOperations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface ProviderOperations {


    /**
     * Result of the request to list REST API operations
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ProviderOperationResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ProviderOperationResult>>;

    /**
     * Result of the request to list REST API operations
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ProviderOperationResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ProviderOperationResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ProviderOperationResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ProviderOperationResult>;
    list(callback: ServiceCallback<models.ProviderOperationResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ProviderOperationResult>): void;


    /**
     * Result of the request to list REST API operations
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ProviderOperationResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ProviderOperationResult>>;

    /**
     * Result of the request to list REST API operations
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ProviderOperationResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ProviderOperationResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ProviderOperationResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ProviderOperationResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ProviderOperationResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ProviderOperationResult>): void;
}

/**
 * @class
 * Labs
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Labs {


    /**
     * List labs in a subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=defaultStorageAccount)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionWithHttpOperationResponse(options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabList>>;

    /**
     * List labs in a subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=defaultStorageAccount)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscription(options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.LabList>;
    listBySubscription(callback: ServiceCallback<models.LabList>): void;
    listBySubscription(options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabList>): void;


    /**
     * List labs in a resource group.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=defaultStorageAccount)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabList>>;

    /**
     * List labs in a resource group.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=defaultStorageAccount)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.LabList>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.LabList>): void;
    listByResourceGroup(resourceGroupName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabList>): void;


    /**
     * Get lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=defaultStorageAccount)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Lab>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Lab>>;

    /**
     * Get lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=defaultStorageAccount)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Lab} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Lab} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Lab} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Lab>;
    get(resourceGroupName: string, name: string, callback: ServiceCallback<models.Lab>): void;
    get(resourceGroupName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Lab>): void;


    /**
     * Create or replace an existing lab. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} lab A lab.
     *
     * @param {string} [lab.labStorageType] Type of storage used by the lab. It can
     * be either Premium or Standard. Default is Premium. Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsLinux] The ordered list of
     * artifact resource IDs that should be applied on all Linux VM creations by
     * default, prior to the artifacts specified by the user.
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsWindows] The ordered list
     * of artifact resource IDs that should be applied on all Windows VM creations
     * by default, prior to the artifacts specified by the user.
     *
     * @param {string} [lab.premiumDataDisks] The setting to enable usage of
     * premium data disks.
     * When its value is 'Enabled', creation of standard or premium data disks is
     * allowed.
     * When its value is 'Disabled', only creation of standard data disks is
     * allowed. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} [lab.environmentPermission] The access rights to be granted
     * to the user when provisioning an environment. Possible values include:
     * 'Reader', 'Contributor'
     *
     * @param {object} [lab.announcement] The properties of any lab announcement
     * associated with this lab
     *
     * @param {string} [lab.announcement.title] The plain text title for the lab
     * announcement
     *
     * @param {string} [lab.announcement.markdown] The markdown text (if any) that
     * this lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {string} [lab.announcement.enabled] Is the lab announcement
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {date} [lab.announcement.expirationDate] The time at which the
     * announcement expires (null for never)
     *
     * @param {boolean} [lab.announcement.expired] Has this announcement expired?
     *
     * @param {object} [lab.support] The properties of any lab support message
     * associated with this lab
     *
     * @param {string} [lab.support.enabled] Is the lab support banner
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [lab.support.markdown] The markdown text (if any) that this
     * lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {object} [lab.extendedProperties] Extended properties of the lab used
     * for experimental features
     *
     * @param {string} [lab.location] The location of the resource.
     *
     * @param {object} [lab.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Lab>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, name: string, lab: models.Lab, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Lab>>;

    /**
     * Create or replace an existing lab. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} lab A lab.
     *
     * @param {string} [lab.labStorageType] Type of storage used by the lab. It can
     * be either Premium or Standard. Default is Premium. Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsLinux] The ordered list of
     * artifact resource IDs that should be applied on all Linux VM creations by
     * default, prior to the artifacts specified by the user.
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsWindows] The ordered list
     * of artifact resource IDs that should be applied on all Windows VM creations
     * by default, prior to the artifacts specified by the user.
     *
     * @param {string} [lab.premiumDataDisks] The setting to enable usage of
     * premium data disks.
     * When its value is 'Enabled', creation of standard or premium data disks is
     * allowed.
     * When its value is 'Disabled', only creation of standard data disks is
     * allowed. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} [lab.environmentPermission] The access rights to be granted
     * to the user when provisioning an environment. Possible values include:
     * 'Reader', 'Contributor'
     *
     * @param {object} [lab.announcement] The properties of any lab announcement
     * associated with this lab
     *
     * @param {string} [lab.announcement.title] The plain text title for the lab
     * announcement
     *
     * @param {string} [lab.announcement.markdown] The markdown text (if any) that
     * this lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {string} [lab.announcement.enabled] Is the lab announcement
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {date} [lab.announcement.expirationDate] The time at which the
     * announcement expires (null for never)
     *
     * @param {boolean} [lab.announcement.expired] Has this announcement expired?
     *
     * @param {object} [lab.support] The properties of any lab support message
     * associated with this lab
     *
     * @param {string} [lab.support.enabled] Is the lab support banner
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [lab.support.markdown] The markdown text (if any) that this
     * lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {object} [lab.extendedProperties] Extended properties of the lab used
     * for experimental features
     *
     * @param {string} [lab.location] The location of the resource.
     *
     * @param {object} [lab.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Lab} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Lab} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Lab} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, name: string, lab: models.Lab, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Lab>;
    createOrUpdate(resourceGroupName: string, name: string, lab: models.Lab, callback: ServiceCallback<models.Lab>): void;
    createOrUpdate(resourceGroupName: string, name: string, lab: models.Lab, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Lab>): void;


    /**
     * Delete lab. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete lab. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of labs.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} lab A lab.
     *
     * @param {string} [lab.labStorageType] Type of storage used by the lab. It can
     * be either Premium or Standard. Default is Premium. Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsLinux] The ordered list of
     * artifact resource IDs that should be applied on all Linux VM creations by
     * default, prior to the artifacts specified by the user.
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsWindows] The ordered list
     * of artifact resource IDs that should be applied on all Windows VM creations
     * by default, prior to the artifacts specified by the user.
     *
     * @param {string} [lab.premiumDataDisks] The setting to enable usage of
     * premium data disks.
     * When its value is 'Enabled', creation of standard or premium data disks is
     * allowed.
     * When its value is 'Disabled', only creation of standard data disks is
     * allowed. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} [lab.environmentPermission] The access rights to be granted
     * to the user when provisioning an environment. Possible values include:
     * 'Reader', 'Contributor'
     *
     * @param {object} [lab.announcement] The properties of any lab announcement
     * associated with this lab
     *
     * @param {string} [lab.announcement.title] The plain text title for the lab
     * announcement
     *
     * @param {string} [lab.announcement.markdown] The markdown text (if any) that
     * this lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {string} [lab.announcement.enabled] Is the lab announcement
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {date} [lab.announcement.expirationDate] The time at which the
     * announcement expires (null for never)
     *
     * @param {boolean} [lab.announcement.expired] Has this announcement expired?
     *
     * @param {object} [lab.support] The properties of any lab support message
     * associated with this lab
     *
     * @param {string} [lab.support.enabled] Is the lab support banner
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [lab.support.markdown] The markdown text (if any) that this
     * lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {object} [lab.extendedProperties] Extended properties of the lab used
     * for experimental features
     *
     * @param {object} [lab.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Lab>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, name: string, lab: models.LabFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Lab>>;

    /**
     * Modify properties of labs.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} lab A lab.
     *
     * @param {string} [lab.labStorageType] Type of storage used by the lab. It can
     * be either Premium or Standard. Default is Premium. Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsLinux] The ordered list of
     * artifact resource IDs that should be applied on all Linux VM creations by
     * default, prior to the artifacts specified by the user.
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsWindows] The ordered list
     * of artifact resource IDs that should be applied on all Windows VM creations
     * by default, prior to the artifacts specified by the user.
     *
     * @param {string} [lab.premiumDataDisks] The setting to enable usage of
     * premium data disks.
     * When its value is 'Enabled', creation of standard or premium data disks is
     * allowed.
     * When its value is 'Disabled', only creation of standard data disks is
     * allowed. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} [lab.environmentPermission] The access rights to be granted
     * to the user when provisioning an environment. Possible values include:
     * 'Reader', 'Contributor'
     *
     * @param {object} [lab.announcement] The properties of any lab announcement
     * associated with this lab
     *
     * @param {string} [lab.announcement.title] The plain text title for the lab
     * announcement
     *
     * @param {string} [lab.announcement.markdown] The markdown text (if any) that
     * this lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {string} [lab.announcement.enabled] Is the lab announcement
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {date} [lab.announcement.expirationDate] The time at which the
     * announcement expires (null for never)
     *
     * @param {boolean} [lab.announcement.expired] Has this announcement expired?
     *
     * @param {object} [lab.support] The properties of any lab support message
     * associated with this lab
     *
     * @param {string} [lab.support.enabled] Is the lab support banner
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [lab.support.markdown] The markdown text (if any) that this
     * lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {object} [lab.extendedProperties] Extended properties of the lab used
     * for experimental features
     *
     * @param {object} [lab.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Lab} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Lab} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Lab} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, name: string, lab: models.LabFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Lab>;
    update(resourceGroupName: string, name: string, lab: models.LabFragment, callback: ServiceCallback<models.Lab>): void;
    update(resourceGroupName: string, name: string, lab: models.LabFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Lab>): void;


    /**
     * Claim a random claimable virtual machine in the lab. This operation can take
     * a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    claimAnyVmWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Claim a random claimable virtual machine in the lab. This operation can take
     * a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    claimAnyVm(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    claimAnyVm(resourceGroupName: string, name: string, callback: ServiceCallback<void>): void;
    claimAnyVm(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Create virtual machines in a lab. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} labVirtualMachineCreationParameter Properties for creating a
     * virtual machine.
     *
     * @param {object} [labVirtualMachineCreationParameter.bulkCreationParameters]
     * The number of virtual machine instances to create.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.bulkCreationParameters.instanceCount]
     * The number of virtual machine instances to create.
     *
     * @param {string} [labVirtualMachineCreationParameter.notes] The notes of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.ownerObjectId] The
     * object identifier of the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.ownerUserPrincipalName]
     * The user principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachineCreationParameter.createdByUserId] The
     * object identifier of the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.createdByUser] The email
     * address of creator of the virtual machine.
     *
     * @param {date} [labVirtualMachineCreationParameter.createdDate] The creation
     * date of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.computeId] The resource
     * identifier (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.customImageId] The
     * custom image identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.osType] The OS type of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.size] The size of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.userName] The user name
     * of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.password] The password
     * of the virtual machine administrator.
     *
     * @param {string} [labVirtualMachineCreationParameter.sshKey] The SSH key of
     * the virtual machine administrator.
     *
     * @param {boolean}
     * [labVirtualMachineCreationParameter.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachineCreationParameter.fqdn] The
     * fully-qualified domain name of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.labSubnetName] The lab
     * subnet name of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.labVirtualNetworkId] The
     * lab virtual network identifier of the virtual machine.
     *
     * @param {boolean}
     * [labVirtualMachineCreationParameter.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachineCreationParameter.artifacts] The artifacts
     * to be installed on the virtual machine.
     *
     * @param {object}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.deploymentStatus]
     * The deployment status of the artifact.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.artifactsApplied]
     * The total count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachineCreationParameter.galleryImageReference]
     * The Microsoft Azure Marketplace image reference of the virtual machine.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.offer] The offer
     * of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.sku] The SKU of
     * the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachineCreationParameter.planId] The id of the
     * plan associated with the virtual machine image
     *
     * @param {object} [labVirtualMachineCreationParameter.networkInterface] The
     * network interface properties.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.dnsName] The DNS name.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachineCreationParameter.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachineCreationParameter.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachineCreationParameter.expirationDate] The
     * expiration date for VM.
     *
     * @param {boolean} [labVirtualMachineCreationParameter.allowClaim] Indicates
     * whether another user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachineCreationParameter.storageType] Storage
     * type to use for virtual machine (i.e. Standard, Premium).
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachineCreationParameter.environmentId] The
     * resource ID of the environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachineCreationParameter.dataDiskParameters] New
     * or existing data disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachineCreationParameter.scheduleParameters]
     * Virtual Machine schedules to be created
     *
     * @param {string} [labVirtualMachineCreationParameter.lastKnownPowerState]
     * Last known compute power state captured in DTL
     *
     * @param {string} [labVirtualMachineCreationParameter.name] The name of the
     * virtual machine or environment
     *
     * @param {string} [labVirtualMachineCreationParameter.location] The location
     * of the new virtual machine or environment
     *
     * @param {object} [labVirtualMachineCreationParameter.tags] The tags of the
     * resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createEnvironmentWithHttpOperationResponse(resourceGroupName: string, name: string, labVirtualMachineCreationParameter: models.LabVirtualMachineCreationParameter, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Create virtual machines in a lab. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} labVirtualMachineCreationParameter Properties for creating a
     * virtual machine.
     *
     * @param {object} [labVirtualMachineCreationParameter.bulkCreationParameters]
     * The number of virtual machine instances to create.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.bulkCreationParameters.instanceCount]
     * The number of virtual machine instances to create.
     *
     * @param {string} [labVirtualMachineCreationParameter.notes] The notes of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.ownerObjectId] The
     * object identifier of the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.ownerUserPrincipalName]
     * The user principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachineCreationParameter.createdByUserId] The
     * object identifier of the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.createdByUser] The email
     * address of creator of the virtual machine.
     *
     * @param {date} [labVirtualMachineCreationParameter.createdDate] The creation
     * date of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.computeId] The resource
     * identifier (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.customImageId] The
     * custom image identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.osType] The OS type of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.size] The size of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.userName] The user name
     * of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.password] The password
     * of the virtual machine administrator.
     *
     * @param {string} [labVirtualMachineCreationParameter.sshKey] The SSH key of
     * the virtual machine administrator.
     *
     * @param {boolean}
     * [labVirtualMachineCreationParameter.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachineCreationParameter.fqdn] The
     * fully-qualified domain name of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.labSubnetName] The lab
     * subnet name of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.labVirtualNetworkId] The
     * lab virtual network identifier of the virtual machine.
     *
     * @param {boolean}
     * [labVirtualMachineCreationParameter.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachineCreationParameter.artifacts] The artifacts
     * to be installed on the virtual machine.
     *
     * @param {object}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.deploymentStatus]
     * The deployment status of the artifact.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.artifactsApplied]
     * The total count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachineCreationParameter.galleryImageReference]
     * The Microsoft Azure Marketplace image reference of the virtual machine.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.offer] The offer
     * of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.sku] The SKU of
     * the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachineCreationParameter.planId] The id of the
     * plan associated with the virtual machine image
     *
     * @param {object} [labVirtualMachineCreationParameter.networkInterface] The
     * network interface properties.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.dnsName] The DNS name.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachineCreationParameter.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachineCreationParameter.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachineCreationParameter.expirationDate] The
     * expiration date for VM.
     *
     * @param {boolean} [labVirtualMachineCreationParameter.allowClaim] Indicates
     * whether another user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachineCreationParameter.storageType] Storage
     * type to use for virtual machine (i.e. Standard, Premium).
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachineCreationParameter.environmentId] The
     * resource ID of the environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachineCreationParameter.dataDiskParameters] New
     * or existing data disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachineCreationParameter.scheduleParameters]
     * Virtual Machine schedules to be created
     *
     * @param {string} [labVirtualMachineCreationParameter.lastKnownPowerState]
     * Last known compute power state captured in DTL
     *
     * @param {string} [labVirtualMachineCreationParameter.name] The name of the
     * virtual machine or environment
     *
     * @param {string} [labVirtualMachineCreationParameter.location] The location
     * of the new virtual machine or environment
     *
     * @param {object} [labVirtualMachineCreationParameter.tags] The tags of the
     * resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createEnvironment(resourceGroupName: string, name: string, labVirtualMachineCreationParameter: models.LabVirtualMachineCreationParameter, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    createEnvironment(resourceGroupName: string, name: string, labVirtualMachineCreationParameter: models.LabVirtualMachineCreationParameter, callback: ServiceCallback<void>): void;
    createEnvironment(resourceGroupName: string, name: string, labVirtualMachineCreationParameter: models.LabVirtualMachineCreationParameter, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Exports the lab resource usage into a storage account This operation can
     * take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} exportResourceUsageParameters The parameters of the export
     * operation.
     *
     * @param {string} [exportResourceUsageParameters.blobStorageAbsoluteSasUri]
     * The blob storage absolute sas uri with write permission to the container
     * which the usage data needs to be uploaded to.
     *
     * @param {date} [exportResourceUsageParameters.usageStartDate] The start time
     * of the usage. If not provided, usage will be reported since the beginning of
     * data collection.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    exportResourceUsageWithHttpOperationResponse(resourceGroupName: string, name: string, exportResourceUsageParameters: models.ExportResourceUsageParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Exports the lab resource usage into a storage account This operation can
     * take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} exportResourceUsageParameters The parameters of the export
     * operation.
     *
     * @param {string} [exportResourceUsageParameters.blobStorageAbsoluteSasUri]
     * The blob storage absolute sas uri with write permission to the container
     * which the usage data needs to be uploaded to.
     *
     * @param {date} [exportResourceUsageParameters.usageStartDate] The start time
     * of the usage. If not provided, usage will be reported since the beginning of
     * data collection.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    exportResourceUsage(resourceGroupName: string, name: string, exportResourceUsageParameters: models.ExportResourceUsageParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    exportResourceUsage(resourceGroupName: string, name: string, exportResourceUsageParameters: models.ExportResourceUsageParameters, callback: ServiceCallback<void>): void;
    exportResourceUsage(resourceGroupName: string, name: string, exportResourceUsageParameters: models.ExportResourceUsageParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Generate a URI for uploading custom disk images to a Lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} generateUploadUriParameter Properties for generating an
     * upload URI.
     *
     * @param {string} [generateUploadUriParameter.blobName] The blob name of the
     * upload URI.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<GenerateUploadUriResponse>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    generateUploadUriWithHttpOperationResponse(resourceGroupName: string, name: string, generateUploadUriParameter: models.GenerateUploadUriParameter, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.GenerateUploadUriResponse>>;

    /**
     * Generate a URI for uploading custom disk images to a Lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} generateUploadUriParameter Properties for generating an
     * upload URI.
     *
     * @param {string} [generateUploadUriParameter.blobName] The blob name of the
     * upload URI.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {GenerateUploadUriResponse} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {GenerateUploadUriResponse} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link GenerateUploadUriResponse} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    generateUploadUri(resourceGroupName: string, name: string, generateUploadUriParameter: models.GenerateUploadUriParameter, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.GenerateUploadUriResponse>;
    generateUploadUri(resourceGroupName: string, name: string, generateUploadUriParameter: models.GenerateUploadUriParameter, callback: ServiceCallback<models.GenerateUploadUriResponse>): void;
    generateUploadUri(resourceGroupName: string, name: string, generateUploadUriParameter: models.GenerateUploadUriParameter, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.GenerateUploadUriResponse>): void;


    /**
     * Import a virtual machine into a different lab. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} importLabVirtualMachineRequest This represents the payload
     * required to import a virtual machine from a different lab into the current
     * one
     *
     * @param {string}
     * [importLabVirtualMachineRequest.sourceVirtualMachineResourceId] The full
     * resource ID of the virtual machine to be imported.
     *
     * @param {string}
     * [importLabVirtualMachineRequest.destinationVirtualMachineName] The name of
     * the virtual machine in the destination lab
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    importVirtualMachineWithHttpOperationResponse(resourceGroupName: string, name: string, importLabVirtualMachineRequest: models.ImportLabVirtualMachineRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Import a virtual machine into a different lab. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} importLabVirtualMachineRequest This represents the payload
     * required to import a virtual machine from a different lab into the current
     * one
     *
     * @param {string}
     * [importLabVirtualMachineRequest.sourceVirtualMachineResourceId] The full
     * resource ID of the virtual machine to be imported.
     *
     * @param {string}
     * [importLabVirtualMachineRequest.destinationVirtualMachineName] The name of
     * the virtual machine in the destination lab
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    importVirtualMachine(resourceGroupName: string, name: string, importLabVirtualMachineRequest: models.ImportLabVirtualMachineRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    importVirtualMachine(resourceGroupName: string, name: string, importLabVirtualMachineRequest: models.ImportLabVirtualMachineRequest, callback: ServiceCallback<void>): void;
    importVirtualMachine(resourceGroupName: string, name: string, importLabVirtualMachineRequest: models.ImportLabVirtualMachineRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List disk images available for custom image creation.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabVhdList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listVhdsWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabVhdList>>;

    /**
     * List disk images available for custom image creation.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabVhdList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabVhdList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabVhdList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listVhds(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.LabVhdList>;
    listVhds(resourceGroupName: string, name: string, callback: ServiceCallback<models.LabVhdList>): void;
    listVhds(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabVhdList>): void;


    /**
     * Create or replace an existing lab. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} lab A lab.
     *
     * @param {string} [lab.labStorageType] Type of storage used by the lab. It can
     * be either Premium or Standard. Default is Premium. Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsLinux] The ordered list of
     * artifact resource IDs that should be applied on all Linux VM creations by
     * default, prior to the artifacts specified by the user.
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsWindows] The ordered list
     * of artifact resource IDs that should be applied on all Windows VM creations
     * by default, prior to the artifacts specified by the user.
     *
     * @param {string} [lab.premiumDataDisks] The setting to enable usage of
     * premium data disks.
     * When its value is 'Enabled', creation of standard or premium data disks is
     * allowed.
     * When its value is 'Disabled', only creation of standard data disks is
     * allowed. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} [lab.environmentPermission] The access rights to be granted
     * to the user when provisioning an environment. Possible values include:
     * 'Reader', 'Contributor'
     *
     * @param {object} [lab.announcement] The properties of any lab announcement
     * associated with this lab
     *
     * @param {string} [lab.announcement.title] The plain text title for the lab
     * announcement
     *
     * @param {string} [lab.announcement.markdown] The markdown text (if any) that
     * this lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {string} [lab.announcement.enabled] Is the lab announcement
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {date} [lab.announcement.expirationDate] The time at which the
     * announcement expires (null for never)
     *
     * @param {boolean} [lab.announcement.expired] Has this announcement expired?
     *
     * @param {object} [lab.support] The properties of any lab support message
     * associated with this lab
     *
     * @param {string} [lab.support.enabled] Is the lab support banner
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [lab.support.markdown] The markdown text (if any) that this
     * lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {object} [lab.extendedProperties] Extended properties of the lab used
     * for experimental features
     *
     * @param {string} [lab.location] The location of the resource.
     *
     * @param {object} [lab.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Lab>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, name: string, lab: models.Lab, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Lab>>;

    /**
     * Create or replace an existing lab. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} lab A lab.
     *
     * @param {string} [lab.labStorageType] Type of storage used by the lab. It can
     * be either Premium or Standard. Default is Premium. Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsLinux] The ordered list of
     * artifact resource IDs that should be applied on all Linux VM creations by
     * default, prior to the artifacts specified by the user.
     *
     * @param {array} [lab.mandatoryArtifactsResourceIdsWindows] The ordered list
     * of artifact resource IDs that should be applied on all Windows VM creations
     * by default, prior to the artifacts specified by the user.
     *
     * @param {string} [lab.premiumDataDisks] The setting to enable usage of
     * premium data disks.
     * When its value is 'Enabled', creation of standard or premium data disks is
     * allowed.
     * When its value is 'Disabled', only creation of standard data disks is
     * allowed. Possible values include: 'Disabled', 'Enabled'
     *
     * @param {string} [lab.environmentPermission] The access rights to be granted
     * to the user when provisioning an environment. Possible values include:
     * 'Reader', 'Contributor'
     *
     * @param {object} [lab.announcement] The properties of any lab announcement
     * associated with this lab
     *
     * @param {string} [lab.announcement.title] The plain text title for the lab
     * announcement
     *
     * @param {string} [lab.announcement.markdown] The markdown text (if any) that
     * this lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {string} [lab.announcement.enabled] Is the lab announcement
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {date} [lab.announcement.expirationDate] The time at which the
     * announcement expires (null for never)
     *
     * @param {boolean} [lab.announcement.expired] Has this announcement expired?
     *
     * @param {object} [lab.support] The properties of any lab support message
     * associated with this lab
     *
     * @param {string} [lab.support.enabled] Is the lab support banner
     * active/enabled at this time?. Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [lab.support.markdown] The markdown text (if any) that this
     * lab displays in the UI. If left empty/null, nothing will be shown.
     *
     * @param {object} [lab.extendedProperties] Extended properties of the lab used
     * for experimental features
     *
     * @param {string} [lab.location] The location of the resource.
     *
     * @param {object} [lab.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Lab} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Lab} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Lab} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, name: string, lab: models.Lab, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Lab>;
    beginCreateOrUpdate(resourceGroupName: string, name: string, lab: models.Lab, callback: ServiceCallback<models.Lab>): void;
    beginCreateOrUpdate(resourceGroupName: string, name: string, lab: models.Lab, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Lab>): void;


    /**
     * Delete lab. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete lab. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, name: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Claim a random claimable virtual machine in the lab. This operation can take
     * a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginClaimAnyVmWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Claim a random claimable virtual machine in the lab. This operation can take
     * a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginClaimAnyVm(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginClaimAnyVm(resourceGroupName: string, name: string, callback: ServiceCallback<void>): void;
    beginClaimAnyVm(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Create virtual machines in a lab. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} labVirtualMachineCreationParameter Properties for creating a
     * virtual machine.
     *
     * @param {object} [labVirtualMachineCreationParameter.bulkCreationParameters]
     * The number of virtual machine instances to create.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.bulkCreationParameters.instanceCount]
     * The number of virtual machine instances to create.
     *
     * @param {string} [labVirtualMachineCreationParameter.notes] The notes of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.ownerObjectId] The
     * object identifier of the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.ownerUserPrincipalName]
     * The user principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachineCreationParameter.createdByUserId] The
     * object identifier of the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.createdByUser] The email
     * address of creator of the virtual machine.
     *
     * @param {date} [labVirtualMachineCreationParameter.createdDate] The creation
     * date of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.computeId] The resource
     * identifier (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.customImageId] The
     * custom image identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.osType] The OS type of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.size] The size of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.userName] The user name
     * of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.password] The password
     * of the virtual machine administrator.
     *
     * @param {string} [labVirtualMachineCreationParameter.sshKey] The SSH key of
     * the virtual machine administrator.
     *
     * @param {boolean}
     * [labVirtualMachineCreationParameter.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachineCreationParameter.fqdn] The
     * fully-qualified domain name of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.labSubnetName] The lab
     * subnet name of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.labVirtualNetworkId] The
     * lab virtual network identifier of the virtual machine.
     *
     * @param {boolean}
     * [labVirtualMachineCreationParameter.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachineCreationParameter.artifacts] The artifacts
     * to be installed on the virtual machine.
     *
     * @param {object}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.deploymentStatus]
     * The deployment status of the artifact.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.artifactsApplied]
     * The total count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachineCreationParameter.galleryImageReference]
     * The Microsoft Azure Marketplace image reference of the virtual machine.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.offer] The offer
     * of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.sku] The SKU of
     * the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachineCreationParameter.planId] The id of the
     * plan associated with the virtual machine image
     *
     * @param {object} [labVirtualMachineCreationParameter.networkInterface] The
     * network interface properties.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.dnsName] The DNS name.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachineCreationParameter.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachineCreationParameter.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachineCreationParameter.expirationDate] The
     * expiration date for VM.
     *
     * @param {boolean} [labVirtualMachineCreationParameter.allowClaim] Indicates
     * whether another user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachineCreationParameter.storageType] Storage
     * type to use for virtual machine (i.e. Standard, Premium).
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachineCreationParameter.environmentId] The
     * resource ID of the environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachineCreationParameter.dataDiskParameters] New
     * or existing data disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachineCreationParameter.scheduleParameters]
     * Virtual Machine schedules to be created
     *
     * @param {string} [labVirtualMachineCreationParameter.lastKnownPowerState]
     * Last known compute power state captured in DTL
     *
     * @param {string} [labVirtualMachineCreationParameter.name] The name of the
     * virtual machine or environment
     *
     * @param {string} [labVirtualMachineCreationParameter.location] The location
     * of the new virtual machine or environment
     *
     * @param {object} [labVirtualMachineCreationParameter.tags] The tags of the
     * resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateEnvironmentWithHttpOperationResponse(resourceGroupName: string, name: string, labVirtualMachineCreationParameter: models.LabVirtualMachineCreationParameter, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Create virtual machines in a lab. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} labVirtualMachineCreationParameter Properties for creating a
     * virtual machine.
     *
     * @param {object} [labVirtualMachineCreationParameter.bulkCreationParameters]
     * The number of virtual machine instances to create.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.bulkCreationParameters.instanceCount]
     * The number of virtual machine instances to create.
     *
     * @param {string} [labVirtualMachineCreationParameter.notes] The notes of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.ownerObjectId] The
     * object identifier of the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.ownerUserPrincipalName]
     * The user principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachineCreationParameter.createdByUserId] The
     * object identifier of the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.createdByUser] The email
     * address of creator of the virtual machine.
     *
     * @param {date} [labVirtualMachineCreationParameter.createdDate] The creation
     * date of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.computeId] The resource
     * identifier (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.customImageId] The
     * custom image identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.osType] The OS type of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.size] The size of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.userName] The user name
     * of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.password] The password
     * of the virtual machine administrator.
     *
     * @param {string} [labVirtualMachineCreationParameter.sshKey] The SSH key of
     * the virtual machine administrator.
     *
     * @param {boolean}
     * [labVirtualMachineCreationParameter.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachineCreationParameter.fqdn] The
     * fully-qualified domain name of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.labSubnetName] The lab
     * subnet name of the virtual machine.
     *
     * @param {string} [labVirtualMachineCreationParameter.labVirtualNetworkId] The
     * lab virtual network identifier of the virtual machine.
     *
     * @param {boolean}
     * [labVirtualMachineCreationParameter.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachineCreationParameter.artifacts] The artifacts
     * to be installed on the virtual machine.
     *
     * @param {object}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.deploymentStatus]
     * The deployment status of the artifact.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.artifactsApplied]
     * The total count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [labVirtualMachineCreationParameter.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachineCreationParameter.galleryImageReference]
     * The Microsoft Azure Marketplace image reference of the virtual machine.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.offer] The offer
     * of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.sku] The SKU of
     * the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachineCreationParameter.planId] The id of the
     * plan associated with the virtual machine image
     *
     * @param {object} [labVirtualMachineCreationParameter.networkInterface] The
     * network interface properties.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.dnsName] The DNS name.
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachineCreationParameter.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachineCreationParameter.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachineCreationParameter.expirationDate] The
     * expiration date for VM.
     *
     * @param {boolean} [labVirtualMachineCreationParameter.allowClaim] Indicates
     * whether another user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachineCreationParameter.storageType] Storage
     * type to use for virtual machine (i.e. Standard, Premium).
     *
     * @param {string}
     * [labVirtualMachineCreationParameter.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachineCreationParameter.environmentId] The
     * resource ID of the environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachineCreationParameter.dataDiskParameters] New
     * or existing data disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachineCreationParameter.scheduleParameters]
     * Virtual Machine schedules to be created
     *
     * @param {string} [labVirtualMachineCreationParameter.lastKnownPowerState]
     * Last known compute power state captured in DTL
     *
     * @param {string} [labVirtualMachineCreationParameter.name] The name of the
     * virtual machine or environment
     *
     * @param {string} [labVirtualMachineCreationParameter.location] The location
     * of the new virtual machine or environment
     *
     * @param {object} [labVirtualMachineCreationParameter.tags] The tags of the
     * resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateEnvironment(resourceGroupName: string, name: string, labVirtualMachineCreationParameter: models.LabVirtualMachineCreationParameter, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginCreateEnvironment(resourceGroupName: string, name: string, labVirtualMachineCreationParameter: models.LabVirtualMachineCreationParameter, callback: ServiceCallback<void>): void;
    beginCreateEnvironment(resourceGroupName: string, name: string, labVirtualMachineCreationParameter: models.LabVirtualMachineCreationParameter, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Exports the lab resource usage into a storage account This operation can
     * take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} exportResourceUsageParameters The parameters of the export
     * operation.
     *
     * @param {string} [exportResourceUsageParameters.blobStorageAbsoluteSasUri]
     * The blob storage absolute sas uri with write permission to the container
     * which the usage data needs to be uploaded to.
     *
     * @param {date} [exportResourceUsageParameters.usageStartDate] The start time
     * of the usage. If not provided, usage will be reported since the beginning of
     * data collection.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginExportResourceUsageWithHttpOperationResponse(resourceGroupName: string, name: string, exportResourceUsageParameters: models.ExportResourceUsageParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Exports the lab resource usage into a storage account This operation can
     * take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} exportResourceUsageParameters The parameters of the export
     * operation.
     *
     * @param {string} [exportResourceUsageParameters.blobStorageAbsoluteSasUri]
     * The blob storage absolute sas uri with write permission to the container
     * which the usage data needs to be uploaded to.
     *
     * @param {date} [exportResourceUsageParameters.usageStartDate] The start time
     * of the usage. If not provided, usage will be reported since the beginning of
     * data collection.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginExportResourceUsage(resourceGroupName: string, name: string, exportResourceUsageParameters: models.ExportResourceUsageParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginExportResourceUsage(resourceGroupName: string, name: string, exportResourceUsageParameters: models.ExportResourceUsageParameters, callback: ServiceCallback<void>): void;
    beginExportResourceUsage(resourceGroupName: string, name: string, exportResourceUsageParameters: models.ExportResourceUsageParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Import a virtual machine into a different lab. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} importLabVirtualMachineRequest This represents the payload
     * required to import a virtual machine from a different lab into the current
     * one
     *
     * @param {string}
     * [importLabVirtualMachineRequest.sourceVirtualMachineResourceId] The full
     * resource ID of the virtual machine to be imported.
     *
     * @param {string}
     * [importLabVirtualMachineRequest.destinationVirtualMachineName] The name of
     * the virtual machine in the destination lab
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginImportVirtualMachineWithHttpOperationResponse(resourceGroupName: string, name: string, importLabVirtualMachineRequest: models.ImportLabVirtualMachineRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Import a virtual machine into a different lab. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the lab.
     *
     * @param {object} importLabVirtualMachineRequest This represents the payload
     * required to import a virtual machine from a different lab into the current
     * one
     *
     * @param {string}
     * [importLabVirtualMachineRequest.sourceVirtualMachineResourceId] The full
     * resource ID of the virtual machine to be imported.
     *
     * @param {string}
     * [importLabVirtualMachineRequest.destinationVirtualMachineName] The name of
     * the virtual machine in the destination lab
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginImportVirtualMachine(resourceGroupName: string, name: string, importLabVirtualMachineRequest: models.ImportLabVirtualMachineRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginImportVirtualMachine(resourceGroupName: string, name: string, importLabVirtualMachineRequest: models.ImportLabVirtualMachineRequest, callback: ServiceCallback<void>): void;
    beginImportVirtualMachine(resourceGroupName: string, name: string, importLabVirtualMachineRequest: models.ImportLabVirtualMachineRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List labs in a subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabList>>;

    /**
     * List labs in a subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscriptionNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.LabList>;
    listBySubscriptionNext(nextPageLink: string, callback: ServiceCallback<models.LabList>): void;
    listBySubscriptionNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabList>): void;


    /**
     * List labs in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabList>>;

    /**
     * List labs in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.LabList>;
    listByResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.LabList>): void;
    listByResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabList>): void;


    /**
     * List disk images available for custom image creation.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabVhdList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listVhdsNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabVhdList>>;

    /**
     * List disk images available for custom image creation.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabVhdList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabVhdList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabVhdList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listVhdsNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.LabVhdList>;
    listVhdsNext(nextPageLink: string, callback: ServiceCallback<models.LabVhdList>): void;
    listVhdsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabVhdList>): void;
}

/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Operations {


    /**
     * Get operation.
     *
     * @param {string} locationName The name of the location.
     *
     * @param {string} name The name of the operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(locationName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationResult>>;

    /**
     * Get operation.
     *
     * @param {string} locationName The name of the location.
     *
     * @param {string} name The name of the operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(locationName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationResult>;
    get(locationName: string, name: string, callback: ServiceCallback<models.OperationResult>): void;
    get(locationName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationResult>): void;
}

/**
 * @class
 * GlobalSchedules
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface GlobalSchedules {


    /**
     * List schedules in a subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionWithHttpOperationResponse(options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscription(options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    listBySubscription(callback: ServiceCallback<models.ScheduleList>): void;
    listBySubscription(options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;


    /**
     * List schedules in a resource group.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a resource group.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.ScheduleList>): void;
    listByResourceGroup(resourceGroupName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;


    /**
     * Get schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Get schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    get(resourceGroupName: string, name: string, callback: ServiceCallback<models.Schedule>): void;
    get(resourceGroupName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Create or replace an existing schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {string} [schedule.location] The location of the resource.
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, name: string, schedule: models.Schedule, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Create or replace an existing schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {string} [schedule.location] The location of the resource.
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, name: string, schedule: models.Schedule, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    createOrUpdate(resourceGroupName: string, name: string, schedule: models.Schedule, callback: ServiceCallback<models.Schedule>): void;
    createOrUpdate(resourceGroupName: string, name: string, schedule: models.Schedule, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Delete schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of schedules.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, name: string, schedule: models.ScheduleFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Modify properties of schedules.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, name: string, schedule: models.ScheduleFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    update(resourceGroupName: string, name: string, schedule: models.ScheduleFragment, callback: ServiceCallback<models.Schedule>): void;
    update(resourceGroupName: string, name: string, schedule: models.ScheduleFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    executeWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    execute(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    execute(resourceGroupName: string, name: string, callback: ServiceCallback<void>): void;
    execute(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a schedule's target resource Id. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} retargetScheduleProperties Properties for retargeting a
     * virtual machine schedule.
     *
     * @param {string} [retargetScheduleProperties.currentResourceId] The resource
     * Id of the virtual machine on which the schedule operates
     *
     * @param {string} [retargetScheduleProperties.targetResourceId] The resource
     * Id of the virtual machine that the schedule should be retargeted to
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    retargetWithHttpOperationResponse(resourceGroupName: string, name: string, retargetScheduleProperties: models.RetargetScheduleProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Updates a schedule's target resource Id. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} retargetScheduleProperties Properties for retargeting a
     * virtual machine schedule.
     *
     * @param {string} [retargetScheduleProperties.currentResourceId] The resource
     * Id of the virtual machine on which the schedule operates
     *
     * @param {string} [retargetScheduleProperties.targetResourceId] The resource
     * Id of the virtual machine that the schedule should be retargeted to
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    retarget(resourceGroupName: string, name: string, retargetScheduleProperties: models.RetargetScheduleProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    retarget(resourceGroupName: string, name: string, retargetScheduleProperties: models.RetargetScheduleProperties, callback: ServiceCallback<void>): void;
    retarget(resourceGroupName: string, name: string, retargetScheduleProperties: models.RetargetScheduleProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginExecuteWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginExecute(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginExecute(resourceGroupName: string, name: string, callback: ServiceCallback<void>): void;
    beginExecute(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a schedule's target resource Id. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} retargetScheduleProperties Properties for retargeting a
     * virtual machine schedule.
     *
     * @param {string} [retargetScheduleProperties.currentResourceId] The resource
     * Id of the virtual machine on which the schedule operates
     *
     * @param {string} [retargetScheduleProperties.targetResourceId] The resource
     * Id of the virtual machine that the schedule should be retargeted to
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginRetargetWithHttpOperationResponse(resourceGroupName: string, name: string, retargetScheduleProperties: models.RetargetScheduleProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Updates a schedule's target resource Id. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} retargetScheduleProperties Properties for retargeting a
     * virtual machine schedule.
     *
     * @param {string} [retargetScheduleProperties.currentResourceId] The resource
     * Id of the virtual machine on which the schedule operates
     *
     * @param {string} [retargetScheduleProperties.targetResourceId] The resource
     * Id of the virtual machine that the schedule should be retargeted to
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginRetarget(resourceGroupName: string, name: string, retargetScheduleProperties: models.RetargetScheduleProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginRetarget(resourceGroupName: string, name: string, retargetScheduleProperties: models.RetargetScheduleProperties, callback: ServiceCallback<void>): void;
    beginRetarget(resourceGroupName: string, name: string, retargetScheduleProperties: models.RetargetScheduleProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List schedules in a subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscriptionNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    listBySubscriptionNext(nextPageLink: string, callback: ServiceCallback<models.ScheduleList>): void;
    listBySubscriptionNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;


    /**
     * List schedules in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    listByResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.ScheduleList>): void;
    listByResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;
}

/**
 * @class
 * ArtifactSources
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface ArtifactSources {


    /**
     * List artifact sources in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=displayName)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArtifactSourceList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArtifactSourceList>>;

    /**
     * List artifact sources in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=displayName)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArtifactSourceList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArtifactSourceList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArtifactSourceList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ArtifactSourceList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.ArtifactSourceList>): void;
    list(resourceGroupName: string, labName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArtifactSourceList>): void;


    /**
     * Get artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the artifact source.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=displayName)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArtifactSource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArtifactSource>>;

    /**
     * Get artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the artifact source.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=displayName)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArtifactSource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArtifactSource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArtifactSource} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ArtifactSource>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.ArtifactSource>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArtifactSource>): void;


    /**
     * Create or replace an existing artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the artifact source.
     *
     * @param {object} artifactSource Properties of an artifact source.
     *
     * @param {string} [artifactSource.displayName] The artifact source's display
     * name.
     *
     * @param {string} [artifactSource.uri] The artifact source's URI.
     *
     * @param {string} [artifactSource.sourceType] The artifact source's type.
     * Possible values include: 'VsoGit', 'GitHub'
     *
     * @param {string} [artifactSource.folderPath] The folder containing artifacts.
     *
     * @param {string} [artifactSource.armTemplateFolderPath] The folder containing
     * Azure Resource Manager templates.
     *
     * @param {string} [artifactSource.branchRef] The artifact source's branch
     * reference.
     *
     * @param {string} [artifactSource.securityToken] The security token to
     * authenticate to the artifact source.
     *
     * @param {string} [artifactSource.status] Indicates if the artifact source is
     * enabled (values: Enabled, Disabled). Possible values include: 'Enabled',
     * 'Disabled'
     *
     * @param {string} [artifactSource.location] The location of the resource.
     *
     * @param {object} [artifactSource.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArtifactSource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, artifactSource: models.ArtifactSource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArtifactSource>>;

    /**
     * Create or replace an existing artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the artifact source.
     *
     * @param {object} artifactSource Properties of an artifact source.
     *
     * @param {string} [artifactSource.displayName] The artifact source's display
     * name.
     *
     * @param {string} [artifactSource.uri] The artifact source's URI.
     *
     * @param {string} [artifactSource.sourceType] The artifact source's type.
     * Possible values include: 'VsoGit', 'GitHub'
     *
     * @param {string} [artifactSource.folderPath] The folder containing artifacts.
     *
     * @param {string} [artifactSource.armTemplateFolderPath] The folder containing
     * Azure Resource Manager templates.
     *
     * @param {string} [artifactSource.branchRef] The artifact source's branch
     * reference.
     *
     * @param {string} [artifactSource.securityToken] The security token to
     * authenticate to the artifact source.
     *
     * @param {string} [artifactSource.status] Indicates if the artifact source is
     * enabled (values: Enabled, Disabled). Possible values include: 'Enabled',
     * 'Disabled'
     *
     * @param {string} [artifactSource.location] The location of the resource.
     *
     * @param {object} [artifactSource.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArtifactSource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArtifactSource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArtifactSource} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, artifactSource: models.ArtifactSource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ArtifactSource>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, artifactSource: models.ArtifactSource, callback: ServiceCallback<models.ArtifactSource>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, artifactSource: models.ArtifactSource, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArtifactSource>): void;


    /**
     * Delete artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the artifact source.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the artifact source.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of artifact sources.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the artifact source.
     *
     * @param {object} artifactSource Properties of an artifact source.
     *
     * @param {string} [artifactSource.displayName] The artifact source's display
     * name.
     *
     * @param {string} [artifactSource.uri] The artifact source's URI.
     *
     * @param {string} [artifactSource.sourceType] The artifact source's type.
     * Possible values include: 'VsoGit', 'GitHub'
     *
     * @param {string} [artifactSource.folderPath] The folder containing artifacts.
     *
     * @param {string} [artifactSource.armTemplateFolderPath] The folder containing
     * Azure Resource Manager templates.
     *
     * @param {string} [artifactSource.branchRef] The artifact source's branch
     * reference.
     *
     * @param {string} [artifactSource.securityToken] The security token to
     * authenticate to the artifact source.
     *
     * @param {string} [artifactSource.status] Indicates if the artifact source is
     * enabled (values: Enabled, Disabled). Possible values include: 'Enabled',
     * 'Disabled'
     *
     * @param {object} [artifactSource.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArtifactSource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, artifactSource: models.ArtifactSourceFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArtifactSource>>;

    /**
     * Modify properties of artifact sources.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the artifact source.
     *
     * @param {object} artifactSource Properties of an artifact source.
     *
     * @param {string} [artifactSource.displayName] The artifact source's display
     * name.
     *
     * @param {string} [artifactSource.uri] The artifact source's URI.
     *
     * @param {string} [artifactSource.sourceType] The artifact source's type.
     * Possible values include: 'VsoGit', 'GitHub'
     *
     * @param {string} [artifactSource.folderPath] The folder containing artifacts.
     *
     * @param {string} [artifactSource.armTemplateFolderPath] The folder containing
     * Azure Resource Manager templates.
     *
     * @param {string} [artifactSource.branchRef] The artifact source's branch
     * reference.
     *
     * @param {string} [artifactSource.securityToken] The security token to
     * authenticate to the artifact source.
     *
     * @param {string} [artifactSource.status] Indicates if the artifact source is
     * enabled (values: Enabled, Disabled). Possible values include: 'Enabled',
     * 'Disabled'
     *
     * @param {object} [artifactSource.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArtifactSource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArtifactSource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArtifactSource} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, name: string, artifactSource: models.ArtifactSourceFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ArtifactSource>;
    update(resourceGroupName: string, labName: string, name: string, artifactSource: models.ArtifactSourceFragment, callback: ServiceCallback<models.ArtifactSource>): void;
    update(resourceGroupName: string, labName: string, name: string, artifactSource: models.ArtifactSourceFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArtifactSource>): void;


    /**
     * List artifact sources in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArtifactSourceList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArtifactSourceList>>;

    /**
     * List artifact sources in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArtifactSourceList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArtifactSourceList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArtifactSourceList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ArtifactSourceList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ArtifactSourceList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArtifactSourceList>): void;
}

/**
 * @class
 * ArmTemplates
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface ArmTemplates {


    /**
     * List azure resource manager templates in a given artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=displayName)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArmTemplateList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, artifactSourceName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArmTemplateList>>;

    /**
     * List azure resource manager templates in a given artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=displayName)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArmTemplateList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArmTemplateList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArmTemplateList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, artifactSourceName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ArmTemplateList>;
    list(resourceGroupName: string, labName: string, artifactSourceName: string, callback: ServiceCallback<models.ArmTemplateList>): void;
    list(resourceGroupName: string, labName: string, artifactSourceName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArmTemplateList>): void;


    /**
     * Get azure resource manager template.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {string} name The name of the azure Resource Manager template.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=displayName)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArmTemplate>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArmTemplate>>;

    /**
     * Get azure resource manager template.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {string} name The name of the azure Resource Manager template.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=displayName)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArmTemplate} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArmTemplate} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArmTemplate} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ArmTemplate>;
    get(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, callback: ServiceCallback<models.ArmTemplate>): void;
    get(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArmTemplate>): void;


    /**
     * List azure resource manager templates in a given artifact source.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArmTemplateList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArmTemplateList>>;

    /**
     * List azure resource manager templates in a given artifact source.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArmTemplateList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArmTemplateList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArmTemplateList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ArmTemplateList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ArmTemplateList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArmTemplateList>): void;
}

/**
 * @class
 * Artifacts
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Artifacts {


    /**
     * List artifacts in a given artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=title)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArtifactList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, artifactSourceName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArtifactList>>;

    /**
     * List artifacts in a given artifact source.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=title)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArtifactList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArtifactList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArtifactList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, artifactSourceName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ArtifactList>;
    list(resourceGroupName: string, labName: string, artifactSourceName: string, callback: ServiceCallback<models.ArtifactList>): void;
    list(resourceGroupName: string, labName: string, artifactSourceName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArtifactList>): void;


    /**
     * Get artifact.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {string} name The name of the artifact.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=title)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Artifact>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Artifact>>;

    /**
     * Get artifact.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {string} name The name of the artifact.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=title)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Artifact} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Artifact} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Artifact} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Artifact>;
    get(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, callback: ServiceCallback<models.Artifact>): void;
    get(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Artifact>): void;


    /**
     * Generates an ARM template for the given artifact, uploads the required files
     * to a storage account, and validates the generated artifact.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {string} name The name of the artifact.
     *
     * @param {object} generateArmTemplateRequest Parameters for generating an ARM
     * template for deploying artifacts.
     *
     * @param {string} [generateArmTemplateRequest.virtualMachineName] The resource
     * name of the virtual machine.
     *
     * @param {array} [generateArmTemplateRequest.parameters] The parameters of the
     * ARM template.
     *
     * @param {string} [generateArmTemplateRequest.location] The location of the
     * virtual machine.
     *
     * @param {string} [generateArmTemplateRequest.fileUploadOptions] Options for
     * uploading the files for the artifact. UploadFilesAndGenerateSasTokens is the
     * default value. Possible values include: 'UploadFilesAndGenerateSasTokens',
     * 'None'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArmTemplateInfo>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    generateArmTemplateWithHttpOperationResponse(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, generateArmTemplateRequest: models.GenerateArmTemplateRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArmTemplateInfo>>;

    /**
     * Generates an ARM template for the given artifact, uploads the required files
     * to a storage account, and validates the generated artifact.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} artifactSourceName The name of the artifact source.
     *
     * @param {string} name The name of the artifact.
     *
     * @param {object} generateArmTemplateRequest Parameters for generating an ARM
     * template for deploying artifacts.
     *
     * @param {string} [generateArmTemplateRequest.virtualMachineName] The resource
     * name of the virtual machine.
     *
     * @param {array} [generateArmTemplateRequest.parameters] The parameters of the
     * ARM template.
     *
     * @param {string} [generateArmTemplateRequest.location] The location of the
     * virtual machine.
     *
     * @param {string} [generateArmTemplateRequest.fileUploadOptions] Options for
     * uploading the files for the artifact. UploadFilesAndGenerateSasTokens is the
     * default value. Possible values include: 'UploadFilesAndGenerateSasTokens',
     * 'None'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArmTemplateInfo} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArmTemplateInfo} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArmTemplateInfo} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    generateArmTemplate(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, generateArmTemplateRequest: models.GenerateArmTemplateRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ArmTemplateInfo>;
    generateArmTemplate(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, generateArmTemplateRequest: models.GenerateArmTemplateRequest, callback: ServiceCallback<models.ArmTemplateInfo>): void;
    generateArmTemplate(resourceGroupName: string, labName: string, artifactSourceName: string, name: string, generateArmTemplateRequest: models.GenerateArmTemplateRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArmTemplateInfo>): void;


    /**
     * List artifacts in a given artifact source.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ArtifactList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ArtifactList>>;

    /**
     * List artifacts in a given artifact source.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ArtifactList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ArtifactList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ArtifactList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ArtifactList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ArtifactList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ArtifactList>): void;
}

/**
 * @class
 * Costs
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Costs {


    /**
     * Get cost.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the cost.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=labCostDetails)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabCost>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabCost>>;

    /**
     * Get cost.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the cost.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=labCostDetails)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabCost} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabCost} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabCost} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.LabCost>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.LabCost>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabCost>): void;


    /**
     * Create or replace an existing cost.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the cost.
     *
     * @param {object} labCost A cost item.
     *
     * @param {object} [labCost.targetCost] The target cost properties
     *
     * @param {string} [labCost.targetCost.status] Target cost status. Possible
     * values include: 'Enabled', 'Disabled'
     *
     * @param {number} [labCost.targetCost.target] Lab target cost
     *
     * @param {array} [labCost.targetCost.costThresholds] Cost thresholds.
     *
     * @param {date} [labCost.targetCost.cycleStartDateTime] Reporting cycle start
     * date.
     *
     * @param {date} [labCost.targetCost.cycleEndDateTime] Reporting cycle end
     * date.
     *
     * @param {string} [labCost.targetCost.cycleType] Reporting cycle type.
     * Possible values include: 'CalendarMonth', 'Custom'
     *
     * @param {string} [labCost.currencyCode] The currency code of the cost.
     *
     * @param {date} [labCost.startDateTime] The start time of the cost data.
     *
     * @param {date} [labCost.endDateTime] The end time of the cost data.
     *
     * @param {date} [labCost.createdDate] The creation date of the cost.
     *
     * @param {string} [labCost.location] The location of the resource.
     *
     * @param {object} [labCost.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabCost>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, labCost: models.LabCost, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabCost>>;

    /**
     * Create or replace an existing cost.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the cost.
     *
     * @param {object} labCost A cost item.
     *
     * @param {object} [labCost.targetCost] The target cost properties
     *
     * @param {string} [labCost.targetCost.status] Target cost status. Possible
     * values include: 'Enabled', 'Disabled'
     *
     * @param {number} [labCost.targetCost.target] Lab target cost
     *
     * @param {array} [labCost.targetCost.costThresholds] Cost thresholds.
     *
     * @param {date} [labCost.targetCost.cycleStartDateTime] Reporting cycle start
     * date.
     *
     * @param {date} [labCost.targetCost.cycleEndDateTime] Reporting cycle end
     * date.
     *
     * @param {string} [labCost.targetCost.cycleType] Reporting cycle type.
     * Possible values include: 'CalendarMonth', 'Custom'
     *
     * @param {string} [labCost.currencyCode] The currency code of the cost.
     *
     * @param {date} [labCost.startDateTime] The start time of the cost data.
     *
     * @param {date} [labCost.endDateTime] The end time of the cost data.
     *
     * @param {date} [labCost.createdDate] The creation date of the cost.
     *
     * @param {string} [labCost.location] The location of the resource.
     *
     * @param {object} [labCost.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabCost} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabCost} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabCost} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, labCost: models.LabCost, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.LabCost>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, labCost: models.LabCost, callback: ServiceCallback<models.LabCost>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, labCost: models.LabCost, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabCost>): void;
}

/**
 * @class
 * CustomImages
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface CustomImages {


    /**
     * List custom images in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=vm)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CustomImageList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CustomImageList>>;

    /**
     * List custom images in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=vm)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CustomImageList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CustomImageList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CustomImageList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.CustomImageList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.CustomImageList>): void;
    list(resourceGroupName: string, labName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CustomImageList>): void;


    /**
     * Get custom image.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=vm)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CustomImage>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CustomImage>>;

    /**
     * Get custom image.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=vm)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CustomImage} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CustomImage} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CustomImage} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.CustomImage>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.CustomImage>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CustomImage>): void;


    /**
     * Create or replace an existing custom image. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} customImage A custom image.
     *
     * @param {object} [customImage.vm] The virtual machine from which the image is
     * to be created.
     *
     * @param {string} [customImage.vm.sourceVmId] The source vm identifier.
     *
     * @param {object} [customImage.vm.windowsOsInfo] The Windows OS information of
     * the VM.
     *
     * @param {string} [customImage.vm.windowsOsInfo.windowsOsState] The state of
     * the Windows OS (i.e. NonSysprepped, SysprepRequested, SysprepApplied).
     * Possible values include: 'NonSysprepped', 'SysprepRequested',
     * 'SysprepApplied'
     *
     * @param {object} [customImage.vm.linuxOsInfo] The Linux OS information of the
     * VM.
     *
     * @param {string} [customImage.vm.linuxOsInfo.linuxOsState] The state of the
     * Linux OS (i.e. NonDeprovisioned, DeprovisionRequested, DeprovisionApplied).
     * Possible values include: 'NonDeprovisioned', 'DeprovisionRequested',
     * 'DeprovisionApplied'
     *
     * @param {object} [customImage.vhd] The VHD from which the image is to be
     * created.
     *
     * @param {string} [customImage.vhd.imageName] The image name.
     *
     * @param {boolean} [customImage.vhd.sysPrep] Indicates whether sysprep has
     * been run on the VHD.
     *
     * @param {string} customImage.vhd.osType The OS type of the custom image (i.e.
     * Windows, Linux). Possible values include: 'Windows', 'Linux', 'None'
     *
     * @param {string} [customImage.description] The description of the custom
     * image.
     *
     * @param {string} [customImage.author] The author of the custom image.
     *
     * @param {string} [customImage.managedImageId] The Managed Image Id backing
     * the custom image.
     *
     * @param {string} [customImage.managedSnapshotId] The Managed Snapshot Id
     * backing the custom image.
     *
     * @param {array} [customImage.dataDiskStorageInfo] Storage information about
     * the data disks present in the custom image
     *
     * @param {object} [customImage.customImagePlan] Storage information about the
     * plan related to this custom image
     *
     * @param {string} [customImage.customImagePlan.id] The id of the plan,
     * equivalent to name of the plan
     *
     * @param {string} [customImage.customImagePlan.publisher] The publisher for
     * the plan from the marketplace image the custom image is derived from
     *
     * @param {string} [customImage.customImagePlan.offer] The offer for the plan
     * from the marketplace image the custom image is derived from
     *
     * @param {boolean} [customImage.isPlanAuthorized] Whether or not the custom
     * images underlying offer/plan has been enabled for programmatic deployment
     *
     * @param {string} [customImage.location] The location of the resource.
     *
     * @param {object} [customImage.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CustomImage>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImage, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CustomImage>>;

    /**
     * Create or replace an existing custom image. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} customImage A custom image.
     *
     * @param {object} [customImage.vm] The virtual machine from which the image is
     * to be created.
     *
     * @param {string} [customImage.vm.sourceVmId] The source vm identifier.
     *
     * @param {object} [customImage.vm.windowsOsInfo] The Windows OS information of
     * the VM.
     *
     * @param {string} [customImage.vm.windowsOsInfo.windowsOsState] The state of
     * the Windows OS (i.e. NonSysprepped, SysprepRequested, SysprepApplied).
     * Possible values include: 'NonSysprepped', 'SysprepRequested',
     * 'SysprepApplied'
     *
     * @param {object} [customImage.vm.linuxOsInfo] The Linux OS information of the
     * VM.
     *
     * @param {string} [customImage.vm.linuxOsInfo.linuxOsState] The state of the
     * Linux OS (i.e. NonDeprovisioned, DeprovisionRequested, DeprovisionApplied).
     * Possible values include: 'NonDeprovisioned', 'DeprovisionRequested',
     * 'DeprovisionApplied'
     *
     * @param {object} [customImage.vhd] The VHD from which the image is to be
     * created.
     *
     * @param {string} [customImage.vhd.imageName] The image name.
     *
     * @param {boolean} [customImage.vhd.sysPrep] Indicates whether sysprep has
     * been run on the VHD.
     *
     * @param {string} customImage.vhd.osType The OS type of the custom image (i.e.
     * Windows, Linux). Possible values include: 'Windows', 'Linux', 'None'
     *
     * @param {string} [customImage.description] The description of the custom
     * image.
     *
     * @param {string} [customImage.author] The author of the custom image.
     *
     * @param {string} [customImage.managedImageId] The Managed Image Id backing
     * the custom image.
     *
     * @param {string} [customImage.managedSnapshotId] The Managed Snapshot Id
     * backing the custom image.
     *
     * @param {array} [customImage.dataDiskStorageInfo] Storage information about
     * the data disks present in the custom image
     *
     * @param {object} [customImage.customImagePlan] Storage information about the
     * plan related to this custom image
     *
     * @param {string} [customImage.customImagePlan.id] The id of the plan,
     * equivalent to name of the plan
     *
     * @param {string} [customImage.customImagePlan.publisher] The publisher for
     * the plan from the marketplace image the custom image is derived from
     *
     * @param {string} [customImage.customImagePlan.offer] The offer for the plan
     * from the marketplace image the custom image is derived from
     *
     * @param {boolean} [customImage.isPlanAuthorized] Whether or not the custom
     * images underlying offer/plan has been enabled for programmatic deployment
     *
     * @param {string} [customImage.location] The location of the resource.
     *
     * @param {object} [customImage.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CustomImage} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CustomImage} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CustomImage} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImage, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CustomImage>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImage, callback: ServiceCallback<models.CustomImage>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImage, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CustomImage>): void;


    /**
     * Delete custom image. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete custom image. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of custom images.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} customImage A custom image.
     *
     * @param {object} [customImage.vm] The virtual machine from which the image is
     * to be created.
     *
     * @param {string} [customImage.vm.sourceVmId] The source vm identifier.
     *
     * @param {object} [customImage.vm.windowsOsInfo] The Windows OS information of
     * the VM.
     *
     * @param {string} [customImage.vm.windowsOsInfo.windowsOsState] The state of
     * the Windows OS (i.e. NonSysprepped, SysprepRequested, SysprepApplied).
     * Possible values include: 'NonSysprepped', 'SysprepRequested',
     * 'SysprepApplied'
     *
     * @param {object} [customImage.vm.linuxOsInfo] The Linux OS information of the
     * VM.
     *
     * @param {string} [customImage.vm.linuxOsInfo.linuxOsState] The state of the
     * Linux OS (i.e. NonDeprovisioned, DeprovisionRequested, DeprovisionApplied).
     * Possible values include: 'NonDeprovisioned', 'DeprovisionRequested',
     * 'DeprovisionApplied'
     *
     * @param {object} [customImage.vhd] The VHD from which the image is to be
     * created.
     *
     * @param {string} [customImage.vhd.imageName] The image name.
     *
     * @param {boolean} [customImage.vhd.sysPrep] Indicates whether sysprep has
     * been run on the VHD.
     *
     * @param {string} [customImage.vhd.osType] The OS type of the custom image
     * (i.e. Windows, Linux). Possible values include: 'Windows', 'Linux', 'None'
     *
     * @param {string} [customImage.description] The description of the custom
     * image.
     *
     * @param {string} [customImage.author] The author of the custom image.
     *
     * @param {string} [customImage.managedImageId] The Managed Image Id backing
     * the custom image.
     *
     * @param {string} [customImage.managedSnapshotId] The Managed Snapshot Id
     * backing the custom image.
     *
     * @param {array} [customImage.dataDiskStorageInfo] Storage information about
     * the data disks present in the custom image
     *
     * @param {object} [customImage.customImagePlan] Storage information about the
     * plan related to this custom image
     *
     * @param {string} [customImage.customImagePlan.id] The id of the plan,
     * equivalent to name of the plan
     *
     * @param {string} [customImage.customImagePlan.publisher] The publisher for
     * the plan from the marketplace image the custom image is derived from
     *
     * @param {string} [customImage.customImagePlan.offer] The offer for the plan
     * from the marketplace image the custom image is derived from
     *
     * @param {boolean} [customImage.isPlanAuthorized] Whether or not the custom
     * images underlying offer/plan has been enabled for programmatic deployment
     *
     * @param {object} [customImage.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CustomImage>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImageFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CustomImage>>;

    /**
     * Modify properties of custom images.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} customImage A custom image.
     *
     * @param {object} [customImage.vm] The virtual machine from which the image is
     * to be created.
     *
     * @param {string} [customImage.vm.sourceVmId] The source vm identifier.
     *
     * @param {object} [customImage.vm.windowsOsInfo] The Windows OS information of
     * the VM.
     *
     * @param {string} [customImage.vm.windowsOsInfo.windowsOsState] The state of
     * the Windows OS (i.e. NonSysprepped, SysprepRequested, SysprepApplied).
     * Possible values include: 'NonSysprepped', 'SysprepRequested',
     * 'SysprepApplied'
     *
     * @param {object} [customImage.vm.linuxOsInfo] The Linux OS information of the
     * VM.
     *
     * @param {string} [customImage.vm.linuxOsInfo.linuxOsState] The state of the
     * Linux OS (i.e. NonDeprovisioned, DeprovisionRequested, DeprovisionApplied).
     * Possible values include: 'NonDeprovisioned', 'DeprovisionRequested',
     * 'DeprovisionApplied'
     *
     * @param {object} [customImage.vhd] The VHD from which the image is to be
     * created.
     *
     * @param {string} [customImage.vhd.imageName] The image name.
     *
     * @param {boolean} [customImage.vhd.sysPrep] Indicates whether sysprep has
     * been run on the VHD.
     *
     * @param {string} [customImage.vhd.osType] The OS type of the custom image
     * (i.e. Windows, Linux). Possible values include: 'Windows', 'Linux', 'None'
     *
     * @param {string} [customImage.description] The description of the custom
     * image.
     *
     * @param {string} [customImage.author] The author of the custom image.
     *
     * @param {string} [customImage.managedImageId] The Managed Image Id backing
     * the custom image.
     *
     * @param {string} [customImage.managedSnapshotId] The Managed Snapshot Id
     * backing the custom image.
     *
     * @param {array} [customImage.dataDiskStorageInfo] Storage information about
     * the data disks present in the custom image
     *
     * @param {object} [customImage.customImagePlan] Storage information about the
     * plan related to this custom image
     *
     * @param {string} [customImage.customImagePlan.id] The id of the plan,
     * equivalent to name of the plan
     *
     * @param {string} [customImage.customImagePlan.publisher] The publisher for
     * the plan from the marketplace image the custom image is derived from
     *
     * @param {string} [customImage.customImagePlan.offer] The offer for the plan
     * from the marketplace image the custom image is derived from
     *
     * @param {boolean} [customImage.isPlanAuthorized] Whether or not the custom
     * images underlying offer/plan has been enabled for programmatic deployment
     *
     * @param {object} [customImage.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CustomImage} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CustomImage} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CustomImage} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImageFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CustomImage>;
    update(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImageFragment, callback: ServiceCallback<models.CustomImage>): void;
    update(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImageFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CustomImage>): void;


    /**
     * Create or replace an existing custom image. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} customImage A custom image.
     *
     * @param {object} [customImage.vm] The virtual machine from which the image is
     * to be created.
     *
     * @param {string} [customImage.vm.sourceVmId] The source vm identifier.
     *
     * @param {object} [customImage.vm.windowsOsInfo] The Windows OS information of
     * the VM.
     *
     * @param {string} [customImage.vm.windowsOsInfo.windowsOsState] The state of
     * the Windows OS (i.e. NonSysprepped, SysprepRequested, SysprepApplied).
     * Possible values include: 'NonSysprepped', 'SysprepRequested',
     * 'SysprepApplied'
     *
     * @param {object} [customImage.vm.linuxOsInfo] The Linux OS information of the
     * VM.
     *
     * @param {string} [customImage.vm.linuxOsInfo.linuxOsState] The state of the
     * Linux OS (i.e. NonDeprovisioned, DeprovisionRequested, DeprovisionApplied).
     * Possible values include: 'NonDeprovisioned', 'DeprovisionRequested',
     * 'DeprovisionApplied'
     *
     * @param {object} [customImage.vhd] The VHD from which the image is to be
     * created.
     *
     * @param {string} [customImage.vhd.imageName] The image name.
     *
     * @param {boolean} [customImage.vhd.sysPrep] Indicates whether sysprep has
     * been run on the VHD.
     *
     * @param {string} customImage.vhd.osType The OS type of the custom image (i.e.
     * Windows, Linux). Possible values include: 'Windows', 'Linux', 'None'
     *
     * @param {string} [customImage.description] The description of the custom
     * image.
     *
     * @param {string} [customImage.author] The author of the custom image.
     *
     * @param {string} [customImage.managedImageId] The Managed Image Id backing
     * the custom image.
     *
     * @param {string} [customImage.managedSnapshotId] The Managed Snapshot Id
     * backing the custom image.
     *
     * @param {array} [customImage.dataDiskStorageInfo] Storage information about
     * the data disks present in the custom image
     *
     * @param {object} [customImage.customImagePlan] Storage information about the
     * plan related to this custom image
     *
     * @param {string} [customImage.customImagePlan.id] The id of the plan,
     * equivalent to name of the plan
     *
     * @param {string} [customImage.customImagePlan.publisher] The publisher for
     * the plan from the marketplace image the custom image is derived from
     *
     * @param {string} [customImage.customImagePlan.offer] The offer for the plan
     * from the marketplace image the custom image is derived from
     *
     * @param {boolean} [customImage.isPlanAuthorized] Whether or not the custom
     * images underlying offer/plan has been enabled for programmatic deployment
     *
     * @param {string} [customImage.location] The location of the resource.
     *
     * @param {object} [customImage.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CustomImage>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImage, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CustomImage>>;

    /**
     * Create or replace an existing custom image. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} customImage A custom image.
     *
     * @param {object} [customImage.vm] The virtual machine from which the image is
     * to be created.
     *
     * @param {string} [customImage.vm.sourceVmId] The source vm identifier.
     *
     * @param {object} [customImage.vm.windowsOsInfo] The Windows OS information of
     * the VM.
     *
     * @param {string} [customImage.vm.windowsOsInfo.windowsOsState] The state of
     * the Windows OS (i.e. NonSysprepped, SysprepRequested, SysprepApplied).
     * Possible values include: 'NonSysprepped', 'SysprepRequested',
     * 'SysprepApplied'
     *
     * @param {object} [customImage.vm.linuxOsInfo] The Linux OS information of the
     * VM.
     *
     * @param {string} [customImage.vm.linuxOsInfo.linuxOsState] The state of the
     * Linux OS (i.e. NonDeprovisioned, DeprovisionRequested, DeprovisionApplied).
     * Possible values include: 'NonDeprovisioned', 'DeprovisionRequested',
     * 'DeprovisionApplied'
     *
     * @param {object} [customImage.vhd] The VHD from which the image is to be
     * created.
     *
     * @param {string} [customImage.vhd.imageName] The image name.
     *
     * @param {boolean} [customImage.vhd.sysPrep] Indicates whether sysprep has
     * been run on the VHD.
     *
     * @param {string} customImage.vhd.osType The OS type of the custom image (i.e.
     * Windows, Linux). Possible values include: 'Windows', 'Linux', 'None'
     *
     * @param {string} [customImage.description] The description of the custom
     * image.
     *
     * @param {string} [customImage.author] The author of the custom image.
     *
     * @param {string} [customImage.managedImageId] The Managed Image Id backing
     * the custom image.
     *
     * @param {string} [customImage.managedSnapshotId] The Managed Snapshot Id
     * backing the custom image.
     *
     * @param {array} [customImage.dataDiskStorageInfo] Storage information about
     * the data disks present in the custom image
     *
     * @param {object} [customImage.customImagePlan] Storage information about the
     * plan related to this custom image
     *
     * @param {string} [customImage.customImagePlan.id] The id of the plan,
     * equivalent to name of the plan
     *
     * @param {string} [customImage.customImagePlan.publisher] The publisher for
     * the plan from the marketplace image the custom image is derived from
     *
     * @param {string} [customImage.customImagePlan.offer] The offer for the plan
     * from the marketplace image the custom image is derived from
     *
     * @param {boolean} [customImage.isPlanAuthorized] Whether or not the custom
     * images underlying offer/plan has been enabled for programmatic deployment
     *
     * @param {string} [customImage.location] The location of the resource.
     *
     * @param {object} [customImage.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CustomImage} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CustomImage} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CustomImage} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImage, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CustomImage>;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImage, callback: ServiceCallback<models.CustomImage>): void;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, customImage: models.CustomImage, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CustomImage>): void;


    /**
     * Delete custom image. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete custom image. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the custom image.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List custom images in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CustomImageList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CustomImageList>>;

    /**
     * List custom images in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CustomImageList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CustomImageList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CustomImageList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CustomImageList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.CustomImageList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CustomImageList>): void;
}

/**
 * @class
 * Formulas
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Formulas {


    /**
     * List formulas in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=description)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<FormulaList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.FormulaList>>;

    /**
     * List formulas in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=description)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {FormulaList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {FormulaList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link FormulaList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.FormulaList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.FormulaList>): void;
    list(resourceGroupName: string, labName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.FormulaList>): void;


    /**
     * Get formula.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=description)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Formula>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Formula>>;

    /**
     * Get formula.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=description)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Formula} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Formula} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Formula} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Formula>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.Formula>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Formula>): void;


    /**
     * Create or replace an existing Formula. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} formula A formula for creating a VM, specifying an image
     * base and other parameters
     *
     * @param {string} [formula.description] The description of the formula.
     *
     * @param {string} [formula.author] The author of the formula.
     *
     * @param {string} [formula.osType] The OS type of the formula.
     *
     * @param {object} [formula.formulaContent] The content of the formula.
     *
     * @param {object} [formula.formulaContent.bulkCreationParameters] The number
     * of virtual machine instances to create.
     *
     * @param {number}
     * [formula.formulaContent.bulkCreationParameters.instanceCount] The number of
     * virtual machine instances to create.
     *
     * @param {string} [formula.formulaContent.notes] The notes of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.ownerObjectId] The object identifier
     * of the owner of the virtual machine.
     *
     * @param {string} [formula.formulaContent.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [formula.formulaContent.createdByUserId] The object
     * identifier of the creator of the virtual machine.
     *
     * @param {string} [formula.formulaContent.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [formula.formulaContent.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [formula.formulaContent.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [formula.formulaContent.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.size] The size of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.userName] The user name of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.password] The password of the
     * virtual machine administrator.
     *
     * @param {string} [formula.formulaContent.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [formula.formulaContent.isAuthenticationWithSshKey]
     * Indicates whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [formula.formulaContent.fqdn] The fully-qualified domain
     * name of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labSubnetName] The lab subnet name
     * of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [formula.formulaContent.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [formula.formulaContent.artifacts] The artifacts to be
     * installed on the virtual machine.
     *
     * @param {object} [formula.formulaContent.artifactDeploymentStatus] The
     * artifact deployment status for the virtual machine.
     *
     * @param {string}
     * [formula.formulaContent.artifactDeploymentStatus.deploymentStatus] The
     * deployment status of the artifact.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.totalArtifacts] The total
     * count of the artifacts that were tentatively applied.
     *
     * @param {object} [formula.formulaContent.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.offer] The
     * offer of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.sku] The SKU
     * of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [formula.formulaContent.planId] The id of the plan
     * associated with the virtual machine image
     *
     * @param {object} [formula.formulaContent.networkInterface] The network
     * interface properties.
     *
     * @param {string} [formula.formulaContent.networkInterface.virtualNetworkId]
     * The resource ID of the virtual network.
     *
     * @param {string} [formula.formulaContent.networkInterface.subnetId] The
     * resource ID of the sub net.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddressId]
     * The resource ID of the public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddress]
     * The public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.privateIpAddress]
     * The private IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.dnsName] The DNS
     * name.
     *
     * @param {string} [formula.formulaContent.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [formula.formulaContent.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [formula.formulaContent.expirationDate] The expiration date
     * for VM.
     *
     * @param {boolean} [formula.formulaContent.allowClaim] Indicates whether
     * another user can take ownership of the virtual machine
     *
     * @param {string} [formula.formulaContent.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [formula.formulaContent.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [formula.formulaContent.environmentId] The resource ID of
     * the environment that contains this virtual machine, if any.
     *
     * @param {array} [formula.formulaContent.dataDiskParameters] New or existing
     * data disks to attach to the virtual machine after creation
     *
     * @param {array} [formula.formulaContent.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [formula.formulaContent.lastKnownPowerState] Last known
     * compute power state captured in DTL
     *
     * @param {string} [formula.formulaContent.name] The name of the virtual
     * machine or environment
     *
     * @param {string} [formula.formulaContent.location] The location of the new
     * virtual machine or environment
     *
     * @param {object} [formula.formulaContent.tags] The tags of the resource.
     *
     * @param {object} [formula.vm] Information about a VM from which a formula is
     * to be created.
     *
     * @param {string} [formula.vm.labVmId] The identifier of the VM from which a
     * formula is to be created.
     *
     * @param {string} [formula.location] The location of the resource.
     *
     * @param {object} [formula.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Formula>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, formula: models.Formula, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Formula>>;

    /**
     * Create or replace an existing Formula. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} formula A formula for creating a VM, specifying an image
     * base and other parameters
     *
     * @param {string} [formula.description] The description of the formula.
     *
     * @param {string} [formula.author] The author of the formula.
     *
     * @param {string} [formula.osType] The OS type of the formula.
     *
     * @param {object} [formula.formulaContent] The content of the formula.
     *
     * @param {object} [formula.formulaContent.bulkCreationParameters] The number
     * of virtual machine instances to create.
     *
     * @param {number}
     * [formula.formulaContent.bulkCreationParameters.instanceCount] The number of
     * virtual machine instances to create.
     *
     * @param {string} [formula.formulaContent.notes] The notes of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.ownerObjectId] The object identifier
     * of the owner of the virtual machine.
     *
     * @param {string} [formula.formulaContent.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [formula.formulaContent.createdByUserId] The object
     * identifier of the creator of the virtual machine.
     *
     * @param {string} [formula.formulaContent.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [formula.formulaContent.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [formula.formulaContent.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [formula.formulaContent.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.size] The size of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.userName] The user name of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.password] The password of the
     * virtual machine administrator.
     *
     * @param {string} [formula.formulaContent.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [formula.formulaContent.isAuthenticationWithSshKey]
     * Indicates whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [formula.formulaContent.fqdn] The fully-qualified domain
     * name of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labSubnetName] The lab subnet name
     * of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [formula.formulaContent.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [formula.formulaContent.artifacts] The artifacts to be
     * installed on the virtual machine.
     *
     * @param {object} [formula.formulaContent.artifactDeploymentStatus] The
     * artifact deployment status for the virtual machine.
     *
     * @param {string}
     * [formula.formulaContent.artifactDeploymentStatus.deploymentStatus] The
     * deployment status of the artifact.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.totalArtifacts] The total
     * count of the artifacts that were tentatively applied.
     *
     * @param {object} [formula.formulaContent.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.offer] The
     * offer of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.sku] The SKU
     * of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [formula.formulaContent.planId] The id of the plan
     * associated with the virtual machine image
     *
     * @param {object} [formula.formulaContent.networkInterface] The network
     * interface properties.
     *
     * @param {string} [formula.formulaContent.networkInterface.virtualNetworkId]
     * The resource ID of the virtual network.
     *
     * @param {string} [formula.formulaContent.networkInterface.subnetId] The
     * resource ID of the sub net.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddressId]
     * The resource ID of the public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddress]
     * The public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.privateIpAddress]
     * The private IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.dnsName] The DNS
     * name.
     *
     * @param {string} [formula.formulaContent.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [formula.formulaContent.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [formula.formulaContent.expirationDate] The expiration date
     * for VM.
     *
     * @param {boolean} [formula.formulaContent.allowClaim] Indicates whether
     * another user can take ownership of the virtual machine
     *
     * @param {string} [formula.formulaContent.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [formula.formulaContent.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [formula.formulaContent.environmentId] The resource ID of
     * the environment that contains this virtual machine, if any.
     *
     * @param {array} [formula.formulaContent.dataDiskParameters] New or existing
     * data disks to attach to the virtual machine after creation
     *
     * @param {array} [formula.formulaContent.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [formula.formulaContent.lastKnownPowerState] Last known
     * compute power state captured in DTL
     *
     * @param {string} [formula.formulaContent.name] The name of the virtual
     * machine or environment
     *
     * @param {string} [formula.formulaContent.location] The location of the new
     * virtual machine or environment
     *
     * @param {object} [formula.formulaContent.tags] The tags of the resource.
     *
     * @param {object} [formula.vm] Information about a VM from which a formula is
     * to be created.
     *
     * @param {string} [formula.vm.labVmId] The identifier of the VM from which a
     * formula is to be created.
     *
     * @param {string} [formula.location] The location of the resource.
     *
     * @param {object} [formula.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Formula} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Formula} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Formula} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, formula: models.Formula, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Formula>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, formula: models.Formula, callback: ServiceCallback<models.Formula>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, formula: models.Formula, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Formula>): void;


    /**
     * Delete formula.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete formula.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of formulas.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} formula A formula for creating a VM, specifying an image
     * base and other parameters
     *
     * @param {string} [formula.description] The description of the formula.
     *
     * @param {string} [formula.author] The author of the formula.
     *
     * @param {string} [formula.osType] The OS type of the formula.
     *
     * @param {object} [formula.formulaContent] The content of the formula.
     *
     * @param {object} [formula.formulaContent.bulkCreationParameters] The number
     * of virtual machine instances to create.
     *
     * @param {number}
     * [formula.formulaContent.bulkCreationParameters.instanceCount] The number of
     * virtual machine instances to create.
     *
     * @param {string} [formula.formulaContent.notes] The notes of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.ownerObjectId] The object identifier
     * of the owner of the virtual machine.
     *
     * @param {string} [formula.formulaContent.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [formula.formulaContent.createdByUserId] The object
     * identifier of the creator of the virtual machine.
     *
     * @param {string} [formula.formulaContent.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [formula.formulaContent.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [formula.formulaContent.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [formula.formulaContent.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.size] The size of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.userName] The user name of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.password] The password of the
     * virtual machine administrator.
     *
     * @param {string} [formula.formulaContent.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [formula.formulaContent.isAuthenticationWithSshKey]
     * Indicates whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [formula.formulaContent.fqdn] The fully-qualified domain
     * name of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labSubnetName] The lab subnet name
     * of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [formula.formulaContent.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [formula.formulaContent.artifacts] The artifacts to be
     * installed on the virtual machine.
     *
     * @param {object} [formula.formulaContent.artifactDeploymentStatus] The
     * artifact deployment status for the virtual machine.
     *
     * @param {string}
     * [formula.formulaContent.artifactDeploymentStatus.deploymentStatus] The
     * deployment status of the artifact.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.totalArtifacts] The total
     * count of the artifacts that were tentatively applied.
     *
     * @param {object} [formula.formulaContent.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.offer] The
     * offer of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.sku] The SKU
     * of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [formula.formulaContent.planId] The id of the plan
     * associated with the virtual machine image
     *
     * @param {object} [formula.formulaContent.networkInterface] The network
     * interface properties.
     *
     * @param {string} [formula.formulaContent.networkInterface.virtualNetworkId]
     * The resource ID of the virtual network.
     *
     * @param {string} [formula.formulaContent.networkInterface.subnetId] The
     * resource ID of the sub net.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddressId]
     * The resource ID of the public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddress]
     * The public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.privateIpAddress]
     * The private IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.dnsName] The DNS
     * name.
     *
     * @param {string} [formula.formulaContent.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [formula.formulaContent.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [formula.formulaContent.expirationDate] The expiration date
     * for VM.
     *
     * @param {boolean} [formula.formulaContent.allowClaim] Indicates whether
     * another user can take ownership of the virtual machine
     *
     * @param {string} [formula.formulaContent.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [formula.formulaContent.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [formula.formulaContent.environmentId] The resource ID of
     * the environment that contains this virtual machine, if any.
     *
     * @param {array} [formula.formulaContent.dataDiskParameters] New or existing
     * data disks to attach to the virtual machine after creation
     *
     * @param {array} [formula.formulaContent.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [formula.formulaContent.lastKnownPowerState] Last known
     * compute power state captured in DTL
     *
     * @param {string} [formula.formulaContent.name] The name of the virtual
     * machine or environment
     *
     * @param {string} [formula.formulaContent.location] The location of the new
     * virtual machine or environment
     *
     * @param {object} [formula.formulaContent.tags] The tags of the resource.
     *
     * @param {object} [formula.vm] Information about a VM from which a formula is
     * to be created.
     *
     * @param {string} [formula.vm.labVmId] The identifier of the VM from which a
     * formula is to be created.
     *
     * @param {object} [formula.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Formula>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, formula: models.FormulaFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Formula>>;

    /**
     * Modify properties of formulas.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} formula A formula for creating a VM, specifying an image
     * base and other parameters
     *
     * @param {string} [formula.description] The description of the formula.
     *
     * @param {string} [formula.author] The author of the formula.
     *
     * @param {string} [formula.osType] The OS type of the formula.
     *
     * @param {object} [formula.formulaContent] The content of the formula.
     *
     * @param {object} [formula.formulaContent.bulkCreationParameters] The number
     * of virtual machine instances to create.
     *
     * @param {number}
     * [formula.formulaContent.bulkCreationParameters.instanceCount] The number of
     * virtual machine instances to create.
     *
     * @param {string} [formula.formulaContent.notes] The notes of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.ownerObjectId] The object identifier
     * of the owner of the virtual machine.
     *
     * @param {string} [formula.formulaContent.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [formula.formulaContent.createdByUserId] The object
     * identifier of the creator of the virtual machine.
     *
     * @param {string} [formula.formulaContent.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [formula.formulaContent.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [formula.formulaContent.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [formula.formulaContent.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.size] The size of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.userName] The user name of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.password] The password of the
     * virtual machine administrator.
     *
     * @param {string} [formula.formulaContent.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [formula.formulaContent.isAuthenticationWithSshKey]
     * Indicates whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [formula.formulaContent.fqdn] The fully-qualified domain
     * name of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labSubnetName] The lab subnet name
     * of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [formula.formulaContent.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [formula.formulaContent.artifacts] The artifacts to be
     * installed on the virtual machine.
     *
     * @param {object} [formula.formulaContent.artifactDeploymentStatus] The
     * artifact deployment status for the virtual machine.
     *
     * @param {string}
     * [formula.formulaContent.artifactDeploymentStatus.deploymentStatus] The
     * deployment status of the artifact.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.totalArtifacts] The total
     * count of the artifacts that were tentatively applied.
     *
     * @param {object} [formula.formulaContent.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.offer] The
     * offer of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.sku] The SKU
     * of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [formula.formulaContent.planId] The id of the plan
     * associated with the virtual machine image
     *
     * @param {object} [formula.formulaContent.networkInterface] The network
     * interface properties.
     *
     * @param {string} [formula.formulaContent.networkInterface.virtualNetworkId]
     * The resource ID of the virtual network.
     *
     * @param {string} [formula.formulaContent.networkInterface.subnetId] The
     * resource ID of the sub net.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddressId]
     * The resource ID of the public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddress]
     * The public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.privateIpAddress]
     * The private IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.dnsName] The DNS
     * name.
     *
     * @param {string} [formula.formulaContent.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [formula.formulaContent.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [formula.formulaContent.expirationDate] The expiration date
     * for VM.
     *
     * @param {boolean} [formula.formulaContent.allowClaim] Indicates whether
     * another user can take ownership of the virtual machine
     *
     * @param {string} [formula.formulaContent.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [formula.formulaContent.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [formula.formulaContent.environmentId] The resource ID of
     * the environment that contains this virtual machine, if any.
     *
     * @param {array} [formula.formulaContent.dataDiskParameters] New or existing
     * data disks to attach to the virtual machine after creation
     *
     * @param {array} [formula.formulaContent.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [formula.formulaContent.lastKnownPowerState] Last known
     * compute power state captured in DTL
     *
     * @param {string} [formula.formulaContent.name] The name of the virtual
     * machine or environment
     *
     * @param {string} [formula.formulaContent.location] The location of the new
     * virtual machine or environment
     *
     * @param {object} [formula.formulaContent.tags] The tags of the resource.
     *
     * @param {object} [formula.vm] Information about a VM from which a formula is
     * to be created.
     *
     * @param {string} [formula.vm.labVmId] The identifier of the VM from which a
     * formula is to be created.
     *
     * @param {object} [formula.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Formula} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Formula} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Formula} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, name: string, formula: models.FormulaFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Formula>;
    update(resourceGroupName: string, labName: string, name: string, formula: models.FormulaFragment, callback: ServiceCallback<models.Formula>): void;
    update(resourceGroupName: string, labName: string, name: string, formula: models.FormulaFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Formula>): void;


    /**
     * Create or replace an existing Formula. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} formula A formula for creating a VM, specifying an image
     * base and other parameters
     *
     * @param {string} [formula.description] The description of the formula.
     *
     * @param {string} [formula.author] The author of the formula.
     *
     * @param {string} [formula.osType] The OS type of the formula.
     *
     * @param {object} [formula.formulaContent] The content of the formula.
     *
     * @param {object} [formula.formulaContent.bulkCreationParameters] The number
     * of virtual machine instances to create.
     *
     * @param {number}
     * [formula.formulaContent.bulkCreationParameters.instanceCount] The number of
     * virtual machine instances to create.
     *
     * @param {string} [formula.formulaContent.notes] The notes of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.ownerObjectId] The object identifier
     * of the owner of the virtual machine.
     *
     * @param {string} [formula.formulaContent.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [formula.formulaContent.createdByUserId] The object
     * identifier of the creator of the virtual machine.
     *
     * @param {string} [formula.formulaContent.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [formula.formulaContent.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [formula.formulaContent.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [formula.formulaContent.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.size] The size of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.userName] The user name of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.password] The password of the
     * virtual machine administrator.
     *
     * @param {string} [formula.formulaContent.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [formula.formulaContent.isAuthenticationWithSshKey]
     * Indicates whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [formula.formulaContent.fqdn] The fully-qualified domain
     * name of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labSubnetName] The lab subnet name
     * of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [formula.formulaContent.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [formula.formulaContent.artifacts] The artifacts to be
     * installed on the virtual machine.
     *
     * @param {object} [formula.formulaContent.artifactDeploymentStatus] The
     * artifact deployment status for the virtual machine.
     *
     * @param {string}
     * [formula.formulaContent.artifactDeploymentStatus.deploymentStatus] The
     * deployment status of the artifact.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.totalArtifacts] The total
     * count of the artifacts that were tentatively applied.
     *
     * @param {object} [formula.formulaContent.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.offer] The
     * offer of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.sku] The SKU
     * of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [formula.formulaContent.planId] The id of the plan
     * associated with the virtual machine image
     *
     * @param {object} [formula.formulaContent.networkInterface] The network
     * interface properties.
     *
     * @param {string} [formula.formulaContent.networkInterface.virtualNetworkId]
     * The resource ID of the virtual network.
     *
     * @param {string} [formula.formulaContent.networkInterface.subnetId] The
     * resource ID of the sub net.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddressId]
     * The resource ID of the public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddress]
     * The public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.privateIpAddress]
     * The private IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.dnsName] The DNS
     * name.
     *
     * @param {string} [formula.formulaContent.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [formula.formulaContent.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [formula.formulaContent.expirationDate] The expiration date
     * for VM.
     *
     * @param {boolean} [formula.formulaContent.allowClaim] Indicates whether
     * another user can take ownership of the virtual machine
     *
     * @param {string} [formula.formulaContent.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [formula.formulaContent.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [formula.formulaContent.environmentId] The resource ID of
     * the environment that contains this virtual machine, if any.
     *
     * @param {array} [formula.formulaContent.dataDiskParameters] New or existing
     * data disks to attach to the virtual machine after creation
     *
     * @param {array} [formula.formulaContent.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [formula.formulaContent.lastKnownPowerState] Last known
     * compute power state captured in DTL
     *
     * @param {string} [formula.formulaContent.name] The name of the virtual
     * machine or environment
     *
     * @param {string} [formula.formulaContent.location] The location of the new
     * virtual machine or environment
     *
     * @param {object} [formula.formulaContent.tags] The tags of the resource.
     *
     * @param {object} [formula.vm] Information about a VM from which a formula is
     * to be created.
     *
     * @param {string} [formula.vm.labVmId] The identifier of the VM from which a
     * formula is to be created.
     *
     * @param {string} [formula.location] The location of the resource.
     *
     * @param {object} [formula.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Formula>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, formula: models.Formula, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Formula>>;

    /**
     * Create or replace an existing Formula. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the formula.
     *
     * @param {object} formula A formula for creating a VM, specifying an image
     * base and other parameters
     *
     * @param {string} [formula.description] The description of the formula.
     *
     * @param {string} [formula.author] The author of the formula.
     *
     * @param {string} [formula.osType] The OS type of the formula.
     *
     * @param {object} [formula.formulaContent] The content of the formula.
     *
     * @param {object} [formula.formulaContent.bulkCreationParameters] The number
     * of virtual machine instances to create.
     *
     * @param {number}
     * [formula.formulaContent.bulkCreationParameters.instanceCount] The number of
     * virtual machine instances to create.
     *
     * @param {string} [formula.formulaContent.notes] The notes of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.ownerObjectId] The object identifier
     * of the owner of the virtual machine.
     *
     * @param {string} [formula.formulaContent.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [formula.formulaContent.createdByUserId] The object
     * identifier of the creator of the virtual machine.
     *
     * @param {string} [formula.formulaContent.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [formula.formulaContent.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [formula.formulaContent.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [formula.formulaContent.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.size] The size of the virtual
     * machine.
     *
     * @param {string} [formula.formulaContent.userName] The user name of the
     * virtual machine.
     *
     * @param {string} [formula.formulaContent.password] The password of the
     * virtual machine administrator.
     *
     * @param {string} [formula.formulaContent.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [formula.formulaContent.isAuthenticationWithSshKey]
     * Indicates whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [formula.formulaContent.fqdn] The fully-qualified domain
     * name of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labSubnetName] The lab subnet name
     * of the virtual machine.
     *
     * @param {string} [formula.formulaContent.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [formula.formulaContent.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [formula.formulaContent.artifacts] The artifacts to be
     * installed on the virtual machine.
     *
     * @param {object} [formula.formulaContent.artifactDeploymentStatus] The
     * artifact deployment status for the virtual machine.
     *
     * @param {string}
     * [formula.formulaContent.artifactDeploymentStatus.deploymentStatus] The
     * deployment status of the artifact.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number}
     * [formula.formulaContent.artifactDeploymentStatus.totalArtifacts] The total
     * count of the artifacts that were tentatively applied.
     *
     * @param {object} [formula.formulaContent.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.offer] The
     * offer of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.sku] The SKU
     * of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.osType] The OS
     * type of the gallery image.
     *
     * @param {string} [formula.formulaContent.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [formula.formulaContent.planId] The id of the plan
     * associated with the virtual machine image
     *
     * @param {object} [formula.formulaContent.networkInterface] The network
     * interface properties.
     *
     * @param {string} [formula.formulaContent.networkInterface.virtualNetworkId]
     * The resource ID of the virtual network.
     *
     * @param {string} [formula.formulaContent.networkInterface.subnetId] The
     * resource ID of the sub net.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddressId]
     * The resource ID of the public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.publicIpAddress]
     * The public IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.privateIpAddress]
     * The private IP address.
     *
     * @param {string} [formula.formulaContent.networkInterface.dnsName] The DNS
     * name.
     *
     * @param {string} [formula.formulaContent.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [formula.formulaContent.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration]
     * The configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [formula.formulaContent.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [formula.formulaContent.expirationDate] The expiration date
     * for VM.
     *
     * @param {boolean} [formula.formulaContent.allowClaim] Indicates whether
     * another user can take ownership of the virtual machine
     *
     * @param {string} [formula.formulaContent.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [formula.formulaContent.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [formula.formulaContent.environmentId] The resource ID of
     * the environment that contains this virtual machine, if any.
     *
     * @param {array} [formula.formulaContent.dataDiskParameters] New or existing
     * data disks to attach to the virtual machine after creation
     *
     * @param {array} [formula.formulaContent.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [formula.formulaContent.lastKnownPowerState] Last known
     * compute power state captured in DTL
     *
     * @param {string} [formula.formulaContent.name] The name of the virtual
     * machine or environment
     *
     * @param {string} [formula.formulaContent.location] The location of the new
     * virtual machine or environment
     *
     * @param {object} [formula.formulaContent.tags] The tags of the resource.
     *
     * @param {object} [formula.vm] Information about a VM from which a formula is
     * to be created.
     *
     * @param {string} [formula.vm.labVmId] The identifier of the VM from which a
     * formula is to be created.
     *
     * @param {string} [formula.location] The location of the resource.
     *
     * @param {object} [formula.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Formula} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Formula} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Formula} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, formula: models.Formula, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Formula>;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, formula: models.Formula, callback: ServiceCallback<models.Formula>): void;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, formula: models.Formula, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Formula>): void;


    /**
     * List formulas in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<FormulaList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.FormulaList>>;

    /**
     * List formulas in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {FormulaList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {FormulaList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link FormulaList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.FormulaList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.FormulaList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.FormulaList>): void;
}

/**
 * @class
 * GalleryImages
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface GalleryImages {


    /**
     * List gallery images in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=author)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<GalleryImageList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.GalleryImageList>>;

    /**
     * List gallery images in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=author)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {GalleryImageList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {GalleryImageList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link GalleryImageList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.GalleryImageList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.GalleryImageList>): void;
    list(resourceGroupName: string, labName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.GalleryImageList>): void;


    /**
     * List gallery images in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<GalleryImageList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.GalleryImageList>>;

    /**
     * List gallery images in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {GalleryImageList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {GalleryImageList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link GalleryImageList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.GalleryImageList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.GalleryImageList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.GalleryImageList>): void;
}

/**
 * @class
 * NotificationChannels
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface NotificationChannels {


    /**
     * List notification channels in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=webHookUrl)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<NotificationChannelList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.NotificationChannelList>>;

    /**
     * List notification channels in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=webHookUrl)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {NotificationChannelList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {NotificationChannelList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link NotificationChannelList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.NotificationChannelList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.NotificationChannelList>): void;
    list(resourceGroupName: string, labName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.NotificationChannelList>): void;


    /**
     * Get notification channel.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=webHookUrl)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<NotificationChannel>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.NotificationChannel>>;

    /**
     * Get notification channel.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=webHookUrl)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {NotificationChannel} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {NotificationChannel} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link NotificationChannel} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.NotificationChannel>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.NotificationChannel>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.NotificationChannel>): void;


    /**
     * Create or replace an existing notificationChannel.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} notificationChannel A notification.
     *
     * @param {string} [notificationChannel.webHookUrl] The webhook URL to send
     * notifications to.
     *
     * @param {string} [notificationChannel.emailRecipient] The email recipient to
     * send notifications to (can be a list of semi-colon separated email
     * addresses).
     *
     * @param {string} [notificationChannel.notificationLocale] The locale to use
     * when sending a notification (fallback for unsupported languages is EN).
     *
     * @param {string} [notificationChannel.description] Description of
     * notification.
     *
     * @param {array} [notificationChannel.events] The list of event for which this
     * notification is enabled.
     *
     * @param {string} [notificationChannel.location] The location of the resource.
     *
     * @param {object} [notificationChannel.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<NotificationChannel>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, notificationChannel: models.NotificationChannel, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.NotificationChannel>>;

    /**
     * Create or replace an existing notificationChannel.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} notificationChannel A notification.
     *
     * @param {string} [notificationChannel.webHookUrl] The webhook URL to send
     * notifications to.
     *
     * @param {string} [notificationChannel.emailRecipient] The email recipient to
     * send notifications to (can be a list of semi-colon separated email
     * addresses).
     *
     * @param {string} [notificationChannel.notificationLocale] The locale to use
     * when sending a notification (fallback for unsupported languages is EN).
     *
     * @param {string} [notificationChannel.description] Description of
     * notification.
     *
     * @param {array} [notificationChannel.events] The list of event for which this
     * notification is enabled.
     *
     * @param {string} [notificationChannel.location] The location of the resource.
     *
     * @param {object} [notificationChannel.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {NotificationChannel} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {NotificationChannel} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link NotificationChannel} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, notificationChannel: models.NotificationChannel, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.NotificationChannel>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, notificationChannel: models.NotificationChannel, callback: ServiceCallback<models.NotificationChannel>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, notificationChannel: models.NotificationChannel, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.NotificationChannel>): void;


    /**
     * Delete notification channel.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete notification channel.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of notification channels.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} notificationChannel A notification.
     *
     * @param {string} [notificationChannel.webHookUrl] The webhook URL to send
     * notifications to.
     *
     * @param {string} [notificationChannel.emailRecipient] The email recipient to
     * send notifications to (can be a list of semi-colon separated email
     * addresses).
     *
     * @param {string} [notificationChannel.notificationLocale] The locale to use
     * when sending a notification (fallback for unsupported languages is EN).
     *
     * @param {string} [notificationChannel.description] Description of
     * notification.
     *
     * @param {array} [notificationChannel.events] The list of event for which this
     * notification is enabled.
     *
     * @param {object} [notificationChannel.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<NotificationChannel>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, notificationChannel: models.NotificationChannelFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.NotificationChannel>>;

    /**
     * Modify properties of notification channels.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} notificationChannel A notification.
     *
     * @param {string} [notificationChannel.webHookUrl] The webhook URL to send
     * notifications to.
     *
     * @param {string} [notificationChannel.emailRecipient] The email recipient to
     * send notifications to (can be a list of semi-colon separated email
     * addresses).
     *
     * @param {string} [notificationChannel.notificationLocale] The locale to use
     * when sending a notification (fallback for unsupported languages is EN).
     *
     * @param {string} [notificationChannel.description] Description of
     * notification.
     *
     * @param {array} [notificationChannel.events] The list of event for which this
     * notification is enabled.
     *
     * @param {object} [notificationChannel.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {NotificationChannel} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {NotificationChannel} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link NotificationChannel} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, name: string, notificationChannel: models.NotificationChannelFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.NotificationChannel>;
    update(resourceGroupName: string, labName: string, name: string, notificationChannel: models.NotificationChannelFragment, callback: ServiceCallback<models.NotificationChannel>): void;
    update(resourceGroupName: string, labName: string, name: string, notificationChannel: models.NotificationChannelFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.NotificationChannel>): void;


    /**
     * Send notification to provided channel.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} notifyParameters Properties for generating a Notification.
     *
     * @param {string} [notifyParameters.eventName] The type of event (i.e.
     * AutoShutdown, Cost). Possible values include: 'AutoShutdown', 'Cost'
     *
     * @param {string} [notifyParameters.jsonPayload] Properties for the
     * notification in json format.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    notifyWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, notifyParameters: models.NotifyParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Send notification to provided channel.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the notificationChannel.
     *
     * @param {object} notifyParameters Properties for generating a Notification.
     *
     * @param {string} [notifyParameters.eventName] The type of event (i.e.
     * AutoShutdown, Cost). Possible values include: 'AutoShutdown', 'Cost'
     *
     * @param {string} [notifyParameters.jsonPayload] Properties for the
     * notification in json format.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    notify(resourceGroupName: string, labName: string, name: string, notifyParameters: models.NotifyParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    notify(resourceGroupName: string, labName: string, name: string, notifyParameters: models.NotifyParameters, callback: ServiceCallback<void>): void;
    notify(resourceGroupName: string, labName: string, name: string, notifyParameters: models.NotifyParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List notification channels in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<NotificationChannelList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.NotificationChannelList>>;

    /**
     * List notification channels in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {NotificationChannelList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {NotificationChannelList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link NotificationChannelList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.NotificationChannelList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.NotificationChannelList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.NotificationChannelList>): void;
}

/**
 * @class
 * PolicySets
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface PolicySets {


    /**
     * Evaluates lab policy.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the policy set.
     *
     * @param {object} evaluatePoliciesRequest Request body for evaluating a policy
     * set.
     *
     * @param {array} [evaluatePoliciesRequest.policies] Policies to evaluate.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EvaluatePoliciesResponse>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    evaluatePoliciesWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, evaluatePoliciesRequest: models.EvaluatePoliciesRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EvaluatePoliciesResponse>>;

    /**
     * Evaluates lab policy.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the policy set.
     *
     * @param {object} evaluatePoliciesRequest Request body for evaluating a policy
     * set.
     *
     * @param {array} [evaluatePoliciesRequest.policies] Policies to evaluate.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EvaluatePoliciesResponse} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EvaluatePoliciesResponse} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EvaluatePoliciesResponse} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    evaluatePolicies(resourceGroupName: string, labName: string, name: string, evaluatePoliciesRequest: models.EvaluatePoliciesRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EvaluatePoliciesResponse>;
    evaluatePolicies(resourceGroupName: string, labName: string, name: string, evaluatePoliciesRequest: models.EvaluatePoliciesRequest, callback: ServiceCallback<models.EvaluatePoliciesResponse>): void;
    evaluatePolicies(resourceGroupName: string, labName: string, name: string, evaluatePoliciesRequest: models.EvaluatePoliciesRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EvaluatePoliciesResponse>): void;
}

/**
 * @class
 * Policies
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Policies {


    /**
     * List policies in a given policy set.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=description)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<PolicyList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, policySetName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.PolicyList>>;

    /**
     * List policies in a given policy set.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=description)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {PolicyList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {PolicyList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link PolicyList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, policySetName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.PolicyList>;
    list(resourceGroupName: string, labName: string, policySetName: string, callback: ServiceCallback<models.PolicyList>): void;
    list(resourceGroupName: string, labName: string, policySetName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.PolicyList>): void;


    /**
     * Get policy.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {string} name The name of the policy.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=description)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Policy>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, policySetName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Policy>>;

    /**
     * Get policy.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {string} name The name of the policy.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=description)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Policy} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Policy} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Policy} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, policySetName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Policy>;
    get(resourceGroupName: string, labName: string, policySetName: string, name: string, callback: ServiceCallback<models.Policy>): void;
    get(resourceGroupName: string, labName: string, policySetName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Policy>): void;


    /**
     * Create or replace an existing policy.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {string} name The name of the policy.
     *
     * @param {object} policy A Policy.
     *
     * @param {string} [policy.description] The description of the policy.
     *
     * @param {string} [policy.status] The status of the policy. Possible values
     * include: 'Enabled', 'Disabled'
     *
     * @param {string} [policy.factName] The fact name of the policy (e.g.
     * LabVmCount, LabVmSize, MaxVmsAllowedPerLab, etc. Possible values include:
     * 'UserOwnedLabVmCount', 'UserOwnedLabPremiumVmCount', 'LabVmCount',
     * 'LabPremiumVmCount', 'LabVmSize', 'GalleryImage',
     * 'UserOwnedLabVmCountInSubnet', 'LabTargetCost', 'EnvironmentTemplate',
     * 'ScheduleEditPermission'
     *
     * @param {string} [policy.factData] The fact data of the policy.
     *
     * @param {string} [policy.threshold] The threshold of the policy (i.e. a
     * number for MaxValuePolicy, and a JSON array of values for
     * AllowedValuesPolicy).
     *
     * @param {string} [policy.evaluatorType] The evaluator type of the policy
     * (i.e. AllowedValuesPolicy, MaxValuePolicy). Possible values include:
     * 'AllowedValuesPolicy', 'MaxValuePolicy'
     *
     * @param {string} [policy.location] The location of the resource.
     *
     * @param {object} [policy.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Policy>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, policySetName: string, name: string, policy: models.Policy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Policy>>;

    /**
     * Create or replace an existing policy.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {string} name The name of the policy.
     *
     * @param {object} policy A Policy.
     *
     * @param {string} [policy.description] The description of the policy.
     *
     * @param {string} [policy.status] The status of the policy. Possible values
     * include: 'Enabled', 'Disabled'
     *
     * @param {string} [policy.factName] The fact name of the policy (e.g.
     * LabVmCount, LabVmSize, MaxVmsAllowedPerLab, etc. Possible values include:
     * 'UserOwnedLabVmCount', 'UserOwnedLabPremiumVmCount', 'LabVmCount',
     * 'LabPremiumVmCount', 'LabVmSize', 'GalleryImage',
     * 'UserOwnedLabVmCountInSubnet', 'LabTargetCost', 'EnvironmentTemplate',
     * 'ScheduleEditPermission'
     *
     * @param {string} [policy.factData] The fact data of the policy.
     *
     * @param {string} [policy.threshold] The threshold of the policy (i.e. a
     * number for MaxValuePolicy, and a JSON array of values for
     * AllowedValuesPolicy).
     *
     * @param {string} [policy.evaluatorType] The evaluator type of the policy
     * (i.e. AllowedValuesPolicy, MaxValuePolicy). Possible values include:
     * 'AllowedValuesPolicy', 'MaxValuePolicy'
     *
     * @param {string} [policy.location] The location of the resource.
     *
     * @param {object} [policy.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Policy} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Policy} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Policy} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, policySetName: string, name: string, policy: models.Policy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Policy>;
    createOrUpdate(resourceGroupName: string, labName: string, policySetName: string, name: string, policy: models.Policy, callback: ServiceCallback<models.Policy>): void;
    createOrUpdate(resourceGroupName: string, labName: string, policySetName: string, name: string, policy: models.Policy, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Policy>): void;


    /**
     * Delete policy.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {string} name The name of the policy.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, policySetName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete policy.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {string} name The name of the policy.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, policySetName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, policySetName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, policySetName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of policies.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {string} name The name of the policy.
     *
     * @param {object} policy A Policy.
     *
     * @param {string} [policy.description] The description of the policy.
     *
     * @param {string} [policy.status] The status of the policy. Possible values
     * include: 'Enabled', 'Disabled'
     *
     * @param {string} [policy.factName] The fact name of the policy (e.g.
     * LabVmCount, LabVmSize, MaxVmsAllowedPerLab, etc. Possible values include:
     * 'UserOwnedLabVmCount', 'UserOwnedLabPremiumVmCount', 'LabVmCount',
     * 'LabPremiumVmCount', 'LabVmSize', 'GalleryImage',
     * 'UserOwnedLabVmCountInSubnet', 'LabTargetCost', 'EnvironmentTemplate',
     * 'ScheduleEditPermission'
     *
     * @param {string} [policy.factData] The fact data of the policy.
     *
     * @param {string} [policy.threshold] The threshold of the policy (i.e. a
     * number for MaxValuePolicy, and a JSON array of values for
     * AllowedValuesPolicy).
     *
     * @param {string} [policy.evaluatorType] The evaluator type of the policy
     * (i.e. AllowedValuesPolicy, MaxValuePolicy). Possible values include:
     * 'AllowedValuesPolicy', 'MaxValuePolicy'
     *
     * @param {object} [policy.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Policy>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, policySetName: string, name: string, policy: models.PolicyFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Policy>>;

    /**
     * Modify properties of policies.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} policySetName The name of the policy set.
     *
     * @param {string} name The name of the policy.
     *
     * @param {object} policy A Policy.
     *
     * @param {string} [policy.description] The description of the policy.
     *
     * @param {string} [policy.status] The status of the policy. Possible values
     * include: 'Enabled', 'Disabled'
     *
     * @param {string} [policy.factName] The fact name of the policy (e.g.
     * LabVmCount, LabVmSize, MaxVmsAllowedPerLab, etc. Possible values include:
     * 'UserOwnedLabVmCount', 'UserOwnedLabPremiumVmCount', 'LabVmCount',
     * 'LabPremiumVmCount', 'LabVmSize', 'GalleryImage',
     * 'UserOwnedLabVmCountInSubnet', 'LabTargetCost', 'EnvironmentTemplate',
     * 'ScheduleEditPermission'
     *
     * @param {string} [policy.factData] The fact data of the policy.
     *
     * @param {string} [policy.threshold] The threshold of the policy (i.e. a
     * number for MaxValuePolicy, and a JSON array of values for
     * AllowedValuesPolicy).
     *
     * @param {string} [policy.evaluatorType] The evaluator type of the policy
     * (i.e. AllowedValuesPolicy, MaxValuePolicy). Possible values include:
     * 'AllowedValuesPolicy', 'MaxValuePolicy'
     *
     * @param {object} [policy.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Policy} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Policy} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Policy} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, policySetName: string, name: string, policy: models.PolicyFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Policy>;
    update(resourceGroupName: string, labName: string, policySetName: string, name: string, policy: models.PolicyFragment, callback: ServiceCallback<models.Policy>): void;
    update(resourceGroupName: string, labName: string, policySetName: string, name: string, policy: models.PolicyFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Policy>): void;


    /**
     * List policies in a given policy set.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<PolicyList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.PolicyList>>;

    /**
     * List policies in a given policy set.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {PolicyList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {PolicyList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link PolicyList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.PolicyList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.PolicyList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.PolicyList>): void;
}

/**
 * @class
 * Schedules
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Schedules {


    /**
     * List schedules in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.ScheduleList>): void;
    list(resourceGroupName: string, labName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;


    /**
     * Get schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Get schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.Schedule>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Create or replace an existing schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {string} [schedule.location] The location of the resource.
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, schedule: models.Schedule, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Create or replace an existing schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {string} [schedule.location] The location of the resource.
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, schedule: models.Schedule, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, schedule: models.Schedule, callback: ServiceCallback<models.Schedule>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, schedule: models.Schedule, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Delete schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of schedules.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, schedule: models.ScheduleFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Modify properties of schedules.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, name: string, schedule: models.ScheduleFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    update(resourceGroupName: string, labName: string, name: string, schedule: models.ScheduleFragment, callback: ServiceCallback<models.Schedule>): void;
    update(resourceGroupName: string, labName: string, name: string, schedule: models.ScheduleFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    executeWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    execute(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    execute(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    execute(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Lists all applicable schedules
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listApplicableWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * Lists all applicable schedules
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listApplicable(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    listApplicable(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.ScheduleList>): void;
    listApplicable(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;


    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginExecuteWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginExecute(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginExecute(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginExecute(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List schedules in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ScheduleList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;


    /**
     * Lists all applicable schedules
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listApplicableNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * Lists all applicable schedules
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listApplicableNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    listApplicableNext(nextPageLink: string, callback: ServiceCallback<models.ScheduleList>): void;
    listApplicableNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;
}

/**
 * @class
 * ServiceRunners
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface ServiceRunners {


    /**
     * List service runners in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceRunnerList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceRunnerList>>;

    /**
     * List service runners in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceRunnerList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceRunnerList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceRunnerList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceRunnerList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.ServiceRunnerList>): void;
    list(resourceGroupName: string, labName: string, options: { filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceRunnerList>): void;


    /**
     * Get service runner.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the service runner.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceRunner>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceRunner>>;

    /**
     * Get service runner.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the service runner.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceRunner} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceRunner} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceRunner} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceRunner>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.ServiceRunner>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceRunner>): void;


    /**
     * Create or replace an existing Service runner.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the service runner.
     *
     * @param {object} serviceRunner A container for a managed identity to execute
     * DevTest lab services.
     *
     * @param {object} [serviceRunner.identity] The identity of the resource.
     *
     * @param {string} [serviceRunner.identity.type] Managed identity.
     *
     * @param {string} [serviceRunner.identity.principalId] The principal id of
     * resource identity.
     *
     * @param {string} [serviceRunner.identity.tenantId] The tenant identifier of
     * resource.
     *
     * @param {string} [serviceRunner.identity.clientSecretUrl] The client secret
     * URL of the identity.
     *
     * @param {string} [serviceRunner.location] The location of the resource.
     *
     * @param {object} [serviceRunner.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceRunner>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, serviceRunner: models.ServiceRunner, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceRunner>>;

    /**
     * Create or replace an existing Service runner.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the service runner.
     *
     * @param {object} serviceRunner A container for a managed identity to execute
     * DevTest lab services.
     *
     * @param {object} [serviceRunner.identity] The identity of the resource.
     *
     * @param {string} [serviceRunner.identity.type] Managed identity.
     *
     * @param {string} [serviceRunner.identity.principalId] The principal id of
     * resource identity.
     *
     * @param {string} [serviceRunner.identity.tenantId] The tenant identifier of
     * resource.
     *
     * @param {string} [serviceRunner.identity.clientSecretUrl] The client secret
     * URL of the identity.
     *
     * @param {string} [serviceRunner.location] The location of the resource.
     *
     * @param {object} [serviceRunner.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceRunner} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceRunner} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceRunner} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, serviceRunner: models.ServiceRunner, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceRunner>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, serviceRunner: models.ServiceRunner, callback: ServiceCallback<models.ServiceRunner>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, serviceRunner: models.ServiceRunner, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceRunner>): void;


    /**
     * Delete service runner.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the service runner.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete service runner.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the service runner.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List service runners in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceRunnerList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceRunnerList>>;

    /**
     * List service runners in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceRunnerList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceRunnerList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceRunnerList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceRunnerList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ServiceRunnerList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceRunnerList>): void;
}

/**
 * @class
 * Users
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Users {


    /**
     * List user profiles in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=identity)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<UserList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.UserList>>;

    /**
     * List user profiles in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=identity)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {UserList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {UserList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link UserList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.UserList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.UserList>): void;
    list(resourceGroupName: string, labName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.UserList>): void;


    /**
     * Get user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=identity)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<User>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.User>>;

    /**
     * Get user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=identity)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {User} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {User} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link User} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.User>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.User>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.User>): void;


    /**
     * Create or replace an existing user profile. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} user Profile of a lab user.
     *
     * @param {object} [user.identity] The identity of the user.
     *
     * @param {string} [user.identity.principalName] Set to the principal name /
     * UPN of the client JWT making the request.
     *
     * @param {string} [user.identity.principalId] Set to the principal Id of the
     * client JWT making the request. Service principal will not have the principal
     * Id.
     *
     * @param {string} [user.identity.tenantId] Set to the tenant ID of the client
     * JWT making the request.
     *
     * @param {string} [user.identity.objectId] Set to the object Id of the client
     * JWT making the request. Not all users have object Id. For CSP (reseller)
     * scenarios for example, object Id is not available.
     *
     * @param {string} [user.identity.appId] Set to the app Id of the client JWT
     * making the request.
     *
     * @param {object} [user.secretStore] The secret store of the user.
     *
     * @param {string} [user.secretStore.keyVaultUri] The URI of the user's Key
     * vault.
     *
     * @param {string} [user.secretStore.keyVaultId] The ID of the user's Key
     * vault.
     *
     * @param {string} [user.location] The location of the resource.
     *
     * @param {object} [user.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<User>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, user: models.User, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.User>>;

    /**
     * Create or replace an existing user profile. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} user Profile of a lab user.
     *
     * @param {object} [user.identity] The identity of the user.
     *
     * @param {string} [user.identity.principalName] Set to the principal name /
     * UPN of the client JWT making the request.
     *
     * @param {string} [user.identity.principalId] Set to the principal Id of the
     * client JWT making the request. Service principal will not have the principal
     * Id.
     *
     * @param {string} [user.identity.tenantId] Set to the tenant ID of the client
     * JWT making the request.
     *
     * @param {string} [user.identity.objectId] Set to the object Id of the client
     * JWT making the request. Not all users have object Id. For CSP (reseller)
     * scenarios for example, object Id is not available.
     *
     * @param {string} [user.identity.appId] Set to the app Id of the client JWT
     * making the request.
     *
     * @param {object} [user.secretStore] The secret store of the user.
     *
     * @param {string} [user.secretStore.keyVaultUri] The URI of the user's Key
     * vault.
     *
     * @param {string} [user.secretStore.keyVaultId] The ID of the user's Key
     * vault.
     *
     * @param {string} [user.location] The location of the resource.
     *
     * @param {object} [user.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {User} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {User} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link User} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, user: models.User, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.User>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, user: models.User, callback: ServiceCallback<models.User>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, user: models.User, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.User>): void;


    /**
     * Delete user profile. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete user profile. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of user profiles.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} user Profile of a lab user.
     *
     * @param {object} [user.identity] The identity of the user.
     *
     * @param {string} [user.identity.principalName] Set to the principal name /
     * UPN of the client JWT making the request.
     *
     * @param {string} [user.identity.principalId] Set to the principal Id of the
     * client JWT making the request. Service principal will not have the principal
     * Id.
     *
     * @param {string} [user.identity.tenantId] Set to the tenant ID of the client
     * JWT making the request.
     *
     * @param {string} [user.identity.objectId] Set to the object Id of the client
     * JWT making the request. Not all users have object Id. For CSP (reseller)
     * scenarios for example, object Id is not available.
     *
     * @param {string} [user.identity.appId] Set to the app Id of the client JWT
     * making the request.
     *
     * @param {object} [user.secretStore] The secret store of the user.
     *
     * @param {string} [user.secretStore.keyVaultUri] The URI of the user's Key
     * vault.
     *
     * @param {string} [user.secretStore.keyVaultId] The ID of the user's Key
     * vault.
     *
     * @param {object} [user.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<User>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, user: models.UserFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.User>>;

    /**
     * Modify properties of user profiles.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} user Profile of a lab user.
     *
     * @param {object} [user.identity] The identity of the user.
     *
     * @param {string} [user.identity.principalName] Set to the principal name /
     * UPN of the client JWT making the request.
     *
     * @param {string} [user.identity.principalId] Set to the principal Id of the
     * client JWT making the request. Service principal will not have the principal
     * Id.
     *
     * @param {string} [user.identity.tenantId] Set to the tenant ID of the client
     * JWT making the request.
     *
     * @param {string} [user.identity.objectId] Set to the object Id of the client
     * JWT making the request. Not all users have object Id. For CSP (reseller)
     * scenarios for example, object Id is not available.
     *
     * @param {string} [user.identity.appId] Set to the app Id of the client JWT
     * making the request.
     *
     * @param {object} [user.secretStore] The secret store of the user.
     *
     * @param {string} [user.secretStore.keyVaultUri] The URI of the user's Key
     * vault.
     *
     * @param {string} [user.secretStore.keyVaultId] The ID of the user's Key
     * vault.
     *
     * @param {object} [user.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {User} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {User} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link User} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, name: string, user: models.UserFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.User>;
    update(resourceGroupName: string, labName: string, name: string, user: models.UserFragment, callback: ServiceCallback<models.User>): void;
    update(resourceGroupName: string, labName: string, name: string, user: models.UserFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.User>): void;


    /**
     * Create or replace an existing user profile. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} user Profile of a lab user.
     *
     * @param {object} [user.identity] The identity of the user.
     *
     * @param {string} [user.identity.principalName] Set to the principal name /
     * UPN of the client JWT making the request.
     *
     * @param {string} [user.identity.principalId] Set to the principal Id of the
     * client JWT making the request. Service principal will not have the principal
     * Id.
     *
     * @param {string} [user.identity.tenantId] Set to the tenant ID of the client
     * JWT making the request.
     *
     * @param {string} [user.identity.objectId] Set to the object Id of the client
     * JWT making the request. Not all users have object Id. For CSP (reseller)
     * scenarios for example, object Id is not available.
     *
     * @param {string} [user.identity.appId] Set to the app Id of the client JWT
     * making the request.
     *
     * @param {object} [user.secretStore] The secret store of the user.
     *
     * @param {string} [user.secretStore.keyVaultUri] The URI of the user's Key
     * vault.
     *
     * @param {string} [user.secretStore.keyVaultId] The ID of the user's Key
     * vault.
     *
     * @param {string} [user.location] The location of the resource.
     *
     * @param {object} [user.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<User>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, user: models.User, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.User>>;

    /**
     * Create or replace an existing user profile. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} user Profile of a lab user.
     *
     * @param {object} [user.identity] The identity of the user.
     *
     * @param {string} [user.identity.principalName] Set to the principal name /
     * UPN of the client JWT making the request.
     *
     * @param {string} [user.identity.principalId] Set to the principal Id of the
     * client JWT making the request. Service principal will not have the principal
     * Id.
     *
     * @param {string} [user.identity.tenantId] Set to the tenant ID of the client
     * JWT making the request.
     *
     * @param {string} [user.identity.objectId] Set to the object Id of the client
     * JWT making the request. Not all users have object Id. For CSP (reseller)
     * scenarios for example, object Id is not available.
     *
     * @param {string} [user.identity.appId] Set to the app Id of the client JWT
     * making the request.
     *
     * @param {object} [user.secretStore] The secret store of the user.
     *
     * @param {string} [user.secretStore.keyVaultUri] The URI of the user's Key
     * vault.
     *
     * @param {string} [user.secretStore.keyVaultId] The ID of the user's Key
     * vault.
     *
     * @param {string} [user.location] The location of the resource.
     *
     * @param {object} [user.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {User} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {User} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link User} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, user: models.User, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.User>;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, user: models.User, callback: ServiceCallback<models.User>): void;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, user: models.User, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.User>): void;


    /**
     * Delete user profile. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete user profile. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List user profiles in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<UserList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.UserList>>;

    /**
     * List user profiles in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {UserList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {UserList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link UserList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.UserList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.UserList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.UserList>): void;
}

/**
 * @class
 * Disks
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Disks {


    /**
     * List disks in a given user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=diskType)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DiskList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DiskList>>;

    /**
     * List disks in a given user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=diskType)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DiskList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DiskList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DiskList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, userName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.DiskList>;
    list(resourceGroupName: string, labName: string, userName: string, callback: ServiceCallback<models.DiskList>): void;
    list(resourceGroupName: string, labName: string, userName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DiskList>): void;


    /**
     * Get disk.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=diskType)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Disk>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Disk>>;

    /**
     * Get disk.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=diskType)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Disk} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Disk} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Disk} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, userName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Disk>;
    get(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<models.Disk>): void;
    get(resourceGroupName: string, labName: string, userName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Disk>): void;


    /**
     * Create or replace an existing disk. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} disk A Disk.
     *
     * @param {string} [disk.diskType] The storage type for the disk (i.e.
     * Standard, Premium). Possible values include: 'Standard', 'Premium'
     *
     * @param {number} [disk.diskSizeGiB] The size of the disk in Gibibytes.
     *
     * @param {string} [disk.leasedByLabVmId] The resource ID of the VM to which
     * this disk is leased.
     *
     * @param {string} [disk.diskBlobName] When backed by a blob, the name of the
     * VHD blob without extension.
     *
     * @param {string} [disk.diskUri] When backed by a blob, the URI of underlying
     * blob.
     *
     * @param {string} [disk.hostCaching] The host caching policy of the disk (i.e.
     * None, ReadOnly, ReadWrite).
     *
     * @param {string} [disk.managedDiskId] When backed by managed disk, this is
     * the ID of the compute disk resource.
     *
     * @param {string} [disk.location] The location of the resource.
     *
     * @param {object} [disk.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Disk>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.Disk, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Disk>>;

    /**
     * Create or replace an existing disk. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} disk A Disk.
     *
     * @param {string} [disk.diskType] The storage type for the disk (i.e.
     * Standard, Premium). Possible values include: 'Standard', 'Premium'
     *
     * @param {number} [disk.diskSizeGiB] The size of the disk in Gibibytes.
     *
     * @param {string} [disk.leasedByLabVmId] The resource ID of the VM to which
     * this disk is leased.
     *
     * @param {string} [disk.diskBlobName] When backed by a blob, the name of the
     * VHD blob without extension.
     *
     * @param {string} [disk.diskUri] When backed by a blob, the URI of underlying
     * blob.
     *
     * @param {string} [disk.hostCaching] The host caching policy of the disk (i.e.
     * None, ReadOnly, ReadWrite).
     *
     * @param {string} [disk.managedDiskId] When backed by managed disk, this is
     * the ID of the compute disk resource.
     *
     * @param {string} [disk.location] The location of the resource.
     *
     * @param {object} [disk.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Disk} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Disk} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Disk} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.Disk, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Disk>;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.Disk, callback: ServiceCallback<models.Disk>): void;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.Disk, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Disk>): void;


    /**
     * Delete disk. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete disk. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of disks.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} disk A Disk.
     *
     * @param {string} [disk.diskType] The storage type for the disk (i.e.
     * Standard, Premium). Possible values include: 'Standard', 'Premium'
     *
     * @param {number} [disk.diskSizeGiB] The size of the disk in Gibibytes.
     *
     * @param {string} [disk.leasedByLabVmId] The resource ID of the VM to which
     * this disk is leased.
     *
     * @param {string} [disk.diskBlobName] When backed by a blob, the name of the
     * VHD blob without extension.
     *
     * @param {string} [disk.diskUri] When backed by a blob, the URI of underlying
     * blob.
     *
     * @param {string} [disk.hostCaching] The host caching policy of the disk (i.e.
     * None, ReadOnly, ReadWrite).
     *
     * @param {string} [disk.managedDiskId] When backed by managed disk, this is
     * the ID of the compute disk resource.
     *
     * @param {object} [disk.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Disk>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.DiskFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Disk>>;

    /**
     * Modify properties of disks.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} disk A Disk.
     *
     * @param {string} [disk.diskType] The storage type for the disk (i.e.
     * Standard, Premium). Possible values include: 'Standard', 'Premium'
     *
     * @param {number} [disk.diskSizeGiB] The size of the disk in Gibibytes.
     *
     * @param {string} [disk.leasedByLabVmId] The resource ID of the VM to which
     * this disk is leased.
     *
     * @param {string} [disk.diskBlobName] When backed by a blob, the name of the
     * VHD blob without extension.
     *
     * @param {string} [disk.diskUri] When backed by a blob, the URI of underlying
     * blob.
     *
     * @param {string} [disk.hostCaching] The host caching policy of the disk (i.e.
     * None, ReadOnly, ReadWrite).
     *
     * @param {string} [disk.managedDiskId] When backed by managed disk, this is
     * the ID of the compute disk resource.
     *
     * @param {object} [disk.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Disk} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Disk} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Disk} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.DiskFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Disk>;
    update(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.DiskFragment, callback: ServiceCallback<models.Disk>): void;
    update(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.DiskFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Disk>): void;


    /**
     * Attach and create the lease of the disk to the virtual machine. This
     * operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} attachDiskProperties Properties of the disk to attach.
     *
     * @param {string} [attachDiskProperties.leasedByLabVmId] The resource ID of
     * the Lab virtual machine to which the disk is attached.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    attachWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, attachDiskProperties: models.AttachDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Attach and create the lease of the disk to the virtual machine. This
     * operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} attachDiskProperties Properties of the disk to attach.
     *
     * @param {string} [attachDiskProperties.leasedByLabVmId] The resource ID of
     * the Lab virtual machine to which the disk is attached.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    attach(resourceGroupName: string, labName: string, userName: string, name: string, attachDiskProperties: models.AttachDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    attach(resourceGroupName: string, labName: string, userName: string, name: string, attachDiskProperties: models.AttachDiskProperties, callback: ServiceCallback<void>): void;
    attach(resourceGroupName: string, labName: string, userName: string, name: string, attachDiskProperties: models.AttachDiskProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Detach and break the lease of the disk attached to the virtual machine. This
     * operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} detachDiskProperties Properties of the disk to detach.
     *
     * @param {string} [detachDiskProperties.leasedByLabVmId] The resource ID of
     * the Lab VM to which the disk is attached.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    detachWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, detachDiskProperties: models.DetachDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Detach and break the lease of the disk attached to the virtual machine. This
     * operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} detachDiskProperties Properties of the disk to detach.
     *
     * @param {string} [detachDiskProperties.leasedByLabVmId] The resource ID of
     * the Lab VM to which the disk is attached.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    detach(resourceGroupName: string, labName: string, userName: string, name: string, detachDiskProperties: models.DetachDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    detach(resourceGroupName: string, labName: string, userName: string, name: string, detachDiskProperties: models.DetachDiskProperties, callback: ServiceCallback<void>): void;
    detach(resourceGroupName: string, labName: string, userName: string, name: string, detachDiskProperties: models.DetachDiskProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Create or replace an existing disk. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} disk A Disk.
     *
     * @param {string} [disk.diskType] The storage type for the disk (i.e.
     * Standard, Premium). Possible values include: 'Standard', 'Premium'
     *
     * @param {number} [disk.diskSizeGiB] The size of the disk in Gibibytes.
     *
     * @param {string} [disk.leasedByLabVmId] The resource ID of the VM to which
     * this disk is leased.
     *
     * @param {string} [disk.diskBlobName] When backed by a blob, the name of the
     * VHD blob without extension.
     *
     * @param {string} [disk.diskUri] When backed by a blob, the URI of underlying
     * blob.
     *
     * @param {string} [disk.hostCaching] The host caching policy of the disk (i.e.
     * None, ReadOnly, ReadWrite).
     *
     * @param {string} [disk.managedDiskId] When backed by managed disk, this is
     * the ID of the compute disk resource.
     *
     * @param {string} [disk.location] The location of the resource.
     *
     * @param {object} [disk.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Disk>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.Disk, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Disk>>;

    /**
     * Create or replace an existing disk. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} disk A Disk.
     *
     * @param {string} [disk.diskType] The storage type for the disk (i.e.
     * Standard, Premium). Possible values include: 'Standard', 'Premium'
     *
     * @param {number} [disk.diskSizeGiB] The size of the disk in Gibibytes.
     *
     * @param {string} [disk.leasedByLabVmId] The resource ID of the VM to which
     * this disk is leased.
     *
     * @param {string} [disk.diskBlobName] When backed by a blob, the name of the
     * VHD blob without extension.
     *
     * @param {string} [disk.diskUri] When backed by a blob, the URI of underlying
     * blob.
     *
     * @param {string} [disk.hostCaching] The host caching policy of the disk (i.e.
     * None, ReadOnly, ReadWrite).
     *
     * @param {string} [disk.managedDiskId] When backed by managed disk, this is
     * the ID of the compute disk resource.
     *
     * @param {string} [disk.location] The location of the resource.
     *
     * @param {object} [disk.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Disk} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Disk} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Disk} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.Disk, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Disk>;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.Disk, callback: ServiceCallback<models.Disk>): void;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, disk: models.Disk, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Disk>): void;


    /**
     * Delete disk. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete disk. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Attach and create the lease of the disk to the virtual machine. This
     * operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} attachDiskProperties Properties of the disk to attach.
     *
     * @param {string} [attachDiskProperties.leasedByLabVmId] The resource ID of
     * the Lab virtual machine to which the disk is attached.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginAttachWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, attachDiskProperties: models.AttachDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Attach and create the lease of the disk to the virtual machine. This
     * operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} attachDiskProperties Properties of the disk to attach.
     *
     * @param {string} [attachDiskProperties.leasedByLabVmId] The resource ID of
     * the Lab virtual machine to which the disk is attached.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginAttach(resourceGroupName: string, labName: string, userName: string, name: string, attachDiskProperties: models.AttachDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginAttach(resourceGroupName: string, labName: string, userName: string, name: string, attachDiskProperties: models.AttachDiskProperties, callback: ServiceCallback<void>): void;
    beginAttach(resourceGroupName: string, labName: string, userName: string, name: string, attachDiskProperties: models.AttachDiskProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Detach and break the lease of the disk attached to the virtual machine. This
     * operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} detachDiskProperties Properties of the disk to detach.
     *
     * @param {string} [detachDiskProperties.leasedByLabVmId] The resource ID of
     * the Lab VM to which the disk is attached.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDetachWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, detachDiskProperties: models.DetachDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Detach and break the lease of the disk attached to the virtual machine. This
     * operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the disk.
     *
     * @param {object} detachDiskProperties Properties of the disk to detach.
     *
     * @param {string} [detachDiskProperties.leasedByLabVmId] The resource ID of
     * the Lab VM to which the disk is attached.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDetach(resourceGroupName: string, labName: string, userName: string, name: string, detachDiskProperties: models.DetachDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDetach(resourceGroupName: string, labName: string, userName: string, name: string, detachDiskProperties: models.DetachDiskProperties, callback: ServiceCallback<void>): void;
    beginDetach(resourceGroupName: string, labName: string, userName: string, name: string, detachDiskProperties: models.DetachDiskProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List disks in a given user profile.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DiskList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DiskList>>;

    /**
     * List disks in a given user profile.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DiskList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DiskList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DiskList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DiskList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.DiskList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DiskList>): void;
}

/**
 * @class
 * Environments
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Environments {


    /**
     * List environments in a given user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=deploymentProperties)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DtlEnvironmentList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DtlEnvironmentList>>;

    /**
     * List environments in a given user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=deploymentProperties)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DtlEnvironmentList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DtlEnvironmentList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DtlEnvironmentList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, userName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.DtlEnvironmentList>;
    list(resourceGroupName: string, labName: string, userName: string, callback: ServiceCallback<models.DtlEnvironmentList>): void;
    list(resourceGroupName: string, labName: string, userName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DtlEnvironmentList>): void;


    /**
     * Get environment.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=deploymentProperties)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DtlEnvironment>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DtlEnvironment>>;

    /**
     * Get environment.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=deploymentProperties)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DtlEnvironment} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DtlEnvironment} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DtlEnvironment} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, userName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.DtlEnvironment>;
    get(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<models.DtlEnvironment>): void;
    get(resourceGroupName: string, labName: string, userName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DtlEnvironment>): void;


    /**
     * Create or replace an existing environment. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} dtlEnvironment An environment, which is essentially an ARM
     * template deployment.
     *
     * @param {object} [dtlEnvironment.deploymentProperties] The deployment
     * properties of the environment.
     *
     * @param {string} [dtlEnvironment.deploymentProperties.armTemplateId] The
     * Azure Resource Manager template's identifier.
     *
     * @param {array} [dtlEnvironment.deploymentProperties.parameters] The
     * parameters of the Azure Resource Manager template.
     *
     * @param {string} [dtlEnvironment.armTemplateDisplayName] The display name of
     * the Azure Resource Manager template that produced the environment.
     *
     * @param {string} [dtlEnvironment.location] The location of the resource.
     *
     * @param {object} [dtlEnvironment.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DtlEnvironment>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DtlEnvironment>>;

    /**
     * Create or replace an existing environment. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} dtlEnvironment An environment, which is essentially an ARM
     * template deployment.
     *
     * @param {object} [dtlEnvironment.deploymentProperties] The deployment
     * properties of the environment.
     *
     * @param {string} [dtlEnvironment.deploymentProperties.armTemplateId] The
     * Azure Resource Manager template's identifier.
     *
     * @param {array} [dtlEnvironment.deploymentProperties.parameters] The
     * parameters of the Azure Resource Manager template.
     *
     * @param {string} [dtlEnvironment.armTemplateDisplayName] The display name of
     * the Azure Resource Manager template that produced the environment.
     *
     * @param {string} [dtlEnvironment.location] The location of the resource.
     *
     * @param {object} [dtlEnvironment.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DtlEnvironment} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DtlEnvironment} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DtlEnvironment} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DtlEnvironment>;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironment, callback: ServiceCallback<models.DtlEnvironment>): void;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DtlEnvironment>): void;


    /**
     * Delete environment. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete environment. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of environments.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} dtlEnvironment An environment, which is essentially an ARM
     * template deployment.
     *
     * @param {object} [dtlEnvironment.deploymentProperties] The deployment
     * properties of the environment.
     *
     * @param {string} [dtlEnvironment.deploymentProperties.armTemplateId] The
     * Azure Resource Manager template's identifier.
     *
     * @param {array} [dtlEnvironment.deploymentProperties.parameters] The
     * parameters of the Azure Resource Manager template.
     *
     * @param {string} [dtlEnvironment.armTemplateDisplayName] The display name of
     * the Azure Resource Manager template that produced the environment.
     *
     * @param {object} [dtlEnvironment.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DtlEnvironment>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironmentFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DtlEnvironment>>;

    /**
     * Modify properties of environments.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} dtlEnvironment An environment, which is essentially an ARM
     * template deployment.
     *
     * @param {object} [dtlEnvironment.deploymentProperties] The deployment
     * properties of the environment.
     *
     * @param {string} [dtlEnvironment.deploymentProperties.armTemplateId] The
     * Azure Resource Manager template's identifier.
     *
     * @param {array} [dtlEnvironment.deploymentProperties.parameters] The
     * parameters of the Azure Resource Manager template.
     *
     * @param {string} [dtlEnvironment.armTemplateDisplayName] The display name of
     * the Azure Resource Manager template that produced the environment.
     *
     * @param {object} [dtlEnvironment.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DtlEnvironment} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DtlEnvironment} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DtlEnvironment} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironmentFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DtlEnvironment>;
    update(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironmentFragment, callback: ServiceCallback<models.DtlEnvironment>): void;
    update(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironmentFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DtlEnvironment>): void;


    /**
     * Create or replace an existing environment. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} dtlEnvironment An environment, which is essentially an ARM
     * template deployment.
     *
     * @param {object} [dtlEnvironment.deploymentProperties] The deployment
     * properties of the environment.
     *
     * @param {string} [dtlEnvironment.deploymentProperties.armTemplateId] The
     * Azure Resource Manager template's identifier.
     *
     * @param {array} [dtlEnvironment.deploymentProperties.parameters] The
     * parameters of the Azure Resource Manager template.
     *
     * @param {string} [dtlEnvironment.armTemplateDisplayName] The display name of
     * the Azure Resource Manager template that produced the environment.
     *
     * @param {string} [dtlEnvironment.location] The location of the resource.
     *
     * @param {object} [dtlEnvironment.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DtlEnvironment>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DtlEnvironment>>;

    /**
     * Create or replace an existing environment. This operation can take a while
     * to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} dtlEnvironment An environment, which is essentially an ARM
     * template deployment.
     *
     * @param {object} [dtlEnvironment.deploymentProperties] The deployment
     * properties of the environment.
     *
     * @param {string} [dtlEnvironment.deploymentProperties.armTemplateId] The
     * Azure Resource Manager template's identifier.
     *
     * @param {array} [dtlEnvironment.deploymentProperties.parameters] The
     * parameters of the Azure Resource Manager template.
     *
     * @param {string} [dtlEnvironment.armTemplateDisplayName] The display name of
     * the Azure Resource Manager template that produced the environment.
     *
     * @param {string} [dtlEnvironment.location] The location of the resource.
     *
     * @param {object} [dtlEnvironment.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DtlEnvironment} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DtlEnvironment} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DtlEnvironment} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DtlEnvironment>;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironment, callback: ServiceCallback<models.DtlEnvironment>): void;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, dtlEnvironment: models.DtlEnvironment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DtlEnvironment>): void;


    /**
     * Delete environment. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete environment. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the environment.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List environments in a given user profile.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DtlEnvironmentList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DtlEnvironmentList>>;

    /**
     * List environments in a given user profile.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DtlEnvironmentList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DtlEnvironmentList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DtlEnvironmentList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DtlEnvironmentList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.DtlEnvironmentList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DtlEnvironmentList>): void;
}

/**
 * @class
 * Secrets
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface Secrets {


    /**
     * List secrets in a given user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=value)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SecretList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SecretList>>;

    /**
     * List secrets in a given user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=value)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SecretList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SecretList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link SecretList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, userName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.SecretList>;
    list(resourceGroupName: string, labName: string, userName: string, callback: ServiceCallback<models.SecretList>): void;
    list(resourceGroupName: string, labName: string, userName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SecretList>): void;


    /**
     * Get secret.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=value)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Secret>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Secret>>;

    /**
     * Get secret.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=value)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Secret} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Secret} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Secret} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, userName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Secret>;
    get(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<models.Secret>): void;
    get(resourceGroupName: string, labName: string, userName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Secret>): void;


    /**
     * Create or replace an existing secret. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} secret A secret.
     *
     * @param {string} [secret.value] The value of the secret for secret creation.
     *
     * @param {string} [secret.location] The location of the resource.
     *
     * @param {object} [secret.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Secret>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.Secret, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Secret>>;

    /**
     * Create or replace an existing secret. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} secret A secret.
     *
     * @param {string} [secret.value] The value of the secret for secret creation.
     *
     * @param {string} [secret.location] The location of the resource.
     *
     * @param {object} [secret.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Secret} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Secret} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Secret} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.Secret, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Secret>;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.Secret, callback: ServiceCallback<models.Secret>): void;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.Secret, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Secret>): void;


    /**
     * Delete secret.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete secret.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of secrets.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} secret A secret.
     *
     * @param {string} [secret.value] The value of the secret for secret creation.
     *
     * @param {object} [secret.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Secret>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.SecretFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Secret>>;

    /**
     * Modify properties of secrets.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} secret A secret.
     *
     * @param {string} [secret.value] The value of the secret for secret creation.
     *
     * @param {object} [secret.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Secret} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Secret} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Secret} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.SecretFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Secret>;
    update(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.SecretFragment, callback: ServiceCallback<models.Secret>): void;
    update(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.SecretFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Secret>): void;


    /**
     * Create or replace an existing secret. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} secret A secret.
     *
     * @param {string} [secret.value] The value of the secret for secret creation.
     *
     * @param {string} [secret.location] The location of the resource.
     *
     * @param {object} [secret.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Secret>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.Secret, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Secret>>;

    /**
     * Create or replace an existing secret. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the secret.
     *
     * @param {object} secret A secret.
     *
     * @param {string} [secret.value] The value of the secret for secret creation.
     *
     * @param {string} [secret.location] The location of the resource.
     *
     * @param {object} [secret.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Secret} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Secret} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Secret} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.Secret, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Secret>;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.Secret, callback: ServiceCallback<models.Secret>): void;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, secret: models.Secret, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Secret>): void;


    /**
     * List secrets in a given user profile.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SecretList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SecretList>>;

    /**
     * List secrets in a given user profile.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SecretList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SecretList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link SecretList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.SecretList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.SecretList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SecretList>): void;
}

/**
 * @class
 * ServiceFabrics
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface ServiceFabrics {


    /**
     * List service fabrics in a given user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=applicableSchedule)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceFabricList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceFabricList>>;

    /**
     * List service fabrics in a given user profile.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=applicableSchedule)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceFabricList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceFabricList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceFabricList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, userName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceFabricList>;
    list(resourceGroupName: string, labName: string, userName: string, callback: ServiceCallback<models.ServiceFabricList>): void;
    list(resourceGroupName: string, labName: string, userName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceFabricList>): void;


    /**
     * Get service fabric.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=applicableSchedule)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceFabric>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceFabric>>;

    /**
     * Get service fabric.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=applicableSchedule)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceFabric} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceFabric} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceFabric} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, userName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceFabric>;
    get(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<models.ServiceFabric>): void;
    get(resourceGroupName: string, labName: string, userName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceFabric>): void;


    /**
     * Create or replace an existing Service Fabric. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} serviceFabric A Service Fabric.
     *
     * @param {string} [serviceFabric.externalServiceFabricId] The backing service
     * fabric resource's id
     *
     * @param {string} [serviceFabric.environmentId] The resource id of the
     * environment under which the service fabric resource is present
     *
     * @param {string} [serviceFabric.location] The location of the resource.
     *
     * @param {object} [serviceFabric.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceFabric>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabric, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceFabric>>;

    /**
     * Create or replace an existing Service Fabric. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} serviceFabric A Service Fabric.
     *
     * @param {string} [serviceFabric.externalServiceFabricId] The backing service
     * fabric resource's id
     *
     * @param {string} [serviceFabric.environmentId] The resource id of the
     * environment under which the service fabric resource is present
     *
     * @param {string} [serviceFabric.location] The location of the resource.
     *
     * @param {object} [serviceFabric.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceFabric} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceFabric} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceFabric} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabric, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceFabric>;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabric, callback: ServiceCallback<models.ServiceFabric>): void;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabric, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceFabric>): void;


    /**
     * Delete service fabric. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete service fabric. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of service fabrics.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} serviceFabric A Service Fabric.
     *
     * @param {string} [serviceFabric.externalServiceFabricId] The backing service
     * fabric resource's id
     *
     * @param {string} [serviceFabric.environmentId] The resource id of the
     * environment under which the service fabric resource is present
     *
     * @param {object} [serviceFabric.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceFabric>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabricFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceFabric>>;

    /**
     * Modify properties of service fabrics.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} serviceFabric A Service Fabric.
     *
     * @param {string} [serviceFabric.externalServiceFabricId] The backing service
     * fabric resource's id
     *
     * @param {string} [serviceFabric.environmentId] The resource id of the
     * environment under which the service fabric resource is present
     *
     * @param {object} [serviceFabric.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceFabric} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceFabric} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceFabric} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabricFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceFabric>;
    update(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabricFragment, callback: ServiceCallback<models.ServiceFabric>): void;
    update(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabricFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceFabric>): void;


    /**
     * Lists the applicable start/stop schedules, if any.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ApplicableSchedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listApplicableSchedulesWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ApplicableSchedule>>;

    /**
     * Lists the applicable start/stop schedules, if any.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ApplicableSchedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ApplicableSchedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ApplicableSchedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listApplicableSchedules(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ApplicableSchedule>;
    listApplicableSchedules(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<models.ApplicableSchedule>): void;
    listApplicableSchedules(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ApplicableSchedule>): void;


    /**
     * Start a service fabric. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    startWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Start a service fabric. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    start(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    start(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    start(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Stop a service fabric This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    stopWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Stop a service fabric This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    stop(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    stop(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    stop(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Create or replace an existing Service Fabric. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} serviceFabric A Service Fabric.
     *
     * @param {string} [serviceFabric.externalServiceFabricId] The backing service
     * fabric resource's id
     *
     * @param {string} [serviceFabric.environmentId] The resource id of the
     * environment under which the service fabric resource is present
     *
     * @param {string} [serviceFabric.location] The location of the resource.
     *
     * @param {object} [serviceFabric.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceFabric>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabric, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceFabric>>;

    /**
     * Create or replace an existing Service Fabric. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} serviceFabric A Service Fabric.
     *
     * @param {string} [serviceFabric.externalServiceFabricId] The backing service
     * fabric resource's id
     *
     * @param {string} [serviceFabric.environmentId] The resource id of the
     * environment under which the service fabric resource is present
     *
     * @param {string} [serviceFabric.location] The location of the resource.
     *
     * @param {object} [serviceFabric.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceFabric} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceFabric} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceFabric} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabric, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceFabric>;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabric, callback: ServiceCallback<models.ServiceFabric>): void;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, userName: string, name: string, serviceFabric: models.ServiceFabric, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceFabric>): void;


    /**
     * Delete service fabric. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete service fabric. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Start a service fabric. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginStartWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Start a service fabric. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginStart(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginStart(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    beginStart(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Stop a service fabric This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginStopWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Stop a service fabric This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} name The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginStop(resourceGroupName: string, labName: string, userName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginStop(resourceGroupName: string, labName: string, userName: string, name: string, callback: ServiceCallback<void>): void;
    beginStop(resourceGroupName: string, labName: string, userName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List service fabrics in a given user profile.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ServiceFabricList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ServiceFabricList>>;

    /**
     * List service fabrics in a given user profile.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ServiceFabricList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ServiceFabricList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ServiceFabricList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ServiceFabricList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ServiceFabricList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ServiceFabricList>): void;
}

/**
 * @class
 * ServiceFabricSchedules
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface ServiceFabricSchedules {


    /**
     * List schedules in a given service fabric.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a given service fabric.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    list(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, callback: ServiceCallback<models.ScheduleList>): void;
    list(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;


    /**
     * Get schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Get schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    get(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, callback: ServiceCallback<models.Schedule>): void;
    get(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Create or replace an existing schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {string} [schedule.location] The location of the resource.
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, schedule: models.Schedule, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Create or replace an existing schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {string} [schedule.location] The location of the resource.
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, schedule: models.Schedule, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, schedule: models.Schedule, callback: ServiceCallback<models.Schedule>): void;
    createOrUpdate(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, schedule: models.Schedule, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Delete schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of schedules.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, schedule: models.ScheduleFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Modify properties of schedules.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, schedule: models.ScheduleFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    update(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, schedule: models.ScheduleFragment, callback: ServiceCallback<models.Schedule>): void;
    update(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, schedule: models.ScheduleFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    executeWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    execute(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    execute(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, callback: ServiceCallback<void>): void;
    execute(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginExecuteWithHttpOperationResponse(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} userName The name of the user profile.
     *
     * @param {string} serviceFabricName The name of the service Fabric.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginExecute(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginExecute(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, callback: ServiceCallback<void>): void;
    beginExecute(resourceGroupName: string, labName: string, userName: string, serviceFabricName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List schedules in a given service fabric.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a given service fabric.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ScheduleList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;
}

/**
 * @class
 * VirtualMachines
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface VirtualMachines {


    /**
     * List virtual machines in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=artifacts,computeVm,networkInterface,applicableSchedule)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabVirtualMachineList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabVirtualMachineList>>;

    /**
     * List virtual machines in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=artifacts,computeVm,networkInterface,applicableSchedule)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabVirtualMachineList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabVirtualMachineList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabVirtualMachineList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.LabVirtualMachineList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.LabVirtualMachineList>): void;
    list(resourceGroupName: string, labName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabVirtualMachineList>): void;


    /**
     * Get virtual machine.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=artifacts,computeVm,networkInterface,applicableSchedule)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabVirtualMachine>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabVirtualMachine>>;

    /**
     * Get virtual machine.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=artifacts,computeVm,networkInterface,applicableSchedule)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabVirtualMachine} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabVirtualMachine} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabVirtualMachine} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.LabVirtualMachine>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.LabVirtualMachine>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabVirtualMachine>): void;


    /**
     * Create or replace an existing Virtual machine. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} labVirtualMachine A virtual machine.
     *
     * @param {string} [labVirtualMachine.notes] The notes of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerObjectId] The object identifier of
     * the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachine.createdByUserId] The object identifier of
     * the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachine.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [labVirtualMachine.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachine.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachine.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.size] The size of the virtual machine.
     *
     * @param {string} [labVirtualMachine.userName] The user name of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.password] The password of the virtual
     * machine administrator.
     *
     * @param {string} [labVirtualMachine.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [labVirtualMachine.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachine.fqdn] The fully-qualified domain name of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachine.labSubnetName] The lab subnet name of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [labVirtualMachine.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachine.artifacts] The artifacts to be installed
     * on the virtual machine.
     *
     * @param {object} [labVirtualMachine.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachine.artifactDeploymentStatus.deploymentStatus] The deployment
     * status of the artifact.
     *
     * @param {number}
     * [labVirtualMachine.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number} [labVirtualMachine.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachine.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.offer] The offer of
     * the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.sku] The SKU of the
     * gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.osType] The OS type
     * of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachine.planId] The id of the plan associated
     * with the virtual machine image
     *
     * @param {object} [labVirtualMachine.networkInterface] The network interface
     * properties.
     *
     * @param {string} [labVirtualMachine.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string} [labVirtualMachine.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.dnsName] The DNS name.
     *
     * @param {string} [labVirtualMachine.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [labVirtualMachine.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration] The
     * configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachine.expirationDate] The expiration date for VM.
     *
     * @param {boolean} [labVirtualMachine.allowClaim] Indicates whether another
     * user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachine.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [labVirtualMachine.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachine.environmentId] The resource ID of the
     * environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachine.dataDiskParameters] New or existing data
     * disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachine.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [labVirtualMachine.lastKnownPowerState] Last known compute
     * power state captured in DTL
     *
     * @param {string} [labVirtualMachine.location] The location of the resource.
     *
     * @param {object} [labVirtualMachine.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabVirtualMachine>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachine, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabVirtualMachine>>;

    /**
     * Create or replace an existing Virtual machine. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} labVirtualMachine A virtual machine.
     *
     * @param {string} [labVirtualMachine.notes] The notes of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerObjectId] The object identifier of
     * the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachine.createdByUserId] The object identifier of
     * the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachine.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [labVirtualMachine.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachine.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachine.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.size] The size of the virtual machine.
     *
     * @param {string} [labVirtualMachine.userName] The user name of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.password] The password of the virtual
     * machine administrator.
     *
     * @param {string} [labVirtualMachine.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [labVirtualMachine.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachine.fqdn] The fully-qualified domain name of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachine.labSubnetName] The lab subnet name of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [labVirtualMachine.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachine.artifacts] The artifacts to be installed
     * on the virtual machine.
     *
     * @param {object} [labVirtualMachine.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachine.artifactDeploymentStatus.deploymentStatus] The deployment
     * status of the artifact.
     *
     * @param {number}
     * [labVirtualMachine.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number} [labVirtualMachine.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachine.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.offer] The offer of
     * the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.sku] The SKU of the
     * gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.osType] The OS type
     * of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachine.planId] The id of the plan associated
     * with the virtual machine image
     *
     * @param {object} [labVirtualMachine.networkInterface] The network interface
     * properties.
     *
     * @param {string} [labVirtualMachine.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string} [labVirtualMachine.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.dnsName] The DNS name.
     *
     * @param {string} [labVirtualMachine.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [labVirtualMachine.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration] The
     * configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachine.expirationDate] The expiration date for VM.
     *
     * @param {boolean} [labVirtualMachine.allowClaim] Indicates whether another
     * user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachine.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [labVirtualMachine.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachine.environmentId] The resource ID of the
     * environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachine.dataDiskParameters] New or existing data
     * disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachine.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [labVirtualMachine.lastKnownPowerState] Last known compute
     * power state captured in DTL
     *
     * @param {string} [labVirtualMachine.location] The location of the resource.
     *
     * @param {object} [labVirtualMachine.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabVirtualMachine} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabVirtualMachine} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabVirtualMachine} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachine, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.LabVirtualMachine>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachine, callback: ServiceCallback<models.LabVirtualMachine>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachine, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabVirtualMachine>): void;


    /**
     * Delete virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of virtual machines.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} labVirtualMachine A virtual machine.
     *
     * @param {string} [labVirtualMachine.notes] The notes of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerObjectId] The object identifier of
     * the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachine.createdByUserId] The object identifier of
     * the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachine.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [labVirtualMachine.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachine.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachine.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.size] The size of the virtual machine.
     *
     * @param {string} [labVirtualMachine.userName] The user name of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.password] The password of the virtual
     * machine administrator.
     *
     * @param {string} [labVirtualMachine.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [labVirtualMachine.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachine.fqdn] The fully-qualified domain name of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachine.labSubnetName] The lab subnet name of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [labVirtualMachine.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachine.artifacts] The artifacts to be installed
     * on the virtual machine.
     *
     * @param {object} [labVirtualMachine.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachine.artifactDeploymentStatus.deploymentStatus] The deployment
     * status of the artifact.
     *
     * @param {number}
     * [labVirtualMachine.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number} [labVirtualMachine.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachine.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.offer] The offer of
     * the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.sku] The SKU of the
     * gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.osType] The OS type
     * of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachine.planId] The id of the plan associated
     * with the virtual machine image
     *
     * @param {object} [labVirtualMachine.networkInterface] The network interface
     * properties.
     *
     * @param {string} [labVirtualMachine.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string} [labVirtualMachine.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.dnsName] The DNS name.
     *
     * @param {string} [labVirtualMachine.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [labVirtualMachine.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration] The
     * configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachine.expirationDate] The expiration date for VM.
     *
     * @param {boolean} [labVirtualMachine.allowClaim] Indicates whether another
     * user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachine.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [labVirtualMachine.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachine.environmentId] The resource ID of the
     * environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachine.dataDiskParameters] New or existing data
     * disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachine.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [labVirtualMachine.lastKnownPowerState] Last known compute
     * power state captured in DTL
     *
     * @param {object} [labVirtualMachine.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabVirtualMachine>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachineFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabVirtualMachine>>;

    /**
     * Modify properties of virtual machines.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} labVirtualMachine A virtual machine.
     *
     * @param {string} [labVirtualMachine.notes] The notes of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerObjectId] The object identifier of
     * the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachine.createdByUserId] The object identifier of
     * the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachine.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [labVirtualMachine.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachine.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachine.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.size] The size of the virtual machine.
     *
     * @param {string} [labVirtualMachine.userName] The user name of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.password] The password of the virtual
     * machine administrator.
     *
     * @param {string} [labVirtualMachine.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [labVirtualMachine.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachine.fqdn] The fully-qualified domain name of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachine.labSubnetName] The lab subnet name of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [labVirtualMachine.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachine.artifacts] The artifacts to be installed
     * on the virtual machine.
     *
     * @param {object} [labVirtualMachine.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachine.artifactDeploymentStatus.deploymentStatus] The deployment
     * status of the artifact.
     *
     * @param {number}
     * [labVirtualMachine.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number} [labVirtualMachine.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachine.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.offer] The offer of
     * the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.sku] The SKU of the
     * gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.osType] The OS type
     * of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachine.planId] The id of the plan associated
     * with the virtual machine image
     *
     * @param {object} [labVirtualMachine.networkInterface] The network interface
     * properties.
     *
     * @param {string} [labVirtualMachine.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string} [labVirtualMachine.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.dnsName] The DNS name.
     *
     * @param {string} [labVirtualMachine.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [labVirtualMachine.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration] The
     * configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachine.expirationDate] The expiration date for VM.
     *
     * @param {boolean} [labVirtualMachine.allowClaim] Indicates whether another
     * user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachine.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [labVirtualMachine.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachine.environmentId] The resource ID of the
     * environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachine.dataDiskParameters] New or existing data
     * disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachine.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [labVirtualMachine.lastKnownPowerState] Last known compute
     * power state captured in DTL
     *
     * @param {object} [labVirtualMachine.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabVirtualMachine} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabVirtualMachine} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabVirtualMachine} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachineFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.LabVirtualMachine>;
    update(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachineFragment, callback: ServiceCallback<models.LabVirtualMachine>): void;
    update(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachineFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabVirtualMachine>): void;


    /**
     * Attach a new or existing data disk to virtual machine. This operation can
     * take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} dataDiskProperties Request body for adding a new or existing
     * data disk to a virtual machine.
     *
     * @param {object} [dataDiskProperties.attachNewDataDiskOptions] Specifies
     * options to attach a new disk to the virtual machine.
     *
     * @param {number} [dataDiskProperties.attachNewDataDiskOptions.diskSizeGiB]
     * Size of the disk to be attached in Gibibytes.
     *
     * @param {string} [dataDiskProperties.attachNewDataDiskOptions.diskName] The
     * name of the disk to be attached.
     *
     * @param {string} [dataDiskProperties.attachNewDataDiskOptions.diskType] The
     * storage type for the disk (i.e. Standard, Premium). Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {string} [dataDiskProperties.existingLabDiskId] Specifies the
     * existing lab disk id to attach to virtual machine.
     *
     * @param {string} [dataDiskProperties.hostCaching] Caching option for a data
     * disk (i.e. None, ReadOnly, ReadWrite). Possible values include: 'None',
     * 'ReadOnly', 'ReadWrite'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    addDataDiskWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, dataDiskProperties: models.DataDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Attach a new or existing data disk to virtual machine. This operation can
     * take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} dataDiskProperties Request body for adding a new or existing
     * data disk to a virtual machine.
     *
     * @param {object} [dataDiskProperties.attachNewDataDiskOptions] Specifies
     * options to attach a new disk to the virtual machine.
     *
     * @param {number} [dataDiskProperties.attachNewDataDiskOptions.diskSizeGiB]
     * Size of the disk to be attached in Gibibytes.
     *
     * @param {string} [dataDiskProperties.attachNewDataDiskOptions.diskName] The
     * name of the disk to be attached.
     *
     * @param {string} [dataDiskProperties.attachNewDataDiskOptions.diskType] The
     * storage type for the disk (i.e. Standard, Premium). Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {string} [dataDiskProperties.existingLabDiskId] Specifies the
     * existing lab disk id to attach to virtual machine.
     *
     * @param {string} [dataDiskProperties.hostCaching] Caching option for a data
     * disk (i.e. None, ReadOnly, ReadWrite). Possible values include: 'None',
     * 'ReadOnly', 'ReadWrite'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    addDataDisk(resourceGroupName: string, labName: string, name: string, dataDiskProperties: models.DataDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    addDataDisk(resourceGroupName: string, labName: string, name: string, dataDiskProperties: models.DataDiskProperties, callback: ServiceCallback<void>): void;
    addDataDisk(resourceGroupName: string, labName: string, name: string, dataDiskProperties: models.DataDiskProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Apply artifacts to virtual machine. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} applyArtifactsRequest Request body for applying artifacts to
     * a virtual machine.
     *
     * @param {array} [applyArtifactsRequest.artifacts] The list of artifacts to
     * apply.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    applyArtifactsWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, applyArtifactsRequest: models.ApplyArtifactsRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Apply artifacts to virtual machine. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} applyArtifactsRequest Request body for applying artifacts to
     * a virtual machine.
     *
     * @param {array} [applyArtifactsRequest.artifacts] The list of artifacts to
     * apply.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    applyArtifacts(resourceGroupName: string, labName: string, name: string, applyArtifactsRequest: models.ApplyArtifactsRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    applyArtifacts(resourceGroupName: string, labName: string, name: string, applyArtifactsRequest: models.ApplyArtifactsRequest, callback: ServiceCallback<void>): void;
    applyArtifacts(resourceGroupName: string, labName: string, name: string, applyArtifactsRequest: models.ApplyArtifactsRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Take ownership of an existing virtual machine This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    claimWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Take ownership of an existing virtual machine This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    claim(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    claim(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    claim(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Detach the specified disk from the virtual machine. This operation can take
     * a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} detachDataDiskProperties Request body for detaching data
     * disk from a virtual machine.
     *
     * @param {string} [detachDataDiskProperties.existingLabDiskId] Specifies the
     * disk resource ID to detach from virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    detachDataDiskWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, detachDataDiskProperties: models.DetachDataDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Detach the specified disk from the virtual machine. This operation can take
     * a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} detachDataDiskProperties Request body for detaching data
     * disk from a virtual machine.
     *
     * @param {string} [detachDataDiskProperties.existingLabDiskId] Specifies the
     * disk resource ID to detach from virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    detachDataDisk(resourceGroupName: string, labName: string, name: string, detachDataDiskProperties: models.DetachDataDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    detachDataDisk(resourceGroupName: string, labName: string, name: string, detachDataDiskProperties: models.DetachDataDiskProperties, callback: ServiceCallback<void>): void;
    detachDataDisk(resourceGroupName: string, labName: string, name: string, detachDataDiskProperties: models.DetachDataDiskProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Gets a string that represents the contents of the RDP file for the virtual
     * machine
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RdpConnection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getRdpFileContentsWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RdpConnection>>;

    /**
     * Gets a string that represents the contents of the RDP file for the virtual
     * machine
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RdpConnection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RdpConnection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RdpConnection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getRdpFileContents(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RdpConnection>;
    getRdpFileContents(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.RdpConnection>): void;
    getRdpFileContents(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RdpConnection>): void;


    /**
     * Lists the applicable start/stop schedules, if any.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ApplicableSchedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listApplicableSchedulesWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ApplicableSchedule>>;

    /**
     * Lists the applicable start/stop schedules, if any.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ApplicableSchedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ApplicableSchedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ApplicableSchedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listApplicableSchedules(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ApplicableSchedule>;
    listApplicableSchedules(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.ApplicableSchedule>): void;
    listApplicableSchedules(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ApplicableSchedule>): void;


    /**
     * Redeploy a virtual machine This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    redeployWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Redeploy a virtual machine This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    redeploy(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    redeploy(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    redeploy(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Resize Virtual Machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} resizeLabVirtualMachineProperties Request body for resizing
     * a virtual machine.
     *
     * @param {string} [resizeLabVirtualMachineProperties.size] Specifies the size
     * of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    resizeWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, resizeLabVirtualMachineProperties: models.ResizeLabVirtualMachineProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Resize Virtual Machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} resizeLabVirtualMachineProperties Request body for resizing
     * a virtual machine.
     *
     * @param {string} [resizeLabVirtualMachineProperties.size] Specifies the size
     * of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    resize(resourceGroupName: string, labName: string, name: string, resizeLabVirtualMachineProperties: models.ResizeLabVirtualMachineProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    resize(resourceGroupName: string, labName: string, name: string, resizeLabVirtualMachineProperties: models.ResizeLabVirtualMachineProperties, callback: ServiceCallback<void>): void;
    resize(resourceGroupName: string, labName: string, name: string, resizeLabVirtualMachineProperties: models.ResizeLabVirtualMachineProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Restart a virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    restartWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Restart a virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    restart(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    restart(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    restart(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Start a virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    startWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Start a virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    start(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    start(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    start(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Stop a virtual machine This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    stopWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Stop a virtual machine This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    stop(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    stop(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    stop(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Transfers all data disks attached to the virtual machine to be owned by the
     * current user. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    transferDisksWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Transfers all data disks attached to the virtual machine to be owned by the
     * current user. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    transferDisks(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    transferDisks(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    transferDisks(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Release ownership of an existing virtual machine This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    unClaimWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Release ownership of an existing virtual machine This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    unClaim(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    unClaim(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    unClaim(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Create or replace an existing Virtual machine. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} labVirtualMachine A virtual machine.
     *
     * @param {string} [labVirtualMachine.notes] The notes of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerObjectId] The object identifier of
     * the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachine.createdByUserId] The object identifier of
     * the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachine.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [labVirtualMachine.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachine.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachine.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.size] The size of the virtual machine.
     *
     * @param {string} [labVirtualMachine.userName] The user name of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.password] The password of the virtual
     * machine administrator.
     *
     * @param {string} [labVirtualMachine.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [labVirtualMachine.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachine.fqdn] The fully-qualified domain name of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachine.labSubnetName] The lab subnet name of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [labVirtualMachine.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachine.artifacts] The artifacts to be installed
     * on the virtual machine.
     *
     * @param {object} [labVirtualMachine.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachine.artifactDeploymentStatus.deploymentStatus] The deployment
     * status of the artifact.
     *
     * @param {number}
     * [labVirtualMachine.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number} [labVirtualMachine.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachine.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.offer] The offer of
     * the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.sku] The SKU of the
     * gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.osType] The OS type
     * of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachine.planId] The id of the plan associated
     * with the virtual machine image
     *
     * @param {object} [labVirtualMachine.networkInterface] The network interface
     * properties.
     *
     * @param {string} [labVirtualMachine.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string} [labVirtualMachine.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.dnsName] The DNS name.
     *
     * @param {string} [labVirtualMachine.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [labVirtualMachine.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration] The
     * configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachine.expirationDate] The expiration date for VM.
     *
     * @param {boolean} [labVirtualMachine.allowClaim] Indicates whether another
     * user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachine.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [labVirtualMachine.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachine.environmentId] The resource ID of the
     * environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachine.dataDiskParameters] New or existing data
     * disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachine.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [labVirtualMachine.lastKnownPowerState] Last known compute
     * power state captured in DTL
     *
     * @param {string} [labVirtualMachine.location] The location of the resource.
     *
     * @param {object} [labVirtualMachine.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabVirtualMachine>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachine, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabVirtualMachine>>;

    /**
     * Create or replace an existing Virtual machine. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} labVirtualMachine A virtual machine.
     *
     * @param {string} [labVirtualMachine.notes] The notes of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerObjectId] The object identifier of
     * the owner of the virtual machine.
     *
     * @param {string} [labVirtualMachine.ownerUserPrincipalName] The user
     * principal name of the virtual machine owner.
     *
     * @param {string} [labVirtualMachine.createdByUserId] The object identifier of
     * the creator of the virtual machine.
     *
     * @param {string} [labVirtualMachine.createdByUser] The email address of
     * creator of the virtual machine.
     *
     * @param {date} [labVirtualMachine.createdDate] The creation date of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.computeId] The resource identifier
     * (Microsoft.Compute) of the virtual machine.
     *
     * @param {string} [labVirtualMachine.customImageId] The custom image
     * identifier of the virtual machine.
     *
     * @param {string} [labVirtualMachine.osType] The OS type of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.size] The size of the virtual machine.
     *
     * @param {string} [labVirtualMachine.userName] The user name of the virtual
     * machine.
     *
     * @param {string} [labVirtualMachine.password] The password of the virtual
     * machine administrator.
     *
     * @param {string} [labVirtualMachine.sshKey] The SSH key of the virtual
     * machine administrator.
     *
     * @param {boolean} [labVirtualMachine.isAuthenticationWithSshKey] Indicates
     * whether this virtual machine uses an SSH key for authentication.
     *
     * @param {string} [labVirtualMachine.fqdn] The fully-qualified domain name of
     * the virtual machine.
     *
     * @param {string} [labVirtualMachine.labSubnetName] The lab subnet name of the
     * virtual machine.
     *
     * @param {string} [labVirtualMachine.labVirtualNetworkId] The lab virtual
     * network identifier of the virtual machine.
     *
     * @param {boolean} [labVirtualMachine.disallowPublicIpAddress] Indicates
     * whether the virtual machine is to be created without a public IP address.
     *
     * @param {array} [labVirtualMachine.artifacts] The artifacts to be installed
     * on the virtual machine.
     *
     * @param {object} [labVirtualMachine.artifactDeploymentStatus] The artifact
     * deployment status for the virtual machine.
     *
     * @param {string}
     * [labVirtualMachine.artifactDeploymentStatus.deploymentStatus] The deployment
     * status of the artifact.
     *
     * @param {number}
     * [labVirtualMachine.artifactDeploymentStatus.artifactsApplied] The total
     * count of the artifacts that were successfully applied.
     *
     * @param {number} [labVirtualMachine.artifactDeploymentStatus.totalArtifacts]
     * The total count of the artifacts that were tentatively applied.
     *
     * @param {object} [labVirtualMachine.galleryImageReference] The Microsoft
     * Azure Marketplace image reference of the virtual machine.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.offer] The offer of
     * the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.publisher] The
     * publisher of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.sku] The SKU of the
     * gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.osType] The OS type
     * of the gallery image.
     *
     * @param {string} [labVirtualMachine.galleryImageReference.version] The
     * version of the gallery image.
     *
     * @param {string} [labVirtualMachine.planId] The id of the plan associated
     * with the virtual machine image
     *
     * @param {object} [labVirtualMachine.networkInterface] The network interface
     * properties.
     *
     * @param {string} [labVirtualMachine.networkInterface.virtualNetworkId] The
     * resource ID of the virtual network.
     *
     * @param {string} [labVirtualMachine.networkInterface.subnetId] The resource
     * ID of the sub net.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddressId] The
     * resource ID of the public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.publicIpAddress] The
     * public IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.privateIpAddress] The
     * private IP address.
     *
     * @param {string} [labVirtualMachine.networkInterface.dnsName] The DNS name.
     *
     * @param {string} [labVirtualMachine.networkInterface.rdpAuthority] The
     * RdpAuthority property is a server DNS host name or IP address followed by
     * the service port number for RDP (Remote Desktop Protocol).
     *
     * @param {string} [labVirtualMachine.networkInterface.sshAuthority] The
     * SshAuthority property is a server DNS host name or IP address followed by
     * the service port number for SSH.
     *
     * @param {object}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration] The
     * configuration for sharing a public IP address across multiple virtual
     * machines.
     *
     * @param {array}
     * [labVirtualMachine.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
     * The incoming NAT rules
     *
     * @param {date} [labVirtualMachine.expirationDate] The expiration date for VM.
     *
     * @param {boolean} [labVirtualMachine.allowClaim] Indicates whether another
     * user can take ownership of the virtual machine
     *
     * @param {string} [labVirtualMachine.storageType] Storage type to use for
     * virtual machine (i.e. Standard, Premium).
     *
     * @param {string} [labVirtualMachine.virtualMachineCreationSource] Tells
     * source of creation of lab virtual machine. Output property only. Possible
     * values include: 'FromCustomImage', 'FromGalleryImage'
     *
     * @param {string} [labVirtualMachine.environmentId] The resource ID of the
     * environment that contains this virtual machine, if any.
     *
     * @param {array} [labVirtualMachine.dataDiskParameters] New or existing data
     * disks to attach to the virtual machine after creation
     *
     * @param {array} [labVirtualMachine.scheduleParameters] Virtual Machine
     * schedules to be created
     *
     * @param {string} [labVirtualMachine.lastKnownPowerState] Last known compute
     * power state captured in DTL
     *
     * @param {string} [labVirtualMachine.location] The location of the resource.
     *
     * @param {object} [labVirtualMachine.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabVirtualMachine} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabVirtualMachine} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabVirtualMachine} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachine, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.LabVirtualMachine>;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachine, callback: ServiceCallback<models.LabVirtualMachine>): void;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, labVirtualMachine: models.LabVirtualMachine, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabVirtualMachine>): void;


    /**
     * Delete virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Attach a new or existing data disk to virtual machine. This operation can
     * take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} dataDiskProperties Request body for adding a new or existing
     * data disk to a virtual machine.
     *
     * @param {object} [dataDiskProperties.attachNewDataDiskOptions] Specifies
     * options to attach a new disk to the virtual machine.
     *
     * @param {number} [dataDiskProperties.attachNewDataDiskOptions.diskSizeGiB]
     * Size of the disk to be attached in Gibibytes.
     *
     * @param {string} [dataDiskProperties.attachNewDataDiskOptions.diskName] The
     * name of the disk to be attached.
     *
     * @param {string} [dataDiskProperties.attachNewDataDiskOptions.diskType] The
     * storage type for the disk (i.e. Standard, Premium). Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {string} [dataDiskProperties.existingLabDiskId] Specifies the
     * existing lab disk id to attach to virtual machine.
     *
     * @param {string} [dataDiskProperties.hostCaching] Caching option for a data
     * disk (i.e. None, ReadOnly, ReadWrite). Possible values include: 'None',
     * 'ReadOnly', 'ReadWrite'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginAddDataDiskWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, dataDiskProperties: models.DataDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Attach a new or existing data disk to virtual machine. This operation can
     * take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} dataDiskProperties Request body for adding a new or existing
     * data disk to a virtual machine.
     *
     * @param {object} [dataDiskProperties.attachNewDataDiskOptions] Specifies
     * options to attach a new disk to the virtual machine.
     *
     * @param {number} [dataDiskProperties.attachNewDataDiskOptions.diskSizeGiB]
     * Size of the disk to be attached in Gibibytes.
     *
     * @param {string} [dataDiskProperties.attachNewDataDiskOptions.diskName] The
     * name of the disk to be attached.
     *
     * @param {string} [dataDiskProperties.attachNewDataDiskOptions.diskType] The
     * storage type for the disk (i.e. Standard, Premium). Possible values include:
     * 'Standard', 'Premium'
     *
     * @param {string} [dataDiskProperties.existingLabDiskId] Specifies the
     * existing lab disk id to attach to virtual machine.
     *
     * @param {string} [dataDiskProperties.hostCaching] Caching option for a data
     * disk (i.e. None, ReadOnly, ReadWrite). Possible values include: 'None',
     * 'ReadOnly', 'ReadWrite'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginAddDataDisk(resourceGroupName: string, labName: string, name: string, dataDiskProperties: models.DataDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginAddDataDisk(resourceGroupName: string, labName: string, name: string, dataDiskProperties: models.DataDiskProperties, callback: ServiceCallback<void>): void;
    beginAddDataDisk(resourceGroupName: string, labName: string, name: string, dataDiskProperties: models.DataDiskProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Apply artifacts to virtual machine. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} applyArtifactsRequest Request body for applying artifacts to
     * a virtual machine.
     *
     * @param {array} [applyArtifactsRequest.artifacts] The list of artifacts to
     * apply.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginApplyArtifactsWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, applyArtifactsRequest: models.ApplyArtifactsRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Apply artifacts to virtual machine. This operation can take a while to
     * complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} applyArtifactsRequest Request body for applying artifacts to
     * a virtual machine.
     *
     * @param {array} [applyArtifactsRequest.artifacts] The list of artifacts to
     * apply.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginApplyArtifacts(resourceGroupName: string, labName: string, name: string, applyArtifactsRequest: models.ApplyArtifactsRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginApplyArtifacts(resourceGroupName: string, labName: string, name: string, applyArtifactsRequest: models.ApplyArtifactsRequest, callback: ServiceCallback<void>): void;
    beginApplyArtifacts(resourceGroupName: string, labName: string, name: string, applyArtifactsRequest: models.ApplyArtifactsRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Take ownership of an existing virtual machine This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginClaimWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Take ownership of an existing virtual machine This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginClaim(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginClaim(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginClaim(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Detach the specified disk from the virtual machine. This operation can take
     * a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} detachDataDiskProperties Request body for detaching data
     * disk from a virtual machine.
     *
     * @param {string} [detachDataDiskProperties.existingLabDiskId] Specifies the
     * disk resource ID to detach from virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDetachDataDiskWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, detachDataDiskProperties: models.DetachDataDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Detach the specified disk from the virtual machine. This operation can take
     * a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} detachDataDiskProperties Request body for detaching data
     * disk from a virtual machine.
     *
     * @param {string} [detachDataDiskProperties.existingLabDiskId] Specifies the
     * disk resource ID to detach from virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDetachDataDisk(resourceGroupName: string, labName: string, name: string, detachDataDiskProperties: models.DetachDataDiskProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDetachDataDisk(resourceGroupName: string, labName: string, name: string, detachDataDiskProperties: models.DetachDataDiskProperties, callback: ServiceCallback<void>): void;
    beginDetachDataDisk(resourceGroupName: string, labName: string, name: string, detachDataDiskProperties: models.DetachDataDiskProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Redeploy a virtual machine This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginRedeployWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Redeploy a virtual machine This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginRedeploy(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginRedeploy(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginRedeploy(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Resize Virtual Machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} resizeLabVirtualMachineProperties Request body for resizing
     * a virtual machine.
     *
     * @param {string} [resizeLabVirtualMachineProperties.size] Specifies the size
     * of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginResizeWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, resizeLabVirtualMachineProperties: models.ResizeLabVirtualMachineProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Resize Virtual Machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} resizeLabVirtualMachineProperties Request body for resizing
     * a virtual machine.
     *
     * @param {string} [resizeLabVirtualMachineProperties.size] Specifies the size
     * of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginResize(resourceGroupName: string, labName: string, name: string, resizeLabVirtualMachineProperties: models.ResizeLabVirtualMachineProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginResize(resourceGroupName: string, labName: string, name: string, resizeLabVirtualMachineProperties: models.ResizeLabVirtualMachineProperties, callback: ServiceCallback<void>): void;
    beginResize(resourceGroupName: string, labName: string, name: string, resizeLabVirtualMachineProperties: models.ResizeLabVirtualMachineProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Restart a virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginRestartWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Restart a virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginRestart(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginRestart(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginRestart(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Start a virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginStartWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Start a virtual machine. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginStart(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginStart(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginStart(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Stop a virtual machine This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginStopWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Stop a virtual machine This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginStop(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginStop(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginStop(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Transfers all data disks attached to the virtual machine to be owned by the
     * current user. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginTransferDisksWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Transfers all data disks attached to the virtual machine to be owned by the
     * current user. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginTransferDisks(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginTransferDisks(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginTransferDisks(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Release ownership of an existing virtual machine This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUnClaimWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Release ownership of an existing virtual machine This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUnClaim(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginUnClaim(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginUnClaim(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List virtual machines in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<LabVirtualMachineList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.LabVirtualMachineList>>;

    /**
     * List virtual machines in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {LabVirtualMachineList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {LabVirtualMachineList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link LabVirtualMachineList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.LabVirtualMachineList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.LabVirtualMachineList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.LabVirtualMachineList>): void;
}

/**
 * @class
 * VirtualMachineSchedules
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface VirtualMachineSchedules {


    /**
     * List schedules in a given virtual machine.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, virtualMachineName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a given virtual machine.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, virtualMachineName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    list(resourceGroupName: string, labName: string, virtualMachineName: string, callback: ServiceCallback<models.ScheduleList>): void;
    list(resourceGroupName: string, labName: string, virtualMachineName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;


    /**
     * Get schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Get schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($select=status)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    get(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, callback: ServiceCallback<models.Schedule>): void;
    get(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Create or replace an existing schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {string} [schedule.location] The location of the resource.
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, schedule: models.Schedule, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Create or replace an existing schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {string} [schedule.location] The location of the resource.
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, schedule: models.Schedule, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    createOrUpdate(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, schedule: models.Schedule, callback: ServiceCallback<models.Schedule>): void;
    createOrUpdate(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, schedule: models.Schedule, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Delete schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete schedule.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of schedules.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Schedule>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, schedule: models.ScheduleFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Schedule>>;

    /**
     * Modify properties of schedules.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} schedule A schedule.
     *
     * @param {string} [schedule.status] The status of the schedule (i.e. Enabled,
     * Disabled). Possible values include: 'Enabled', 'Disabled'
     *
     * @param {string} [schedule.taskType] The task type of the schedule (e.g.
     * LabVmsShutdownTask, LabVmAutoStart).
     *
     * @param {object} [schedule.weeklyRecurrence] If the schedule will occur only
     * some days of the week, specify the weekly recurrence.
     *
     * @param {array} [schedule.weeklyRecurrence.weekdays] The days of the week for
     * which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).
     *
     * @param {string} [schedule.weeklyRecurrence.time] The time of the day the
     * schedule will occur.
     *
     * @param {object} [schedule.dailyRecurrence] If the schedule will occur once
     * each day of the week, specify the daily recurrence.
     *
     * @param {string} [schedule.dailyRecurrence.time] The time of day the schedule
     * will occur.
     *
     * @param {object} [schedule.hourlyRecurrence] If the schedule will occur
     * multiple times a day, specify the hourly recurrence.
     *
     * @param {number} [schedule.hourlyRecurrence.minute] Minutes of the hour the
     * schedule will run.
     *
     * @param {string} [schedule.timeZoneId] The time zone ID (e.g. Pacific
     * Standard time).
     *
     * @param {object} [schedule.notificationSettings] Notification settings.
     *
     * @param {string} [schedule.notificationSettings.status] If notifications are
     * enabled for this schedule (i.e. Enabled, Disabled). Possible values include:
     * 'Enabled', 'Disabled'
     *
     * @param {number} [schedule.notificationSettings.timeInMinutes] Time in
     * minutes before event at which notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.webhookUrl] The webhook URL
     * to which the notification will be sent.
     *
     * @param {string} [schedule.notificationSettings.emailRecipient] The email
     * recipient to send notifications to (can be a list of semi-colon separated
     * email addresses).
     *
     * @param {string} [schedule.notificationSettings.notificationLocale] The
     * locale to use when sending a notification (fallback for unsupported
     * languages is EN).
     *
     * @param {string} [schedule.targetResourceId] The resource ID to which the
     * schedule belongs
     *
     * @param {object} [schedule.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Schedule} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Schedule} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Schedule} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, schedule: models.ScheduleFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Schedule>;
    update(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, schedule: models.ScheduleFragment, callback: ServiceCallback<models.Schedule>): void;
    update(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, schedule: models.ScheduleFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Schedule>): void;


    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    executeWithHttpOperationResponse(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    execute(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    execute(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, callback: ServiceCallback<void>): void;
    execute(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginExecuteWithHttpOperationResponse(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Execute a schedule. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} virtualMachineName The name of the virtual machine.
     *
     * @param {string} name The name of the schedule.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginExecute(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginExecute(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, callback: ServiceCallback<void>): void;
    beginExecute(resourceGroupName: string, labName: string, virtualMachineName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List schedules in a given virtual machine.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScheduleList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScheduleList>>;

    /**
     * List schedules in a given virtual machine.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScheduleList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScheduleList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScheduleList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScheduleList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ScheduleList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScheduleList>): void;
}

/**
 * @class
 * VirtualNetworks
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevTestLabsClient.
 */
export interface VirtualNetworks {


    /**
     * List virtual networks in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=externalSubnets)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VirtualNetworkList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VirtualNetworkList>>;

    /**
     * List virtual networks in a given lab.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=externalSubnets)'
     *
     * @param {string} [options.filter] The filter to apply to the operation.
     * Example: '$filter=contains(name,'myName')
     *
     * @param {number} [options.top] The maximum number of resources to return from
     * the operation. Example: '$top=10'
     *
     * @param {string} [options.orderby] The ordering expression for the results,
     * using OData notation. Example: '$orderby=name desc'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VirtualNetworkList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VirtualNetworkList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VirtualNetworkList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, labName: string, options?: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.VirtualNetworkList>;
    list(resourceGroupName: string, labName: string, callback: ServiceCallback<models.VirtualNetworkList>): void;
    list(resourceGroupName: string, labName: string, options: { expand? : string, filter? : string, top? : number, orderby? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VirtualNetworkList>): void;


    /**
     * Get virtual network.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=externalSubnets)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VirtualNetwork>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VirtualNetwork>>;

    /**
     * Get virtual network.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.expand] Specify the $expand query. Example:
     * 'properties($expand=externalSubnets)'
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VirtualNetwork} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VirtualNetwork} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VirtualNetwork} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, labName: string, name: string, options?: { expand? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.VirtualNetwork>;
    get(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<models.VirtualNetwork>): void;
    get(resourceGroupName: string, labName: string, name: string, options: { expand? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VirtualNetwork>): void;


    /**
     * Create or replace an existing virtual network. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} virtualNetwork A virtual network.
     *
     * @param {array} [virtualNetwork.allowedSubnets] The allowed subnets of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.description] The description of the virtual
     * network.
     *
     * @param {string} [virtualNetwork.externalProviderResourceId] The
     * Microsoft.Network resource identifier of the virtual network.
     *
     * @param {array} [virtualNetwork.subnetOverrides] The subnet overrides of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.location] The location of the resource.
     *
     * @param {object} [virtualNetwork.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VirtualNetwork>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetwork, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VirtualNetwork>>;

    /**
     * Create or replace an existing virtual network. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} virtualNetwork A virtual network.
     *
     * @param {array} [virtualNetwork.allowedSubnets] The allowed subnets of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.description] The description of the virtual
     * network.
     *
     * @param {string} [virtualNetwork.externalProviderResourceId] The
     * Microsoft.Network resource identifier of the virtual network.
     *
     * @param {array} [virtualNetwork.subnetOverrides] The subnet overrides of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.location] The location of the resource.
     *
     * @param {object} [virtualNetwork.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VirtualNetwork} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VirtualNetwork} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VirtualNetwork} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetwork, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VirtualNetwork>;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetwork, callback: ServiceCallback<models.VirtualNetwork>): void;
    createOrUpdate(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetwork, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VirtualNetwork>): void;


    /**
     * Delete virtual network. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete virtual network. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Modify properties of virtual networks.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} virtualNetwork A virtual network.
     *
     * @param {array} [virtualNetwork.allowedSubnets] The allowed subnets of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.description] The description of the virtual
     * network.
     *
     * @param {string} [virtualNetwork.externalProviderResourceId] The
     * Microsoft.Network resource identifier of the virtual network.
     *
     * @param {array} [virtualNetwork.subnetOverrides] The subnet overrides of the
     * virtual network.
     *
     * @param {object} [virtualNetwork.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VirtualNetwork>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetworkFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VirtualNetwork>>;

    /**
     * Modify properties of virtual networks.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} virtualNetwork A virtual network.
     *
     * @param {array} [virtualNetwork.allowedSubnets] The allowed subnets of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.description] The description of the virtual
     * network.
     *
     * @param {string} [virtualNetwork.externalProviderResourceId] The
     * Microsoft.Network resource identifier of the virtual network.
     *
     * @param {array} [virtualNetwork.subnetOverrides] The subnet overrides of the
     * virtual network.
     *
     * @param {object} [virtualNetwork.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VirtualNetwork} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VirtualNetwork} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VirtualNetwork} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetworkFragment, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VirtualNetwork>;
    update(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetworkFragment, callback: ServiceCallback<models.VirtualNetwork>): void;
    update(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetworkFragment, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VirtualNetwork>): void;


    /**
     * Create or replace an existing virtual network. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} virtualNetwork A virtual network.
     *
     * @param {array} [virtualNetwork.allowedSubnets] The allowed subnets of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.description] The description of the virtual
     * network.
     *
     * @param {string} [virtualNetwork.externalProviderResourceId] The
     * Microsoft.Network resource identifier of the virtual network.
     *
     * @param {array} [virtualNetwork.subnetOverrides] The subnet overrides of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.location] The location of the resource.
     *
     * @param {object} [virtualNetwork.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VirtualNetwork>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetwork, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VirtualNetwork>>;

    /**
     * Create or replace an existing virtual network. This operation can take a
     * while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} virtualNetwork A virtual network.
     *
     * @param {array} [virtualNetwork.allowedSubnets] The allowed subnets of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.description] The description of the virtual
     * network.
     *
     * @param {string} [virtualNetwork.externalProviderResourceId] The
     * Microsoft.Network resource identifier of the virtual network.
     *
     * @param {array} [virtualNetwork.subnetOverrides] The subnet overrides of the
     * virtual network.
     *
     * @param {string} [virtualNetwork.location] The location of the resource.
     *
     * @param {object} [virtualNetwork.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VirtualNetwork} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VirtualNetwork} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VirtualNetwork} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetwork, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VirtualNetwork>;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetwork, callback: ServiceCallback<models.VirtualNetwork>): void;
    beginCreateOrUpdate(resourceGroupName: string, labName: string, name: string, virtualNetwork: models.VirtualNetwork, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VirtualNetwork>): void;


    /**
     * Delete virtual network. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete virtual network. This operation can take a while to complete.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} labName The name of the lab.
     *
     * @param {string} name The name of the virtual network.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, labName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List virtual networks in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<VirtualNetworkList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.VirtualNetworkList>>;

    /**
     * List virtual networks in a given lab.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {VirtualNetworkList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {VirtualNetworkList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link VirtualNetworkList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.VirtualNetworkList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.VirtualNetworkList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.VirtualNetworkList>): void;
}
