/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import * as moment from "moment";

/**
 * Error object containing error code and error message.
 */
export interface FabricErrorError {
  /**
   * Defines the fabric error codes that be returned as part of the error object in response to
   * Service Fabric API operations that are not successful. Following are the error code values
   * that can be returned for a specific HTTP status code.
   *
   * - Possible values of the error code for HTTP status code 400 (Bad Request)
   * - "FABRIC_E_INVALID_PARTITION_KEY"
   * - "FABRIC_E_IMAGEBUILDER_VALIDATION_ERROR"
   * - "FABRIC_E_INVALID_ADDRESS"
   * - "FABRIC_E_APPLICATION_NOT_UPGRADING"
   * - "FABRIC_E_APPLICATION_UPGRADE_VALIDATION_ERROR"
   * - "FABRIC_E_FABRIC_NOT_UPGRADING"
   * - "FABRIC_E_FABRIC_UPGRADE_VALIDATION_ERROR"
   * - "FABRIC_E_INVALID_CONFIGURATION"
   * - "FABRIC_E_INVALID_NAME_URI"
   * - "FABRIC_E_PATH_TOO_LONG"
   * - "FABRIC_E_KEY_TOO_LARGE"
   * - "FABRIC_E_SERVICE_AFFINITY_CHAIN_NOT_SUPPORTED"
   * - "FABRIC_E_INVALID_ATOMIC_GROUP"
   * - "FABRIC_E_VALUE_EMPTY"
   * - "FABRIC_E_BACKUP_IS_ENABLED"
   * - "FABRIC_E_RESTORE_SOURCE_TARGET_PARTITION_MISMATCH"
   * - "FABRIC_E_INVALID_FOR_STATELESS_SERVICES"
   * - "FABRIC_E_INVALID_SERVICE_SCALING_POLICY"
   * - "E_INVALIDARG"
   *
   * - Possible values of the error code for HTTP status code 404 (Not Found)
   * - "FABRIC_E_NODE_NOT_FOUND"
   * - "FABRIC_E_APPLICATION_TYPE_NOT_FOUND"
   * - "FABRIC_E_APPLICATION_NOT_FOUND"
   * - "FABRIC_E_SERVICE_TYPE_NOT_FOUND"
   * - "FABRIC_E_SERVICE_DOES_NOT_EXIST"
   * - "FABRIC_E_SERVICE_TYPE_TEMPLATE_NOT_FOUND"
   * - "FABRIC_E_CONFIGURATION_SECTION_NOT_FOUND"
   * - "FABRIC_E_PARTITION_NOT_FOUND"
   * - "FABRIC_E_REPLICA_DOES_NOT_EXIST"
   * - "FABRIC_E_SERVICE_GROUP_DOES_NOT_EXIST"
   * - "FABRIC_E_CONFIGURATION_PARAMETER_NOT_FOUND"
   * - "FABRIC_E_DIRECTORY_NOT_FOUND"
   * - "FABRIC_E_FABRIC_VERSION_NOT_FOUND"
   * - "FABRIC_E_FILE_NOT_FOUND"
   * - "FABRIC_E_NAME_DOES_NOT_EXIST"
   * - "FABRIC_E_PROPERTY_DOES_NOT_EXIST"
   * - "FABRIC_E_ENUMERATION_COMPLETED"
   * - "FABRIC_E_SERVICE_MANIFEST_NOT_FOUND"
   * - "FABRIC_E_KEY_NOT_FOUND"
   * - "FABRIC_E_HEALTH_ENTITY_NOT_FOUND"
   * - "FABRIC_E_BACKUP_NOT_ENABLED"
   * - "FABRIC_E_BACKUP_POLICY_NOT_EXISTING"
   * - "FABRIC_E_FAULT_ANALYSIS_SERVICE_NOT_EXISTING"
   * - "FABRIC_E_IMAGEBUILDER_RESERVED_DIRECTORY_ERROR"
   *
   * - Possible values of the error code for HTTP status code 409 (Conflict)
   * - "FABRIC_E_APPLICATION_TYPE_ALREADY_EXISTS"
   * - "FABRIC_E_APPLICATION_ALREADY_EXISTS"
   * - "FABRIC_E_APPLICATION_ALREADY_IN_TARGET_VERSION"
   * - "FABRIC_E_APPLICATION_TYPE_PROVISION_IN_PROGRESS"
   * - "FABRIC_E_APPLICATION_UPGRADE_IN_PROGRESS"
   * - "FABRIC_E_SERVICE_ALREADY_EXISTS"
   * - "FABRIC_E_SERVICE_GROUP_ALREADY_EXISTS"
   * - "FABRIC_E_APPLICATION_TYPE_IN_USE"
   * - "FABRIC_E_FABRIC_ALREADY_IN_TARGET_VERSION"
   * - "FABRIC_E_FABRIC_VERSION_ALREADY_EXISTS"
   * - "FABRIC_E_FABRIC_VERSION_IN_USE"
   * - "FABRIC_E_FABRIC_UPGRADE_IN_PROGRESS"
   * - "FABRIC_E_NAME_ALREADY_EXISTS"
   * - "FABRIC_E_NAME_NOT_EMPTY"
   * - "FABRIC_E_PROPERTY_CHECK_FAILED"
   * - "FABRIC_E_SERVICE_METADATA_MISMATCH"
   * - "FABRIC_E_SERVICE_TYPE_MISMATCH"
   * - "FABRIC_E_HEALTH_STALE_REPORT"
   * - "FABRIC_E_SEQUENCE_NUMBER_CHECK_FAILED"
   * - "FABRIC_E_NODE_HAS_NOT_STOPPED_YET"
   * - "FABRIC_E_INSTANCE_ID_MISMATCH"
   * - "FABRIC_E_BACKUP_IN_PROGRESS"
   * - "FABRIC_E_RESTORE_IN_PROGRESS"
   * - "FABRIC_E_BACKUP_POLICY_ALREADY_EXISTING"
   *
   * - Possible values of the error code for HTTP status code 413 (Request Entity Too Large)
   * - "FABRIC_E_VALUE_TOO_LARGE"
   *
   * - Possible values of the error code for HTTP status code 500 (Internal Server Error)
   * - "FABRIC_E_NODE_IS_UP"
   * - "E_FAIL"
   * - "FABRIC_E_SINGLE_INSTANCE_APPLICATION_ALREADY_EXISTS"
   * - "FABRIC_E_SINGLE_INSTANCE_APPLICATION_NOT_FOUND"
   * - "FABRIC_E_VOLUME_ALREADY_EXISTS"
   * - "ABRIC_E_VOLUME_NOT_FOUND"
   * - "SerializationError"
   *
   * - Possible values of the error code for HTTP status code 503 (Service Unavailable)
   * - "FABRIC_E_NO_WRITE_QUORUM"
   * - "FABRIC_E_NOT_PRIMARY"
   * - "FABRIC_E_NOT_READY"
   * - "FABRIC_E_RECONFIGURATION_PENDING"
   * - "FABRIC_E_SERVICE_OFFLINE"
   * - "E_ABORT"
   * - "FABRIC_E_VALUE_TOO_LARGE"
   *
   * - Possible values of the error code for HTTP status code 504 (Gateway Timeout)
   * - "FABRIC_E_COMMUNICATION_ERROR"
   * - "FABRIC_E_OPERATION_NOT_COMPLETE"
   * - "FABRIC_E_TIMEOUT". Possible values include: 'FABRIC_E_INVALID_PARTITION_KEY',
   * 'FABRIC_E_IMAGEBUILDER_VALIDATION_ERROR', 'FABRIC_E_INVALID_ADDRESS',
   * 'FABRIC_E_APPLICATION_NOT_UPGRADING', 'FABRIC_E_APPLICATION_UPGRADE_VALIDATION_ERROR',
   * 'FABRIC_E_FABRIC_NOT_UPGRADING', 'FABRIC_E_FABRIC_UPGRADE_VALIDATION_ERROR',
   * 'FABRIC_E_INVALID_CONFIGURATION', 'FABRIC_E_INVALID_NAME_URI', 'FABRIC_E_PATH_TOO_LONG',
   * 'FABRIC_E_KEY_TOO_LARGE', 'FABRIC_E_SERVICE_AFFINITY_CHAIN_NOT_SUPPORTED',
   * 'FABRIC_E_INVALID_ATOMIC_GROUP', 'FABRIC_E_VALUE_EMPTY', 'FABRIC_E_NODE_NOT_FOUND',
   * 'FABRIC_E_APPLICATION_TYPE_NOT_FOUND', 'FABRIC_E_APPLICATION_NOT_FOUND',
   * 'FABRIC_E_SERVICE_TYPE_NOT_FOUND', 'FABRIC_E_SERVICE_DOES_NOT_EXIST',
   * 'FABRIC_E_SERVICE_TYPE_TEMPLATE_NOT_FOUND', 'FABRIC_E_CONFIGURATION_SECTION_NOT_FOUND',
   * 'FABRIC_E_PARTITION_NOT_FOUND', 'FABRIC_E_REPLICA_DOES_NOT_EXIST',
   * 'FABRIC_E_SERVICE_GROUP_DOES_NOT_EXIST', 'FABRIC_E_CONFIGURATION_PARAMETER_NOT_FOUND',
   * 'FABRIC_E_DIRECTORY_NOT_FOUND', 'FABRIC_E_FABRIC_VERSION_NOT_FOUND',
   * 'FABRIC_E_FILE_NOT_FOUND', 'FABRIC_E_NAME_DOES_NOT_EXIST', 'FABRIC_E_PROPERTY_DOES_NOT_EXIST',
   * 'FABRIC_E_ENUMERATION_COMPLETED', 'FABRIC_E_SERVICE_MANIFEST_NOT_FOUND',
   * 'FABRIC_E_KEY_NOT_FOUND', 'FABRIC_E_HEALTH_ENTITY_NOT_FOUND',
   * 'FABRIC_E_APPLICATION_TYPE_ALREADY_EXISTS', 'FABRIC_E_APPLICATION_ALREADY_EXISTS',
   * 'FABRIC_E_APPLICATION_ALREADY_IN_TARGET_VERSION',
   * 'FABRIC_E_APPLICATION_TYPE_PROVISION_IN_PROGRESS', 'FABRIC_E_APPLICATION_UPGRADE_IN_PROGRESS',
   * 'FABRIC_E_SERVICE_ALREADY_EXISTS', 'FABRIC_E_SERVICE_GROUP_ALREADY_EXISTS',
   * 'FABRIC_E_APPLICATION_TYPE_IN_USE', 'FABRIC_E_FABRIC_ALREADY_IN_TARGET_VERSION',
   * 'FABRIC_E_FABRIC_VERSION_ALREADY_EXISTS', 'FABRIC_E_FABRIC_VERSION_IN_USE',
   * 'FABRIC_E_FABRIC_UPGRADE_IN_PROGRESS', 'FABRIC_E_NAME_ALREADY_EXISTS',
   * 'FABRIC_E_NAME_NOT_EMPTY', 'FABRIC_E_PROPERTY_CHECK_FAILED',
   * 'FABRIC_E_SERVICE_METADATA_MISMATCH', 'FABRIC_E_SERVICE_TYPE_MISMATCH',
   * 'FABRIC_E_HEALTH_STALE_REPORT', 'FABRIC_E_SEQUENCE_NUMBER_CHECK_FAILED',
   * 'FABRIC_E_NODE_HAS_NOT_STOPPED_YET', 'FABRIC_E_INSTANCE_ID_MISMATCH',
   * 'FABRIC_E_VALUE_TOO_LARGE', 'FABRIC_E_NO_WRITE_QUORUM', 'FABRIC_E_NOT_PRIMARY',
   * 'FABRIC_E_NOT_READY', 'FABRIC_E_RECONFIGURATION_PENDING', 'FABRIC_E_SERVICE_OFFLINE',
   * 'E_ABORT', 'FABRIC_E_COMMUNICATION_ERROR', 'FABRIC_E_OPERATION_NOT_COMPLETE',
   * 'FABRIC_E_TIMEOUT', 'FABRIC_E_NODE_IS_UP', 'E_FAIL', 'FABRIC_E_BACKUP_IS_ENABLED',
   * 'FABRIC_E_RESTORE_SOURCE_TARGET_PARTITION_MISMATCH',
   * 'FABRIC_E_INVALID_FOR_STATELESS_SERVICES', 'FABRIC_E_BACKUP_NOT_ENABLED',
   * 'FABRIC_E_BACKUP_POLICY_NOT_EXISTING', 'FABRIC_E_FAULT_ANALYSIS_SERVICE_NOT_EXISTING',
   * 'FABRIC_E_BACKUP_IN_PROGRESS', 'FABRIC_E_RESTORE_IN_PROGRESS',
   * 'FABRIC_E_BACKUP_POLICY_ALREADY_EXISTING', 'FABRIC_E_INVALID_SERVICE_SCALING_POLICY',
   * 'E_INVALIDARG', 'FABRIC_E_SINGLE_INSTANCE_APPLICATION_ALREADY_EXISTS',
   * 'FABRIC_E_SINGLE_INSTANCE_APPLICATION_NOT_FOUND', 'FABRIC_E_VOLUME_ALREADY_EXISTS',
   * 'FABRIC_E_VOLUME_NOT_FOUND', 'SerializationError',
   * 'FABRIC_E_IMAGEBUILDER_RESERVED_DIRECTORY_ERROR'
   */
  code: string;
  /**
   * Error message.
   */
  message?: string;
}

/**
 * The REST API operations for Service Fabric return standard HTTP status codes. This type defines
 * the additional information returned from the Service Fabric API operations that are not
 * successful.
 */
export interface FabricError {
  /**
   * Error object containing error code and error message.
   */
  error: FabricErrorError;
}

/**
 * Container logs.
 */
export interface ContainerLogs {
  /**
   * Container logs.
   */
  content?: string;
}

/**
 * Azure Active Directory metadata used for secured connection to cluster.
 */
export interface AadMetadata {
  /**
   * The AAD authority url.
   */
  authority?: string;
  /**
   * The AAD client application Id.
   */
  client?: string;
  /**
   * The AAD cluster application Id.
   */
  cluster?: string;
  /**
   * The AAD login url.
   */
  login?: string;
  /**
   * The client application redirect address.
   */
  redirect?: string;
  /**
   * The AAD tenant Id.
   */
  tenant?: string;
}

/**
 * Azure Active Directory metadata object used for secured connection to cluster.
 */
export interface AadMetadataObject {
  /**
   * The client authentication method.
   */
  type?: string;
  /**
   * Azure Active Directory metadata used for secured connection to cluster.
   */
  metadata?: AadMetadata;
}

/**
 * Metadata about an Analysis Event.
 */
export interface AnalysisEventMetadata {
  /**
   * The analysis delay.
   */
  delay?: moment.Duration;
  /**
   * The duration of analysis.
   */
  duration?: moment.Duration;
}

/**
 * Represents the base for all Fabric Events.
 */
export interface FabricEvent {
  /**
   * The identifier for the FabricEvent instance.
   */
  eventInstanceId: string;
  /**
   * The category of event.
   */
  category?: string;
  /**
   * The time event was logged.
   */
  timeStamp: Date;
  /**
   * Shows there is existing related events available.
   */
  hasCorrelatedEvents?: boolean;
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Represents the base for all Application Events.
 */
export interface ApplicationEvent extends FabricEvent {
  /**
   * The identity of the application. This is an encoded representation of the application name.
   * This is used in the REST APIs to identify the application resource.
   * Starting in version 6.0, hierarchical names are delimited with the "\~" character. For
   * example, if the application name is "fabric:/myapp/app1",
   * the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
   */
  applicationId: string;
}

/**
 * A base type for the health state of various entities in the cluster. It contains the aggregated
 * health state.
 */
export interface EntityHealthState {
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  aggregatedHealthState?: string;
}

/**
 * Represents the health state of a service, which contains the service identifier and its
 * aggregated health state.
 */
export interface ServiceHealthState extends EntityHealthState {
  /**
   * Name of the service whose health state is represented by this object.
   */
  serviceName?: string;
}

/**
 * Represents the health state of a deployed application, which contains the entity identifier and
 * the aggregated health state.
 */
export interface DeployedApplicationHealthState extends EntityHealthState {
  /**
   * Name of the node on which the service package is deployed.
   */
  nodeName?: string;
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
}

/**
 * Health information common to all entities in the cluster. It contains the aggregated health
 * state, health events and unhealthy evaluation.
 */
export interface EntityHealth {
  /**
   * The HealthState representing the aggregated health state of the entity computed by Health
   * Manager.
   * The health evaluation of the entity reflects all events reported on the entity and its
   * children (if any).
   * The aggregation is done by applying the desired health policy. Possible values include:
   * 'Invalid', 'Ok', 'Warning', 'Error', 'Unknown'
   */
  aggregatedHealthState?: string;
  /**
   * The list of health events reported on the entity.
   */
  healthEvents?: HealthEvent[];
  /**
   * The unhealthy evaluations that show why the current aggregated health state was returned by
   * Health Manager.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
  /**
   * Shows the health statistics for all children types of the queried entity.
   */
  healthStatistics?: HealthStatistics;
}

/**
 * Represents the health of the application. Contains the application aggregated health state and
 * the service and deployed application health states.
 */
export interface ApplicationHealth extends EntityHealth {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  name?: string;
  /**
   * Service health states as found in the health store.
   */
  serviceHealthStates?: ServiceHealthState[];
  /**
   * Deployed application health states as found in the health store.
   */
  deployedApplicationHealthStates?: DeployedApplicationHealthState[];
}

/**
 * Represents a health evaluation which describes the data and the algorithm used by health manager
 * to evaluate the health of an entity.
 */
export interface HealthEvaluation {
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  aggregatedHealthState?: string;
  /**
   * Description of the health evaluation, which represents a summary of the evaluation process.
   */
  description?: string;
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Wrapper object for health evaluation.
 */
export interface HealthEvaluationWrapper {
  /**
   * Represents a health evaluation which describes the data and the algorithm used by health
   * manager to evaluate the health of an entity.
   */
  healthEvaluation?: HealthEvaluation;
}

/**
 * Represents health evaluation for an application, containing information about the data and the
 * algorithm used by the health store to evaluate health.
 */
export interface ApplicationHealthEvaluation extends HealthEvaluation {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
  /**
   * List of unhealthy evaluations that led to the current aggregated health state of the
   * application. The types of the unhealthy evaluations can be
   * DeployedApplicationsHealthEvaluation, ServicesHealthEvaluation or EventHealthEvaluation.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Represents the health policy used to evaluate the health of services belonging to a service
 * type.
 */
export interface ServiceTypeHealthPolicy {
  /**
   * The maximum allowed percentage of unhealthy partitions per service. Allowed values are Byte
   * values from zero to 100
   *
   * The percentage represents the maximum tolerated percentage of partitions that can be unhealthy
   * before the service is considered in error.
   * If the percentage is respected but there is at least one unhealthy partition, the health is
   * evaluated as Warning.
   * The percentage is calculated by dividing the number of unhealthy partitions over the total
   * number of partitions in the service.
   * The computation rounds up to tolerate one failure on small numbers of partitions. Default
   * percentage is zero.
   */
  maxPercentUnhealthyPartitionsPerService?: number;
  /**
   * The maximum allowed percentage of unhealthy replicas per partition. Allowed values are Byte
   * values from zero to 100.
   *
   * The percentage represents the maximum tolerated percentage of replicas that can be unhealthy
   * before the partition is considered in error.
   * If the percentage is respected but there is at least one unhealthy replica, the health is
   * evaluated as Warning.
   * The percentage is calculated by dividing the number of unhealthy replicas over the total
   * number of replicas in the partition.
   * The computation rounds up to tolerate one failure on small numbers of replicas. Default
   * percentage is zero.
   */
  maxPercentUnhealthyReplicasPerPartition?: number;
  /**
   * The maximum allowed percentage of unhealthy services. Allowed values are Byte values from zero
   * to 100.
   *
   * The percentage represents the maximum tolerated percentage of services that can be unhealthy
   * before the application is considered in error.
   * If the percentage is respected but there is at least one unhealthy service, the health is
   * evaluated as Warning.
   * This is calculated by dividing the number of unhealthy services of the specific service type
   * over the total number of services of the specific service type.
   * The computation rounds up to tolerate one failure on small numbers of services. Default
   * percentage is zero.
   */
  maxPercentUnhealthyServices?: number;
}

/**
 * Defines an item in ServiceTypeHealthPolicyMap.
 */
export interface ServiceTypeHealthPolicyMapItem {
  /**
   * The key of the service type health policy map item. This is the name of the service type.
   */
  key: string;
  /**
   * The value of the service type health policy map item. This is the ServiceTypeHealthPolicy for
   * this service type.
   */
  value: ServiceTypeHealthPolicy;
}

/**
 * Defines a health policy used to evaluate the health of an application or one of its children
 * entities.
 */
export interface ApplicationHealthPolicy {
  /**
   * Indicates whether warnings are treated with the same severity as errors.
   */
  considerWarningAsError?: boolean;
  /**
   * The maximum allowed percentage of unhealthy deployed applications. Allowed values are Byte
   * values from zero to 100.
   * The percentage represents the maximum tolerated percentage of deployed applications that can
   * be unhealthy before the application is considered in error.
   * This is calculated by dividing the number of unhealthy deployed applications over the number
   * of nodes where the application is currently deployed on in the cluster.
   * The computation rounds up to tolerate one failure on small numbers of nodes. Default
   * percentage is zero.
   */
  maxPercentUnhealthyDeployedApplications?: number;
  /**
   * The health policy used by default to evaluate the health of a service type.
   */
  defaultServiceTypeHealthPolicy?: ServiceTypeHealthPolicy;
  /**
   * The map with service type health policy per service type name. The map is empty by default.
   */
  serviceTypeHealthPolicyMap?: ServiceTypeHealthPolicyMapItem[];
}

/**
 * Defines an item in ApplicationHealthPolicyMap.
 */
export interface ApplicationHealthPolicyMapItem {
  /**
   * The key of the application health policy map item. This is the name of the application.
   */
  key: string;
  /**
   * The value of the application health policy map item. This is the ApplicationHealthPolicy for
   * this application.
   */
  value: ApplicationHealthPolicy;
}

/**
 * Defines the application health policy map used to evaluate the health of an application or one
 * of its children entities.
 */
export interface ApplicationHealthPolicies {
  /**
   * The wrapper that contains the map with application health policies used to evaluate specific
   * applications in the cluster.
   */
  applicationHealthPolicyMap?: ApplicationHealthPolicyMapItem[];
}

/**
 * Represents the health state of an application, which contains the application identifier and the
 * aggregated health state.
 */
export interface ApplicationHealthState extends EntityHealthState {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  name?: string;
}

/**
 * A base type for the health state chunk of various entities in the cluster. It contains the
 * aggregated health state.
 */
export interface EntityHealthStateChunk {
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  healthState?: string;
}

/**
 * Represents the health state chunk of a stateful service replica or a stateless service instance.
 * The replica health state contains the replica ID and its aggregated health state.
 */
export interface ReplicaHealthStateChunk extends EntityHealthStateChunk {
  /**
   * Id of a stateful service replica or a stateless service instance. This ID is used in the
   * queries that apply to both stateful and stateless services. It is used by Service Fabric to
   * uniquely identify a replica of a partition of a stateful service or an instance of a stateless
   * service partition. It is unique within a partition and does not change for the lifetime of the
   * replica or the instance. If a stateful replica gets dropped and another replica gets created
   * on the same node for the same partition, it will get a different value for the ID. If a
   * stateless instance is failed over on the same or different node it will get a different value
   * for the ID.
   */
  replicaOrInstanceId?: string;
}

/**
 * The list of replica health state chunks that respect the input filters in the chunk query.
 * Returned by get cluster health state chunks query.
 */
export interface ReplicaHealthStateChunkList {
  /**
   * The list of replica health state chunks that respect the input filters in the chunk query.
   */
  items?: ReplicaHealthStateChunk[];
}

/**
 * Represents the health state chunk of a partition, which contains the partition ID, its
 * aggregated health state and any replicas that respect the filters in the cluster health chunk
 * query description.
 */
export interface PartitionHealthStateChunk extends EntityHealthStateChunk {
  /**
   * The Id of the partition.
   */
  partitionId?: string;
  /**
   * The list of replica health state chunks belonging to the partition that respect the filters in
   * the cluster health chunk query description.
   */
  replicaHealthStateChunks?: ReplicaHealthStateChunkList;
}

/**
 * The list of partition health state chunks that respect the input filters in the chunk query
 * description.
 * Returned by get cluster health state chunks query as part of the parent application hierarchy.
 */
export interface PartitionHealthStateChunkList {
  /**
   * The list of partition health state chunks that respect the input filters in the chunk query.
   */
  items?: PartitionHealthStateChunk[];
}

/**
 * Represents the health state chunk of a service, which contains the service name, its aggregated
 * health state and any partitions that respect the filters in the cluster health chunk query
 * description.
 */
export interface ServiceHealthStateChunk extends EntityHealthStateChunk {
  /**
   * The name of the service whose health state chunk is provided in this object.
   */
  serviceName?: string;
  /**
   * The list of partition health state chunks belonging to the service that respect the filters in
   * the cluster health chunk query description.
   */
  partitionHealthStateChunks?: PartitionHealthStateChunkList;
}

/**
 * The list of service health state chunks that respect the input filters in the chunk query.
 * Returned by get cluster health state chunks query.
 */
export interface ServiceHealthStateChunkList {
  /**
   * The list of service health state chunks that respect the input filters in the chunk query.
   */
  items?: ServiceHealthStateChunk[];
}

/**
 * Represents the health state chunk of a deployed service package, which contains the service
 * manifest name and the service package aggregated health state.
 */
export interface DeployedServicePackageHealthStateChunk extends EntityHealthStateChunk {
  /**
   * The name of the service manifest.
   */
  serviceManifestName?: string;
  /**
   * The ActivationId of a deployed service package. If ServicePackageActivationMode specified at
   * the time of creating the service
   * is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'),
   * then value of ServicePackageActivationId
   * is always an empty string.
   */
  servicePackageActivationId?: string;
}

/**
 * The list of deployed service package health state chunks that respect the input filters in the
 * chunk query. Returned by get cluster health state chunks query.
 */
export interface DeployedServicePackageHealthStateChunkList {
  /**
   * The list of deployed service package health state chunks that respect the input filters in the
   * chunk query.
   */
  items?: DeployedServicePackageHealthStateChunk[];
}

/**
 * Represents the health state chunk of a deployed application, which contains the node where the
 * application is deployed, the aggregated health state and any deployed service packages that
 * respect the chunk query description filters.
 */
export interface DeployedApplicationHealthStateChunk extends EntityHealthStateChunk {
  /**
   * The name of node where the application is deployed.
   */
  nodeName?: string;
  /**
   * The list of deployed service package health state chunks belonging to the deployed application
   * that respect the filters in the cluster health chunk query description.
   */
  deployedServicePackageHealthStateChunks?: DeployedServicePackageHealthStateChunkList;
}

/**
 * The list of deployed application health state chunks that respect the input filters in the chunk
 * query. Returned by get cluster health state chunks query.
 */
export interface DeployedApplicationHealthStateChunkList {
  /**
   * The list of deployed application health state chunks that respect the input filters in the
   * chunk query.
   */
  items?: DeployedApplicationHealthStateChunk[];
}

/**
 * Represents the health state chunk of a application.
 * The application health state chunk contains the application name, its aggregated health state
 * and any children services and deployed applications that respect the filters in cluster health
 * chunk query description.
 */
export interface ApplicationHealthStateChunk extends EntityHealthStateChunk {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
  /**
   * The application type name as defined in the application manifest.
   */
  applicationTypeName?: string;
  /**
   * The list of service health state chunks in the cluster that respect the filters in the cluster
   * health chunk query description.
   */
  serviceHealthStateChunks?: ServiceHealthStateChunkList;
  /**
   * The list of deployed application health state chunks in the cluster that respect the filters
   * in the cluster health chunk query description.
   */
  deployedApplicationHealthStateChunks?: DeployedApplicationHealthStateChunkList;
}

/**
 * A base type for the list of health state chunks found in the cluster. It contains the total
 * number of health states that match the input filters.
 */
export interface EntityHealthStateChunkList {
  /**
   * Total number of entity health state objects that match the specified filters from the cluster
   * health chunk query description.
   */
  totalCount?: number;
}

/**
 * The list of application health state chunks in the cluster that respect the input filters in the
 * chunk query. Returned by get cluster health state chunks query.
 */
export interface ApplicationHealthStateChunkList extends EntityHealthStateChunkList {
  /**
   * The list of application health state chunks that respect the input filters in the chunk query.
   */
  items?: ApplicationHealthStateChunk[];
}

/**
 * Defines matching criteria to determine whether a replica should be included as a child of a
 * partition in the cluster health chunk.
 * The replicas are only returned if the parent entities match a filter specified in the cluster
 * health chunk query description. The parent partition, service and application must be included
 * in the cluster health chunk.
 * One filter can match zero, one or multiple replicas, depending on its properties.
 */
export interface ReplicaHealthStateFilter {
  /**
   * Id of the stateful service replica or stateless service instance that matches the filter. The
   * filter is applied only to the specified replica, if it exists.
   * If the replica doesn't exist, no replica is returned in the cluster health chunk based on this
   * filter.
   * If the replica exists, it is included in the cluster health chunk if it respects the other
   * filter properties.
   * If not specified, all replicas that match the parent filters (if any) are taken into
   * consideration and matched against the other filter members, like health state filter.
   */
  replicaOrInstanceIdFilter?: string;
  /**
   * The filter for the health state of the replicas. It allows selecting replicas if they match
   * the desired health states.
   * The possible values are integer value of one of the following health states. Only replicas
   * that match the filter are returned. All replicas are used to evaluate the parent partition
   * aggregated health state.
   * If not specified, default value is None, unless the replica ID is specified. If the filter has
   * default value and replica ID is specified, the matching replica is returned.
   * The state values are flag-based enumeration, so the value could be a combination of these
   * values obtained using bitwise 'OR' operator.
   * For example, if the provided value is 6, it matches replicas with HealthState value of OK (2)
   * and Warning (4).
   *
   * - Default - Default value. Matches any HealthState. The value is zero.
   * - None - Filter that doesn't match any HealthState value. Used in order to return no results
   * on a given collection of states. The value is 1.
   * - Ok - Filter that matches input with HealthState value Ok. The value is 2.
   * - Warning - Filter that matches input with HealthState value Warning. The value is 4.
   * - Error - Filter that matches input with HealthState value Error. The value is 8.
   * - All - Filter that matches input with any HealthState value. The value is 65535.
   */
  healthStateFilter?: number;
}

/**
 * Defines matching criteria to determine whether a partition should be included as a child of a
 * service in the cluster health chunk.
 * The partitions are only returned if the parent entities match a filter specified in the cluster
 * health chunk query description. The parent service and application must be included in the
 * cluster health chunk.
 * One filter can match zero, one or multiple partitions, depending on its properties.
 */
export interface PartitionHealthStateFilter {
  /**
   * ID of the partition that matches the filter. The filter is applied only to the specified
   * partition, if it exists.
   * If the partition doesn't exist, no partition is returned in the cluster health chunk based on
   * this filter.
   * If the partition exists, it is included in the cluster health chunk if it respects the other
   * filter properties.
   * If not specified, all partitions that match the parent filters (if any) are taken into
   * consideration and matched against the other filter members, like health state filter.
   */
  partitionIdFilter?: string;
  /**
   * The filter for the health state of the partitions. It allows selecting partitions if they
   * match the desired health states.
   * The possible values are integer value of one of the following health states. Only partitions
   * that match the filter are returned. All partitions are used to evaluate the cluster aggregated
   * health state.
   * If not specified, default value is None, unless the partition ID is specified. If the filter
   * has default value and partition ID is specified, the matching partition is returned.
   * The state values are flag-based enumeration, so the value could be a combination of these
   * values obtained using bitwise 'OR' operator.
   * For example, if the provided value is 6, it matches partitions with HealthState value of OK
   * (2) and Warning (4).
   *
   * - Default - Default value. Matches any HealthState. The value is zero.
   * - None - Filter that doesn't match any HealthState value. Used in order to return no results
   * on a given collection of states. The value is 1.
   * - Ok - Filter that matches input with HealthState value Ok. The value is 2.
   * - Warning - Filter that matches input with HealthState value Warning. The value is 4.
   * - Error - Filter that matches input with HealthState value Error. The value is 8.
   * - All - Filter that matches input with any HealthState value. The value is 65535.
   */
  healthStateFilter?: number;
  /**
   * Defines a list of filters that specify which replicas to be included in the returned cluster
   * health chunk as children of the parent partition. The replicas are returned only if the parent
   * partition matches a filter.
   * If the list is empty, no replicas are returned. All the replicas are used to evaluate the
   * parent partition aggregated health state, regardless of the input filters.
   * The partition filter may specify multiple replica filters.
   * For example, it can specify a filter to return all replicas with health state Error and
   * another filter to always include a replica identified by its replica id.
   */
  replicaFilters?: ReplicaHealthStateFilter[];
}

/**
 * Defines matching criteria to determine whether a service should be included as a child of an
 * application in the cluster health chunk.
 * The services are only returned if the parent application matches a filter specified in the
 * cluster health chunk query description.
 * One filter can match zero, one or multiple services, depending on its properties.
 */
export interface ServiceHealthStateFilter {
  /**
   * The name of the service that matches the filter. The filter is applied only to the specified
   * service, if it exists.
   * If the service doesn't exist, no service is returned in the cluster health chunk based on this
   * filter.
   * If the service exists, it is included as the application's child if the health state matches
   * the other filter properties.
   * If not specified, all services that match the parent filters (if any) are taken into
   * consideration and matched against the other filter members, like health state filter.
   */
  serviceNameFilter?: string;
  /**
   * The filter for the health state of the services. It allows selecting services if they match
   * the desired health states.
   * The possible values are integer value of one of the following health states. Only services
   * that match the filter are returned. All services are used to evaluate the cluster aggregated
   * health state.
   * If not specified, default value is None, unless the service name is specified. If the filter
   * has default value and service name is specified, the matching service is returned.
   * The state values are flag-based enumeration, so the value could be a combination of these
   * values obtained using bitwise 'OR' operator.
   * For example, if the provided value is 6, it matches services with HealthState value of OK (2)
   * and Warning (4).
   *
   * - Default - Default value. Matches any HealthState. The value is zero.
   * - None - Filter that doesn't match any HealthState value. Used in order to return no results
   * on a given collection of states. The value is 1.
   * - Ok - Filter that matches input with HealthState value Ok. The value is 2.
   * - Warning - Filter that matches input with HealthState value Warning. The value is 4.
   * - Error - Filter that matches input with HealthState value Error. The value is 8.
   * - All - Filter that matches input with any HealthState value. The value is 65535.
   */
  healthStateFilter?: number;
  /**
   * Defines a list of filters that specify which partitions to be included in the returned cluster
   * health chunk as children of the service. The partitions are returned only if the parent
   * service matches a filter.
   * If the list is empty, no partitions are returned. All the partitions are used to evaluate the
   * parent service aggregated health state, regardless of the input filters.
   * The service filter may specify multiple partition filters.
   * For example, it can specify a filter to return all partitions with health state Error and
   * another filter to always include a partition identified by its partition ID.
   */
  partitionFilters?: PartitionHealthStateFilter[];
}

/**
 * Defines matching criteria to determine whether a deployed service package should be included as
 * a child of a deployed application in the cluster health chunk.
 * The deployed service packages are only returned if the parent entities match a filter specified
 * in the cluster health chunk query description. The parent deployed application and its parent
 * application must be included in the cluster health chunk.
 * One filter can match zero, one or multiple deployed service packages, depending on its
 * properties.
 */
export interface DeployedServicePackageHealthStateFilter {
  /**
   * The name of the service manifest which identifies the deployed service packages that matches
   * the filter.
   * If specified, the filter is applied only to the specified deployed service packages, if any.
   * If no deployed service packages with specified manifest name exist, nothing is returned in the
   * cluster health chunk based on this filter.
   * If any deployed service package exists, they are included in the cluster health chunk if it
   * respects the other filter properties.
   * If not specified, all deployed service packages that match the parent filters (if any) are
   * taken into consideration and matched against the other filter members, like health state
   * filter.
   */
  serviceManifestNameFilter?: string;
  /**
   * The activation ID of a deployed service package that matches the filter.
   * If not specified, the filter applies to all deployed service packages that match the other
   * parameters.
   * If specified, the filter matches only the deployed service package with the specified
   * activation ID.
   */
  servicePackageActivationIdFilter?: string;
  /**
   * The filter for the health state of the deployed service packages. It allows selecting deployed
   * service packages if they match the desired health states.
   * The possible values are integer value of one of the following health states. Only deployed
   * service packages that match the filter are returned. All deployed service packages are used to
   * evaluate the parent deployed application aggregated health state.
   * If not specified, default value is None, unless the deployed service package ID is specified.
   * If the filter has default value and deployed service package ID is specified, the matching
   * deployed service package is returned.
   * The state values are flag-based enumeration, so the value could be a combination of these
   * values obtained using bitwise 'OR' operator.
   * For example, if the provided value is 6, it matches deployed service packages with HealthState
   * value of OK (2) and Warning (4).
   *
   * - Default - Default value. Matches any HealthState. The value is zero.
   * - None - Filter that doesn't match any HealthState value. Used in order to return no results
   * on a given collection of states. The value is 1.
   * - Ok - Filter that matches input with HealthState value Ok. The value is 2.
   * - Warning - Filter that matches input with HealthState value Warning. The value is 4.
   * - Error - Filter that matches input with HealthState value Error. The value is 8.
   * - All - Filter that matches input with any HealthState value. The value is 65535.
   */
  healthStateFilter?: number;
}

/**
 * Defines matching criteria to determine whether a deployed application should be included as a
 * child of an application in the cluster health chunk.
 * The deployed applications are only returned if the parent application matches a filter specified
 * in the cluster health chunk query description.
 * One filter can match zero, one or multiple deployed applications, depending on its properties.
 */
export interface DeployedApplicationHealthStateFilter {
  /**
   * The name of the node where the application is deployed in order to match the filter.
   * If specified, the filter is applied only to the application deployed on the specified node.
   * If the application is not deployed on the node with the specified name, no deployed
   * application is returned in the cluster health chunk based on this filter.
   * Otherwise, the deployed application is included in the cluster health chunk if it respects the
   * other filter properties.
   * If not specified, all deployed applications that match the parent filters (if any) are taken
   * into consideration and matched against the other filter members, like health state filter.
   */
  nodeNameFilter?: string;
  /**
   * The filter for the health state of the deployed applications. It allows selecting deployed
   * applications if they match the desired health states.
   * The possible values are integer value of one of the following health states. Only deployed
   * applications that match the filter are returned. All deployed applications are used to
   * evaluate the cluster aggregated health state.
   * If not specified, default value is None, unless the node name is specified. If the filter has
   * default value and node name is specified, the matching deployed application is returned.
   * The state values are flag-based enumeration, so the value could be a combination of these
   * values obtained using bitwise 'OR' operator.
   * For example, if the provided value is 6, it matches deployed applications with HealthState
   * value of OK (2) and Warning (4).
   *
   * - Default - Default value. Matches any HealthState. The value is zero.
   * - None - Filter that doesn't match any HealthState value. Used in order to return no results
   * on a given collection of states. The value is 1.
   * - Ok - Filter that matches input with HealthState value Ok. The value is 2.
   * - Warning - Filter that matches input with HealthState value Warning. The value is 4.
   * - Error - Filter that matches input with HealthState value Error. The value is 8.
   * - All - Filter that matches input with any HealthState value. The value is 65535.
   */
  healthStateFilter?: number;
  /**
   * Defines a list of filters that specify which deployed service packages to be included in the
   * returned cluster health chunk as children of the parent deployed application. The deployed
   * service packages are returned only if the parent deployed application matches a filter.
   * If the list is empty, no deployed service packages are returned. All the deployed service
   * packages are used to evaluate the parent deployed application aggregated health state,
   * regardless of the input filters.
   * The deployed application filter may specify multiple deployed service package filters.
   * For example, it can specify a filter to return all deployed service packages with health state
   * Error and another filter to always include a deployed service package on a node.
   */
  deployedServicePackageFilters?: DeployedServicePackageHealthStateFilter[];
}

/**
 * Defines matching criteria to determine whether a application should be included in the cluster
 * health chunk.
 * One filter can match zero, one or multiple applications, depending on its properties.
 */
export interface ApplicationHealthStateFilter {
  /**
   * The name of the application that matches the filter, as a fabric uri. The filter is applied
   * only to the specified application, if it exists.
   * If the application doesn't exist, no application is returned in the cluster health chunk based
   * on this filter.
   * If the application exists, it is included in the cluster health chunk if it respects the other
   * filter properties.
   * If not specified, all applications are matched against the other filter members, like health
   * state filter.
   */
  applicationNameFilter?: string;
  /**
   * The name of the application type that matches the filter.
   * If specified, the filter is applied only to applications of the selected application type, if
   * any exists.
   * If no applications of the specified application type exists, no application is returned in the
   * cluster health chunk based on this filter.
   * Each application of the specified application type is included in the cluster health chunk if
   * it respects the other filter properties.
   * If not specified, all applications are matched against the other filter members, like health
   * state filter.
   */
  applicationTypeNameFilter?: string;
  /**
   * The filter for the health state of the applications. It allows selecting applications if they
   * match the desired health states.
   * The possible values are integer value of one of the following health states. Only applications
   * that match the filter are returned. All applications are used to evaluate the cluster
   * aggregated health state.
   * If not specified, default value is None, unless the application name or the application type
   * name are specified. If the filter has default value and application name is specified, the
   * matching application is returned.
   * The state values are flag-based enumeration, so the value could be a combination of these
   * values obtained using bitwise 'OR' operator.
   * For example, if the provided value is 6, it matches applications with HealthState value of OK
   * (2) and Warning (4).
   *
   * - Default - Default value. Matches any HealthState. The value is zero.
   * - None - Filter that doesn't match any HealthState value. Used in order to return no results
   * on a given collection of states. The value is 1.
   * - Ok - Filter that matches input with HealthState value Ok. The value is 2.
   * - Warning - Filter that matches input with HealthState value Warning. The value is 4.
   * - Error - Filter that matches input with HealthState value Error. The value is 8.
   * - All - Filter that matches input with any HealthState value. The value is 65535.
   */
  healthStateFilter?: number;
  /**
   * Defines a list of filters that specify which services to be included in the returned cluster
   * health chunk as children of the application. The services are returned only if the parent
   * application matches a filter.
   * If the list is empty, no services are returned. All the services are used to evaluate the
   * parent application aggregated health state, regardless of the input filters.
   * The application filter may specify multiple service filters.
   * For example, it can specify a filter to return all services with health state Error and
   * another filter to always include a service identified by its service name.
   */
  serviceFilters?: ServiceHealthStateFilter[];
  /**
   * Defines a list of filters that specify which deployed applications to be included in the
   * returned cluster health chunk as children of the application. The deployed applications are
   * returned only if the parent application matches a filter.
   * If the list is empty, no deployed applications are returned. All the deployed applications are
   * used to evaluate the parent application aggregated health state, regardless of the input
   * filters.
   * The application filter may specify multiple deployed application filters.
   * For example, it can specify a filter to return all deployed applications with health state
   * Error and another filter to always include a deployed application on a specified node.
   */
  deployedApplicationFilters?: DeployedApplicationHealthStateFilter[];
}

/**
 * Describes an application parameter override to be applied when creating or upgrading an
 * application.
 */
export interface ApplicationParameter {
  /**
   * The name of the parameter.
   */
  key: string;
  /**
   * The value of the parameter.
   */
  value: string;
}

/**
 * Information about a Service Fabric application.
 */
export interface ApplicationInfo {
  /**
   * The identity of the application. This is an encoded representation of the application name.
   * This is used in the REST APIs to identify the application resource.
   * Starting in version 6.0, hierarchical names are delimited with the "\~" character. For
   * example, if the application name is "fabric:/myapp/app1",
   * the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
   */
  id?: string;
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  name?: string;
  /**
   * The application type name as defined in the application manifest.
   */
  typeName?: string;
  /**
   * The version of the application type as defined in the application manifest.
   */
  typeVersion?: string;
  /**
   * The status of the application. Possible values include: 'Invalid', 'Ready', 'Upgrading',
   * 'Creating', 'Deleting', 'Failed'
   */
  status?: string;
  /**
   * List of application parameters with overridden values from their default values specified in
   * the application manifest.
   */
  parameters?: ApplicationParameter[];
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  healthState?: string;
  /**
   * The mechanism used to define a Service Fabric application. Possible values include: 'Invalid',
   * 'ServiceFabricApplicationDescription', 'Compose'
   */
  applicationDefinitionKind?: string;
}

/**
 * Describes capacity information for a custom resource balancing metric. This can be used to limit
 * the total consumption of this metric by the services of this application.
 */
export interface ApplicationMetricDescription {
  /**
   * The name of the metric.
   */
  name?: string;
  /**
   * The maximum node capacity for Service Fabric application.
   * This is the maximum Load for an instance of this application on a single node. Even if the
   * capacity of node is greater than this value, Service Fabric will limit the total load of
   * services within the application on each node to this value.
   * If set to zero, capacity for this metric is unlimited on each node.
   * When creating a new application with application capacity defined, the product of MaximumNodes
   * and this value must always be smaller than or equal to TotalApplicationCapacity.
   * When updating existing application with application capacity, the product of MaximumNodes and
   * this value must always be smaller than or equal to TotalApplicationCapacity.
   */
  maximumCapacity?: number;
  /**
   * The node reservation capacity for Service Fabric application.
   * This is the amount of load which is reserved on nodes which have instances of this
   * application.
   * If MinimumNodes is specified, then the product of these values will be the capacity reserved
   * in the cluster for the application.
   * If set to zero, no capacity is reserved for this metric.
   * When setting application capacity or when updating application capacity; this value must be
   * smaller than or equal to MaximumCapacity for each metric.
   */
  reservationCapacity?: number;
  /**
   * The total metric capacity for Service Fabric application.
   * This is the total metric capacity for this application in the cluster. Service Fabric will try
   * to limit the sum of loads of services within the application to this value.
   * When creating a new application with application capacity defined, the product of MaximumNodes
   * and MaximumCapacity must always be smaller than or equal to this value.
   */
  totalApplicationCapacity?: number;
}

/**
 * Load Information about a Service Fabric application.
 */
export interface ApplicationLoadInfo {
  /**
   * The identity of the application. This is an encoded representation of the application name.
   * This is used in the REST APIs to identify the application resource.
   * Starting in version 6.0, hierarchical names are delimited with the "\~" character. For
   * example, if the application name is "fabric:/myapp/app1",
   * the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
   */
  id?: string;
  /**
   * The minimum number of nodes for this application.
   * It is the number of nodes where Service Fabric will reserve Capacity in the cluster which
   * equals to ReservedLoad * MinimumNodes for this Application instance.
   * For applications that do not have application capacity defined this value will be zero.
   */
  minimumNodes?: number;
  /**
   * The maximum number of nodes where this application can be instantiated.
   * It is the number of nodes this application is allowed to span.
   * For applications that do not have application capacity defined this value will be zero.
   */
  maximumNodes?: number;
  /**
   * The number of nodes on which this application is instantiated.
   * For applications that do not have application capacity defined this value will be zero.
   */
  nodeCount?: number;
  /**
   * List of application capacity metric description.
   */
  applicationLoadMetricInformation?: ApplicationMetricDescription[];
}

/**
 * Information about the application name.
 */
export interface ApplicationNameInfo {
  /**
   * The identity of the application. This is an encoded representation of the application name.
   * This is used in the REST APIs to identify the application resource.
   * Starting in version 6.0, hierarchical names are delimited with the "\~" character. For
   * example, if the application name is "fabric:/myapp/app1",
   * the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
   */
  id?: string;
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  name?: string;
}

/**
 * Represents health evaluation for applications, containing health evaluations for each unhealthy
 * application that impacted current aggregated health state.
 */
export interface ApplicationsHealthEvaluation extends HealthEvaluation {
  /**
   * Maximum allowed percentage of unhealthy applications from the ClusterHealthPolicy.
   */
  maxPercentUnhealthyApplications?: number;
  /**
   * Total number of applications from the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy ApplicationHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Represents health evaluation for applications of a particular application type. The application
 * type applications evaluation can be returned when cluster health evaluation returns unhealthy
 * aggregated health state, either Error or Warning. It contains health evaluations for each
 * unhealthy application of the included application type that impacted current aggregated health
 * state.
 */
export interface ApplicationTypeApplicationsHealthEvaluation extends HealthEvaluation {
  /**
   * The application type name as defined in the application manifest.
   */
  applicationTypeName?: string;
  /**
   * Maximum allowed percentage of unhealthy applications for the application type, specified as an
   * entry in ApplicationTypeHealthPolicyMap.
   */
  maxPercentUnhealthyApplications?: number;
  /**
   * Total number of applications of the application type found in the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy ApplicationHealthEvaluation of this application type that impacted the aggregated
   * health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Defines an item in ApplicationTypeHealthPolicyMap.
 */
export interface ApplicationTypeHealthPolicyMapItem {
  /**
   * The key of the application type health policy map item. This is the name of the application
   * type.
   */
  key: string;
  /**
   * The value of the application type health policy map item.
   * The max percent unhealthy applications allowed for the application type. Must be between zero
   * and 100.
   */
  value: number;
}

/**
 * Information about an application type.
 */
export interface ApplicationTypeInfo {
  /**
   * The application type name as defined in the application manifest.
   */
  name?: string;
  /**
   * The version of the application type as defined in the application manifest.
   */
  version?: string;
  /**
   * List of application type parameters that can be overridden when creating or updating the
   * application.
   */
  defaultParameterList?: ApplicationParameter[];
  /**
   * The status of the application type. Possible values include: 'Invalid', 'Provisioning',
   * 'Available', 'Unprovisioning', 'Failed'
   */
  status?: string;
  /**
   * Additional detailed information about the status of the application type.
   */
  statusDetails?: string;
  /**
   * The mechanism used to define a Service Fabric application type. Possible values include:
   * 'Invalid', 'ServiceFabricApplicationPackage', 'Compose'
   */
  applicationTypeDefinitionKind?: string;
}

/**
 * The list of application types that are provisioned or being provisioned in the cluster. The list
 * is paged when all of the results cannot fit in a single message. The next set of results can be
 * obtained by executing the same query with the continuation token provided in this list.
 */
export interface PagedApplicationTypeInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of application type information.
   */
  items?: ApplicationTypeInfo[];
}

/**
 * Contains the manifest describing an application type registered in a Service Fabric cluster.
 */
export interface ApplicationTypeManifest {
  /**
   * The XML manifest as a string.
   */
  manifest?: string;
}

/**
 * Describes the parameters for monitoring an upgrade in Monitored mode.
 */
export interface MonitoringPolicyDescription {
  /**
   * The compensating action to perform when a Monitored upgrade encounters monitoring policy or
   * health policy violations.
   * Invalid indicates the failure action is invalid. Rollback specifies that the upgrade will
   * start rolling back automatically.
   * Manual indicates that the upgrade will switch to UnmonitoredManual upgrade mode. Possible
   * values include: 'Invalid', 'Rollback', 'Manual'
   */
  failureAction?: string;
  /**
   * The amount of time to wait after completing an upgrade domain before applying health policies.
   * It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it
   * is interpreted as a number representing the total number of milliseconds.
   */
  healthCheckWaitDurationInMilliseconds?: string;
  /**
   * The amount of time that the application or cluster must remain healthy before the upgrade
   * proceeds to the next upgrade domain. It is first interpreted as a string representing an ISO
   * 8601 duration. If that fails, then it is interpreted as a number representing the total number
   * of milliseconds.
   */
  healthCheckStableDurationInMilliseconds?: string;
  /**
   * The amount of time to retry health evaluation when the application or cluster is unhealthy
   * before FailureAction is executed. It is first interpreted as a string representing an ISO 8601
   * duration. If that fails, then it is interpreted as a number representing the total number of
   * milliseconds.
   */
  healthCheckRetryTimeoutInMilliseconds?: string;
  /**
   * The amount of time the overall upgrade has to complete before FailureAction is executed. It is
   * first interpreted as a string representing an ISO 8601 duration. If that fails, then it is
   * interpreted as a number representing the total number of milliseconds.
   */
  upgradeTimeoutInMilliseconds?: string;
  /**
   * The amount of time each upgrade domain has to complete before FailureAction is executed. It is
   * first interpreted as a string representing an ISO 8601 duration. If that fails, then it is
   * interpreted as a number representing the total number of milliseconds.
   */
  upgradeDomainTimeoutInMilliseconds?: string;
}

/**
 * Describes the parameters for an application upgrade. Note that upgrade description replaces the
 * existing application description. This means that if the parameters are not specified, the
 * existing parameters on the applications will be overwritten with the empty parameters list. This
 * would result in the application using the default value of the parameters from the application
 * manifest. If you do not want to change any existing parameter values, please get the application
 * parameters first using the GetApplicationInfo query and then supply those values as Parameters
 * in this ApplicationUpgradeDescription.
 */
export interface ApplicationUpgradeDescription {
  /**
   * The name of the target application, including the 'fabric:' URI scheme.
   */
  name: string;
  /**
   * The target application type version (found in the application manifest) for the application
   * upgrade.
   */
  targetApplicationTypeVersion: string;
  /**
   * List of application parameters with overridden values from their default values specified in
   * the application manifest.
   */
  parameters: ApplicationParameter[];
  /**
   * The kind of upgrade out of the following possible values. Possible values include: 'Invalid',
   * 'Rolling'
   */
  upgradeKind: string;
  /**
   * The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto,
   * UnmonitoredManual, and Monitored. Possible values include: 'Invalid', 'UnmonitoredAuto',
   * 'UnmonitoredManual', 'Monitored'
   */
  rollingUpgradeMode?: string;
  /**
   * The maximum amount of time to block processing of an upgrade domain and prevent loss of
   * availability when there are unexpected issues. When this timeout expires, processing of the
   * upgrade domain will proceed regardless of availability loss issues. The timeout is reset at
   * the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive.
   * (unsigned 32-bit integer).
   */
  upgradeReplicaSetCheckTimeoutInSeconds?: number;
  /**
   * If true, then processes are forcefully restarted during upgrade even when the code version has
   * not changed (the upgrade only changes configuration or data).
   */
  forceRestart?: boolean;
  /**
   * Describes the parameters for monitoring an upgrade in Monitored mode.
   */
  monitoringPolicy?: MonitoringPolicyDescription;
  /**
   * Defines a health policy used to evaluate the health of an application or one of its children
   * entities.
   */
  applicationHealthPolicy?: ApplicationHealthPolicy;
}

/**
 * Information about an upgrade domain.
 */
export interface UpgradeDomainInfo {
  /**
   * The name of the upgrade domain
   */
  name?: string;
  /**
   * The state of the upgrade domain. Possible values include: 'Invalid', 'Pending', 'InProgress',
   * 'Completed'
   */
  state?: string;
}

/**
 * Represents a safety check performed by service fabric before continuing with the operations.
 * These checks ensure the availability of the service and the reliability of the state.
 */
export interface SafetyCheck {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * A wrapper for the safety check object. Safety checks are performed by service fabric before
 * continuing with the operations. These checks ensure the availability of the service and the
 * reliability of the state.
 */
export interface SafetyCheckWrapper {
  /**
   * Represents a safety check performed by service fabric before continuing with the operations.
   * These checks ensure the availability of the service and the reliability of the state.
   */
  safetyCheck?: SafetyCheck;
}

/**
 * Information about the upgrading node and its status
 */
export interface NodeUpgradeProgressInfo {
  /**
   * The name of a Service Fabric node.
   */
  nodeName?: string;
  /**
   * The state of the upgrading node. Possible values include: 'Invalid', 'PreUpgradeSafetyCheck',
   * 'Upgrading', 'PostUpgradeSafetyCheck'
   */
  upgradePhase?: string;
  /**
   * List of pending safety checks
   */
  pendingSafetyChecks?: SafetyCheckWrapper[];
}

/**
 * Information about the current in-progress upgrade domain.
 */
export interface CurrentUpgradeDomainProgressInfo {
  /**
   * The name of the upgrade domain
   */
  domainName?: string;
  /**
   * List of upgrading nodes and their statuses
   */
  nodeUpgradeProgressList?: NodeUpgradeProgressInfo[];
}

/**
 * Information about the upgrade domain progress at the time of upgrade failure.
 */
export interface FailureUpgradeDomainProgressInfo {
  /**
   * The name of the upgrade domain
   */
  domainName?: string;
  /**
   * List of upgrading nodes and their statuses
   */
  nodeUpgradeProgressList?: NodeUpgradeProgressInfo[];
}

/**
 * Describes the parameters for an application upgrade.
 */
export interface ApplicationUpgradeProgressInfo {
  /**
   * The name of the target application, including the 'fabric:' URI scheme.
   */
  name?: string;
  /**
   * The application type name as defined in the application manifest.
   */
  typeName?: string;
  /**
   * The target application type version (found in the application manifest) for the application
   * upgrade.
   */
  targetApplicationTypeVersion?: string;
  /**
   * List of upgrade domains and their statuses.
   */
  upgradeDomains?: UpgradeDomainInfo[];
  /**
   * The state of the upgrade domain. Possible values include: 'Invalid', 'RollingBackInProgress',
   * 'RollingBackCompleted', 'RollingForwardPending', 'RollingForwardInProgress',
   * 'RollingForwardCompleted', 'Failed'
   */
  upgradeState?: string;
  /**
   * The name of the next upgrade domain to be processed.
   */
  nextUpgradeDomain?: string;
  /**
   * The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto,
   * UnmonitoredManual, and Monitored. Possible values include: 'Invalid', 'UnmonitoredAuto',
   * 'UnmonitoredManual', 'Monitored'
   */
  rollingUpgradeMode?: string;
  /**
   * Describes the parameters for an application upgrade. Note that upgrade description replaces
   * the existing application description. This means that if the parameters are not specified, the
   * existing parameters on the applications will be overwritten with the empty parameters list.
   * This would result in the application using the default value of the parameters from the
   * application manifest. If you do not want to change any existing parameter values, please get
   * the application parameters first using the GetApplicationInfo query and then supply those
   * values as Parameters in this ApplicationUpgradeDescription.
   */
  upgradeDescription?: ApplicationUpgradeDescription;
  /**
   * The estimated total amount of time spent processing the overall upgrade.
   */
  upgradeDurationInMilliseconds?: string;
  /**
   * The estimated total amount of time spent processing the current upgrade domain.
   */
  upgradeDomainDurationInMilliseconds?: string;
  /**
   * List of health evaluations that resulted in the current aggregated health state.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
  /**
   * Information about the current in-progress upgrade domain.
   */
  currentUpgradeDomainProgress?: CurrentUpgradeDomainProgressInfo;
  /**
   * The estimated UTC datetime when the upgrade started.
   */
  startTimestampUtc?: string;
  /**
   * The estimated UTC datetime when the upgrade failed and FailureAction was executed.
   */
  failureTimestampUtc?: string;
  /**
   * The cause of an upgrade failure that resulted in FailureAction being executed. Possible values
   * include: 'None', 'Interrupted', 'HealthCheck', 'UpgradeDomainTimeout', 'OverallUpgradeTimeout'
   */
  failureReason?: string;
  /**
   * Information about the upgrade domain progress at the time of upgrade failure.
   */
  upgradeDomainProgressAtFailure?: FailureUpgradeDomainProgressInfo;
  /**
   * Additional detailed information about the status of the pending upgrade.
   */
  upgradeStatusDetails?: string;
}

/**
 * Information about the standalone cluster configuration.
 */
export interface ClusterConfiguration {
  /**
   * The contents of the cluster configuration file.
   */
  clusterConfiguration?: string;
}

/**
 * Represents the base for all Cluster Events.
 */
export interface ClusterEvent extends FabricEvent {
}

/**
 * An internal ID used by Service Fabric to uniquely identify a node. Node Id is deterministically
 * generated from node name.
 */
export interface NodeId {
  /**
   * Value of the node Id. This is a 128 bit integer.
   */
  id?: string;
}

/**
 * Represents the health state of a node, which contains the node identifier and its aggregated
 * health state.
 */
export interface NodeHealthState extends EntityHealthState {
  /**
   * The name of a Service Fabric node.
   */
  name?: string;
  /**
   * An internal ID used by Service Fabric to uniquely identify a node. Node Id is
   * deterministically generated from node name.
   */
  id?: NodeId;
}

/**
 * Represents the health of the cluster.
 * Contains the cluster aggregated health state, the cluster application and node health states as
 * well as the health events and the unhealthy evaluations.
 */
export interface ClusterHealth extends EntityHealth {
  /**
   * Cluster node health states as found in the health store.
   */
  nodeHealthStates?: NodeHealthState[];
  /**
   * Cluster application health states as found in the health store.
   */
  applicationHealthStates?: ApplicationHealthState[];
}

/**
 * Represents the health state chunk of a node, which contains the node name and its aggregated
 * health state.
 */
export interface NodeHealthStateChunk extends EntityHealthStateChunk {
  /**
   * The name of a Service Fabric node.
   */
  nodeName?: string;
}

/**
 * The list of node health state chunks in the cluster that respect the input filters in the chunk
 * query. Returned by get cluster health state chunks query.
 */
export interface NodeHealthStateChunkList extends EntityHealthStateChunkList {
  /**
   * The list of node health state chunks that respect the input filters in the chunk query.
   */
  items?: NodeHealthStateChunk[];
}

/**
 * Represents the health chunk of the cluster.
 * Contains the cluster aggregated health state, and the cluster entities that respect the input
 * filter.
 */
export interface ClusterHealthChunk {
  /**
   * The HealthState representing the aggregated health state of the cluster computed by Health
   * Manager.
   * The health evaluation of the entity reflects all events reported on the entity and its
   * children (if any).
   * The aggregation is done by applying the desired cluster health policy and the application
   * health policies. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error', 'Unknown'
   */
  healthState?: string;
  /**
   * The list of node health state chunks in the cluster that respect the filters in the cluster
   * health chunk query description.
   */
  nodeHealthStateChunks?: NodeHealthStateChunkList;
  /**
   * The list of application health state chunks in the cluster that respect the filters in the
   * cluster health chunk query description.
   */
  applicationHealthStateChunks?: ApplicationHealthStateChunkList;
}

/**
 * Defines matching criteria to determine whether a node should be included in the returned cluster
 * health chunk.
 * One filter can match zero, one or multiple nodes, depending on its properties.
 * Can be specified in the cluster health chunk query description.
 */
export interface NodeHealthStateFilter {
  /**
   * Name of the node that matches the filter. The filter is applied only to the specified node, if
   * it exists.
   * If the node doesn't exist, no node is returned in the cluster health chunk based on this
   * filter.
   * If the node exists, it is included in the cluster health chunk if the health state matches the
   * other filter properties.
   * If not specified, all nodes that match the parent filters (if any) are taken into
   * consideration and matched against the other filter members, like health state filter.
   */
  nodeNameFilter?: string;
  /**
   * The filter for the health state of the nodes. It allows selecting nodes if they match the
   * desired health states.
   * The possible values are integer value of one of the following health states. Only nodes that
   * match the filter are returned. All nodes are used to evaluate the cluster aggregated health
   * state.
   * If not specified, default value is None, unless the node name is specified. If the filter has
   * default value and node name is specified, the matching node is returned.
   * The state values are flag-based enumeration, so the value could be a combination of these
   * values obtained using bitwise 'OR' operator.
   * For example, if the provided value is 6, it matches nodes with HealthState value of OK (2) and
   * Warning (4).
   *
   * - Default - Default value. Matches any HealthState. The value is zero.
   * - None - Filter that doesn't match any HealthState value. Used in order to return no results
   * on a given collection of states. The value is 1.
   * - Ok - Filter that matches input with HealthState value Ok. The value is 2.
   * - Warning - Filter that matches input with HealthState value Warning. The value is 4.
   * - Error - Filter that matches input with HealthState value Error. The value is 8.
   * - All - Filter that matches input with any HealthState value. The value is 65535.
   */
  healthStateFilter?: number;
}

/**
 * Defines a health policy used to evaluate the health of the cluster or of a cluster node.
 */
export interface ClusterHealthPolicy {
  /**
   * Indicates whether warnings are treated with the same severity as errors.
   */
  considerWarningAsError?: boolean;
  /**
   * The maximum allowed percentage of unhealthy nodes before reporting an error. For example, to
   * allow 10% of nodes to be unhealthy, this value would be 10.
   *
   * The percentage represents the maximum tolerated percentage of nodes that can be unhealthy
   * before the cluster is considered in error.
   * If the percentage is respected but there is at least one unhealthy node, the health is
   * evaluated as Warning.
   * The percentage is calculated by dividing the number of unhealthy nodes over the total number
   * of nodes in the cluster.
   * The computation rounds up to tolerate one failure on small numbers of nodes. Default
   * percentage is zero.
   *
   * In large clusters, some nodes will always be down or out for repairs, so this percentage
   * should be configured to tolerate that.
   */
  maxPercentUnhealthyNodes?: number;
  /**
   * The maximum allowed percentage of unhealthy applications before reporting an error. For
   * example, to allow 10% of applications to be unhealthy, this value would be 10.
   *
   * The percentage represents the maximum tolerated percentage of applications that can be
   * unhealthy before the cluster is considered in error.
   * If the percentage is respected but there is at least one unhealthy application, the health is
   * evaluated as Warning.
   * This is calculated by dividing the number of unhealthy applications over the total number of
   * application instances in the cluster, excluding applications of application types that are
   * included in the ApplicationTypeHealthPolicyMap.
   * The computation rounds up to tolerate one failure on small numbers of applications. Default
   * percentage is zero.
   */
  maxPercentUnhealthyApplications?: number;
  /**
   * Defines a map with max percentage unhealthy applications for specific application types.
   * Each entry specifies as key the application type name and as value an integer that represents
   * the MaxPercentUnhealthyApplications percentage used to evaluate the applications of the
   * specified application type.
   *
   * The application type health policy map can be used during cluster health evaluation to
   * describe special application types.
   * The application types included in the map are evaluated against the percentage specified in
   * the map, and not with the global MaxPercentUnhealthyApplications defined in the cluster health
   * policy.
   * The applications of application types specified in the map are not counted against the global
   * pool of applications.
   * For example, if some applications of a type are critical, the cluster administrator can add an
   * entry to the map for that application type
   * and assign it a value of 0% (that is, do not tolerate any failures).
   * All other applications can be evaluated with MaxPercentUnhealthyApplications set to 20% to
   * tolerate some failures out of the thousands of application instances.
   * The application type health policy map is used only if the cluster manifest enables
   * application type health evaluation using the configuration entry for
   * HealthManager/EnableApplicationTypeHealthEvaluation.
   */
  applicationTypeHealthPolicyMap?: ApplicationTypeHealthPolicyMapItem[];
}

/**
 * The cluster health chunk query description, which can specify the health policies to evaluate
 * cluster health and very expressive filters to select which cluster entities to include in
 * response.
 */
export interface ClusterHealthChunkQueryDescription {
  /**
   * Defines a list of filters that specify which nodes to be included in the returned cluster
   * health chunk.
   * If no filters are specified, no nodes are returned. All the nodes are used to evaluate the
   * cluster's aggregated health state, regardless of the input filters.
   * The cluster health chunk query may specify multiple node filters.
   * For example, it can specify a filter to return all nodes with health state Error and another
   * filter to always include a node identified by its NodeName.
   */
  nodeFilters?: NodeHealthStateFilter[];
  /**
   * Defines a list of filters that specify which applications to be included in the returned
   * cluster health chunk.
   * If no filters are specified, no applications are returned. All the applications are used to
   * evaluate the cluster's aggregated health state, regardless of the input filters.
   * The cluster health chunk query may specify multiple application filters.
   * For example, it can specify a filter to return all applications with health state Error and
   * another filter to always include applications of a specified application type.
   */
  applicationFilters?: ApplicationHealthStateFilter[];
  /**
   * Defines a health policy used to evaluate the health of the cluster or of a cluster node.
   */
  clusterHealthPolicy?: ClusterHealthPolicy;
  /**
   * Defines the application health policy map used to evaluate the health of an application or one
   * of its children entities.
   */
  applicationHealthPolicies?: ApplicationHealthPolicies;
}

/**
 * Health policies to evaluate cluster health.
 */
export interface ClusterHealthPolicies {
  /**
   * Defines a map that contains specific application health policies for different applications.
   * Each entry specifies as key the application name and as value an ApplicationHealthPolicy used
   * to evaluate the application health.
   * If an application is not specified in the map, the application health evaluation uses the
   * ApplicationHealthPolicy found in its application manifest or the default application health
   * policy (if no health policy is defined in the manifest).
   * The map is empty by default.
   */
  applicationHealthPolicyMap?: ApplicationHealthPolicyMapItem[];
  /**
   * Defines a health policy used to evaluate the health of the cluster or of a cluster node.
   */
  clusterHealthPolicy?: ClusterHealthPolicy;
}

/**
 * Information about the cluster manifest.
 */
export interface ClusterManifest {
  /**
   * The contents of the cluster manifest file.
   */
  manifest?: string;
}

/**
 * The cluster version.
 */
export interface ClusterVersion {
  /**
   * The Service Fabric cluster runtime version.
   */
  version?: string;
}

/**
 * parameters for making container API call.
 */
export interface ContainerApiRequestBody {
  /**
   * HTTP verb of container REST API, defaults to "GET"
   */
  httpVerb?: string;
  /**
   * URI path of container REST API
   */
  uriPath: string;
  /**
   * Content type of container REST API request, defaults to "application/json"
   */
  contentType?: string;
  /**
   * HTTP request body of container REST API
   */
  body?: string;
}

/**
 * Container API result.
 */
export interface ContainerApiResult {
  /**
   * HTTP status code returned by the target container API
   */
  status: number;
  /**
   * HTTP content type
   */
  contentType?: string;
  /**
   * HTTP content encoding
   */
  contentEncoding?: string;
  /**
   * container API result body
   */
  body?: string;
}

/**
 * Response body that wraps container API result.
 */
export interface ContainerApiResponse {
  /**
   * Container API result.
   */
  containerApiResult: ContainerApiResult;
}

/**
 * Represents the base for all Container Events.
 */
export interface ContainerInstanceEvent extends FabricEvent {
}

/**
 * Describes the intent or reason for deactivating the node.
 */
export interface DeactivationIntentDescription {
  /**
   * Describes the intent or reason for deactivating the node. The possible values are following.
   * Possible values include: 'Pause', 'Restart', 'RemoveData'
   */
  deactivationIntent?: string;
}

/**
 * Represents health evaluation for delta nodes, containing health evaluations for each unhealthy
 * node that impacted current aggregated health state.
 * Can be returned during cluster upgrade when the aggregated health state of the cluster is
 * Warning or Error.
 */
export interface DeltaNodesCheckHealthEvaluation extends HealthEvaluation {
  /**
   * Number of nodes with aggregated heath state Error in the health store at the beginning of the
   * cluster upgrade.
   */
  baselineErrorCount?: number;
  /**
   * Total number of nodes in the health store at the beginning of the cluster upgrade.
   */
  baselineTotalCount?: number;
  /**
   * Maximum allowed percentage of delta unhealthy nodes from the ClusterUpgradeHealthPolicy.
   */
  maxPercentDeltaUnhealthyNodes?: number;
  /**
   * Total number of nodes in the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state.
   * Includes all the unhealthy NodeHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Represents the health state of a deployed service package, containing the entity identifier and
 * the aggregated health state.
 */
export interface DeployedServicePackageHealthState extends EntityHealthState {
  /**
   * Name of the node on which the service package is deployed.
   */
  nodeName?: string;
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
  /**
   * Name of the manifest describing the service package.
   */
  serviceManifestName?: string;
  /**
   * The ActivationId of a deployed service package. If ServicePackageActivationMode specified at
   * the time of creating the service
   * is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'),
   * then value of ServicePackageActivationId
   * is always an empty string.
   */
  servicePackageActivationId?: string;
}

/**
 * Information about the health of an application deployed on a Service Fabric node.
 */
export interface DeployedApplicationHealth extends EntityHealth {
  /**
   * Name of the application deployed on the node whose health information is described by this
   * object.
   */
  name?: string;
  /**
   * Name of the node where this application is deployed.
   */
  nodeName?: string;
  /**
   * Deployed service package health states for the current deployed application as found in the
   * health store.
   */
  deployedServicePackageHealthStates?: DeployedServicePackageHealthState[];
}

/**
 * Represents health evaluation for a deployed application, containing information about the data
 * and the algorithm used by the health store to evaluate health.
 */
export interface DeployedApplicationHealthEvaluation extends HealthEvaluation {
  /**
   * Name of the node where the application is deployed to.
   */
  nodeName?: string;
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
  /**
   * List of  unhealthy evaluations that led to the current aggregated health state of the deployed
   * application.
   * The types of the unhealthy evaluations can be DeployedServicePackagesHealthEvaluation or
   * EventHealthEvaluation.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Information about application deployed on the node.
 */
export interface DeployedApplicationInfo {
  /**
   * The identity of the application. This is an encoded representation of the application name.
   * This is used in the REST APIs to identify the application resource.
   * Starting in version 6.0, hierarchical names are delimited with the "\~" character. For
   * example, if the application name is "fabric:/myapp/app1",
   * the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
   */
  id?: string;
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  name?: string;
  /**
   * The application type name as defined in the application manifest.
   */
  typeName?: string;
  /**
   * The status of the application deployed on the node. Following are the possible values.
   * Possible values include: 'Invalid', 'Downloading', 'Activating', 'Active', 'Upgrading',
   * 'Deactivating'
   */
  status?: string;
  /**
   * The work directory of the application on the node. The work directory can be used to store
   * application data.
   */
  workDirectory?: string;
  /**
   * The log directory of the application on the node. The log directory can be used to store
   * application logs.
   */
  logDirectory?: string;
  /**
   * The temp directory of the application on the node. The code packages belonging to the
   * application are forked with this directory set as their temporary directory.
   */
  tempDirectory?: string;
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  healthState?: string;
}

/**
 * Represents health evaluation for deployed applications, containing health evaluations for each
 * unhealthy deployed application that impacted current aggregated health state.
 * Can be returned when evaluating application health and the aggregated health state is either
 * Error or Warning.
 */
export interface DeployedApplicationsHealthEvaluation extends HealthEvaluation {
  /**
   * Maximum allowed percentage of unhealthy deployed applications from the
   * ApplicationHealthPolicy.
   */
  maxPercentUnhealthyDeployedApplications?: number;
  /**
   * Total number of deployed applications of the application in the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy DeployedApplicationHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Information about the health of a service package for a specific application deployed on a
 * Service Fabric node.
 */
export interface DeployedServicePackageHealth extends EntityHealth {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
  /**
   * Name of the service manifest.
   */
  serviceManifestName?: string;
  /**
   * Name of the node where this service package is deployed.
   */
  nodeName?: string;
}

/**
 * Represents health evaluation for a deployed service package, containing information about the
 * data and the algorithm used by health store to evaluate health. The evaluation is returned only
 * when the aggregated health state is either Error or Warning.
 */
export interface DeployedServicePackageHealthEvaluation extends HealthEvaluation {
  /**
   * The name of a Service Fabric node.
   */
  nodeName?: string;
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
  /**
   * The name of the service manifest.
   */
  serviceManifestName?: string;
  /**
   * List of unhealthy evaluations that led to the current aggregated health state. The type of the
   * unhealthy evaluations can be EventHealthEvaluation.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Represents health evaluation for deployed service packages, containing health evaluations for
 * each unhealthy deployed service package that impacted current aggregated health state. Can be
 * returned when evaluating deployed application health and the aggregated health state is either
 * Error or Warning.
 */
export interface DeployedServicePackagesHealthEvaluation extends HealthEvaluation {
  /**
   * Total number of deployed service packages of the deployed application in the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy DeployedServicePackageHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Information about a Service Fabric service replica deployed on a node.
 */
export interface DeployedServiceReplicaInfo {
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  serviceName?: string;
  /**
   * Name of the service type as specified in the service manifest.
   */
  serviceTypeName?: string;
  /**
   * The name of the service manifest in which this service type is defined.
   */
  serviceManifestName?: string;
  /**
   * The name of the code package that hosts this replica.
   */
  codePackageName?: string;
  /**
   * An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly
   * generated GUID when the service was created. The partition ID is unique and does not change
   * for the lifetime of the service. If the same service was deleted and recreated the IDs of its
   * partitions would be different.
   */
  partitionId?: string;
  /**
   * The status of a replica of a service. Possible values include: 'Invalid', 'InBuild',
   * 'Standby', 'Ready', 'Down', 'Dropped'
   */
  replicaStatus?: string;
  /**
   * The last address returned by the replica in Open or ChangeRole.
   */
  address?: string;
  /**
   * The ActivationId of a deployed service package. If ServicePackageActivationMode specified at
   * the time of creating the service
   * is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'),
   * then value of ServicePackageActivationId
   * is always an empty string.
   */
  servicePackageActivationId?: string;
  /**
   * Host process ID of the process that is hosting the replica. This will be zero if the replica
   * is down. In hyper-v containers this host process ID will be from different kernel.
   */
  hostProcessId?: string;
  /**
   * Polymorphic Discriminator
   */
  serviceKind: string;
}

/**
 * Information about current reconfiguration like phase, type, previous configuration role of
 * replica and reconfiguration start date time.
 */
export interface ReconfigurationInformation {
  /**
   * Replica role before reconfiguration started. Possible values include: 'Unknown', 'None',
   * 'Primary', 'IdleSecondary', 'ActiveSecondary'
   */
  previousConfigurationRole?: string;
  /**
   * Current phase of ongoing reconfiguration. If no reconfiguration is taking place then this
   * value will be "None". Possible values include: 'Unknown', 'None', 'Phase0', 'Phase1',
   * 'Phase2', 'Phase3', 'Phase4', 'AbortPhaseZero'
   */
  reconfigurationPhase?: string;
  /**
   * Type of current ongoing reconfiguration. If no reconfiguration is taking place then this value
   * will be "None". Possible values include: 'Unknown', 'SwapPrimary', 'Failover', 'Other'
   */
  reconfigurationType?: string;
  /**
   * Start time (in UTC) of the ongoing reconfiguration. If no reconfiguration is taking place then
   * this value will be zero date-time.
   */
  reconfigurationStartTimeUtc?: Date;
}

/**
 * Information about a stateful service replica deployed on a node.
 */
export interface DeployedStatefulServiceReplicaInfo extends DeployedServiceReplicaInfo {
  /**
   * Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a
   * replica of a partition. It is unique within a partition and does not change for the lifetime
   * of the replica. If a replica gets dropped and another replica gets created on the same node
   * for the same partition, it will get a different value for the id. Sometimes the id of a
   * stateless service instance is also referred as a replica id.
   */
  replicaId?: string;
  /**
   * The role of a replica of a stateful service. Possible values include: 'Unknown', 'None',
   * 'Primary', 'IdleSecondary', 'ActiveSecondary'
   */
  replicaRole?: string;
  /**
   * Information about current reconfiguration like phase, type, previous configuration role of
   * replica and reconfiguration start date time.
   */
  reconfigurationInformation?: ReconfigurationInformation;
}

/**
 * Information about a stateless service instance deployed on a node.
 */
export interface DeployedStatelessServiceInstanceInfo extends DeployedServiceReplicaInfo {
  /**
   * Id of a stateless service instance. InstanceId is used by Service Fabric to uniquely identify
   * an instance of a partition of a stateless service. It is unique within a partition and does
   * not change for the lifetime of the instance. If the instance has failed over on the same or
   * different node, it will get a different value for the InstanceId.
   */
  instanceId?: string;
}

/**
 * Represents common health report information. It is included in all health reports sent to health
 * store and in all health events returned by health queries.
 */
export interface HealthInformation {
  /**
   * The source name that identifies the client/watchdog/system component that generated the health
   * information.
   */
  sourceId: string;
  /**
   * The property of the health information. An entity can have health reports for different
   * properties.
   * The property is a string and not a fixed enumeration to allow the reporter flexibility to
   * categorize the state condition that triggers the report.
   * For example, a reporter with SourceId "LocalWatchdog" can monitor the state of the available
   * disk on a node,
   * so it can report "AvailableDisk" property on that node.
   * The same reporter can monitor the node connectivity, so it can report a property
   * "Connectivity" on the same node.
   * In the health store, these reports are treated as separate health events for the specified
   * node.
   *
   * Together with the SourceId, the property uniquely identifies the health information.
   */
  property: string;
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  healthState: string;
  /**
   * The duration for which this health report is valid. This field uses ISO8601 format for
   * specifying the duration.
   * When clients report periodically, they should send reports with higher frequency than time to
   * live.
   * If clients report on transition, they can set the time to live to infinite.
   * When time to live expires, the health event that contains the health information
   * is either removed from health store, if RemoveWhenExpired is true, or evaluated at error, if
   * RemoveWhenExpired false.
   *
   * If not specified, time to live defaults to infinite value.
   */
  timeToLiveInMilliSeconds?: moment.Duration;
  /**
   * The description of the health information. It represents free text used to add human readable
   * information about the report.
   * The maximum string length for the description is 4096 characters.
   * If the provided string is longer, it will be automatically truncated.
   * When truncated, the last characters of the description contain a marker "[Truncated]", and
   * total string size is 4096 characters.
   * The presence of the marker indicates to users that truncation occurred.
   * Note that when truncated, the description has less than 4096 characters from the original
   * string.
   */
  description?: string;
  /**
   * The sequence number for this health report as a numeric string.
   * The report sequence number is used by the health store to detect stale reports.
   * If not specified, a sequence number is auto-generated by the health client when a report is
   * added.
   */
  sequenceNumber?: string;
  /**
   * Value that indicates whether the report is removed from health store when it expires.
   * If set to true, the report is removed from the health store after it expires.
   * If set to false, the report is treated as an error when expired. The value of this property is
   * false by default.
   * When clients report periodically, they should set RemoveWhenExpired false (default).
   * This way, if the reporter has issues (e.g. deadlock) and can't report, the entity is evaluated
   * at error when the health report expires.
   * This flags the entity as being in Error health state.
   */
  removeWhenExpired?: boolean;
}

/**
 * Represents health information reported on a health entity, such as cluster, application or node,
 * with additional metadata added by the Health Manager.
 */
export interface HealthEvent extends HealthInformation {
  /**
   * Returns true if the health event is expired, otherwise false.
   */
  isExpired?: boolean;
  /**
   * The date and time when the health report was sent by the source.
   */
  sourceUtcTimestamp?: Date;
  /**
   * The date and time when the health report was last modified by the health store.
   */
  lastModifiedUtcTimestamp?: Date;
  /**
   * If the current health state is 'Ok', this property returns the time at which the health report
   * was first reported with 'Ok'.
   * For periodic reporting, many reports with the same state may have been generated.
   * This property returns the date and time when the first 'Ok' health report was received.
   *
   * If the current health state is 'Error' or 'Warning', returns the date and time at which the
   * health state was last in 'Ok', before transitioning to a different state.
   *
   * If the health state was never 'Ok', the value will be zero date-time.
   */
  lastOkTransitionAt?: Date;
  /**
   * If the current health state is 'Warning', this property returns the time at which the health
   * report was first reported with 'Warning'. For periodic reporting, many reports with the same
   * state may have been generated however, this property returns only the date and time at the
   * first 'Warning' health report was received.
   *
   * If the current health state is 'Ok' or 'Error', returns the date and time at which the health
   * state was last in 'Warning', before transitioning to a different state.
   *
   * If the health state was never 'Warning', the value will be zero date-time.
   */
  lastWarningTransitionAt?: Date;
  /**
   * If the current health state is 'Error', this property returns the time at which the health
   * report was first reported with 'Error'. For periodic reporting, many reports with the same
   * state may have been generated however, this property returns only the date and time at the
   * first 'Error' health report was received.
   *
   * If the current health state is 'Ok' or 'Warning', returns the date and time at which the
   * health state was last in 'Error', before transitioning to a different state.
   *
   * If the health state was never 'Error', the value will be zero date-time.
   */
  lastErrorTransitionAt?: Date;
}

/**
 * Represents information about how many health entities are in Ok, Warning and Error health state.
 */
export interface HealthStateCount {
  /**
   * The number of health entities with aggregated health state Ok.
   */
  okCount?: number;
  /**
   * The number of health entities with aggregated health state Warning.
   */
  warningCount?: number;
  /**
   * The number of health entities with aggregated health state Error.
   */
  errorCount?: number;
}

/**
 * Represents health state count for entities of the specified entity kind.
 */
export interface EntityKindHealthStateCount {
  /**
   * The entity kind for which health states are evaluated. Possible values include: 'Invalid',
   * 'Node', 'Partition', 'Service', 'Application', 'Replica', 'DeployedApplication',
   * 'DeployedServicePackage', 'Cluster'
   */
  entityKind?: string;
  /**
   * The health state count for the entities of the specified kind.
   */
  healthStateCount?: HealthStateCount;
}

/**
 * The health statistics of an entity, returned as part of the health query result when the query
 * description is configured to include statistics.
 * The statistics include health state counts for all children types of the current entity.
 * For example, for cluster, the health statistics include health state counts for nodes,
 * applications, services, partitions, replicas, deployed applications and deployed service
 * packages.
 * For partition, the health statistics include health counts for replicas.
 */
export interface HealthStatistics {
  /**
   * List of health state counts per entity kind, which keeps track of how many children of the
   * queried entity are in Ok, Warning and Error state.
   */
  healthStateCountList?: EntityKindHealthStateCount[];
}

/**
 * An Epoch is a configuration number for the partition as a whole. When the configuration of the
 * replica set changes, for example when the Primary replica changes, the operations that are
 * replicated from the new Primary replica are said to be a new Epoch from the ones which were sent
 * by the old Primary replica.
 */
export interface Epoch {
  /**
   * The current configuration number of this Epoch. The configuration number is an increasing
   * value that is updated whenever the configuration of this replica set changes.
   */
  configurationVersion?: string;
  /**
   * The current dataloss number of this Epoch. The data loss number property is an increasing
   * value which is updated whenever data loss is suspected, as when loss of a quorum of replicas
   * in the replica set that includes the Primary replica.
   */
  dataLossVersion?: string;
}

/**
 * Represents health evaluation of a HealthEvent that was reported on the entity.
 * The health evaluation is returned when evaluating health of an entity results in Error or
 * Warning.
 */
export interface EventHealthEvaluation extends HealthEvaluation {
  /**
   * Indicates whether warnings are treated with the same severity as errors. The field is
   * specified in the health policy used to evaluate the entity.
   */
  considerWarningAsError?: boolean;
  /**
   * Represents health information reported on a health entity, such as cluster, application or
   * node, with additional metadata added by the Health Manager.
   */
  unhealthyEvent?: HealthEvent;
}

/**
 * Information about a Service Fabric code version.
 */
export interface FabricCodeVersionInfo {
  /**
   * The product version of Service Fabric.
   */
  codeVersion?: string;
}

/**
 * Information about a Service Fabric config version.
 */
export interface FabricConfigVersionInfo {
  /**
   * The config version of Service Fabric.
   */
  configVersion?: string;
}

/**
 * Information about a standalone cluster configuration upgrade status.
 */
export interface ClusterConfigurationUpgradeStatusInfo {
  /**
   * The state of the upgrade domain. Possible values include: 'Invalid', 'RollingBackInProgress',
   * 'RollingBackCompleted', 'RollingForwardPending', 'RollingForwardInProgress',
   * 'RollingForwardCompleted', 'Failed'
   */
  upgradeState?: string;
  /**
   * The cluster manifest version.
   */
  progressStatus?: number;
  /**
   * The cluster configuration version.
   */
  configVersion?: string;
  /**
   * The cluster upgrade status details.
   */
  details?: string;
}

/**
 * Information about the partition identity, partitioning scheme and keys supported by it.
 */
export interface PartitionInformation {
  /**
   * An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly
   * generated GUID when the service was created. The partition ID is unique and does not change
   * for the lifetime of the service. If the same service was deleted and recreated the IDs of its
   * partitions would be different.
   */
  id?: string;
  /**
   * Polymorphic Discriminator
   */
  servicePartitionKind: string;
}

/**
 * Describes the partition information for the integer range that is based on partition schemes.
 */
export interface Int64RangePartitionInformation extends PartitionInformation {
  /**
   * Specifies the minimum key value handled by this partition.
   */
  lowKey?: string;
  /**
   * Specifies the maximum key value handled by this partition.
   */
  highKey?: string;
}

/**
 * Describes the partition information for the name as a string that is based on partition schemes.
 */
export interface NamedPartitionInformation extends PartitionInformation {
  /**
   * Name of the partition.
   */
  name?: string;
}

/**
 * Identity of the task related to deactivation operation on the node.
 */
export interface NodeDeactivationTaskId {
  /**
   * Value of the task id.
   */
  id?: string;
  /**
   * The type of the task that performed the node deactivation. Following are the possible values.
   * Possible values include: 'Invalid', 'Infrastructure', 'Repair', 'Client'
   */
  nodeDeactivationTaskType?: string;
}

/**
 * The task representing the deactivation operation on the node.
 */
export interface NodeDeactivationTask {
  /**
   * Identity of the task related to deactivation operation on the node.
   */
  nodeDeactivationTaskId?: NodeDeactivationTaskId;
  /**
   * The intent or the reason for deactivating the node. Following are the possible values for it.
   * Possible values include: 'Invalid', 'Pause', 'Restart', 'RemoveData', 'RemoveNode'
   */
  nodeDeactivationIntent?: string;
}

/**
 * Information about the node deactivation. This information is valid for a node that is undergoing
 * deactivation or has already been deactivated.
 */
export interface NodeDeactivationInfo {
  /**
   * The intent or the reason for deactivating the node. Following are the possible values for it.
   * Possible values include: 'Invalid', 'Pause', 'Restart', 'RemoveData', 'RemoveNode'
   */
  nodeDeactivationIntent?: string;
  /**
   * The status of node deactivation operation. Following are the possible values. Possible values
   * include: 'None', 'SafetyCheckInProgress', 'SafetyCheckComplete', 'Completed'
   */
  nodeDeactivationStatus?: string;
  /**
   * List of tasks representing the deactivation operation on the node.
   */
  nodeDeactivationTask?: NodeDeactivationTask[];
  /**
   * List of pending safety checks
   */
  pendingSafetyChecks?: SafetyCheckWrapper[];
}

/**
 * Represents the base for all Node Events.
 */
export interface NodeEvent extends FabricEvent {
  /**
   * The name of a Service Fabric node.
   */
  nodeName: string;
}

/**
 * Information about the health of a Service Fabric node.
 */
export interface NodeHealth extends EntityHealth {
  /**
   * Name of the node whose health information is described by this object.
   */
  name?: string;
}

/**
 * Represents health evaluation for a node, containing information about the data and the algorithm
 * used by health store to evaluate health. The evaluation is returned only when the aggregated
 * health state is either Error or Warning.
 */
export interface NodeHealthEvaluation extends HealthEvaluation {
  /**
   * The name of a Service Fabric node.
   */
  nodeName?: string;
  /**
   * List of unhealthy evaluations that led to the current aggregated health state of the node. The
   * types of the unhealthy evaluations can be EventHealthEvaluation.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Information about a node in Service Fabric cluster.
 */
export interface NodeInfo {
  /**
   * The name of a Service Fabric node.
   */
  name?: string;
  /**
   * The IP address or fully qualified domain name of the node.
   */
  ipAddressOrFQDN?: string;
  /**
   * The type of the node.
   */
  type?: string;
  /**
   * The version of Service Fabric binaries that the node is running.
   */
  codeVersion?: string;
  /**
   * The version of Service Fabric cluster manifest that the node is using.
   */
  configVersion?: string;
  /**
   * The status of the node. Possible values include: 'Invalid', 'Up', 'Down', 'Enabling',
   * 'Disabling', 'Disabled', 'Unknown', 'Removed'
   */
  nodeStatus?: string;
  /**
   * Time in seconds since the node has been in NodeStatus Up. Value zero indicates that the node
   * is not Up.
   */
  nodeUpTimeInSeconds?: string;
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  healthState?: string;
  /**
   * Indicates if the node is a seed node or not. Returns true if the node is a seed node,
   * otherwise false. A quorum of seed nodes are required for proper operation of Service Fabric
   * cluster.
   */
  isSeedNode?: boolean;
  /**
   * The upgrade domain of the node.
   */
  upgradeDomain?: string;
  /**
   * The fault domain of the node.
   */
  faultDomain?: string;
  /**
   * An internal ID used by Service Fabric to uniquely identify a node. Node Id is
   * deterministically generated from node name.
   */
  id?: NodeId;
  /**
   * The ID representing the node instance. While the ID of the node is deterministically generated
   * from the node name and remains same across restarts, the InstanceId changes every time node
   * restarts.
   */
  instanceId?: string;
  /**
   * Information about the node deactivation. This information is valid for a node that is
   * undergoing deactivation or has already been deactivated.
   */
  nodeDeactivationInfo?: NodeDeactivationInfo;
  /**
   * Indicates if the node is stopped by calling stop node API or not. Returns true if the node is
   * stopped, otherwise false.
   */
  isStopped?: boolean;
  /**
   * Time in seconds since the node has been in NodeStatus Down. Value zero indicates node is not
   * NodeStatus Down.
   */
  nodeDownTimeInSeconds?: string;
  /**
   * Date time in UTC when the node came up. If the node has never been up then this value will be
   * zero date time.
   */
  nodeUpAt?: Date;
  /**
   * Date time in UTC when the node went down. If node has never been down then this value will be
   * zero date time.
   */
  nodeDownAt?: Date;
}

/**
 * Represents data structure that contains load information for a certain metric on a node.
 */
export interface NodeLoadMetricInformation {
  /**
   * Name of the metric for which this load information is provided.
   */
  name?: string;
  /**
   * Total capacity on the node for this metric.
   */
  nodeCapacity?: string;
  /**
   * Current load on the node for this metric.
   */
  nodeLoad?: string;
  /**
   * The remaining capacity on the node for this metric.
   */
  nodeRemainingCapacity?: string;
  /**
   * Indicates if there is a capacity violation for this metric on the node.
   */
  isCapacityViolation?: boolean;
  /**
   * The value that indicates the reserved capacity for this metric on the node.
   */
  nodeBufferedCapacity?: string;
  /**
   * The remaining reserved capacity for this metric on the node.
   */
  nodeRemainingBufferedCapacity?: string;
}

/**
 * Information about load on a Service Fabric node. It holds a summary of all metrics and their
 * load on a node.
 */
export interface NodeLoadInfo {
  /**
   * Name of the node for which the load information is provided by this object.
   */
  nodeName?: string;
  /**
   * List that contains metrics and their load information on this node.
   */
  nodeLoadMetricInformation?: NodeLoadMetricInformation[];
}

/**
 * Represents health evaluation for nodes, containing health evaluations for each unhealthy node
 * that impacted current aggregated health state. Can be returned when evaluating cluster health
 * and the aggregated health state is either Error or Warning.
 */
export interface NodesHealthEvaluation extends HealthEvaluation {
  /**
   * Maximum allowed percentage of unhealthy nodes from the ClusterHealthPolicy.
   */
  maxPercentUnhealthyNodes?: number;
  /**
   * Total number of nodes found in the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy NodeHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * The list of applications in the cluster. The list is paged when all of the results cannot fit in
 * a single message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in this list.
 */
export interface PagedApplicationInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of application information.
   */
  items?: ApplicationInfo[];
}

/**
 * The list of deployed applications in activating, downloading, or active states on a node.
 * The list is paged when all of the results cannot fit in a single message.
 * The next set of results can be obtained by executing the same query with the continuation token
 * provided in this list.
 */
export interface PagedDeployedApplicationInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of deployed application information.
   */
  items?: DeployedApplicationInfo[];
}

/**
 * The list of nodes in the cluster. The list is paged when all of the results cannot fit in a
 * single message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in this list.
 */
export interface PagedNodeInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of node information.
   */
  items?: NodeInfo[];
}

/**
 * Information about a partition of a Service Fabric service.
 */
export interface ServicePartitionInfo {
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  healthState?: string;
  /**
   * The status of the service fabric service partition. Possible values include: 'Invalid',
   * 'Ready', 'NotReady', 'InQuorumLoss', 'Reconfiguring', 'Deleting'
   */
  partitionStatus?: string;
  /**
   * Information about the partition identity, partitioning scheme and keys supported by it.
   */
  partitionInformation?: PartitionInformation;
  /**
   * Polymorphic Discriminator
   */
  serviceKind: string;
}

/**
 * The list of partition in the cluster for a service. The list is paged when all of the results
 * cannot fit in a single message. The next set of results can be obtained by executing the same
 * query with the continuation token provided in this list.
 */
export interface PagedServicePartitionInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of service partition information.
   */
  items?: ServicePartitionInfo[];
}

/**
 * Information about the identity, status, health, node name, uptime, and other details about the
 * replica.
 */
export interface ReplicaInfo {
  /**
   * The status of a replica of a service. Possible values include: 'Invalid', 'InBuild',
   * 'Standby', 'Ready', 'Down', 'Dropped'
   */
  replicaStatus?: string;
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  healthState?: string;
  /**
   * The name of a Service Fabric node.
   */
  nodeName?: string;
  /**
   * The address the replica is listening on.
   */
  address?: string;
  /**
   * The last in build duration of the replica in seconds.
   */
  lastInBuildDurationInSeconds?: string;
  /**
   * Polymorphic Discriminator
   */
  serviceKind: string;
}

/**
 * The list of replicas in the cluster for a given partition. The list is paged when all of the
 * results cannot fit in a single message. The next set of results can be obtained by executing the
 * same query with the continuation token provided in this list.
 */
export interface PagedReplicaInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of replica information.
   */
  items?: ReplicaInfo[];
}

/**
 * Information about a Service Fabric service.
 */
export interface ServiceInfo {
  /**
   * The identity of the service. This ID is an encoded representation of the service name. This is
   * used in the REST APIs to identify the service resource.
   * Starting in version 6.0, hierarchical names are delimited with the "\~" character. For
   * example, if the service name is "fabric:/myapp/app1/svc1",
   * the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous
   * versions.
   */
  id?: string;
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  name?: string;
  /**
   * Name of the service type as specified in the service manifest.
   */
  typeName?: string;
  /**
   * The version of the service manifest.
   */
  manifestVersion?: string;
  /**
   * The health state of a Service Fabric entity such as Cluster, Node, Application, Service,
   * Partition, Replica etc. Possible values include: 'Invalid', 'Ok', 'Warning', 'Error',
   * 'Unknown'
   */
  healthState?: string;
  /**
   * The status of the application. Possible values include: 'Unknown', 'Active', 'Upgrading',
   * 'Deleting', 'Creating', 'Failed'
   */
  serviceStatus?: string;
  /**
   * Whether the service is in a service group.
   */
  isServiceGroup?: boolean;
  /**
   * Polymorphic Discriminator
   */
  serviceKind: string;
}

/**
 * The list of services in the cluster for an application. The list is paged when all of the
 * results cannot fit in a single message. The next set of results can be obtained by executing the
 * same query with the continuation token provided in this list.
 */
export interface PagedServiceInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of service information.
   */
  items?: ServiceInfo[];
}

/**
 * Represents the base for all Partition Events.
 */
export interface PartitionEvent extends FabricEvent {
  /**
   * An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly
   * generated GUID when the service was created. The partition ID is unique and does not change
   * for the lifetime of the service. If the same service was deleted and recreated the IDs of its
   * partitions would be different.
   */
  partitionId: string;
}

/**
 * Represents the base for all Partition Analysis Events.
 */
export interface PartitionAnalysisEvent extends PartitionEvent {
  /**
   * Metadata about an Analysis Event.
   */
  metadata: AnalysisEventMetadata;
}

/**
 * Represents a base class for stateful service replica or stateless service instance health state.
 */
export interface ReplicaHealthState extends EntityHealthState {
  /**
   * The ID of the partition to which this replica belongs.
   */
  partitionId?: string;
  /**
   * Polymorphic Discriminator
   */
  serviceKind: string;
}

/**
 * Information about the health of a Service Fabric partition.
 */
export interface PartitionHealth extends EntityHealth {
  /**
   * ID of the partition whose health information is described by this object.
   */
  partitionId?: string;
  /**
   * The list of replica health states associated with the partition.
   */
  replicaHealthStates?: ReplicaHealthState[];
}

/**
 * Represents health evaluation for a partition, containing information about the data and the
 * algorithm used by health store to evaluate health. The evaluation is returned only when the
 * aggregated health state is either Error or Warning.
 */
export interface PartitionHealthEvaluation extends HealthEvaluation {
  /**
   * Id of the partition whose health evaluation is described by this object.
   */
  partitionId?: string;
  /**
   * List of unhealthy evaluations that led to the current aggregated health state of the
   * partition. The types of the unhealthy evaluations can be ReplicasHealthEvaluation or
   * EventHealthEvaluation.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Represents the health state of a partition, which contains the partition identifier and its
 * aggregated health state.
 */
export interface PartitionHealthState extends EntityHealthState {
  /**
   * Id of the partition whose health state is described by this object.
   */
  partitionId?: string;
}

/**
 * Describes the parameters for provisioning a cluster.
 */
export interface ProvisionFabricDescription {
  /**
   * The cluster code package file path.
   */
  codeFilePath?: string;
  /**
   * The cluster manifest file path.
   */
  clusterManifestFilePath?: string;
}

/**
 * Represents the type of registration or provision requested, and if the operation needs to be
 * asynchronous or not. Supported types of provision operations are from either image store or
 * external store.
 */
export interface ProvisionApplicationTypeDescriptionBase {
  /**
   * Indicates whether or not provisioning should occur asynchronously. When set to true, the
   * provision operation returns when the request is accepted by the system, and the provision
   * operation continues without any timeout limit. The default value is false. For large
   * application packages, we recommend setting the value to true.
   */
  async: boolean;
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes the operation to register or provision an application type using an application
 * package uploaded to the Service Fabric image store.
 */
export interface ProvisionApplicationTypeDescription extends ProvisionApplicationTypeDescriptionBase {
  /**
   * The relative path for the application package in the image store specified during the prior
   * upload operation.
   */
  applicationTypeBuildPath: string;
  /**
   * The kind of action that needs to be taken for cleaning up the application package after
   * successful provision. Possible values include: 'Invalid', 'Default', 'Automatic', 'Manual'
   */
  applicationPackageCleanupPolicy?: string;
}

/**
 * Describes the operation to register or provision an application type using an application
 * package from an external store instead of a package uploaded to the Service Fabric image store.
 */
export interface ExternalStoreProvisionApplicationTypeDescription extends ProvisionApplicationTypeDescriptionBase {
  /**
   * The path to the '.sfpkg' application package from where the application package can be
   * downloaded using HTTP or HTTPS protocols. The application package can be stored in an external
   * store that provides GET operation to download the file. Supported protocols are HTTP and
   * HTTPS, and the path must allow READ access.
   */
  applicationPackageDownloadUri: string;
  /**
   * The application type name represents the name of the application type found in the application
   * manifest.
   */
  applicationTypeName: string;
  /**
   * The application type version represents the version of the application type found in the
   * application manifest.
   */
  applicationTypeVersion: string;
}

/**
 * Describes the parameters for unprovisioning a cluster.
 */
export interface UnprovisionFabricDescription {
  /**
   * The cluster code package version.
   */
  codeVersion?: string;
  /**
   * The cluster manifest version.
   */
  configVersion?: string;
}

/**
 * Describes the parameters for resuming a cluster upgrade.
 */
export interface ResumeClusterUpgradeDescription {
  /**
   * The next upgrade domain for this cluster upgrade.
   */
  upgradeDomain: string;
}

/**
 * Defines a health policy used to evaluate the health of the cluster during a cluster upgrade.
 */
export interface ClusterUpgradeHealthPolicyObject {
  /**
   * The maximum allowed percentage of nodes health degradation allowed during cluster upgrades.
   * The delta is measured between the state of the nodes at the beginning of upgrade and the state
   * of the nodes at the time of the health evaluation. The check is performed after every upgrade
   * domain upgrade completion to make sure the global state of the cluster is within tolerated
   * limits. The default value is 10%.
   */
  maxPercentDeltaUnhealthyNodes?: number;
  /**
   * The maximum allowed percentage of upgrade domain nodes health degradation allowed during
   * cluster upgrades. The delta is measured between the state of the upgrade domain nodes at the
   * beginning of upgrade and the state of the upgrade domain nodes at the time of the health
   * evaluation. The check is performed after every upgrade domain upgrade completion for all
   * completed upgrade domains to make sure the state of the upgrade domains is within tolerated
   * limits. The default value is 15%.
   */
  maxPercentUpgradeDomainDeltaUnhealthyNodes?: number;
}

/**
 * Describes the parameters for starting a cluster upgrade.
 */
export interface StartClusterUpgradeDescription {
  /**
   * The cluster code version.
   */
  codeVersion?: string;
  /**
   * The cluster configuration version.
   */
  configVersion?: string;
  /**
   * The kind of upgrade out of the following possible values. Possible values include: 'Invalid',
   * 'Rolling'
   */
  upgradeKind?: string;
  /**
   * The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto,
   * UnmonitoredManual, and Monitored. Possible values include: 'Invalid', 'UnmonitoredAuto',
   * 'UnmonitoredManual', 'Monitored'
   */
  rollingUpgradeMode?: string;
  /**
   * The maximum amount of time to block processing of an upgrade domain and prevent loss of
   * availability when there are unexpected issues. When this timeout expires, processing of the
   * upgrade domain will proceed regardless of availability loss issues. The timeout is reset at
   * the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive.
   * (unsigned 32-bit integer).
   */
  upgradeReplicaSetCheckTimeoutInSeconds?: number;
  /**
   * If true, then processes are forcefully restarted during upgrade even when the code version has
   * not changed (the upgrade only changes configuration or data).
   */
  forceRestart?: boolean;
  /**
   * Describes the parameters for monitoring an upgrade in Monitored mode.
   */
  monitoringPolicy?: MonitoringPolicyDescription;
  /**
   * Defines a health policy used to evaluate the health of the cluster or of a cluster node.
   */
  clusterHealthPolicy?: ClusterHealthPolicy;
  /**
   * When true, enables delta health evaluation rather than absolute health evaluation after
   * completion of each upgrade domain.
   */
  enableDeltaHealthEvaluation?: boolean;
  /**
   * Defines a health policy used to evaluate the health of the cluster during a cluster upgrade.
   */
  clusterUpgradeHealthPolicy?: ClusterUpgradeHealthPolicyObject;
  /**
   * Defines the application health policy map used to evaluate the health of an application or one
   * of its children entities.
   */
  applicationHealthPolicyMap?: ApplicationHealthPolicies;
}

/**
 * Describes the parameters for updating a rolling upgrade of application or cluster.
 */
export interface RollingUpgradeUpdateDescription {
  /**
   * The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto,
   * UnmonitoredManual, and Monitored. Possible values include: 'Invalid', 'UnmonitoredAuto',
   * 'UnmonitoredManual', 'Monitored'
   */
  rollingUpgradeMode: string;
  /**
   * If true, then processes are forcefully restarted during upgrade even when the code version has
   * not changed (the upgrade only changes configuration or data).
   */
  forceRestart?: boolean;
  /**
   * The maximum amount of time to block processing of an upgrade domain and prevent loss of
   * availability when there are unexpected issues. When this timeout expires, processing of the
   * upgrade domain will proceed regardless of availability loss issues. The timeout is reset at
   * the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive.
   * (unsigned 32-bit integer).
   */
  replicaSetCheckTimeoutInMilliseconds?: number;
  /**
   * The compensating action to perform when a Monitored upgrade encounters monitoring policy or
   * health policy violations.
   * Invalid indicates the failure action is invalid. Rollback specifies that the upgrade will
   * start rolling back automatically.
   * Manual indicates that the upgrade will switch to UnmonitoredManual upgrade mode. Possible
   * values include: 'Invalid', 'Rollback', 'Manual'
   */
  failureAction?: string;
  /**
   * The amount of time to wait after completing an upgrade domain before applying health policies.
   * It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it
   * is interpreted as a number representing the total number of milliseconds.
   */
  healthCheckWaitDurationInMilliseconds?: string;
  /**
   * The amount of time that the application or cluster must remain healthy before the upgrade
   * proceeds to the next upgrade domain. It is first interpreted as a string representing an ISO
   * 8601 duration. If that fails, then it is interpreted as a number representing the total number
   * of milliseconds.
   */
  healthCheckStableDurationInMilliseconds?: string;
  /**
   * The amount of time to retry health evaluation when the application or cluster is unhealthy
   * before FailureAction is executed. It is first interpreted as a string representing an ISO 8601
   * duration. If that fails, then it is interpreted as a number representing the total number of
   * milliseconds.
   */
  healthCheckRetryTimeoutInMilliseconds?: string;
  /**
   * The amount of time the overall upgrade has to complete before FailureAction is executed. It is
   * first interpreted as a string representing an ISO 8601 duration. If that fails, then it is
   * interpreted as a number representing the total number of milliseconds.
   */
  upgradeTimeoutInMilliseconds?: string;
  /**
   * The amount of time each upgrade domain has to complete before FailureAction is executed. It is
   * first interpreted as a string representing an ISO 8601 duration. If that fails, then it is
   * interpreted as a number representing the total number of milliseconds.
   */
  upgradeDomainTimeoutInMilliseconds?: string;
}

/**
 * Parameters for updating a cluster upgrade.
 */
export interface UpdateClusterUpgradeDescription {
  /**
   * The type of upgrade out of the following possible values. Possible values include: 'Invalid',
   * 'Rolling', 'Rolling_ForceRestart'
   */
  upgradeKind?: string;
  /**
   * Describes the parameters for updating a rolling upgrade of application or cluster.
   */
  updateDescription?: RollingUpgradeUpdateDescription;
  /**
   * Defines a health policy used to evaluate the health of the cluster or of a cluster node.
   */
  clusterHealthPolicy?: ClusterHealthPolicy;
  /**
   * When true, enables delta health evaluation rather than absolute health evaluation after
   * completion of each upgrade domain.
   */
  enableDeltaHealthEvaluation?: boolean;
  /**
   * Defines a health policy used to evaluate the health of the cluster during a cluster upgrade.
   */
  clusterUpgradeHealthPolicy?: ClusterUpgradeHealthPolicyObject;
  /**
   * Defines the application health policy map used to evaluate the health of an application or one
   * of its children entities.
   */
  applicationHealthPolicyMap?: ApplicationHealthPolicies;
}

/**
 * Represents a safety check for the service partition being performed by service fabric before
 * continuing with operations.
 */
export interface PartitionSafetyCheck extends SafetyCheck {
  /**
   * Id of the partition which is undergoing the safety check.
   */
  partitionId?: string;
}

/**
 * Safety check that waits to ensure the availability of the partition. It waits until there are
 * replicas available such that bringing down this replica will not cause availability loss for the
 * partition.
 */
export interface EnsureAvailabilitySafetyCheck extends PartitionSafetyCheck {
}

/**
 * Safety check that ensures that a quorum of replicas are not lost for a partition.
 */
export interface EnsurePartitionQurumSafetyCheck extends PartitionSafetyCheck {
}

/**
 * Represents a safety check for the seed nodes being performed by service fabric before continuing
 * with node level operations.
 */
export interface SeedNodeSafetyCheck extends SafetyCheck {
}

/**
 * Represents health evaluation for the partitions of a service, containing health evaluations for
 * each unhealthy partition that impacts current aggregated health state. Can be returned when
 * evaluating service health and the aggregated health state is either Error or Warning.
 */
export interface PartitionsHealthEvaluation extends HealthEvaluation {
  /**
   * Maximum allowed percentage of unhealthy partitions per service from the
   * ServiceTypeHealthPolicy.
   */
  maxPercentUnhealthyPartitionsPerService?: number;
  /**
   * Total number of partitions of the service from the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy PartitionHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Represents the base for all Replica Events.
 */
export interface ReplicaEvent extends FabricEvent {
  /**
   * An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly
   * generated GUID when the service was created. The partition ID is unique and does not change
   * for the lifetime of the service. If the same service was deleted and recreated the IDs of its
   * partitions would be different.
   */
  partitionId: string;
  /**
   * Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a
   * replica of a partition. It is unique within a partition and does not change for the lifetime
   * of the replica. If a replica gets dropped and another replica gets created on the same node
   * for the same partition, it will get a different value for the id. Sometimes the id of a
   * stateless service instance is also referred as a replica id.
   */
  replicaId: number;
}

/**
 * Represents a base class for stateful service replica or stateless service instance health.
 * Contains the replica aggregated health state, the health events and the unhealthy evaluations.
 */
export interface ReplicaHealth extends EntityHealth {
  /**
   * Id of the partition to which this replica belongs.
   */
  partitionId?: string;
  /**
   * Polymorphic Discriminator
   */
  serviceKind: string;
}

/**
 * Represents health evaluation for a replica, containing information about the data and the
 * algorithm used by health store to evaluate health. The evaluation is returned only when the
 * aggregated health state is either Error or Warning.
 */
export interface ReplicaHealthEvaluation extends HealthEvaluation {
  /**
   * Id of the partition to which the replica belongs.
   */
  partitionId?: string;
  /**
   * Id of a stateful service replica or a stateless service instance. This ID is used in the
   * queries that apply to both stateful and stateless services. It is used by Service Fabric to
   * uniquely identify a replica of a partition of a stateful service or an instance of a stateless
   * service partition. It is unique within a partition and does not change for the lifetime of the
   * replica or the instance. If a stateful replica gets dropped and another replica gets created
   * on the same node for the same partition, it will get a different value for the ID. If a
   * stateless instance is failed over on the same or different node it will get a different value
   * for the ID.
   */
  replicaOrInstanceId?: string;
  /**
   * List of unhealthy evaluations that led to the current aggregated health state of the replica.
   * The types of the unhealthy evaluations can be EventHealthEvaluation.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Represents health evaluation for replicas, containing health evaluations for each unhealthy
 * replica that impacted current aggregated health state. Can be returned when evaluating partition
 * health and the aggregated health state is either Error or Warning.
 */
export interface ReplicasHealthEvaluation extends HealthEvaluation {
  /**
   * Maximum allowed percentage of unhealthy replicas per partition from the
   * ApplicationHealthPolicy.
   */
  maxPercentUnhealthyReplicasPerPartition?: number;
  /**
   * Total number of replicas in the partition from the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy ReplicaHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Describes the parameters to restart a Service Fabric node.
 */
export interface RestartNodeDescription {
  /**
   * The instance ID of the target node. If instance ID is specified the node is restarted only if
   * it matches with the current instance of the node. A default value of "0" would match any
   * instance ID. The instance ID can be obtained using get node query.
   */
  nodeInstanceId: string;
  /**
   * Specify True to create a dump of the fabric node process. This is case-sensitive. Possible
   * values include: 'False', 'True'
   */
  createFabricDump?: string;
}

/**
 * Represents the base for all Service Events.
 */
export interface ServiceEvent extends FabricEvent {
  /**
   * The identity of the service. This ID is an encoded representation of the service name. This is
   * used in the REST APIs to identify the service resource.
   * Starting in version 6.0, hierarchical names are delimited with the "\~" character. For
   * example, if the service name is "fabric:/myapp/app1/svc1",
   * the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous
   * versions.
   */
  serviceId: string;
}

/**
 * Defines description for creating a Service Fabric service from a template defined in the
 * application manifest.
 */
export interface ServiceFromTemplateDescription {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName: string;
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  serviceName: string;
  /**
   * Name of the service type as specified in the service manifest.
   */
  serviceTypeName: string;
  /**
   * The initialization data for the newly created service instance.
   */
  initializationData?: number[];
  /**
   * The activation mode of service package to be used for a service. Possible values include:
   * 'SharedProcess', 'ExclusiveProcess'
   */
  servicePackageActivationMode?: string;
  /**
   * The DNS name of the service. It requires the DNS system service to be enabled in Service
   * Fabric cluster.
   */
  serviceDnsName?: string;
}

/**
 * Represents health evaluation for a service, containing information about the data and the
 * algorithm used by health store to evaluate health. The evaluation is returned only when the
 * aggregated health state is either Error or Warning.
 */
export interface ServiceHealthEvaluation extends HealthEvaluation {
  /**
   * Name of the service whose health evaluation is described by this object.
   */
  serviceName?: string;
  /**
   * List of unhealthy evaluations that led to the current aggregated health state of the service.
   * The types of the unhealthy evaluations can be PartitionsHealthEvaluation or
   * EventHealthEvaluation.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Information about the health of a Service Fabric service.
 */
export interface ServiceHealth extends EntityHealth {
  /**
   * The name of the service whose health information is described by this object.
   */
  name?: string;
  /**
   * The list of partition health states associated with the service.
   */
  partitionHealthStates?: PartitionHealthState[];
}

/**
 * Information about the service name.
 */
export interface ServiceNameInfo {
  /**
   * The identity of the service. This ID is an encoded representation of the service name. This is
   * used in the REST APIs to identify the service resource.
   * Starting in version 6.0, hierarchical names are delimited with the "\~" character. For
   * example, if the service name is "fabric:/myapp/app1/svc1",
   * the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous
   * versions.
   */
  id?: string;
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  name?: string;
}

/**
 * Describes the policy to be used for placement of a Service Fabric service.
 */
export interface ServicePlacementPolicyDescription {
  /**
   * Polymorphic Discriminator
   */
  type: string;
}

/**
 * Describes the policy to be used for placement of a Service Fabric service where a particular
 * fault or upgrade domain should not be used for placement of the instances or replicas of that
 * service.
 */
export interface ServicePlacementInvalidDomainPolicyDescription extends ServicePlacementPolicyDescription {
  /**
   * The name of the domain that should not be used for placement.
   */
  domainName?: string;
}

/**
 * Describes the policy to be used for placement of a Service Fabric service where all replicas
 * must be able to be placed in order for any replicas to be created.
 */
export interface ServicePlacementNonPartiallyPlaceServicePolicyDescription extends ServicePlacementPolicyDescription {
}

/**
 * Describes the policy to be used for placement of a Service Fabric service where the service's
 * Primary replicas should optimally be placed in a particular domain.
 *
 * This placement policy is usually used with fault domains in scenarios where the Service Fabric
 * cluster is geographically distributed in order to indicate that a service's primary replica
 * should be located in a particular fault domain, which in geo-distributed scenarios usually
 * aligns with regional or datacenter boundaries. Note that since this is an optimization it is
 * possible that the Primary replica may not end up located in this domain due to failures,
 * capacity limits, or other constraints.
 */
export interface ServicePlacementPreferPrimaryDomainPolicyDescription extends ServicePlacementPolicyDescription {
  /**
   * The name of the domain that should used for placement as per this policy.
   */
  domainName?: string;
}

/**
 * Describes the policy to be used for placement of a Service Fabric service where the instances or
 * replicas of that service must be placed in a particular domain
 */
export interface ServicePlacementRequiredDomainPolicyDescription extends ServicePlacementPolicyDescription {
  /**
   * The name of the domain that should used for placement as per this policy.
   */
  domainName?: string;
}

/**
 * Describes the policy to be used for placement of a Service Fabric service where two replicas
 * from the same partition should never be placed in the same fault or upgrade domain.
 *
 * While this is not common it can expose the service to an increased risk of concurrent failures
 * due to unplanned outages or other cases of subsequent/concurrent failures. As an example,
 * consider a case where replicas are deployed across different data center, with one replica per
 * location. In the event that one of the datacenters goes offline, normally the replica that was
 * placed in that datacenter will be packed into one of the remaining datacenters. If this is not
 * desirable then this policy should be set.
 */
export interface ServicePlacementRequireDomainDistributionPolicyDescription extends ServicePlacementPolicyDescription {
  /**
   * The name of the domain that should used for placement as per this policy.
   */
  domainName?: string;
}

/**
 * Represents health evaluation for services of a certain service type belonging to an application,
 * containing health evaluations for each unhealthy service that impacted current aggregated health
 * state. Can be returned when evaluating application health and the aggregated health state is
 * either Error or Warning.
 */
export interface ServicesHealthEvaluation extends HealthEvaluation {
  /**
   * Name of the service type of the services.
   */
  serviceTypeName?: string;
  /**
   * Maximum allowed percentage of unhealthy services from the ServiceTypeHealthPolicy.
   */
  maxPercentUnhealthyServices?: number;
  /**
   * Total number of services of the current service type in the application from the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy ServiceHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Specifies a metric to load balance a service during runtime.
 */
export interface ServiceLoadMetricDescription {
  /**
   * The name of the metric. If the service chooses to report load during runtime, the load metric
   * name should match the name that is specified in Name exactly. Note that metric names are
   * case-sensitive.
   */
  name: string;
  /**
   * The service load metric relative weight, compared to other metrics configured for this
   * service, as a number. Possible values include: 'Zero', 'Low', 'Medium', 'High'
   */
  weight?: string;
  /**
   * Used only for Stateful services. The default amount of load, as a number, that this service
   * creates for this metric when it is a Primary replica.
   */
  primaryDefaultLoad?: number;
  /**
   * Used only for Stateful services. The default amount of load, as a number, that this service
   * creates for this metric when it is a Secondary replica.
   */
  secondaryDefaultLoad?: number;
  /**
   * Used only for Stateless services. The default amount of load, as a number, that this service
   * creates for this metric.
   */
  defaultLoad?: number;
}

/**
 * Describes extension of a service type defined in the service manifest.
 */
export interface ServiceTypeExtensionDescription {
  /**
   * The name of the extension.
   */
  key?: string;
  /**
   * The extension value.
   */
  value?: string;
}

/**
 * Describes a service type defined in the service manifest of a provisioned application type. The
 * properties the ones defined in the service manifest.
 */
export interface ServiceTypeDescription {
  /**
   * Indicates whether the service type is a stateful service type or a stateless service type.
   * This property is true if the service type is a stateful service type, false otherwise.
   */
  isStateful?: boolean;
  /**
   * Name of the service type as specified in the service manifest.
   */
  serviceTypeName?: string;
  /**
   * The placement constraint to be used when instantiating this service in a Service Fabric
   * cluster.
   */
  placementConstraints?: string;
  /**
   * The service load metrics is given as an array of ServiceLoadMetricDescription objects.
   */
  loadMetrics?: ServiceLoadMetricDescription[];
  /**
   * List of service placement policy descriptions.
   */
  servicePlacementPolicies?: ServicePlacementPolicyDescription[];
  /**
   * List of service type extensions.
   */
  extensions?: ServiceTypeExtensionDescription[];
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Information about a service type that is defined in a service manifest of a provisioned
 * application type.
 */
export interface ServiceTypeInfo {
  /**
   * Describes a service type defined in the service manifest of a provisioned application type.
   * The properties the ones defined in the service manifest.
   */
  serviceTypeDescription?: ServiceTypeDescription;
  /**
   * The name of the service manifest in which this service type is defined.
   */
  serviceManifestName?: string;
  /**
   * The version of the service manifest in which this service type is defined.
   */
  serviceManifestVersion?: string;
  /**
   * Indicates whether the service is a service group. If it is, the property value is true
   * otherwise false.
   */
  isServiceGroup?: boolean;
}

/**
 * Contains the manifest describing a service type registered as part of an application in a
 * Service Fabric cluster.
 */
export interface ServiceTypeManifest {
  /**
   * The XML manifest as a string.
   */
  manifest?: string;
}

/**
 * Information about a partition that is singleton. The services with singleton partitioning scheme
 * are effectively non-partitioned. They only have one partition.
 */
export interface SingletonPartitionInformation extends PartitionInformation {
}

/**
 * Information about a stateful Service Fabric service.
 */
export interface StatefulServiceInfo extends ServiceInfo {
  /**
   * Whether the service has persisted state.
   */
  hasPersistedState?: boolean;
}

/**
 * Information about a partition of a stateful Service Fabric service..
 */
export interface StatefulServicePartitionInfo extends ServicePartitionInfo {
  /**
   * The target replica set size as a number.
   */
  targetReplicaSetSize?: number;
  /**
   * The minimum replica set size as a number.
   */
  minReplicaSetSize?: number;
  /**
   * The duration for which this partition was in quorum loss. If the partition is currently in
   * quorum loss, it returns the duration since it has been in that state. This field is using
   * ISO8601 format for specifying the duration.
   */
  lastQuorumLossDuration?: moment.Duration;
  /**
   * An Epoch is a configuration number for the partition as a whole. When the configuration of the
   * replica set changes, for example when the Primary replica changes, the operations that are
   * replicated from the new Primary replica are said to be a new Epoch from the ones which were
   * sent by the old Primary replica.
   */
  primaryEpoch?: Epoch;
}

/**
 * Represents the health of the stateful service replica.
 * Contains the replica aggregated health state, the health events and the unhealthy evaluations.
 */
export interface StatefulServiceReplicaHealth extends ReplicaHealth {
  /**
   * Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a
   * replica of a partition. It is unique within a partition and does not change for the lifetime
   * of the replica. If a replica gets dropped and another replica gets created on the same node
   * for the same partition, it will get a different value for the id. Sometimes the id of a
   * stateless service instance is also referred as a replica id.
   */
  replicaId?: string;
}

/**
 * Represents the health state of the stateful service replica, which contains the replica ID and
 * the aggregated health state.
 */
export interface StatefulServiceReplicaHealthState extends ReplicaHealthState {
  /**
   * Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a
   * replica of a partition. It is unique within a partition and does not change for the lifetime
   * of the replica. If a replica gets dropped and another replica gets created on the same node
   * for the same partition, it will get a different value for the id. Sometimes the id of a
   * stateless service instance is also referred as a replica id.
   */
  replicaId?: string;
}

/**
 * Describes a stateful service type defined in the service manifest of a provisioned application
 * type.
 */
export interface StatefulServiceTypeDescription extends ServiceTypeDescription {
  /**
   * A flag indicating whether this is a persistent service which stores states on the local disk.
   * If it is then the value of this property is true, if not it is false.
   */
  hasPersistedState?: boolean;
}

/**
 * Information about a stateless Service Fabric service.
 */
export interface StatelessServiceInfo extends ServiceInfo {
}

/**
 * Represents the health of the stateless service instance.
 * Contains the instance aggregated health state, the health events and the unhealthy evaluations.
 */
export interface StatelessServiceInstanceHealth extends ReplicaHealth {
  /**
   * Id of a stateless service instance. InstanceId is used by Service Fabric to uniquely identify
   * an instance of a partition of a stateless service. It is unique within a partition and does
   * not change for the lifetime of the instance. If the instance has failed over on the same or
   * different node, it will get a different value for the InstanceId.
   */
  instanceId?: string;
}

/**
 * Represents the health state of the stateless service instance, which contains the instance ID
 * and the aggregated health state.
 */
export interface StatelessServiceInstanceHealthState extends ReplicaHealthState {
  /**
   * Id of the stateless service instance on the wire this field is called ReplicaId.
   */
  replicaId?: string;
}

/**
 * Information about a partition of a stateless Service Fabric service.
 */
export interface StatelessServicePartitionInfo extends ServicePartitionInfo {
  /**
   * Number of instances of this partition.
   */
  instanceCount?: number;
}

/**
 * Describes a stateless service type defined in the service manifest of a provisioned application
 * type.
 */
export interface StatelessServiceTypeDescription extends ServiceTypeDescription {
  /**
   * A flag indicating if this type is not implemented and hosted by a user service process, but is
   * implicitly hosted by a system created process. This value is true for services using the guest
   * executable services, false otherwise.
   */
  useImplicitHost?: boolean;
}

/**
 * Represents health evaluation for the fabric:/System application, containing information about
 * the data and the algorithm used by health store to evaluate health. The evaluation is returned
 * only when the aggregated health state of the cluster is either Error or Warning.
 */
export interface SystemApplicationHealthEvaluation extends HealthEvaluation {
  /**
   * List of unhealthy evaluations that led to the current aggregated health state of the system
   * application. The types of the unhealthy evaluations can be
   * DeployedApplicationsHealthEvaluation, ServicesHealthEvaluation or EventHealthEvaluation.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Represents health evaluation for delta unhealthy cluster nodes in an upgrade domain, containing
 * health evaluations for each unhealthy node that impacted current aggregated health state.
 * Can be returned during cluster upgrade when cluster aggregated health state is Warning or Error.
 */
export interface UpgradeDomainDeltaNodesCheckHealthEvaluation extends HealthEvaluation {
  /**
   * Name of the upgrade domain where nodes health is currently evaluated.
   */
  upgradeDomainName?: string;
  /**
   * Number of upgrade domain nodes with aggregated heath state Error in the health store at the
   * beginning of the cluster upgrade.
   */
  baselineErrorCount?: number;
  /**
   * Total number of upgrade domain nodes in the health store at the beginning of the cluster
   * upgrade.
   */
  baselineTotalCount?: number;
  /**
   * Maximum allowed percentage of upgrade domain delta unhealthy nodes from the
   * ClusterUpgradeHealthPolicy.
   */
  maxPercentDeltaUnhealthyNodes?: number;
  /**
   * Total number of upgrade domain nodes in the health store.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy NodeHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Represents health evaluation for cluster nodes in an upgrade domain, containing health
 * evaluations for each unhealthy node that impacted current aggregated health state. Can be
 * returned when evaluating cluster health during cluster upgrade and the aggregated health state
 * is either Error or Warning.
 */
export interface UpgradeDomainNodesHealthEvaluation extends HealthEvaluation {
  /**
   * Name of the upgrade domain where nodes health is currently evaluated.
   */
  upgradeDomainName?: string;
  /**
   * Maximum allowed percentage of unhealthy nodes from the ClusterHealthPolicy.
   */
  maxPercentUnhealthyNodes?: number;
  /**
   * Total number of nodes in the current upgrade domain.
   */
  totalCount?: number;
  /**
   * List of unhealthy evaluations that led to the aggregated health state. Includes all the
   * unhealthy NodeHealthEvaluation that impacted the aggregated health.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
}

/**
 * Safety check that waits for the replica build operation to finish. This indicates that there is
 * a replica that is going through the copy or is providing data for building another replica.
 * Bring the node down will abort this copy operation which are typically expensive involving data
 * movements.
 */
export interface WaitForInbuildReplicaSafetyCheck extends PartitionSafetyCheck {
}

/**
 * Safety check that waits for the primary replica that was moved out of the node due to upgrade to
 * be placed back again on that node.
 */
export interface WaitForPrimaryPlacementSafetyCheck extends PartitionSafetyCheck {
}

/**
 * Safety check that waits for the primary replica to be moved out of the node before starting an
 * upgrade to ensure the availability of the primary replica for the partition.
 */
export interface WaitForPrimarySwapSafetyCheck extends PartitionSafetyCheck {
}

/**
 * Safety check that waits for the current reconfiguration of the partition to be completed before
 * starting an upgrade.
 */
export interface WaitForReconfigurationSafetyCheck extends PartitionSafetyCheck {
}

/**
 * Represents the load metric report which contains the time metric was reported, its name and
 * value.
 */
export interface LoadMetricReport {
  /**
   * Gets the UTC time when the load was reported.
   */
  lastReportedUtc?: Date;
  /**
   * The name of the load metric.
   */
  name?: string;
  /**
   * The value of the load metric. In future releases of Service Fabric this parameter will be
   * deprecated in favor of CurrentValue.
   */
  value?: string;
  /**
   * The value of the load metric.
   */
  currentValue?: string;
}

/**
 * Represents load information for a partition, which contains the primary and secondary reported
 * load metrics.
 * In case there is no load reported, PartitionLoadInformation will contain the default load for
 * the service of the partition.
 * For default loads, LoadMetricReport's LastReportedUtc is set to 0.
 */
export interface PartitionLoadInformation {
  /**
   * Id of the partition.
   */
  partitionId?: string;
  /**
   * Array of load reports from the primary replica for this partition.
   */
  primaryLoadMetricReports?: LoadMetricReport[];
  /**
   * Array of aggregated load reports from all secondary replicas for this partition.
   * Array only contains the latest reported load for each metric.
   */
  secondaryLoadMetricReports?: LoadMetricReport[];
}

/**
 * Represents a stateful service replica. This includes information about the identity, role,
 * status, health, node name, uptime, and other details about the replica.
 */
export interface StatefulServiceReplicaInfo extends ReplicaInfo {
  /**
   * The role of a replica of a stateful service. Possible values include: 'Unknown', 'None',
   * 'Primary', 'IdleSecondary', 'ActiveSecondary'
   */
  replicaRole?: string;
  /**
   * Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a
   * replica of a partition. It is unique within a partition and does not change for the lifetime
   * of the replica. If a replica gets dropped and another replica gets created on the same node
   * for the same partition, it will get a different value for the id. Sometimes the id of a
   * stateless service instance is also referred as a replica id.
   */
  replicaId?: string;
}

/**
 * Represents a stateless service instance. This includes information about the identity, status,
 * health, node name, uptime, and other details about the instance.
 */
export interface StatelessServiceInstanceInfo extends ReplicaInfo {
  /**
   * Id of a stateless service instance. InstanceId is used by Service Fabric to uniquely identify
   * an instance of a partition of a stateless service. It is unique within a partition and does
   * not change for the lifetime of the instance. If the instance has failed over on the same or
   * different node, it will get a different value for the InstanceId.
   */
  instanceId?: string;
}

/**
 * Represents a ServiceFabric cluster upgrade
 */
export interface ClusterUpgradeDescriptionObject {
  /**
   * The cluster configuration version (specified in the cluster manifest).
   */
  configVersion?: string;
  /**
   * The ServiceFabric code version of the cluster.
   */
  codeVersion?: string;
  /**
   * The kind of upgrade out of the following possible values. Possible values include: 'Invalid',
   * 'Rolling'
   */
  upgradeKind?: string;
  /**
   * The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto,
   * UnmonitoredManual, and Monitored. Possible values include: 'Invalid', 'UnmonitoredAuto',
   * 'UnmonitoredManual', 'Monitored'
   */
  rollingUpgradeMode?: string;
  /**
   * The maximum amount of time to block processing of an upgrade domain and prevent loss of
   * availability when there are unexpected issues. When this timeout expires, processing of the
   * upgrade domain will proceed regardless of availability loss issues. The timeout is reset at
   * the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive.
   * (unsigned 32-bit integer).
   */
  upgradeReplicaSetCheckTimeoutInSeconds?: number;
  /**
   * If true, then processes are forcefully restarted during upgrade even when the code version has
   * not changed (the upgrade only changes configuration or data).
   */
  forceRestart?: boolean;
  /**
   * When true, enables delta health evaluation rather than absolute health evaluation after
   * completion of each upgrade domain.
   */
  enableDeltaHealthEvaluation?: boolean;
  /**
   * Describes the parameters for monitoring an upgrade in Monitored mode.
   */
  monitoringPolicy?: MonitoringPolicyDescription;
  /**
   * Defines a health policy used to evaluate the health of the cluster or of a cluster node.
   */
  clusterHealthPolicy?: ClusterHealthPolicy;
  /**
   * Defines a health policy used to evaluate the health of the cluster during a cluster upgrade.
   */
  clusterUpgradeHealthPolicy?: ClusterUpgradeHealthPolicyObject;
  /**
   * Defines a map that contains specific application health policies for different applications.
   * Each entry specifies as key the application name and as value an ApplicationHealthPolicy used
   * to evaluate the application health.
   * If an application is not specified in the map, the application health evaluation uses the
   * ApplicationHealthPolicy found in its application manifest or the default application health
   * policy (if no health policy is defined in the manifest).
   * The map is empty by default.
   */
  applicationHealthPolicyMap?: ApplicationHealthPolicyMapItem[];
}

/**
 * The detailed upgrade progress for nodes in the current upgrade domain at the point of failure.
 */
export interface FailedUpgradeDomainProgressObject {
  /**
   * The name of the upgrade domain
   */
  domainName?: string;
  /**
   * List of upgrading nodes and their statuses
   */
  nodeUpgradeProgressList?: NodeUpgradeProgressInfo[];
}

/**
 * Information about a cluster upgrade.
 */
export interface ClusterUpgradeProgressObject {
  /**
   * The ServiceFabric code version of the cluster.
   */
  codeVersion?: string;
  /**
   * The cluster configuration version (specified in the cluster manifest).
   */
  configVersion?: string;
  /**
   * List of upgrade domains and their statuses.
   */
  upgradeDomains?: UpgradeDomainInfo[];
  /**
   * The state of the upgrade domain. Possible values include: 'Invalid', 'RollingBackInProgress',
   * 'RollingBackCompleted', 'RollingForwardPending', 'RollingForwardInProgress',
   * 'RollingForwardCompleted', 'Failed'
   */
  upgradeState?: string;
  /**
   * The name of the next upgrade domain to be processed.
   */
  nextUpgradeDomain?: string;
  /**
   * The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto,
   * UnmonitoredManual, and Monitored. Possible values include: 'Invalid', 'UnmonitoredAuto',
   * 'UnmonitoredManual', 'Monitored'
   */
  rollingUpgradeMode?: string;
  /**
   * Represents a ServiceFabric cluster upgrade
   */
  upgradeDescription?: ClusterUpgradeDescriptionObject;
  /**
   * The estimated elapsed time spent processing the current overall upgrade.
   */
  upgradeDurationInMilliseconds?: string;
  /**
   * The estimated elapsed time spent processing the current upgrade domain.
   */
  upgradeDomainDurationInMilliseconds?: string;
  /**
   * List of health evaluations that resulted in the current aggregated health state.
   */
  unhealthyEvaluations?: HealthEvaluationWrapper[];
  /**
   * Information about the current in-progress upgrade domain.
   */
  currentUpgradeDomainProgress?: CurrentUpgradeDomainProgressInfo;
  /**
   * The start time of the upgrade in UTC.
   */
  startTimestampUtc?: string;
  /**
   * The failure time of the upgrade in UTC.
   */
  failureTimestampUtc?: string;
  /**
   * The cause of an upgrade failure that resulted in FailureAction being executed. Possible values
   * include: 'None', 'Interrupted', 'HealthCheck', 'UpgradeDomainTimeout', 'OverallUpgradeTimeout'
   */
  failureReason?: string;
  /**
   * The detailed upgrade progress for nodes in the current upgrade domain at the point of failure.
   */
  upgradeDomainProgressAtFailure?: FailedUpgradeDomainProgressObject;
}

/**
 * Describes the parameters for a standalone cluster configuration upgrade.
 */
export interface ClusterConfigurationUpgradeDescription {
  /**
   * The cluster configuration as a JSON string. For example, [this
   * file](https://github.com/Azure-Samples/service-fabric-dotnet-standalone-cluster-configuration/blob/master/Samples/ClusterConfig.Unsecure.DevCluster.json)
   * contains JSON describing the [nodes and other properties of the
   * cluster](https://docs.microsoft.com/azure/service-fabric/service-fabric-cluster-manifest).
   */
  clusterConfig: string;
  /**
   * The length of time between attempts to perform health checks if the application or cluster is
   * not healthy.
   */
  healthCheckRetryTimeout?: moment.Duration;
  /**
   * The length of time to wait after completing an upgrade domain before starting the health
   * checks process.
   */
  healthCheckWaitDurationInSeconds?: moment.Duration;
  /**
   * The length of time that the application or cluster must remain healthy before the upgrade
   * proceeds to the next upgrade domain.
   */
  healthCheckStableDurationInSeconds?: moment.Duration;
  /**
   * The timeout for the upgrade domain.
   */
  upgradeDomainTimeoutInSeconds?: moment.Duration;
  /**
   * The upgrade timeout.
   */
  upgradeTimeoutInSeconds?: moment.Duration;
  /**
   * The maximum allowed percentage of unhealthy applications during the upgrade. Allowed values
   * are integer values from zero to 100.
   */
  maxPercentUnhealthyApplications?: number;
  /**
   * The maximum allowed percentage of unhealthy nodes during the upgrade. Allowed values are
   * integer values from zero to 100.
   */
  maxPercentUnhealthyNodes?: number;
  /**
   * The maximum allowed percentage of delta health degradation during the upgrade. Allowed values
   * are integer values from zero to 100.
   */
  maxPercentDeltaUnhealthyNodes?: number;
  /**
   * The maximum allowed percentage of upgrade domain delta health degradation during the upgrade.
   * Allowed values are integer values from zero to 100.
   */
  maxPercentUpgradeDomainDeltaUnhealthyNodes?: number;
  /**
   * Defines the application health policy map used to evaluate the health of an application or one
   * of its children entities.
   */
  applicationHealthPolicies?: ApplicationHealthPolicies;
}

/**
 * Service state of Service Fabric Upgrade Orchestration Service.
 */
export interface UpgradeOrchestrationServiceState {
  /**
   * The state of Service Fabric Upgrade Orchestration Service.
   */
  serviceState?: string;
}

/**
 * Service state summary of Service Fabric Upgrade Orchestration Service.
 */
export interface UpgradeOrchestrationServiceStateSummary {
  /**
   * The current code version of the cluster.
   */
  currentCodeVersion?: string;
  /**
   * The current manifest version of the cluster.
   */
  currentManifestVersion?: string;
  /**
   * The target code version of  the cluster.
   */
  targetCodeVersion?: string;
  /**
   * The target manifest version of the cluster.
   */
  targetManifestVersion?: string;
  /**
   * The type of the pending upgrade of the cluster.
   */
  pendingUpgradeType?: string;
}

/**
 * Path description for the application package in the image store specified during the prior copy
 * operation.
 */
export interface ApplicationTypeImageStorePath {
  /**
   * The relative image store path to the application package.
   */
  applicationTypeBuildPath: string;
}

/**
 * Describes the operation to unregister or unprovision an application type and its version that
 * was registered with the Service Fabric.
 */
export interface UnprovisionApplicationTypeDescriptionInfo {
  /**
   * The version of the application type as defined in the application manifest.
   */
  applicationTypeVersion: string;
  /**
   * The flag indicating whether or not unprovision should occur asynchronously. When set to true,
   * the unprovision operation returns when the request is accepted by the system, and the
   * unprovision operation continues without any timeout limit. The default value is false.
   * However, we recommend setting it to true for large application packages that were provisioned.
   */
  async?: boolean;
}

/**
 * Statistics about setup or main entry point  of a code package deployed on a Service Fabric node.
 */
export interface CodePackageEntryPointStatistics {
  /**
   * The last exit code of the entry point.
   */
  lastExitCode?: string;
  /**
   * The last time (in UTC) when Service Fabric attempted to run the entry point.
   */
  lastActivationTime?: Date;
  /**
   * The last time (in UTC) when the entry point finished running.
   */
  lastExitTime?: Date;
  /**
   * The last time (in UTC) when the entry point ran successfully.
   */
  lastSuccessfulActivationTime?: Date;
  /**
   * The last time (in UTC) when the entry point finished running gracefully.
   */
  lastSuccessfulExitTime?: Date;
  /**
   * Number of times the entry point has run.
   */
  activationCount?: string;
  /**
   * Number of times the entry point failed to run.
   */
  activationFailureCount?: string;
  /**
   * Number of times the entry point continuously failed to run.
   */
  continuousActivationFailureCount?: string;
  /**
   * Number of times the entry point finished running.
   */
  exitCount?: string;
  /**
   * Number of times the entry point failed to exit gracefully.
   */
  exitFailureCount?: string;
  /**
   * Number of times the entry point continuously failed to exit gracefully.
   */
  continuousExitFailureCount?: string;
}

/**
 * Information about setup or main entry point of a code package deployed on a Service Fabric node.
 */
export interface CodePackageEntryPoint {
  /**
   * The location of entry point executable on the node.
   */
  entryPointLocation?: string;
  /**
   * The process ID of the entry point.
   */
  processId?: string;
  /**
   * The user name under which entry point executable is run on the node.
   */
  runAsUserName?: string;
  /**
   * Statistics about setup or main entry point  of a code package deployed on a Service Fabric
   * node.
   */
  codePackageEntryPointStatistics?: CodePackageEntryPointStatistics;
  /**
   * Specifies the status of the code package entry point deployed on a Service Fabric node.
   * Possible values include: 'Invalid', 'Pending', 'Starting', 'Started', 'Stopping', 'Stopped'
   */
  status?: string;
  /**
   * The time (in UTC) when the entry point executable will be run next.
   */
  nextActivationTime?: Date;
  /**
   * The instance ID for current running entry point. For a code package setup entry point (if
   * specified) runs first and after it finishes main entry point is started. Each time entry point
   * executable is run, its instance id will change.
   */
  instanceId?: string;
}

/**
 * Information about code package deployed on a Service Fabric node.
 */
export interface DeployedCodePackageInfo {
  /**
   * The name of the code package.
   */
  name?: string;
  /**
   * The version of the code package specified in service manifest.
   */
  version?: string;
  /**
   * The name of service manifest that specified this code package.
   */
  serviceManifestName?: string;
  /**
   * The ActivationId of a deployed service package. If ServicePackageActivationMode specified at
   * the time of creating the service
   * is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'),
   * then value of ServicePackageActivationId
   * is always an empty string.
   */
  servicePackageActivationId?: string;
  /**
   * Specifies the type of host for main entry point of a code package as specified in service
   * manifest. Possible values include: 'Invalid', 'ExeHost', 'ContainerHost'
   */
  hostType?: string;
  /**
   * Specifies the isolation mode of main entry point of a code package when it's host type is
   * ContainerHost. This is specified as part of container host policies in application manifest
   * while importing service manifest. Possible values include: 'None', 'Process', 'HyperV'
   */
  hostIsolationMode?: string;
  /**
   * Specifies the status of a deployed application or service package on a Service Fabric node.
   * Possible values include: 'Invalid', 'Downloading', 'Activating', 'Active', 'Upgrading',
   * 'Deactivating'
   */
  status?: string;
  /**
   * The interval at which code package is run. This is used for periodic code package.
   */
  runFrequencyInterval?: string;
  /**
   * Information about setup or main entry point of a code package deployed on a Service Fabric
   * node.
   */
  setupEntryPoint?: CodePackageEntryPoint;
  /**
   * Information about setup or main entry point of a code package deployed on a Service Fabric
   * node.
   */
  mainEntryPoint?: CodePackageEntryPoint;
}

/**
 * Describes a map, which is a collection of (string, string) type key-value pairs. The map can be
 * used to record information about
 * the Chaos run. There cannot be more than 100 such pairs and each string (key or value) can be at
 * most 4095 characters long.
 * This map is set by the starter of the Chaos run to optionally store the context about the
 * specific run.
 */
export interface ChaosContext {
  /**
   * Describes a map that contains a collection of ChaosContextMapItem's.
   */
  map?: { [propertyName: string]: string };
}

/**
 * Defines all filters for targeted Chaos faults, for example, faulting only certain node types or
 * faulting only certain applications.
 * If ChaosTargetFilter is not used, Chaos faults all cluster entities. If ChaosTargetFilter is
 * used, Chaos faults only the entities that meet the ChaosTargetFilter
 * specification. NodeTypeInclusionList and ApplicationInclusionList allow a union semantics only.
 * It is not possible to specify an intersection
 * of NodeTypeInclusionList and ApplicationInclusionList. For example, it is not possible to
 * specify "fault this application only when it is on that node type."
 * Once an entity is included in either NodeTypeInclusionList or ApplicationInclusionList, that
 * entity cannot be excluded using ChaosTargetFilter. Even if
 * applicationX does not appear in ApplicationInclusionList, in some Chaos iteration applicationX
 * can be faulted because it happens to be on a node of nodeTypeY that is included
 * in NodeTypeInclusionList. If both NodeTypeInclusionList and ApplicationInclusionList are null or
 * empty, an ArgumentException is thrown.
 */
export interface ChaosTargetFilter {
  /**
   * A list of node types to include in Chaos faults.
   * All types of faults (restart node, restart code package, remove replica, restart replica, move
   * primary, and move secondary) are enabled for the nodes of these node types.
   * If a nodetype (say NodeTypeX) does not appear in the NodeTypeInclusionList, then node level
   * faults (like NodeRestart) will never be enabled for the nodes of
   * NodeTypeX, but code package and replica faults can still be enabled for NodeTypeX if an
   * application in the ApplicationInclusionList.
   * happens to reside on a node of NodeTypeX.
   * At most 100 node type names can be included in this list, to increase this number, a config
   * upgrade is required for MaxNumberOfNodeTypesInChaosEntityFilter configuration.
   */
  nodeTypeInclusionList?: string[];
  /**
   * A list of application URI's to include in Chaos faults.
   * All replicas belonging to services of these applications are amenable to replica faults
   * (restart replica, remove replica, move primary, and move secondary) by Chaos.
   * Chaos may restart a code package only if the code package hosts replicas of these applications
   * only.
   * If an application does not appear in this list, it can still be faulted in some Chaos
   * iteration if the application ends up on a node of a node type that is included in
   * NodeTypeInclusionList.
   * However, if applicationX is tied to nodeTypeY through placement constraints and applicationX
   * is absent from ApplicationInclusionList and nodeTypeY is absent from NodeTypeInclusionList,
   * then applicationX will never be faulted.
   * At most 1000 application names can be included in this list, to increase this number, a config
   * upgrade is required for MaxNumberOfApplicationsInChaosEntityFilter configuration.
   */
  applicationInclusionList?: string[];
}

/**
 * Defines all the parameters to configure a Chaos run.
 */
export interface ChaosParameters {
  /**
   * Total time (in seconds) for which Chaos will run before automatically stopping. The maximum
   * allowed value is 4,294,967,295 (System.UInt32.MaxValue).
   */
  timeToRunInSeconds?: string;
  /**
   * The maximum amount of time to wait for all cluster entities to become stable and healthy.
   * Chaos executes in iterations and at the start of each iteration it validates the health of
   * cluster entities.
   * During validation if a cluster entity is not stable and healthy within
   * MaxClusterStabilizationTimeoutInSeconds, Chaos generates a validation failed event.
   */
  maxClusterStabilizationTimeoutInSeconds?: number;
  /**
   * MaxConcurrentFaults is the maximum number of concurrent faults induced per iteration.
   * Chaos executes in iterations and two consecutive iterations are separated by a validation
   * phase.
   * The higher the concurrency, the more aggressive the injection of faults, leading to inducing
   * more complex series of states to uncover bugs.
   * The recommendation is to start with a value of 2 or 3 and to exercise caution while moving up.
   */
  maxConcurrentFaults?: number;
  /**
   * Enables or disables the move primary and move secondary faults.
   */
  enableMoveReplicaFaults?: boolean;
  /**
   * Wait time (in seconds) between consecutive faults within a single iteration.
   * The larger the value, the lower the overlapping between faults and the simpler the sequence of
   * state transitions that the cluster goes through.
   * The recommendation is to start with a value between 1 and 5 and exercise caution while moving
   * up.
   */
  waitTimeBetweenFaultsInSeconds?: number;
  /**
   * Time-separation (in seconds) between two consecutive iterations of Chaos.
   * The larger the value, the lower the fault injection rate.
   */
  waitTimeBetweenIterationsInSeconds?: number;
  /**
   * Passed-in cluster health policy is used to validate health of the cluster in between Chaos
   * iterations. If the cluster health is in error or if an unexpected exception happens during
   * fault execution--to provide the cluster with some time to recuperate--Chaos will wait for 30
   * minutes before the next health-check.
   */
  clusterHealthPolicy?: ClusterHealthPolicy;
  /**
   * Describes a map, which is a collection of (string, string) type key-value pairs. The map can
   * be used to record information about
   * the Chaos run. There cannot be more than 100 such pairs and each string (key or value) can be
   * at most 4095 characters long.
   * This map is set by the starter of the Chaos run to optionally store the context about the
   * specific run.
   */
  context?: ChaosContext;
  /**
   * List of cluster entities to target for Chaos faults.
   * This filter can be used to target Chaos faults only to certain node types or only to certain
   * application instances. If ChaosTargetFilter is not used, Chaos faults all cluster entities.
   * If ChaosTargetFilter is used, Chaos faults only the entities that meet the ChaosTargetFilter
   * specification.
   */
  chaosTargetFilter?: ChaosTargetFilter;
}

/**
 * Contains a description of Chaos.
 */
export interface Chaos {
  /**
   * If Chaos is running, these are the parameters Chaos is running with.
   */
  chaosParameters?: ChaosParameters;
  /**
   * Current status of the Chaos run. Possible values include: 'Invalid', 'Running', 'Stopped'
   */
  status?: string;
  /**
   * Current status of the schedule. Possible values include: 'Invalid', 'Stopped', 'Active',
   * 'Expired', 'Pending'
   */
  scheduleStatus?: string;
}

/**
 * Defines an item in ChaosParametersDictionary of the Chaos Schedule.
 */
export interface ChaosParametersDictionaryItem {
  /**
   * The key identifying the Chaos Parameter in the dictionary. This key is referenced by Chaos
   * Schedule Jobs.
   */
  key: string;
  /**
   * Defines all the parameters to configure a Chaos run.
   */
  value: ChaosParameters;
}

/**
 * Represents an event generated during a Chaos run.
 */
export interface ChaosEvent {
  /**
   * The UTC timestamp when this Chaos event was generated.
   */
  timeStampUtc: Date;
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Wrapper object for Chaos event.
 */
export interface ChaosEventWrapper {
  /**
   * Represents an event generated during a Chaos run.
   */
  chaosEvent?: ChaosEvent;
}

/**
 * Contains the list of Chaos events and the continuation token to get the next segment.
 */
export interface ChaosEventsSegment {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of Chaos events that meet the user-supplied criteria.
   */
  history?: ChaosEventWrapper[];
}

/**
 * Defines the days of the week that a Chaos Schedule Job will run for.
 */
export interface ChaosScheduleJobActiveDaysOfWeek {
  /**
   * Indicates if the Chaos Schedule Job will run on Sunday
   */
  sunday?: boolean;
  /**
   * Indicates if the Chaos Schedule Job will run on Monday
   */
  monday?: boolean;
  /**
   * Indicates if the Chaos Schedule Job will run on Tuesday
   */
  tuesday?: boolean;
  /**
   * Indicates if the Chaos Schedule Job will run on Wednesday
   */
  wednesday?: boolean;
  /**
   * Indicates if the Chaos Schedule Job will run on Thursday
   */
  thursday?: boolean;
  /**
   * Indicates if the Chaos Schedule Job will run on Friday
   */
  friday?: boolean;
  /**
   * Indicates if the Chaos Schedule Job will run on Saturday
   */
  saturday?: boolean;
}

/**
 * Defines an hour and minute of the day specified in 24 hour time.
 */
export interface TimeOfDay {
  /**
   * Represents the hour of the day. Value must be between 0 and 23 inclusive.
   */
  hour?: number;
  /**
   * Represents the minute of the hour. Value must be between 0 to 59 inclusive.
   */
  minute?: number;
}

/**
 * Defines a time range in a 24 hour day specified by a start and end time.
 */
export interface TimeRange {
  /**
   * Defines an hour and minute of the day specified in 24 hour time.
   */
  startTime?: TimeOfDay;
  /**
   * Defines an hour and minute of the day specified in 24 hour time.
   */
  endTime?: TimeOfDay;
}

/**
 * Defines a repetition rule and parameters of Chaos to be used with the Chaos Schedule.
 */
export interface ChaosScheduleJob {
  /**
   * A reference to which Chaos Parameters of the Chaos Schedule to use.
   */
  chaosParameters?: string;
  /**
   * Defines the days of the week that a Chaos Schedule Job will run for.
   */
  days?: ChaosScheduleJobActiveDaysOfWeek;
  /**
   * A list of Time Ranges that specify when during active days that this job will run. The times
   * are interpreted as UTC.
   */
  times?: TimeRange[];
}

/**
 * Defines the schedule used by Chaos.
 */
export interface ChaosSchedule {
  /**
   * The date and time Chaos will start using this schedule.
   */
  startDate?: Date;
  /**
   * The date and time Chaos will continue to use this schedule until.
   */
  expiryDate?: Date;
  /**
   * A mapping of string names to Chaos Parameters to be referenced by Chaos Schedule Jobs.
   */
  chaosParametersDictionary?: ChaosParametersDictionaryItem[];
  /**
   * A list of all Chaos Schedule Jobs that will be automated by the schedule.
   */
  jobs?: ChaosScheduleJob[];
}

/**
 * Defines the Chaos Schedule used by Chaos and the version of the Chaos Schedule. The version
 * value wraps back to 0 after surpassing 2,147,483,647.
 */
export interface ChaosScheduleDescription {
  /**
   * The version number of the Schedule.
   */
  version?: number;
  /**
   * Defines the schedule used by Chaos.
   */
  schedule?: ChaosSchedule;
}

/**
 * Describes a Chaos event that gets generated when Chaos has decided on the faults for an
 * iteration. This Chaos event contains the details of the faults as a list of strings.
 */
export interface ExecutingFaultsChaosEvent extends ChaosEvent {
  /**
   * List of string description of the faults that Chaos decided to execute in an iteration.
   */
  faults?: string[];
}

/**
 * Describes a Chaos event that gets generated when Chaos is started.
 */
export interface StartedChaosEvent extends ChaosEvent {
  /**
   * Defines all the parameters to configure a Chaos run.
   */
  chaosParameters?: ChaosParameters;
}

/**
 * Describes a Chaos event that gets generated when Chaos stops because either the user issued a
 * stop or the time to run was up.
 */
export interface StoppedChaosEvent extends ChaosEvent {
  /**
   * Describes why Chaos stopped. Chaos can stop because of StopChaos API call or the timeToRun
   * provided in ChaosParameters is over.
   */
  reason?: string;
}

/**
 * Describes a Chaos event that gets generated when an unexpected event occurs in the Chaos engine.
 * For example, due to the cluster snapshot being inconsistent, while faulting an entity, Chaos
 * found that the entity was already faulted -- which would be an unexpected event.
 */
export interface TestErrorChaosEvent extends ChaosEvent {
  /**
   * Describes why TestErrorChaosEvent was generated. For example, Chaos tries to fault a partition
   * but finds that the partition is no longer fault tolerant, then a TestErrorEvent gets generated
   * with the reason stating that the partition is not fault tolerant.
   */
  reason?: string;
}

/**
 * Chaos event corresponding to a failure during validation.
 */
export interface ValidationFailedChaosEvent extends ChaosEvent {
  /**
   * Describes why the ValidationFailedChaosEvent was generated. This may happen because more than
   * MaxPercentUnhealthyNodes are unhealthy for more than MaxClusterStabilizationTimeout. This
   * reason will be in the Reason property of the ValidationFailedChaosEvent as a string.
   */
  reason?: string;
}

/**
 * Describes a Chaos event that gets generated when Chaos is waiting for the cluster to become
 * ready for faulting, for example, Chaos may be waiting for the on-going upgrade to finish.
 */
export interface WaitingChaosEvent extends ChaosEvent {
  /**
   * Describes why the WaitingChaosEvent was generated, for example, due to a cluster upgrade.
   */
  reason?: string;
}

/**
 * Describes capacity information for services of this application. This description can be used
 * for describing the following.
 * - Reserving the capacity for the services on the nodes
 * - Limiting the total number of nodes that services of this application can run on
 * - Limiting the custom capacity metrics to limit the total consumption of this metric by the
 * services of this application
 */
export interface ApplicationCapacityDescription {
  /**
   * The minimum number of nodes where Service Fabric will reserve capacity for this application.
   * Note that this does not mean that the services of this application will be placed on all of
   * those nodes. If this property is set to zero, no capacity will be reserved. The value of this
   * property cannot be more than the value of the MaximumNodes property.
   */
  minimumNodes?: number;
  /**
   * The maximum number of nodes where Service Fabric will reserve capacity for this application.
   * Note that this does not mean that the services of this application will be placed on all of
   * those nodes. By default, the value of this property is zero and it means that the services can
   * be placed on any node.
   */
  maximumNodes?: number;
  /**
   * List of application capacity metric description.
   */
  applicationMetrics?: ApplicationMetricDescription[];
}

/**
 * Describes a Service Fabric application.
 */
export interface ApplicationDescription {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  name: string;
  /**
   * The application type name as defined in the application manifest.
   */
  typeName: string;
  /**
   * The version of the application type as defined in the application manifest.
   */
  typeVersion: string;
  /**
   * List of application parameters with overridden values from their default values specified in
   * the application manifest.
   */
  parameterList?: ApplicationParameter[];
  /**
   * Describes capacity information for services of this application. This description can be used
   * for describing the following.
   * - Reserving the capacity for the services on the nodes
   * - Limiting the total number of nodes that services of this application can run on
   * - Limiting the custom capacity metrics to limit the total consumption of this metric by the
   * services of this application
   */
  applicationCapacity?: ApplicationCapacityDescription;
}

/**
 * Information about a Service Fabric compose deployment.
 */
export interface ComposeDeploymentStatusInfo {
  /**
   * The name of the deployment.
   */
  name?: string;
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
  /**
   * The status of the compose deployment. Possible values include: 'Invalid', 'Provisioning',
   * 'Creating', 'Ready', 'Unprovisioning', 'Deleting', 'Failed', 'Upgrading'
   */
  status?: string;
  /**
   * The status details of compose deployment including failure message.
   */
  statusDetails?: string;
}

/**
 * Credential information to connect to container registry.
 */
export interface RegistryCredential {
  /**
   * The user name to connect to container registry.
   */
  registryUserName?: string;
  /**
   * The password for supplied username to connect to container registry.
   */
  registryPassword?: string;
  /**
   * Indicates that supplied container registry password is encrypted.
   */
  passwordEncrypted?: boolean;
}

/**
 * Describes the parameters for a compose deployment upgrade.
 */
export interface ComposeDeploymentUpgradeDescription {
  /**
   * The name of the deployment.
   */
  deploymentName: string;
  /**
   * The content of the compose file that describes the deployment to create.
   */
  composeFileContent: string;
  /**
   * Credential information to connect to container registry.
   */
  registryCredential?: RegistryCredential;
  /**
   * The kind of upgrade out of the following possible values. Possible values include: 'Invalid',
   * 'Rolling'
   */
  upgradeKind: string;
  /**
   * The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto,
   * UnmonitoredManual, and Monitored. Possible values include: 'Invalid', 'UnmonitoredAuto',
   * 'UnmonitoredManual', 'Monitored'
   */
  rollingUpgradeMode?: string;
  /**
   * The maximum amount of time to block processing of an upgrade domain and prevent loss of
   * availability when there are unexpected issues. When this timeout expires, processing of the
   * upgrade domain will proceed regardless of availability loss issues. The timeout is reset at
   * the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive.
   * (unsigned 32-bit integer).
   */
  upgradeReplicaSetCheckTimeoutInSeconds?: number;
  /**
   * If true, then processes are forcefully restarted during upgrade even when the code version has
   * not changed (the upgrade only changes configuration or data).
   */
  forceRestart?: boolean;
  /**
   * Describes the parameters for monitoring an upgrade in Monitored mode.
   */
  monitoringPolicy?: MonitoringPolicyDescription;
  /**
   * Defines a health policy used to evaluate the health of an application or one of its children
   * entities.
   */
  applicationHealthPolicy?: ApplicationHealthPolicy;
}

/**
 * Describes the parameters for a compose deployment upgrade.
 */
export interface ComposeDeploymentUpgradeProgressInfo {
  /**
   * The name of the target deployment.
   */
  deploymentName?: string;
  /**
   * The name of the target application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
  /**
   * The state of the compose deployment upgrade. Possible values include: 'Invalid',
   * 'ProvisioningTarget', 'RollingForwardInProgress', 'RollingForwardPending',
   * 'UnprovisioningCurrent', 'RollingForwardCompleted', 'RollingBackInProgress',
   * 'UnprovisioningTarget', 'RollingBackCompleted', 'Failed'
   */
  upgradeState?: string;
  /**
   * Additional detailed information about the status of the pending upgrade.
   */
  upgradeStatusDetails?: string;
  /**
   * The kind of upgrade out of the following possible values. Possible values include: 'Invalid',
   * 'Rolling'
   */
  upgradeKind?: string;
  /**
   * The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto,
   * UnmonitoredManual, and Monitored. Possible values include: 'Invalid', 'UnmonitoredAuto',
   * 'UnmonitoredManual', 'Monitored'
   */
  rollingUpgradeMode?: string;
  /**
   * If true, then processes are forcefully restarted during upgrade even when the code version has
   * not changed (the upgrade only changes configuration or data).
   */
  forceRestart?: boolean;
  /**
   * The maximum amount of time to block processing of an upgrade domain and prevent loss of
   * availability when there are unexpected issues. When this timeout expires, processing of the
   * upgrade domain will proceed regardless of availability loss issues. The timeout is reset at
   * the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive.
   * (unsigned 32-bit integer).
   */
  upgradeReplicaSetCheckTimeoutInSeconds?: number;
  /**
   * Describes the parameters for monitoring an upgrade in Monitored mode.
   */
  monitoringPolicy?: MonitoringPolicyDescription;
  /**
   * Defines a health policy used to evaluate the health of an application or one of its children
   * entities.
   */
  applicationHealthPolicy?: ApplicationHealthPolicy;
  /**
   * The target application type version (found in the application manifest) for the application
   * upgrade.
   */
  targetApplicationTypeVersion?: string;
  /**
   * The estimated amount of time that the overall upgrade elapsed. It is first interpreted as a
   * string representing an ISO 8601 duration. If that fails, then it is interpreted as a number
   * representing the total number of milliseconds.
   */
  upgradeDuration?: string;
  /**
   * The estimated amount of time spent processing current Upgrade Domain. It is first interpreted
   * as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a
   * number representing the total number of milliseconds.
   */
  currentUpgradeDomainDuration?: string;
  /**
   * List of health evaluations that resulted in the current aggregated health state.
   */
  applicationUnhealthyEvaluations?: HealthEvaluationWrapper[];
  /**
   * Information about the current in-progress upgrade domain.
   */
  currentUpgradeDomainProgress?: CurrentUpgradeDomainProgressInfo;
  /**
   * The estimated UTC datetime when the upgrade started.
   */
  startTimestampUtc?: string;
  /**
   * The estimated UTC datetime when the upgrade failed and FailureAction was executed.
   */
  failureTimestampUtc?: string;
  /**
   * The cause of an upgrade failure that resulted in FailureAction being executed. Possible values
   * include: 'None', 'Interrupted', 'HealthCheck', 'UpgradeDomainTimeout', 'OverallUpgradeTimeout'
   */
  failureReason?: string;
  /**
   * Information about the upgrade domain progress at the time of upgrade failure.
   */
  upgradeDomainProgressAtFailure?: FailureUpgradeDomainProgressInfo;
  /**
   * Additional details of application upgrade including failure message.
   */
  applicationUpgradeStatusDetails?: string;
}

/**
 * The list of compose deployments in the cluster. The list is paged when all of the results cannot
 * fit in a single message. The next set of results can be obtained by executing the same query
 * with the continuation token provided in this list.
 */
export interface PagedComposeDeploymentStatusInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of compose deployment status information.
   */
  items?: ComposeDeploymentStatusInfo[];
}

/**
 * Defines description for creating a Service Fabric compose deployment.
 */
export interface CreateComposeDeploymentDescription {
  /**
   * The name of the deployment.
   */
  deploymentName: string;
  /**
   * The content of the compose file that describes the deployment to create.
   */
  composeFileContent: string;
  /**
   * Credential information to connect to container registry.
   */
  registryCredential?: RegistryCredential;
}

/**
 * Information about service package deployed on a Service Fabric node.
 */
export interface DeployedServicePackageInfo {
  /**
   * The name of the service package as specified in the service manifest.
   */
  name?: string;
  /**
   * The version of the service package specified in service manifest.
   */
  version?: string;
  /**
   * Specifies the status of a deployed application or service package on a Service Fabric node.
   * Possible values include: 'Invalid', 'Downloading', 'Activating', 'Active', 'Upgrading',
   * 'Deactivating'
   */
  status?: string;
  /**
   * The ActivationId of a deployed service package. If ServicePackageActivationMode specified at
   * the time of creating the service
   * is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'),
   * then value of ServicePackageActivationId
   * is always an empty string.
   */
  servicePackageActivationId?: string;
}

/**
 * Creates a particular correlation between services.
 */
export interface ServiceCorrelationDescription {
  /**
   * The ServiceCorrelationScheme which describes the relationship between this service and the
   * service specified via ServiceName. Possible values include: 'Invalid', 'Affinity',
   * 'AlignedAffinity', 'NonAlignedAffinity'
   */
  scheme: string;
  /**
   * The name of the service that the correlation relationship is established with.
   */
  serviceName: string;
}

/**
 * Describes how the service is partitioned.
 */
export interface PartitionSchemeDescription {
  /**
   * Polymorphic Discriminator
   */
  partitionScheme: string;
}

/**
 * Describes the named partition scheme of the service.
 */
export interface NamedPartitionSchemeDescription extends PartitionSchemeDescription {
  /**
   * The number of partitions.
   */
  count: number;
  /**
   * Array of size specified by the ‘Count’ parameter, for the names of the partitions.
   */
  names: string[];
}

/**
 * Describes the partition scheme of a singleton-partitioned, or non-partitioned service.
 */
export interface SingletonPartitionSchemeDescription extends PartitionSchemeDescription {
}

/**
 * Describes a partitioning scheme where an integer range is allocated evenly across a number of
 * partitions.
 */
export interface UniformInt64RangePartitionSchemeDescription extends PartitionSchemeDescription {
  /**
   * The number of partitions.
   */
  count: number;
  /**
   * String indicating the lower bound of the partition key range that
   * should be split between the partitions.
   */
  lowKey: string;
  /**
   * String indicating the upper bound of the partition key range that
   * should be split between the partitions.
   */
  highKey: string;
}

/**
 * Describes the trigger for performing a scaling operation.
 */
export interface ScalingTriggerDescription {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes the mechanism for performing a scaling operation.
 */
export interface ScalingMechanismDescription {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes how the scaling should be performed
 */
export interface ScalingPolicyDescription {
  /**
   * Specifies the trigger associated with this scaling policy
   */
  scalingTrigger: ScalingTriggerDescription;
  /**
   * Specifies the mechanism associated with this scaling policy
   */
  scalingMechanism: ScalingMechanismDescription;
}

/**
 * A ServiceDescription contains all of the information necessary to create a service.
 */
export interface ServiceDescription {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  serviceName: string;
  /**
   * Name of the service type as specified in the service manifest.
   */
  serviceTypeName: string;
  /**
   * The initialization data as an array of bytes. Initialization data is passed to service
   * instances or replicas when they are created.
   */
  initializationData?: number[];
  /**
   * The partition description as an object.
   */
  partitionDescription: PartitionSchemeDescription;
  /**
   * The placement constraints as a string. Placement constraints are boolean expressions on node
   * properties and allow for restricting a service to particular nodes based on the service
   * requirements. For example, to place a service on nodes where NodeType is blue specify the
   * following: "NodeColor == blue)".
   */
  placementConstraints?: string;
  /**
   * The correlation scheme.
   */
  correlationScheme?: ServiceCorrelationDescription[];
  /**
   * The service load metrics.
   */
  serviceLoadMetrics?: ServiceLoadMetricDescription[];
  /**
   * The service placement policies.
   */
  servicePlacementPolicies?: ServicePlacementPolicyDescription[];
  /**
   * The move cost for the service. Possible values include: 'Zero', 'Low', 'Medium', 'High'
   */
  defaultMoveCost?: string;
  /**
   * Indicates if the DefaultMoveCost property is specified.
   */
  isDefaultMoveCostSpecified?: boolean;
  /**
   * The activation mode of service package to be used for a service. Possible values include:
   * 'SharedProcess', 'ExclusiveProcess'
   */
  servicePackageActivationMode?: string;
  /**
   * The DNS name of the service. It requires the DNS system service to be enabled in Service
   * Fabric cluster.
   */
  serviceDnsName?: string;
  /**
   * Scaling policies for this service.
   */
  scalingPolicies?: ScalingPolicyDescription[];
  /**
   * Polymorphic Discriminator
   */
  serviceKind: string;
}

/**
 * Describes a stateful service.
 */
export interface StatefulServiceDescription extends ServiceDescription {
  /**
   * The target replica set size as a number.
   */
  targetReplicaSetSize: number;
  /**
   * The minimum replica set size as a number.
   */
  minReplicaSetSize: number;
  /**
   * A flag indicating whether this is a persistent service which stores states on the local disk.
   * If it is then the value of this property is true, if not it is false.
   */
  hasPersistedState: boolean;
  /**
   * Flags indicating whether other properties are set. Each of the associated properties
   * corresponds to a flag, specified below, which, if set, indicate that the property is
   * specified.
   * This property can be a combination of those flags obtained using bitwise 'OR' operator.
   * For example, if the provided value is 6 then the flags for QuorumLossWaitDuration (2) and
   * StandByReplicaKeepDuration(4) are set.
   *
   * - None - Does not indicate any other properties are set. The value is zero.
   * - ReplicaRestartWaitDuration - Indicates the ReplicaRestartWaitDuration property is set. The
   * value is 1.
   * - QuorumLossWaitDuration - Indicates the QuorumLossWaitDuration property is set. The value is
   * 2.
   * - StandByReplicaKeepDuration - Indicates the StandByReplicaKeepDuration property is set. The
   * value is 4.
   */
  flags?: number;
  /**
   * The duration, in seconds, between when a replica goes down and when a new replica is created.
   */
  replicaRestartWaitDurationSeconds?: number;
  /**
   * The maximum duration, in seconds, for which a partition is allowed to be in a state of quorum
   * loss.
   */
  quorumLossWaitDurationSeconds?: number;
  /**
   * The definition on how long StandBy replicas should be maintained before being removed.
   */
  standByReplicaKeepDurationSeconds?: number;
}

/**
 * Describes a stateless service.
 */
export interface StatelessServiceDescription extends ServiceDescription {
  /**
   * The instance count.
   */
  instanceCount: number;
}

/**
 * Provides various statistics of the queue used in the service fabric replicator.
 * Contains information about the service fabric replicator like the replication/copy queue
 * utilization, last acknowledgement received timestamp, etc.
 * Depending on the role of the replicator, the properties in this type imply different meanings.
 */
export interface ReplicatorQueueStatus {
  /**
   * Represents the utilization of the queue. A value of 0 indicates that the queue is empty and a
   * value of 100 indicates the queue is full.
   */
  queueUtilizationPercentage?: number;
  /**
   * Represents the virtual memory consumed by the queue in bytes.
   */
  queueMemorySize?: string;
  /**
   * On a primary replicator, this is semantically the sequence number of the operation for which
   * all the secondary replicas have sent an acknowledgement.
   * On a secondary replicator, this is the smallest sequence number of the operation that is
   * present in the queue.
   */
  firstSequenceNumber?: string;
  /**
   * On a primary replicator, this is semantically the highest sequence number of the operation for
   * which all the secondary replicas have sent an acknowledgement.
   * On a secondary replicator, this is semantically the highest sequence number that has been
   * applied to the persistent state.
   */
  completedSequenceNumber?: string;
  /**
   * On a primary replicator, this is semantically the highest sequence number of the operation for
   * which a write quorum of the secondary replicas have sent an acknowledgement.
   * On a secondary replicator, this is semantically the highest sequence number of the in-order
   * operation received from the primary.
   */
  committedSequenceNumber?: string;
  /**
   * Represents the latest sequence number of the operation that is available in the queue.
   */
  lastSequenceNumber?: string;
}

/**
 * Represents a base class for primary or secondary replicator status.
 * Contains information about the service fabric replicator like the replication/copy queue
 * utilization, last acknowledgement received timestamp, etc.
 */
export interface ReplicatorStatus {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Provides various statistics of the acknowledgements that are being received from the remote
 * replicator.
 */
export interface RemoteReplicatorAcknowledgementDetail {
  /**
   * Represents the average duration it takes for the remote replicator to receive an operation.
   */
  averageReceiveDuration?: string;
  /**
   * Represents the average duration it takes for the remote replicator to apply an operation. This
   * usually entails writing the operation to disk.
   */
  averageApplyDuration?: string;
  /**
   * Represents the number of operations not yet received by a remote replicator.
   */
  notReceivedCount?: string;
  /**
   * Represents the number of operations received and not yet applied by a remote replicator.
   */
  receivedAndNotAppliedCount?: string;
}

/**
 * Provides details about the remote replicators from the primary replicator's point of view.
 */
export interface RemoteReplicatorAcknowledgementStatus {
  /**
   * Details about the acknowledgements for operations that are part of the replication stream
   * data.
   */
  replicationStreamAcknowledgementDetail?: RemoteReplicatorAcknowledgementDetail;
  /**
   * Details about the acknowledgements for operations that are part of the copy stream data.
   */
  copyStreamAcknowledgementDetail?: RemoteReplicatorAcknowledgementDetail;
}

/**
 * Represents the state of the secondary replicator from the primary replicator’s point of view.
 */
export interface RemoteReplicatorStatus {
  /**
   * Represents the replica ID of the remote secondary replicator.
   */
  replicaId?: string;
  /**
   * The last timestamp (in UTC) when an acknowledgement from the secondary replicator was
   * processed on the primary.
   * UTC 0 represents an invalid value, indicating that no acknowledgement messages were ever
   * processed.
   */
  lastAcknowledgementProcessedTimeUtc?: Date;
  /**
   * The highest replication operation sequence number that the secondary has received from the
   * primary.
   */
  lastReceivedReplicationSequenceNumber?: string;
  /**
   * The highest replication operation sequence number that the secondary has applied to its state.
   */
  lastAppliedReplicationSequenceNumber?: string;
  /**
   * A value that indicates whether the secondary replica is in the process of being built.
   */
  isInBuild?: boolean;
  /**
   * The highest copy operation sequence number that the secondary has received from the primary.
   * A value of -1 implies that the secondary has received all copy operations.
   */
  lastReceivedCopySequenceNumber?: string;
  /**
   * The highest copy operation sequence number that the secondary has applied to its state.
   * A value of -1 implies that the secondary has applied all copy operations and the copy process
   * is complete.
   */
  lastAppliedCopySequenceNumber?: string;
  /**
   * Represents the acknowledgment status for the remote secondary replicator.
   */
  remoteReplicatorAcknowledgementStatus?: RemoteReplicatorAcknowledgementStatus;
}

/**
 * Provides statistics about the Service Fabric Replicator, when it is functioning in a Primary
 * role.
 */
export interface PrimaryReplicatorStatus extends ReplicatorStatus {
  /**
   * Details about the replication queue on the primary replicator.
   */
  replicationQueueStatus?: ReplicatorQueueStatus;
  /**
   * The status of all the active and idle secondary replicators that the primary is aware of.
   */
  remoteReplicators?: RemoteReplicatorStatus[];
}

/**
 * Provides statistics about the Service Fabric Replicator, when it is functioning in a
 * ActiveSecondary role.
 */
export interface SecondaryReplicatorStatus extends ReplicatorStatus {
  /**
   * Details about the replication queue on the secondary replicator.
   */
  replicationQueueStatus?: ReplicatorQueueStatus;
  /**
   * The last time-stamp (UTC) at which a replication operation was received from the primary.
   * UTC 0 represents an invalid value, indicating that a replication operation message was never
   * received.
   */
  lastReplicationOperationReceivedTimeUtc?: Date;
  /**
   * Value that indicates whether the replica is currently being built.
   */
  isInBuild?: boolean;
  /**
   * Details about the copy queue on the secondary replicator.
   */
  copyQueueStatus?: ReplicatorQueueStatus;
  /**
   * The last time-stamp (UTC) at which a copy operation was received from the primary.
   * UTC 0 represents an invalid value, indicating that a copy operation message was never
   * received.
   */
  lastCopyOperationReceivedTimeUtc?: Date;
  /**
   * The last time-stamp (UTC) at which an acknowledgment was sent to the primary replicator.
   * UTC 0 represents an invalid value, indicating that an acknowledgment message was never sent.
   */
  lastAcknowledgementSentTimeUtc?: Date;
}

/**
 * Status of the secondary replicator when it is in active mode and is part of the replica set.
 */
export interface SecondaryActiveReplicatorStatus extends SecondaryReplicatorStatus {
}

/**
 * Status of the secondary replicator when it is in idle mode and is being built by the primary.
 */
export interface SecondaryIdleReplicatorStatus extends SecondaryReplicatorStatus {
}

/**
 * Information about load reported by replica.
 */
export interface LoadMetricReportInfo {
  /**
   * The name of the metric.
   */
  name?: string;
  /**
   * The value of the load for the metric. In future releases of Service Fabric this parameter will
   * be deprecated in favor of CurrentValue.
   */
  value?: number;
  /**
   * The double value of the load for the metric.
   */
  currentValue?: string;
  /**
   * The UTC time when the load is reported.
   */
  lastReportedUtc?: Date;
}

/**
 * Information about a Service Fabric service replica deployed on a node.
 */
export interface DeployedServiceReplicaDetailInfo {
  /**
   * Full hierarchical name of the service in URI format starting with `fabric:`.
   */
  serviceName?: string;
  /**
   * An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly
   * generated GUID when the service was created. The partition ID is unique and does not change
   * for the lifetime of the service. If the same service was deleted and recreated the IDs of its
   * partitions would be different.
   */
  partitionId?: string;
  /**
   * Specifies the current active life-cycle operation on a stateful service replica or stateless
   * service instance. Possible values include: 'Unknown', 'None', 'Open', 'ChangeRole', 'Close',
   * 'Abort'
   */
  currentServiceOperation?: string;
  /**
   * The start time of the current service operation in UTC format.
   */
  currentServiceOperationStartTimeUtc?: Date;
  /**
   * List of load reported by replica.
   */
  reportedLoad?: LoadMetricReportInfo[];
  /**
   * Polymorphic Discriminator
   */
  serviceKind: string;
}

/**
 * Information about the replica.
 */
export interface ReplicaStatusBase {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Key value store related information for the replica.
 */
export interface KeyValueStoreReplicaStatus extends ReplicaStatusBase {
  /**
   * Value indicating the estimated number of rows in the underlying database.
   */
  databaseRowCountEstimate?: string;
  /**
   * Value indicating the estimated size of the underlying database.
   */
  databaseLogicalSizeEstimate?: string;
  /**
   * Value indicating the latest key-prefix filter applied to enumeration during the callback. Null
   * if there is no pending callback.
   */
  copyNotificationCurrentKeyFilter?: string;
  /**
   * Value indicating the latest number of keys enumerated during the callback. 0 if there is no
   * pending callback.
   */
  copyNotificationCurrentProgress?: string;
  /**
   * Value indicating the current status details of the replica.
   */
  statusDetails?: string;
}

/**
 * Information about a stateful replica running in a code package. Note
 * DeployedServiceReplicaQueryResult will contain duplicate data like ServiceKind, ServiceName,
 * PartitionId and replicaId.
 */
export interface DeployedStatefulServiceReplicaDetailInfo extends DeployedServiceReplicaDetailInfo {
  /**
   * Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a
   * replica of a partition. It is unique within a partition and does not change for the lifetime
   * of the replica. If a replica gets dropped and another replica gets created on the same node
   * for the same partition, it will get a different value for the id. Sometimes the id of a
   * stateless service instance is also referred as a replica id.
   */
  replicaId?: string;
  /**
   * Specifies the operation currently being executed by the Replicator. Possible values include:
   * 'Invalid', 'None', 'Open', 'ChangeRole', 'UpdateEpoch', 'Close', 'Abort', 'OnDataLoss',
   * 'WaitForCatchup', 'Build'
   */
  currentReplicatorOperation?: string;
  /**
   * Specifies the access status of the partition. Possible values include: 'Invalid', 'Granted',
   * 'ReconfigurationPending', 'NotPrimary', 'NoWriteQuorum'
   */
  readStatus?: string;
  /**
   * Specifies the access status of the partition. Possible values include: 'Invalid', 'Granted',
   * 'ReconfigurationPending', 'NotPrimary', 'NoWriteQuorum'
   */
  writeStatus?: string;
  /**
   * Represents a base class for primary or secondary replicator status.
   * Contains information about the service fabric replicator like the replication/copy queue
   * utilization, last acknowledgement received timestamp, etc.
   */
  replicatorStatus?: ReplicatorStatus;
  /**
   * Key value store related information for the replica.
   */
  replicaStatus?: KeyValueStoreReplicaStatus;
  /**
   * Information about a stateful service replica deployed on a node.
   */
  deployedServiceReplicaQueryResult?: DeployedStatefulServiceReplicaInfo;
}

/**
 * Information about a stateless instance running in a code package. Note that
 * DeployedServiceReplicaQueryResult will contain duplicate data like ServiceKind, ServiceName,
 * PartitionId and InstanceId.
 */
export interface DeployedStatelessServiceInstanceDetailInfo extends DeployedServiceReplicaDetailInfo {
  /**
   * Id of a stateless service instance. InstanceId is used by Service Fabric to uniquely identify
   * an instance of a partition of a stateless service. It is unique within a partition and does
   * not change for the lifetime of the instance. If the instance has failed over on the same or
   * different node, it will get a different value for the InstanceId.
   */
  instanceId?: string;
  /**
   * Information about a stateless service instance deployed on a node.
   */
  deployedServiceReplicaQueryResult?: DeployedStatelessServiceInstanceInfo;
}

/**
 * A ServiceUpdateDescription contains all of the information necessary to update a service.
 */
export interface ServiceUpdateDescription {
  /**
   * Flags indicating whether other properties are set. Each of the associated properties
   * corresponds to a flag, specified below, which, if set, indicate that the property is
   * specified.
   * This property can be a combination of those flags obtained using bitwise 'OR' operator.
   * For example, if the provided value is 6 then the flags for ReplicaRestartWaitDuration (2) and
   * QuorumLossWaitDuration (4) are set.
   *
   * - None - Does not indicate any other properties are set. The value is zero.
   * - TargetReplicaSetSize/InstanceCount - Indicates whether the TargetReplicaSetSize property
   * (for Stateful services) or the InstanceCount property (for Stateless services) is set. The
   * value is 1.
   * - ReplicaRestartWaitDuration - Indicates the ReplicaRestartWaitDuration property is set. The
   * value is  2.
   * - QuorumLossWaitDuration - Indicates the QuorumLossWaitDuration property is set. The value is
   * 4.
   * - StandByReplicaKeepDuration - Indicates the StandByReplicaKeepDuration property is set. The
   * value is 8.
   * - MinReplicaSetSize - Indicates the MinReplicaSetSize property is set. The value is 16.
   * - PlacementConstraints - Indicates the PlacementConstraints property is set. The value is 32.
   * - PlacementPolicyList - Indicates the ServicePlacementPolicies property is set. The value is
   * 64.
   * - Correlation - Indicates the CorrelationScheme property is set. The value is 128.
   * - Metrics - Indicates the ServiceLoadMetrics property is set. The value is 256.
   * - DefaultMoveCost - Indicates the DefaultMoveCost property is set. The value is 512.
   * - ScalingPolicy - Indicates the ScalingPolicies property is set. The value is 1024.
   */
  flags?: string;
  /**
   * The placement constraints as a string. Placement constraints are boolean expressions on node
   * properties and allow for restricting a service to particular nodes based on the service
   * requirements. For example, to place a service on nodes where NodeType is blue specify the
   * following: "NodeColor == blue)".
   */
  placementConstraints?: string;
  /**
   * The correlation scheme.
   */
  correlationScheme?: ServiceCorrelationDescription[];
  /**
   * The service load metrics.
   */
  loadMetrics?: ServiceLoadMetricDescription[];
  /**
   * The service placement policies.
   */
  servicePlacementPolicies?: ServicePlacementPolicyDescription[];
  /**
   * The move cost for the service. Possible values include: 'Zero', 'Low', 'Medium', 'High'
   */
  defaultMoveCost?: string;
  /**
   * Scaling policies for this service.
   */
  scalingPolicies?: ScalingPolicyDescription[];
  /**
   * Polymorphic Discriminator
   */
  serviceKind: string;
}

/**
 * Describes an update for a stateful service.
 */
export interface StatefulServiceUpdateDescription extends ServiceUpdateDescription {
  /**
   * The target replica set size as a number.
   */
  targetReplicaSetSize?: number;
  /**
   * The minimum replica set size as a number.
   */
  minReplicaSetSize?: number;
  /**
   * The duration, in seconds, between when a replica goes down and when a new replica is created.
   */
  replicaRestartWaitDurationSeconds?: string;
  /**
   * The maximum duration, in seconds, for which a partition is allowed to be in a state of quorum
   * loss.
   */
  quorumLossWaitDurationSeconds?: string;
  /**
   * The definition on how long StandBy replicas should be maintained before being removed.
   */
  standByReplicaKeepDurationSeconds?: string;
}

/**
 * Describes an update for a stateless service.
 */
export interface StatelessServiceUpdateDescription extends ServiceUpdateDescription {
  /**
   * The instance count.
   */
  instanceCount?: number;
}

/**
 * Information about the version of image store file.
 */
export interface FileVersion {
  /**
   * The current image store version number for the file is used in image store for checking
   * whether it need to be updated.
   */
  versionNumber?: string;
  /**
   * The epoch data loss number of image store replica when this file entry was updated or created.
   */
  epochDataLossNumber?: string;
  /**
   * The epoch configuration version number of the image store replica when this file entry was
   * created or updated.
   */
  epochConfigurationNumber?: string;
}

/**
 * Information about a image store file.
 */
export interface FileInfo {
  /**
   * The size of file in bytes.
   */
  fileSize?: string;
  /**
   * Information about the version of image store file.
   */
  fileVersion?: FileVersion;
  /**
   * The date and time when the image store file was last modified.
   */
  modifiedDate?: Date;
  /**
   * The file path relative to the image store root path.
   */
  storeRelativePath?: string;
}

/**
 * Information about a image store folder. It includes how many files this folder contains and its
 * image store relative path.
 */
export interface FolderInfo {
  /**
   * The remote location within image store. This path is relative to the image store root.
   */
  storeRelativePath?: string;
  /**
   * The number of files from within the image store folder.
   */
  fileCount?: string;
}

/**
 * Information about the image store content.
 */
export interface ImageStoreContent {
  /**
   * The list of image store file info objects represents files found under the given image store
   * relative path.
   */
  storeFiles?: FileInfo[];
  /**
   * The list of image store folder info objects represents subfolders found under the given image
   * store relative path.
   */
  storeFolders?: FolderInfo[];
}

/**
 * Information about how to copy image store content from one image store relative path to another
 * image store relative path.
 */
export interface ImageStoreCopyDescription {
  /**
   * The relative path of source image store content to be copied from.
   */
  remoteSource: string;
  /**
   * The relative path of destination image store content to be copied to.
   */
  remoteDestination: string;
  /**
   * The list of the file names to be skipped for copying.
   */
  skipFiles?: string[];
  /**
   * Indicates whether to check mark file during copying. The property is true if checking mark
   * file is required, false otherwise. The mark file is used to check whether the folder is well
   * constructed. If the property is true and mark file does not exist, the copy is skipped.
   */
  checkMarkFile?: boolean;
}

/**
 * Defines description for restarting a deployed code package on Service Fabric node.
 */
export interface RestartDeployedCodePackageDescription {
  /**
   * The name of service manifest that specified this code package.
   */
  serviceManifestName: string;
  /**
   * The ActivationId of a deployed service package. If ServicePackageActivationMode specified at
   * the time of creating the service
   * is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'),
   * then value of ServicePackageActivationId
   * is always an empty string.
   */
  servicePackageActivationId?: string;
  /**
   * The name of the code package defined in the service manifest.
   */
  codePackageName: string;
  /**
   * The instance ID for currently running entry point. For a code package setup entry point (if
   * specified) runs first and after it finishes main entry point is started.
   * Each time entry point executable is run, its instance ID will change. If 0 is passed in as the
   * code package instance ID, the API will restart the code package with whatever instance ID it
   * is currently running.
   * If an instance ID other than 0 is passed in, the API will restart the code package only if the
   * current Instance ID matches the passed in instance ID.
   * Note, passing in the exact instance ID (not 0) in the API is safer, because if ensures at most
   * one restart of the code package.
   */
  codePackageInstanceId: string;
}

/**
 * Information about service type deployed on a node, information such as the status of the service
 * type registration on a node.
 */
export interface DeployedServiceTypeInfo {
  /**
   * Name of the service type as specified in the service manifest.
   */
  serviceTypeName?: string;
  /**
   * The name of the service manifest in which this service type is defined.
   */
  serviceManifestName?: string;
  /**
   * The name of the code package that registered the service type.
   */
  codePackageName?: string;
  /**
   * The status of the service type registration on the node. Possible values include: 'Invalid',
   * 'Disabled', 'Enabled', 'Registered'
   */
  status?: string;
  /**
   * The ActivationId of a deployed service package. If ServicePackageActivationMode specified at
   * the time of creating the service
   * is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'),
   * then value of ServicePackageActivationId
   * is always an empty string.
   */
  servicePackageActivationId?: string;
}

/**
 * Endpoint of a resolved service partition.
 */
export interface ResolvedServiceEndpoint {
  /**
   * The role of the replica where the endpoint is reported. Possible values include: 'Invalid',
   * 'Stateless', 'StatefulPrimary', 'StatefulSecondary'
   */
  kind?: string;
  /**
   * The address of the endpoint. If the endpoint has multiple listeners the address is a JSON
   * object with one property per listener with the value as the address of that listener.
   */
  address?: string;
}

/**
 * Information about a service partition and its associated endpoints.
 */
export interface ResolvedServicePartition {
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  name: string;
  /**
   * A representation of the resolved partition.
   */
  partitionInformation: PartitionInformation;
  /**
   * List of resolved service endpoints of a service partition.
   */
  endpoints: ResolvedServiceEndpoint[];
  /**
   * The version of this resolved service partition result. This version should be passed in the
   * next time the ResolveService call is made via the PreviousRspVersion query parameter.
   */
  version: string;
}

/**
 * This class returns information about the partition that the user-induced operation acted upon.
 */
export interface SelectedPartition {
  /**
   * The name of the service the partition belongs to.
   */
  serviceName?: string;
  /**
   * An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly
   * generated GUID when the service was created. The partition ID is unique and does not change
   * for the lifetime of the service. If the same service was deleted and recreated the IDs of its
   * partitions would be different.
   */
  partitionId?: string;
}

/**
 * Represents information about an operation in a terminal state (Completed or Faulted).
 */
export interface InvokeDataLossResult {
  /**
   * If OperationState is Completed, this is 0.  If OperationState is Faulted, this is an error
   * code indicating the reason.
   */
  errorCode?: number;
  /**
   * This class returns information about the partition that the user-induced operation acted upon.
   */
  selectedPartition?: SelectedPartition;
}

/**
 * Represents information about an operation in a terminal state (Completed or Faulted).
 */
export interface InvokeQuorumLossResult {
  /**
   * If OperationState is Completed, this is 0.  If OperationState is Faulted, this is an error
   * code indicating the reason.
   */
  errorCode?: number;
  /**
   * This class returns information about the partition that the user-induced operation acted upon.
   */
  selectedPartition?: SelectedPartition;
}

/**
 * Contains information about a node that was targeted by a user-induced operation.
 */
export interface NodeResult {
  /**
   * The name of a Service Fabric node.
   */
  nodeName?: string;
  /**
   * The node instance id.
   */
  nodeInstanceId?: string;
}

/**
 * Represents information about an operation in a terminal state (Completed or Faulted).
 */
export interface NodeTransitionResult {
  /**
   * If OperationState is Completed, this is 0.  If OperationState is Faulted, this is an error
   * code indicating the reason.
   */
  errorCode?: number;
  /**
   * Contains information about a node that was targeted by a user-induced operation.
   */
  nodeResult?: NodeResult;
}

/**
 * Information about an NodeTransition operation.  This class contains an OperationState and a
 * NodeTransitionResult.  The NodeTransitionResult is not valid until OperationState
 * is Completed or Faulted.
 */
export interface NodeTransitionProgress {
  /**
   * The state of the operation. Possible values include: 'Invalid', 'Running', 'RollingBack',
   * 'Completed', 'Faulted', 'Cancelled', 'ForceCancelled'
   */
  state?: string;
  /**
   * Represents information about an operation in a terminal state (Completed or Faulted).
   */
  nodeTransitionResult?: NodeTransitionResult;
}

/**
 * Contains the OperationId, OperationState, and OperationType for user-induced operations.
 */
export interface OperationStatus {
  /**
   * A GUID that identifies a call to this API.  This is also passed into the corresponding
   * GetProgress API.
   */
  operationId?: string;
  /**
   * The state of the operation. Possible values include: 'Invalid', 'Running', 'RollingBack',
   * 'Completed', 'Faulted', 'Cancelled', 'ForceCancelled'
   */
  state?: string;
  /**
   * The type of the operation. Possible values include: 'Invalid', 'PartitionDataLoss',
   * 'PartitionQuorumLoss', 'PartitionRestart', 'NodeTransition'
   */
  type?: string;
}

/**
 * Information about a partition data loss user-induced operation.
 */
export interface PartitionDataLossProgress {
  /**
   * The state of the operation. Possible values include: 'Invalid', 'Running', 'RollingBack',
   * 'Completed', 'Faulted', 'Cancelled', 'ForceCancelled'
   */
  state?: string;
  /**
   * Represents information about an operation in a terminal state (Completed or Faulted).
   */
  invokeDataLossResult?: InvokeDataLossResult;
}

/**
 * Information about a partition quorum loss user-induced operation.
 */
export interface PartitionQuorumLossProgress {
  /**
   * The state of the operation. Possible values include: 'Invalid', 'Running', 'RollingBack',
   * 'Completed', 'Faulted', 'Cancelled', 'ForceCancelled'
   */
  state?: string;
  /**
   * Represents information about an operation in a terminal state (Completed or Faulted).
   */
  invokeQuorumLossResult?: InvokeQuorumLossResult;
}

/**
 * Represents information about an operation in a terminal state (Completed or Faulted).
 */
export interface RestartPartitionResult {
  /**
   * If OperationState is Completed, this is 0.  If OperationState is Faulted, this is an error
   * code indicating the reason.
   */
  errorCode?: number;
  /**
   * This class returns information about the partition that the user-induced operation acted upon.
   */
  selectedPartition?: SelectedPartition;
}

/**
 * Information about a partition restart user-induced operation.
 */
export interface PartitionRestartProgress {
  /**
   * The state of the operation. Possible values include: 'Invalid', 'Running', 'RollingBack',
   * 'Completed', 'Faulted', 'Cancelled', 'ForceCancelled'
   */
  state?: string;
  /**
   * Represents information about an operation in a terminal state (Completed or Faulted).
   */
  restartPartitionResult?: RestartPartitionResult;
}

/**
 * Represents a policy for the package sharing.
 */
export interface PackageSharingPolicyInfo {
  /**
   * The name of code, configuration or data package that should be shared.
   */
  sharedPackageName?: string;
  /**
   * Represents the scope for PackageSharingPolicy. This is specified during
   * DeployServicePackageToNode operation. Possible values include: 'None', 'All', 'Code',
   * 'Config', 'Data'
   */
  packageSharingScope?: string;
}

/**
 * Defines description for downloading packages associated with a service manifest to image cache
 * on a Service Fabric node.
 */
export interface DeployServicePackageToNodeDescription {
  /**
   * The name of service manifest whose packages need to be downloaded.
   */
  serviceManifestName: string;
  /**
   * The application type name as defined in the application manifest.
   */
  applicationTypeName: string;
  /**
   * The version of the application type as defined in the application manifest.
   */
  applicationTypeVersion: string;
  /**
   * The name of a Service Fabric node.
   */
  nodeName: string;
  /**
   * List of package sharing policy information.
   */
  packageSharingPolicy?: PackageSharingPolicyInfo[];
}

/**
 * Describes the parameters for resuming an unmonitored manual Service Fabric application upgrade
 */
export interface ResumeApplicationUpgradeDescription {
  /**
   * The name of the upgrade domain in which to resume the upgrade.
   */
  upgradeDomainName: string;
}

/**
 * Describes the parameters for updating an ongoing application upgrade.
 */
export interface ApplicationUpgradeUpdateDescription {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  name: string;
  /**
   * The kind of upgrade out of the following possible values. Possible values include: 'Invalid',
   * 'Rolling'
   */
  upgradeKind: string;
  /**
   * Defines a health policy used to evaluate the health of an application or one of its children
   * entities.
   */
  applicationHealthPolicy?: ApplicationHealthPolicy;
  /**
   * Describes the parameters for updating a rolling upgrade of application or cluster.
   */
  updateDescription?: RollingUpgradeUpdateDescription;
}

/**
 * Describes a Service Fabric name.
 */
export interface NameDescription {
  /**
   * The Service Fabric name, including the 'fabric:' URI scheme.
   */
  name: string;
}

/**
 * A paged list of Service Fabric names. The list is paged when all of the results cannot fit in a
 * single message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in this list.
 */
export interface PagedSubNameInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * Indicates whether any name under the given name has been modified during the enumeration. If
   * there was a modification, this property value is false.
   */
  isConsistent?: boolean;
  /**
   * List of the child names.
   */
  subNames?: string[];
}

/**
 * Describes a Service Fabric property value.
 */
export interface PropertyValue {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes a Service Fabric property value of type Binary.
 */
export interface BinaryPropertyValue extends PropertyValue {
  /**
   * Array of bytes to be sent as an integer array. Each element of array is a number between 0 and
   * 255.
   */
  data: number[];
}

/**
 * Describes a Service Fabric property value of type Int64.
 */
export interface Int64PropertyValue extends PropertyValue {
  /**
   * The data of the property value.
   */
  data: string;
}

/**
 * Describes a Service Fabric property value of type Double.
 */
export interface DoublePropertyValue extends PropertyValue {
  /**
   * The data of the property value.
   */
  data: number;
}

/**
 * Describes a Service Fabric property value of type String.
 */
export interface StringPropertyValue extends PropertyValue {
  /**
   * The data of the property value.
   */
  data: string;
}

/**
 * Describes a Service Fabric property value of type Guid.
 */
export interface GuidPropertyValue extends PropertyValue {
  /**
   * The data of the property value.
   */
  data: string;
}

/**
 * The metadata associated with a property, including the property's name.
 */
export interface PropertyMetadata {
  /**
   * The kind of property, determined by the type of data. Following are the possible values.
   * Possible values include: 'Invalid', 'Binary', 'Int64', 'Double', 'String', 'Guid'
   */
  typeId?: string;
  /**
   * The property's custom type ID.
   */
  customTypeId?: string;
  /**
   * The name of the parent Service Fabric Name for the property. It could be thought of as the
   * name-space/table under which the property exists.
   */
  parent?: string;
  /**
   * The length of the serialized property value.
   */
  sizeInBytes?: number;
  /**
   * Represents when the Property was last modified. Only write operations will cause this field to
   * be updated.
   */
  lastModifiedUtcTimestamp?: Date;
  /**
   * The version of the property. Every time a property is modified, its sequence number is
   * increased.
   */
  sequenceNumber?: string;
}

/**
 * Information about a Service Fabric property.
 */
export interface PropertyInfo {
  /**
   * The name of the Service Fabric property.
   */
  name: string;
  /**
   * Describes a Service Fabric property value.
   */
  value?: PropertyValue;
  /**
   * The metadata associated with a property, including the property's name.
   */
  metadata: PropertyMetadata;
}

/**
 * The paged list of Service Fabric properties under a given name. The list is paged when all of
 * the results cannot fit in a single message. The next set of results can be obtained by executing
 * the same query with the continuation token provided in this list.
 */
export interface PagedPropertyInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * Indicates whether any property under the given name has been modified during the enumeration.
   * If there was a modification, this property value is false.
   */
  isConsistent?: boolean;
  /**
   * List of property information.
   */
  properties?: PropertyInfo[];
}

/**
 * Description of a Service Fabric property.
 */
export interface PropertyDescription {
  /**
   * The name of the Service Fabric property.
   */
  propertyName: string;
  /**
   * The property's custom type ID. Using this property, the user is able to tag the type of the
   * value of the property.
   */
  customTypeId?: string;
  /**
   * Describes a Service Fabric property value.
   */
  value: PropertyValue;
}

/**
 * Represents the base type for property operations that can be put into a batch and submitted.
 */
export interface PropertyBatchOperation {
  /**
   * The name of the Service Fabric property.
   */
  propertyName: string;
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes a list of property batch operations to be executed. Either all or none of the
 * operations will be committed.
 */
export interface PropertyBatchDescriptionList {
  /**
   * A list of the property batch operations to be executed.
   */
  operations?: PropertyBatchOperation[];
}

/**
 * Represents a PropertyBatchOperation that compares the Boolean existence of a property with the
 * Exists argument.
 * The PropertyBatchOperation operation fails if the property's existence is not equal to the
 * Exists argument.
 * The CheckExistsPropertyBatchOperation is generally used as a precondition for the write
 * operations in the batch.
 * Note that if one PropertyBatchOperation in a PropertyBatch fails,
 * the entire batch fails and cannot be committed in a transactional manner.
 */
export interface CheckExistsPropertyBatchOperation extends PropertyBatchOperation {
  /**
   * Whether or not the property should exist for the operation to pass.
   */
  exists: boolean;
}

/**
 * Compares the Sequence Number of a property with the SequenceNumber argument.
 * A property's sequence number can be thought of as that property's version.
 * Every time the property is modified, its sequence number is increased.
 * The sequence number can be found in a property's metadata.
 * The comparison fails if the sequence numbers are not equal.
 * CheckSequencePropertyBatchOperation is generally used as a precondition for the write operations
 * in the batch.
 * Note that if one PropertyBatchOperation in a PropertyBatch fails,
 * the entire batch fails and cannot be committed in a transactional manner.
 */
export interface CheckSequencePropertyBatchOperation extends PropertyBatchOperation {
  /**
   * The expected sequence number.
   */
  sequenceNumber: string;
}

/**
 * Represents a PropertyBatchOperation that compares the value of the property with the expected
 * value.
 * The CheckValuePropertyBatchOperation is generally used as a precondition for the write
 * operations in the batch.
 * Note that if one PropertyBatchOperation in a PropertyBatch fails,
 * the entire batch fails and cannot be committed in a transactional manner.
 */
export interface CheckValuePropertyBatchOperation extends PropertyBatchOperation {
  /**
   * The expected property value.
   */
  value: PropertyValue;
}

/**
 * Represents a PropertyBatchOperation that deletes a specified property if it exists.
 * Note that if one PropertyBatchOperation in a PropertyBatch fails,
 * the entire batch fails and cannot be committed in a transactional manner.
 */
export interface DeletePropertyBatchOperation extends PropertyBatchOperation {
}

/**
 * Represents a PropertyBatchOperation that gets the specified property if it exists.
 * Note that if one PropertyBatchOperation in a PropertyBatch fails,
 * the entire batch fails and cannot be committed in a transactional manner.
 */
export interface GetPropertyBatchOperation extends PropertyBatchOperation {
  /**
   * Whether or not to return the property value with the metadata.
   * True if values should be returned with the metadata; False to return only property metadata.
   */
  includeValue?: boolean;
}

/**
 * Puts the specified property under the specified name.
 * Note that if one PropertyBatchOperation in a PropertyBatch fails,
 * the entire batch fails and cannot be committed in a transactional manner.
 */
export interface PutPropertyBatchOperation extends PropertyBatchOperation {
  /**
   * Describes a Service Fabric property value.
   */
  value: PropertyValue;
  /**
   * The property's custom type ID. Using this property, the user is able to tag the type of the
   * value of the property.
   */
  customTypeId?: string;
}

/**
 * Information about the results of a property batch.
 */
export interface PropertyBatchInfo {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Derived from PropertyBatchInfo. Represents the property batch succeeding. Contains the results
 * of any "Get" operations in the batch.
 */
export interface SuccessfulPropertyBatchInfo extends PropertyBatchInfo {
  /**
   * A map containing the properties that were requested through any "Get" property batch
   * operations. The key represents the index of the "Get" operation in the original request, in
   * string form. The value is the property. If a property is not found, it will not be in the map.
   */
  properties?: { [propertyName: string]: PropertyInfo };
}

/**
 * Derived from PropertyBatchInfo. Represents the property batch failing. Contains information
 * about the specific batch failure.
 */
export interface FailedPropertyBatchInfo extends PropertyBatchInfo {
  /**
   * The error message of the failed operation. Describes the exception thrown due to the first
   * unsuccessful operation in the property batch.
   */
  errorMessage?: string;
  /**
   * The index of the unsuccessful operation in the property batch.
   */
  operationIndex?: number;
}

/**
 * Describes the backup schedule parameters.
 */
export interface BackupScheduleDescription {
  /**
   * Polymorphic Discriminator
   */
  scheduleKind: string;
}

/**
 * Describes the parameters for the backup storage.
 */
export interface BackupStorageDescription {
  /**
   * Friendly name for this backup storage.
   */
  friendlyName?: string;
  /**
   * Polymorphic Discriminator
   */
  storageKind: string;
}

/**
 * Describes the retention policy configured.
 */
export interface RetentionPolicyDescription {
  /**
   * Polymorphic Discriminator
   */
  retentionPolicyType: string;
}

/**
 * Describes a backup policy for configuring periodic backup.
 */
export interface BackupPolicyDescription {
  /**
   * The unique name identifying this backup policy.
   */
  name: string;
  /**
   * Specifies whether to trigger restore automatically using the latest available backup in case
   * the partition experiences a data loss event.
   */
  autoRestoreOnDataLoss: boolean;
  /**
   * Defines the maximum number of incremental backups to be taken between two full backups. This
   * is just the upper limit. A full backup may be taken before specified number of incremental
   * backups are completed in one of the following conditions
   * - The replica has never taken a full backup since it has become primary,
   * - Some of the log records since the last backup has been truncated, or
   * - Replica passed the MaxAccumulatedBackupLogSizeInMB limit.
   */
  maxIncrementalBackups: number;
  /**
   * Describes the backup schedule parameters.
   */
  schedule: BackupScheduleDescription;
  /**
   * Describes the details of backup storage where to store the periodic backups.
   */
  storage: BackupStorageDescription;
  /**
   * Describes the policy to retain backups in storage.
   */
  retentionPolicy?: RetentionPolicyDescription;
}

/**
 * The list of backup policies configured in the cluster. The list is paged when all of the results
 * cannot fit in a single message. The next set of results can be obtained by executing the same
 * query with the continuation token provided in this list.
 */
export interface PagedBackupPolicyDescriptionList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * The list of backup policies information.
   */
  items?: BackupPolicyDescription[];
}

/**
 * Describes basic retention policy.
 */
export interface BasicRetentionPolicyDescription extends RetentionPolicyDescription {
  /**
   * It is the minimum duration for which a backup created, will remain stored in the storage and
   * might get deleted after that span of time. It should be specified in ISO8601 format.
   */
  retentionDuration: moment.Duration;
  /**
   * It is the minimum number of backups to be retained at any point of time. If specified with a
   * non zero value, backups will not be deleted even if the backups have gone past retention
   * duration and have number of backups less than or equal to it.
   */
  minimumNumberOfBackups?: number;
}

/**
 * It describes the body parameters while disabling backup of a backup
 * entity(Application/Service/Partition).
 */
export interface DisableBackupDescription {
  /**
   * Boolean flag to delete backups. It can be set to true for deleting all the backups which were
   * created for the backup entity that is getting disabled for backup.
   */
  cleanBackup: boolean;
}

/**
 * Describes the backup configuration information.
 */
export interface BackupConfigurationInfo {
  /**
   * The name of the backup policy which is applicable to this Service Fabric application or
   * service or partition.
   */
  policyName?: string;
  /**
   * Specifies the scope at which the backup policy is applied. Possible values include: 'Invalid',
   * 'Partition', 'Service', 'Application'
   */
  policyInheritedFrom?: string;
  /**
   * Describes the backup suspension details.
   */
  suspensionInfo?: BackupSuspensionInfo;
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Backup configuration information for a specific Service Fabric application specifying what
 * backup policy is being applied and suspend description, if any.
 */
export interface ApplicationBackupConfigurationInfo extends BackupConfigurationInfo {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
}

/**
 * Backup configuration information for a specific Service Fabric service specifying what backup
 * policy is being applied and suspend description, if any.
 */
export interface ServiceBackupConfigurationInfo extends BackupConfigurationInfo {
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  serviceName?: string;
}

/**
 * Describes the backup suspension details.
 */
export interface BackupSuspensionInfo {
  /**
   * Indicates whether periodic backup is suspended at this level or not.
   */
  isSuspended?: boolean;
  /**
   * Specifies the scope at which the backup suspension was applied. Possible values include:
   * 'Invalid', 'Partition', 'Service', 'Application'
   */
  suspensionInheritedFrom?: string;
}

/**
 * The list of backup configuration information. The list is paged when all of the results cannot
 * fit in a single message. The next set of results can be obtained by executing the same query
 * with the continuation token provided in this list.
 */
export interface PagedBackupConfigurationInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of backup configuration information.
   */
  items?: BackupConfigurationInfo[];
}

/**
 * Specifies the parameters needed to trigger a restore of a specific partition.
 */
export interface RestorePartitionDescription {
  /**
   * Unique backup ID.
   */
  backupId: string;
  /**
   * Location of the backup relative to the backup storage specified/ configured.
   */
  backupLocation: string;
  /**
   * Location of the backup from where the partition will be restored.
   */
  backupStorage?: BackupStorageDescription;
}

/**
 * Describes the progress of a restore operation on a partition.
 */
export interface RestoreProgressInfo {
  /**
   * Represents the current state of the partition restore operation. Possible values include:
   * 'Invalid', 'Accepted', 'RestoreInProgress', 'Success', 'Failure', 'Timeout'
   */
  restoreState?: string;
  /**
   * Timestamp when operation succeeded or failed.
   */
  timeStampUtc?: Date;
  /**
   * Describes the epoch at which the partition is restored.
   */
  restoredEpoch?: Epoch;
  /**
   * Restored LSN.
   */
  restoredLsn?: string;
  /**
   * Denotes the failure encountered in performing restore operation.
   */
  failureError?: FabricErrorError;
}

/**
 * Describes the parameters for triggering partition's backup.
 */
export interface BackupPartitionDescription {
  /**
   * Specifies the details of the backup storage where to save the backup.
   */
  backupStorage?: BackupStorageDescription;
}

/**
 * Represents a backup point which can be used to trigger a restore.
 */
export interface BackupInfo {
  /**
   * Unique backup ID .
   */
  backupId?: string;
  /**
   * Unique backup chain ID. All backups part of the same chain has the same backup chain id. A
   * backup chain is comprised of 1 full backup and multiple incremental backups.
   */
  backupChainId?: string;
  /**
   * Name of the Service Fabric application this partition backup belongs to.
   */
  applicationName?: string;
  /**
   * Name of the Service Fabric service this partition backup belongs to.
   */
  serviceName?: string;
  /**
   * Information about the partition to which this backup belongs to
   */
  partitionInformation?: PartitionInformation;
  /**
   * Location of the backup, relative to the backup store.
   */
  backupLocation?: string;
  /**
   * Describes the type of backup, whether its full or incremental. Possible values include:
   * 'Invalid', 'Full', 'Incremental'
   */
  backupType?: string;
  /**
   * Epoch of the last record in this backup.
   */
  epochOfLastBackupRecord?: Epoch;
  /**
   * LSN of the last record in this backup.
   */
  lsnOfLastBackupRecord?: string;
  /**
   * The date time when this backup was taken.
   */
  creationTimeUtc?: Date;
  /**
   * Manifest Version of the service this partition backup belongs to.
   */
  serviceManifestVersion?: string;
  /**
   * Denotes the failure encountered in getting backup point information.
   */
  failureError?: FabricErrorError;
}

/**
 * The list of backups. The list is paged when all of the results cannot fit in a single message.
 * The next set of results can be obtained by executing the same query with the continuation token
 * provided in this list.
 */
export interface PagedBackupInfoList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of backup information.
   */
  items?: BackupInfo[];
}

/**
 * Describes the parameters for Azure blob store used for storing and enumerating backups.
 */
export interface AzureBlobBackupStorageDescription extends BackupStorageDescription {
  /**
   * The connection string to connect to the Azure blob store.
   */
  connectionString: string;
  /**
   * The name of the container in the blob store to store and enumerate backups from.
   */
  containerName: string;
}

/**
 * Describes the parameters for file share storage used for storing or enumerating backups.
 */
export interface FileShareBackupStorageDescription extends BackupStorageDescription {
  /**
   * UNC path of the file share where to store or enumerate backups from.
   */
  path: string;
  /**
   * Primary user name to access the file share.
   */
  primaryUserName?: string;
  /**
   * Primary password to access the share location.
   */
  primaryPassword?: string;
  /**
   * Secondary user name to access the file share.
   */
  secondaryUserName?: string;
  /**
   * Secondary password to access the share location
   */
  secondaryPassword?: string;
}

/**
 * Describes the frequency based backup schedule.
 */
export interface FrequencyBasedBackupScheduleDescription extends BackupScheduleDescription {
  /**
   * Defines the interval with which backups are periodically taken. It should be specified in
   * ISO8601 format. Timespan in seconds is not supported and will be ignored while creating the
   * policy.
   */
  interval: moment.Duration;
}

/**
 * Describes the time based backup schedule.
 */
export interface TimeBasedBackupScheduleDescription extends BackupScheduleDescription {
  /**
   * Describes the frequency with which to run the time based backup schedule. Possible values
   * include: 'Invalid', 'Daily', 'Weekly'
   */
  scheduleFrequencyType: string;
  /**
   * List of days of a week when to trigger the periodic backup. This is valid only when the backup
   * schedule frequency type is weekly.
   */
  runDays?: string[];
  /**
   * Represents the list of exact time during the day in ISO8601 format. Like '19:00:00' will
   * represent '7PM' during the day. Date specified along with time will be ignored.
   */
  runTimes: Date[];
}

/**
 * Describes the progress of a partition's backup.
 */
export interface BackupProgressInfo {
  /**
   * Represents the current state of the partition backup operation. Possible values include:
   * 'Invalid', 'Accepted', 'BackupInProgress', 'Success', 'Failure', 'Timeout'
   */
  backupState?: string;
  /**
   * TimeStamp in UTC when operation succeeded or failed.
   */
  timeStampUtc?: Date;
  /**
   * Unique ID of the newly created backup.
   */
  backupId?: string;
  /**
   * Location, relative to the backup store, of the newly created backup.
   */
  backupLocation?: string;
  /**
   * Specifies the epoch of the last record included in backup.
   */
  epochOfLastBackupRecord?: Epoch;
  /**
   * The LSN of last record included in backup.
   */
  lsnOfLastBackupRecord?: string;
  /**
   * Denotes the failure encountered in performing backup operation.
   */
  failureError?: FabricErrorError;
}

/**
 * Backup configuration information, for a specific partition, specifying what backup policy is
 * being applied and suspend description, if any.
 */
export interface PartitionBackupConfigurationInfo extends BackupConfigurationInfo {
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  serviceName?: string;
  /**
   * The partition ID indentifying the partition.
   */
  partitionId?: string;
}

/**
 * Describes the Service Fabric entity that is configured for backup.
 */
export interface BackupEntity {
  /**
   * Polymorphic Discriminator
   */
  entityKind: string;
}

/**
 * Identifies the Service Fabric application which is being backed up.
 */
export interface ApplicationBackupEntity extends BackupEntity {
  /**
   * The name of the application, including the 'fabric:' URI scheme.
   */
  applicationName?: string;
}

/**
 * Identifies the Service Fabric stateful service which is being backed up.
 */
export interface ServiceBackupEntity extends BackupEntity {
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  serviceName?: string;
}

/**
 * Identifies the Service Fabric stateful partition which is being backed up.
 */
export interface PartitionBackupEntity extends BackupEntity {
  /**
   * The full name of the service with 'fabric:' URI scheme.
   */
  serviceName?: string;
  /**
   * The partition ID indentifying the partition.
   */
  partitionId?: string;
}

/**
 * Specifies the parameters needed to enable periodic backup.
 */
export interface EnableBackupDescription {
  /**
   * Name of the backup policy to be used for enabling periodic backups.
   */
  backupPolicyName: string;
}

/**
 * The list of backup entities that are being periodically backed. The list is paged when all of
 * the results cannot fit in a single message. The next set of results can be obtained by executing
 * the same query with the continuation token provided in this list.
 */
export interface PagedBackupEntityList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of backup entity information.
   */
  items?: BackupEntity[];
}

/**
 * Describes additional filters to be applied, while listing backups, and backup storage details
 * from where to fetch the backups.
 */
export interface GetBackupByStorageQueryDescription {
  /**
   * Specifies the start date time in ISO8601 from which to enumerate backups. If not specified,
   * backups are enumerated from the beginning.
   */
  startDateTimeFilter?: Date;
  /**
   * Specifies the end date time in ISO8601 till which to enumerate backups. If not specified,
   * backups are enumerated till the end.
   */
  endDateTimeFilter?: Date;
  /**
   * If specified as true, gets the most recent backup (within the specified time range) for every
   * partition under the specified backup entity.
   */
  latest?: boolean;
  /**
   * Describes the parameters for the backup storage from where to enumerate backups. This is
   * optional and by default backups are enumerated from the backup storage where this backup
   * entity is currently being backed up (as specified in backup policy). This parameter is useful
   * to be able to enumerate backups from another cluster where you may intend to restore.
   */
  storage: BackupStorageDescription;
  /**
   * Indicates the entity for which to enumerate backups.
   */
  backupEntity: BackupEntity;
}

/**
 * Describes the expected impact of a repair to a particular node.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface NodeImpact {
  /**
   * The name of the impacted node.
   */
  nodeName: string;
  /**
   * The level of impact expected. Possible values include: 'Invalid', 'None', 'Restart',
   * 'RemoveData', 'RemoveNode'
   */
  impactLevel?: string;
}

/**
 * Describes the expected impact of executing a repair task.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface RepairImpactDescriptionBase {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes the expected impact of a repair on a set of nodes.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface NodeRepairImpactDescription extends RepairImpactDescriptionBase {
  /**
   * The list of nodes impacted by a repair action and their respective expected impact.
   */
  nodeImpactList?: NodeImpact[];
}

/**
 * Describes the entities targeted by a repair action.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface RepairTargetDescriptionBase {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes the list of nodes targeted by a repair action.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface NodeRepairTargetDescription extends RepairTargetDescriptionBase {
  /**
   * The list of nodes targeted by a repair action.
   */
  nodeNames?: string[];
}

/**
 * A record of the times when the repair task entered each state.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface RepairTaskHistory {
  /**
   * The time when the repair task entered the Created state.
   */
  createdUtcTimestamp?: Date;
  /**
   * The time when the repair task entered the Claimed state.
   */
  claimedUtcTimestamp?: Date;
  /**
   * The time when the repair task entered the Preparing state.
   */
  preparingUtcTimestamp?: Date;
  /**
   * The time when the repair task entered the Approved state
   */
  approvedUtcTimestamp?: Date;
  /**
   * The time when the repair task entered the Executing state
   */
  executingUtcTimestamp?: Date;
  /**
   * The time when the repair task entered the Restoring state
   */
  restoringUtcTimestamp?: Date;
  /**
   * The time when the repair task entered the Completed state
   */
  completedUtcTimestamp?: Date;
  /**
   * The time when the repair task started the health check in the Preparing state.
   */
  preparingHealthCheckStartUtcTimestamp?: Date;
  /**
   * The time when the repair task completed the health check in the Preparing state.
   */
  preparingHealthCheckEndUtcTimestamp?: Date;
  /**
   * The time when the repair task started the health check in the Restoring state.
   */
  restoringHealthCheckStartUtcTimestamp?: Date;
  /**
   * The time when the repair task completed the health check in the Restoring state.
   */
  restoringHealthCheckEndUtcTimestamp?: Date;
}

/**
 * Represents a repair task, which includes information about what kind of repair was requested,
 * what its progress is, and what its final result was.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface RepairTask {
  /**
   * The ID of the repair task.
   */
  taskId: string;
  /**
   * The version of the repair task.
   * When creating a new repair task, the version must be set to zero.  When updating a repair
   * task,
   * the version is used for optimistic concurrency checks.  If the version is
   * set to zero, the update will not check for write conflicts.  If the version is set to a
   * non-zero value, then the
   * update will only succeed if the actual current version of the repair task matches this value.
   */
  version?: string;
  /**
   * A description of the purpose of the repair task, or other informational details.
   * May be set when the repair task is created, and is immutable once set.
   */
  description?: string;
  /**
   * The workflow state of the repair task. Valid initial states are Created, Claimed, and
   * Preparing. Possible values include: 'Invalid', 'Created', 'Claimed', 'Preparing', 'Approved',
   * 'Executing', 'Restoring', 'Completed'
   */
  state: string;
  /**
   * A bitwise-OR of the following values, which gives additional details about the status of the
   * repair task.
   * - 1 - Cancellation of the repair has been requested
   * - 2 - Abort of the repair has been requested
   * - 4 - Approval of the repair was forced via client request
   */
  flags?: number;
  /**
   * The requested repair action. Must be specified when the repair task is created, and is
   * immutable once set.
   */
  action: string;
  /**
   * The target object determines what actions the system will take to prepare for the impact of
   * the repair, prior to approving execution of the repair.
   * May be set when the repair task is created, and is immutable once set.
   */
  target?: RepairTargetDescriptionBase;
  /**
   * The name of the repair executor. Must be specified in Claimed and later states, and is
   * immutable once set.
   */
  executor?: string;
  /**
   * A data string that the repair executor can use to store its internal state.
   */
  executorData?: string;
  /**
   * The impact object determines what actions the system will take to prepare for the impact of
   * the repair, prior to approving execution of the repair.
   * Impact must be specified by the repair executor when transitioning to the Preparing state, and
   * is immutable once set.
   */
  impact?: RepairImpactDescriptionBase;
  /**
   * A value describing the overall result of the repair task execution. Must be specified in the
   * Restoring and later states, and is immutable once set. Possible values include: 'Invalid',
   * 'Succeeded', 'Cancelled', 'Interrupted', 'Failed', 'Pending'
   */
  resultStatus?: string;
  /**
   * A numeric value providing additional details about the result of the repair task execution.
   * May be specified in the Restoring and later states, and is immutable once set.
   */
  resultCode?: number;
  /**
   * A string providing additional details about the result of the repair task execution.
   * May be specified in the Restoring and later states, and is immutable once set.
   */
  resultDetails?: string;
  /**
   * An object that contains timestamps of the repair task's state transitions.
   * These timestamps are updated by the system, and cannot be directly modified.
   */
  history?: RepairTaskHistory;
  /**
   * The workflow state of the health check when the repair task is in the Preparing state.
   * Possible values include: 'NotStarted', 'InProgress', 'Succeeded', 'Skipped', 'TimedOut'
   */
  preparingHealthCheckState?: string;
  /**
   * The workflow state of the health check when the repair task is in the Restoring state.
   * Possible values include: 'NotStarted', 'InProgress', 'Succeeded', 'Skipped', 'TimedOut'
   */
  restoringHealthCheckState?: string;
  /**
   * A value to determine if health checks will be performed when the repair task enters the
   * Preparing state.
   */
  performPreparingHealthCheck?: boolean;
  /**
   * A value to determine if health checks will be performed when the repair task enters the
   * Restoring state.
   */
  performRestoringHealthCheck?: boolean;
}

/**
 * Describes a request for forced approval of a repair task.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface RepairTaskApproveDescription {
  /**
   * The ID of the repair task.
   */
  taskId: string;
  /**
   * The current version number of the repair task. If non-zero, then the request will only succeed
   * if this value matches the actual current version of the repair task. If zero, then no version
   * check is performed.
   */
  version?: string;
}

/**
 * Describes a request to cancel a repair task.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface RepairTaskCancelDescription {
  /**
   * The ID of the repair task.
   */
  taskId: string;
  /**
   * The current version number of the repair task. If non-zero, then the request will only succeed
   * if this value matches the actual current version of the repair task. If zero, then no version
   * check is performed.
   */
  version?: string;
  /**
   * _True_ if the repair should be stopped as soon as possible even if it has already started
   * executing. _False_ if the repair should be cancelled only if execution has not yet started.
   */
  requestAbort?: boolean;
}

/**
 * Describes a request to delete a completed repair task.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface RepairTaskDeleteDescription {
  /**
   * The ID of the completed repair task to be deleted.
   */
  taskId: string;
  /**
   * The current version number of the repair task. If non-zero, then the request will only succeed
   * if this value matches the actual current version of the repair task. If zero, then no version
   * check is performed.
   */
  version?: string;
}

/**
 * Describes a request to update the health policy of a repair task.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface RepairTaskUpdateHealthPolicyDescription {
  /**
   * The ID of the repair task to be updated.
   */
  taskId: string;
  /**
   * The current version number of the repair task. If non-zero, then the request will only succeed
   * if this value matches the actual current value of the repair task. If zero, then no version
   * check is performed.
   */
  version?: string;
  /**
   * A boolean indicating if health check is to be performed in the Preparing stage of the repair
   * task. If not specified the existing value should not be altered. Otherwise, specify the
   * desired new value.
   */
  performPreparingHealthCheck?: boolean;
  /**
   * A boolean indicating if health check is to be performed in the Restoring stage of the repair
   * task. If not specified the existing value should not be altered. Otherwise, specify the
   * desired new value.
   */
  performRestoringHealthCheck?: boolean;
}

/**
 * Describes the result of an operation that created or updated a repair task.
 *
 * This type supports the Service Fabric platform; it is not meant to be used directly from your
 * code.
 */
export interface RepairTaskUpdateInfo {
  /**
   * The new version of the repair task.
   */
  version: string;
}

/**
 * Information about which portion of the file to upload.
 */
export interface UploadChunkRange {
  /**
   * The start position of the portion of the file. It's represented by the number of bytes.
   */
  startPosition?: string;
  /**
   * The end position of the portion of the file. It's represented by the number of bytes.
   */
  endPosition?: string;
}

/**
 * Information about an image store upload session. A session is associated with a relative path in
 * the image store.
 */
export interface UploadSessionInfo {
  /**
   * The remote location within image store. This path is relative to the image store root.
   */
  storeRelativePath?: string;
  /**
   * A unique ID of the upload session. A session ID can be reused only if the session was
   * committed or removed.
   */
  sessionId?: string;
  /**
   * The date and time when the upload session was last modified.
   */
  modifiedDate?: Date;
  /**
   * The size in bytes of the uploading file.
   */
  fileSize?: string;
  /**
   * List of chunk ranges that image store has not received yet.
   */
  expectedRanges?: UploadChunkRange[];
}

/**
 * Information about a image store upload session
 */
export interface UploadSession {
  /**
   * When querying upload session by upload session ID, the result contains only one upload
   * session. When querying upload session by image store relative path, the result might contain
   * multiple upload sessions.
   */
  uploadSessions?: UploadSessionInfo[];
}

/**
 * Represents a scaling trigger related to an average load of a metric/resource of a partition.
 */
export interface AveragePartitionLoadScalingTrigger extends ScalingTriggerDescription {
  /**
   * The name of the metric for which usage should be tracked.
   */
  metricName: string;
  /**
   * The lower limit of the load below which a scale in operation should be performed.
   */
  lowerLoadThreshold: string;
  /**
   * The upper limit of the load beyond which a scale out operation should be performed.
   */
  upperLoadThreshold: string;
  /**
   * The period in seconds on which a decision is made whether to scale or not.
   */
  scaleIntervalInSeconds: number;
}

/**
 * Represents a scaling policy related to an average load of a metric/resource of a service.
 */
export interface AverageServiceLoadScalingTrigger extends ScalingTriggerDescription {
  /**
   * The name of the metric for which usage should be tracked.
   */
  metricName: string;
  /**
   * The lower limit of the load below which a scale in operation should be performed.
   */
  lowerLoadThreshold: string;
  /**
   * The upper limit of the load beyond which a scale out operation should be performed.
   */
  upperLoadThreshold: string;
  /**
   * The period in seconds on which a decision is made whether to scale or not.
   */
  scaleIntervalInSeconds: number;
}

/**
 * Represents a scaling mechanism for adding or removing instances of stateless service partition.
 */
export interface PartitionInstanceCountScaleMechanism extends ScalingMechanismDescription {
  /**
   * Minimum number of instances of the partition.
   */
  minInstanceCount: number;
  /**
   * Maximum number of instances of the partition.
   */
  maxInstanceCount: number;
  /**
   * The number of instances to add or remove during a scaling operation.
   */
  scaleIncrement: number;
}

/**
 * Represents a scaling mechanism for adding or removing named partitions of a stateless service.
 * Partition names are in the format '0','1''N-1'
 */
export interface AddRemoveIncrementalNamedPartitionScalingMechanism extends ScalingMechanismDescription {
  /**
   * Minimum number of named partitions of the service.
   */
  minPartitionCount: number;
  /**
   * Maximum number of named partitions of the service.
   */
  maxPartitionCount: number;
  /**
   * The number of instances to add or remove during a scaling operation.
   */
  scaleIncrement: number;
}

/**
 * Application Created event.
 */
export interface ApplicationCreatedEvent extends ApplicationEvent {
  /**
   * Application type name.
   */
  applicationTypeName: string;
  /**
   * Application type version.
   */
  applicationTypeVersion: string;
  /**
   * Application definition kind.
   */
  applicationDefinitionKind: string;
}

/**
 * Application Deleted event.
 */
export interface ApplicationDeletedEvent extends ApplicationEvent {
  /**
   * Application type name.
   */
  applicationTypeName: string;
  /**
   * Application type version.
   */
  applicationTypeVersion: string;
}

/**
 * Application Health Report Created event.
 */
export interface ApplicationNewHealthReportEvent extends ApplicationEvent {
  /**
   * Id of Application instance.
   */
  applicationInstanceId: number;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Application Health Report Expired event.
 */
export interface ApplicationHealthReportExpiredEvent extends ApplicationEvent {
  /**
   * Id of Application instance.
   */
  applicationInstanceId: number;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Application Upgrade Completed event.
 */
export interface ApplicationUpgradeCompletedEvent extends ApplicationEvent {
  /**
   * Application type name.
   */
  applicationTypeName: string;
  /**
   * Application type version.
   */
  applicationTypeVersion: string;
  /**
   * Overall upgrade time in milli-seconds.
   */
  overallUpgradeElapsedTimeInMs: number;
}

/**
 * Application Upgrade Domain Completed event.
 */
export interface ApplicationUpgradeDomainCompletedEvent extends ApplicationEvent {
  /**
   * Application type name.
   */
  applicationTypeName: string;
  /**
   * Current Application type version.
   */
  currentApplicationTypeVersion: string;
  /**
   * Target Application type version.
   */
  applicationTypeVersion: string;
  /**
   * State of upgrade.
   */
  upgradeState: string;
  /**
   * Upgrade domains.
   */
  upgradeDomains: string;
  /**
   * Upgrade time of domain in milli-seconds.
   */
  upgradeDomainElapsedTimeInMs: number;
}

/**
 * Application Upgrade Rollback Completed event.
 */
export interface ApplicationUpgradeRollbackCompletedEvent extends ApplicationEvent {
  /**
   * Application type name.
   */
  applicationTypeName: string;
  /**
   * Application type version.
   */
  applicationTypeVersion: string;
  /**
   * Describes reason of failure.
   */
  failureReason: string;
  /**
   * Overall upgrade time in milli-seconds.
   */
  overallUpgradeElapsedTimeInMs: number;
}

/**
 * Application Upgrade Rollback Started event.
 */
export interface ApplicationUpgradeRollbackStartedEvent extends ApplicationEvent {
  /**
   * Application type name.
   */
  applicationTypeName: string;
  /**
   * Current Application type version.
   */
  currentApplicationTypeVersion: string;
  /**
   * Target Application type version.
   */
  applicationTypeVersion: string;
  /**
   * Describes reason of failure.
   */
  failureReason: string;
  /**
   * Overall upgrade time in milli-seconds.
   */
  overallUpgradeElapsedTimeInMs: number;
}

/**
 * Application Upgrade Started event.
 */
export interface ApplicationUpgradeStartedEvent extends ApplicationEvent {
  /**
   * Application type name.
   */
  applicationTypeName: string;
  /**
   * Current Application type version.
   */
  currentApplicationTypeVersion: string;
  /**
   * Target Application type version.
   */
  applicationTypeVersion: string;
  /**
   * Type of upgrade.
   */
  upgradeType: string;
  /**
   * Mode of upgrade.
   */
  rollingUpgradeMode: string;
  /**
   * Action if failed.
   */
  failureAction: string;
}

/**
 * Deployed Application Health Report Created event.
 */
export interface DeployedApplicationNewHealthReportEvent extends ApplicationEvent {
  /**
   * Id of Application instance.
   */
  applicationInstanceId: number;
  /**
   * The name of a Service Fabric node.
   */
  nodeName: string;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Deployed Application Health Report Expired event.
 */
export interface DeployedApplicationHealthReportExpiredEvent extends ApplicationEvent {
  /**
   * Id of Application instance.
   */
  applicationInstanceId: number;
  /**
   * The name of a Service Fabric node.
   */
  nodeName: string;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Process Exited event.
 */
export interface ApplicationProcessExitedEvent extends ApplicationEvent {
  /**
   * Name of Service.
   */
  serviceName: string;
  /**
   * Name of Service package.
   */
  servicePackageName: string;
  /**
   * Activation Id of Service package.
   */
  servicePackageActivationId: string;
  /**
   * Indicates IsExclusive flag.
   */
  isExclusive: boolean;
  /**
   * Name of Code package.
   */
  codePackageName: string;
  /**
   * Type of EntryPoint.
   */
  entryPointType: string;
  /**
   * Name of executable.
   */
  exeName: string;
  /**
   * Process Id.
   */
  processId: number;
  /**
   * Host Id.
   */
  hostId: string;
  /**
   * Exit code of process.
   */
  exitCode: number;
  /**
   * Indicates if termination is unexpected.
   */
  unexpectedTermination: boolean;
  /**
   * Start time of process.
   */
  startTime: Date;
}

/**
 * Container Exited event.
 */
export interface ApplicationContainerInstanceExitedEvent extends ApplicationEvent {
  /**
   * Name of Service.
   */
  serviceName: string;
  /**
   * Name of Service package.
   */
  servicePackageName: string;
  /**
   * Activation Id of Service package.
   */
  servicePackageActivationId: string;
  /**
   * Indicates IsExclusive flag.
   */
  isExclusive: boolean;
  /**
   * Name of Code package.
   */
  codePackageName: string;
  /**
   * Type of EntryPoint.
   */
  entryPointType: string;
  /**
   * Name of Container image.
   */
  imageName: string;
  /**
   * Name of Container.
   */
  containerName: string;
  /**
   * Host Id.
   */
  hostId: string;
  /**
   * Exit code of process.
   */
  exitCode: number;
  /**
   * Indicates if termination is unexpected.
   */
  unexpectedTermination: boolean;
  /**
   * Start time of process.
   */
  startTime: Date;
}

/**
 * Node Aborted event.
 */
export interface NodeAbortedEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Id of Node.
   */
  nodeId: string;
  /**
   * Upgrade domain of Node.
   */
  upgradeDomain: string;
  /**
   * Fault domain of Node.
   */
  faultDomain: string;
  /**
   * IP address or FQDN.
   */
  ipAddressOrFQDN: string;
  /**
   * Name of Host.
   */
  hostname: string;
  /**
   * Indicates if it is seed node.
   */
  isSeedNode: boolean;
  /**
   * Version of Node.
   */
  nodeVersion: string;
}

/**
 * Node Added event.
 */
export interface NodeAddedToClusterEvent extends NodeEvent {
  /**
   * Id of Node.
   */
  nodeId: string;
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Type of Node.
   */
  nodeType: string;
  /**
   * Fabric version.
   */
  fabricVersion: string;
  /**
   * IP address or FQDN.
   */
  ipAddressOrFQDN: string;
  /**
   * Capacities.
   */
  nodeCapacities: string;
}

/**
 * Node Closed event.
 */
export interface NodeClosedEvent extends NodeEvent {
  /**
   * Id of Node.
   */
  nodeId: string;
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Describes error.
   */
  error: string;
}

/**
 * Node Deactivate Completed event.
 */
export interface NodeDeactivateCompletedEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Describes deactivate intent.
   */
  effectiveDeactivateIntent: string;
  /**
   * Batch Ids.
   */
  batchIdsWithDeactivateIntent: string;
  /**
   * Start time.
   */
  startTime: Date;
}

/**
 * Node Deactivate Started event.
 */
export interface NodeDeactivateStartedEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Batch Id.
   */
  batchId: string;
  /**
   * Describes deactivate intent.
   */
  deactivateIntent: string;
}

/**
 * Node Down event.
 */
export interface NodeDownEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Time when Node was last up.
   */
  lastNodeUpAt: Date;
}

/**
 * Node Health Report Created event.
 */
export interface NodeNewHealthReportEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstanceId: number;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Node Health Report Expired event.
 */
export interface NodeHealthReportExpiredEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstanceId: number;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Node Opened Succeeded event.
 */
export interface NodeOpenSucceededEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Id of Node.
   */
  nodeId: string;
  /**
   * Upgrade domain of Node.
   */
  upgradeDomain: string;
  /**
   * Fault domain of Node.
   */
  faultDomain: string;
  /**
   * IP address or FQDN.
   */
  ipAddressOrFQDN: string;
  /**
   * Name of Host.
   */
  hostname: string;
  /**
   * Indicates if it is seed node.
   */
  isSeedNode: boolean;
  /**
   * Version of Node.
   */
  nodeVersion: string;
}

/**
 * Node Open Failed event.
 */
export interface NodeOpenFailedEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Id of Node.
   */
  nodeId: string;
  /**
   * Upgrade domain of Node.
   */
  upgradeDomain: string;
  /**
   * Fault domain of Node.
   */
  faultDomain: string;
  /**
   * IP address or FQDN.
   */
  ipAddressOrFQDN: string;
  /**
   * Name of Host.
   */
  hostname: string;
  /**
   * Indicates if it is seed node.
   */
  isSeedNode: boolean;
  /**
   * Version of Node.
   */
  nodeVersion: string;
  /**
   * Describes the error.
   */
  error: string;
}

/**
 * Node Removed event.
 */
export interface NodeRemovedFromClusterEvent extends NodeEvent {
  /**
   * Id of Node.
   */
  nodeId: string;
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Type of Node.
   */
  nodeType: string;
  /**
   * Fabric version.
   */
  fabricVersion: string;
  /**
   * IP address or FQDN.
   */
  ipAddressOrFQDN: string;
  /**
   * Capacities.
   */
  nodeCapacities: string;
}

/**
 * Node Up event.
 */
export interface NodeUpEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstance: number;
  /**
   * Time when Node was last down.
   */
  lastNodeDownAt: Date;
}

/**
 * Partition Health Report Created event.
 */
export interface PartitionNewHealthReportEvent extends PartitionEvent {
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Partition Health Report Expired event.
 */
export interface PartitionHealthReportExpiredEvent extends PartitionEvent {
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Partition Reconfiguration event.
 */
export interface PartitionReconfiguredEvent extends PartitionEvent {
  /**
   * The name of a Service Fabric node.
   */
  nodeName: string;
  /**
   * Id of Node instance.
   */
  nodeInstanceId: string;
  /**
   * Type of Service.
   */
  serviceType: string;
  /**
   * CcEpochDataLoss version.
   */
  ccEpochDataLossVersion: number;
  /**
   * CcEpochConfig version.
   */
  ccEpochConfigVersion: number;
  /**
   * Type of reconfiguration.
   */
  reconfigType: string;
  /**
   * Describes reconfiguration result.
   */
  result: string;
  /**
   * Duration of Phase0 in milli-seconds.
   */
  phase0DurationMs: number;
  /**
   * Duration of Phase1 in milli-seconds.
   */
  phase1DurationMs: number;
  /**
   * Duration of Phase2 in milli-seconds.
   */
  phase2DurationMs: number;
  /**
   * Duration of Phase3 in milli-seconds.
   */
  phase3DurationMs: number;
  /**
   * Duration of Phase4 in milli-seconds.
   */
  phase4DurationMs: number;
  /**
   * Total duration in milli-seconds.
   */
  totalDurationMs: number;
}

/**
 * Partition Primary Move Analysis event.
 */
export interface PartitionPrimaryMoveAnalysisEvent extends PartitionAnalysisEvent {
  /**
   * Time when the move was completed.
   */
  whenMoveCompleted: Date;
  /**
   * The name of a Service Fabric node.
   */
  previousNode: string;
  /**
   * The name of a Service Fabric node.
   */
  currentNode: string;
  /**
   * Move reason.
   */
  moveReason: string;
  /**
   * Relevant traces.
   */
  relevantTraces: string;
}

/**
 * Service Created event.
 */
export interface ServiceCreatedEvent extends ServiceEvent {
  /**
   * Service type name.
   */
  serviceTypeName: string;
  /**
   * Application name.
   */
  applicationName: string;
  /**
   * Application type name.
   */
  applicationTypeName: string;
  /**
   * Id of Service instance.
   */
  serviceInstance: number;
  /**
   * Indicates if Service is stateful.
   */
  isStateful: boolean;
  /**
   * Number of partitions.
   */
  partitionCount: number;
  /**
   * Size of target replicas set.
   */
  targetReplicaSetSize: number;
  /**
   * Minimum size of replicas set.
   */
  minReplicaSetSize: number;
  /**
   * Version of Service package.
   */
  servicePackageVersion: string;
  /**
   * An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly
   * generated GUID when the service was created. The partition ID is unique and does not change
   * for the lifetime of the service. If the same service was deleted and recreated the IDs of its
   * partitions would be different.
   */
  partitionId: string;
}

/**
 * Service Deleted event.
 */
export interface ServiceDeletedEvent extends ServiceEvent {
  /**
   * Service type name.
   */
  serviceTypeName: string;
  /**
   * Application name.
   */
  applicationName: string;
  /**
   * Application type name.
   */
  applicationTypeName: string;
  /**
   * Id of Service instance.
   */
  serviceInstance: number;
  /**
   * Indicates if Service is stateful.
   */
  isStateful: boolean;
  /**
   * Number of partitions.
   */
  partitionCount: number;
  /**
   * Size of target replicas set.
   */
  targetReplicaSetSize: number;
  /**
   * Minimum size of replicas set.
   */
  minReplicaSetSize: number;
  /**
   * Version of Service package.
   */
  servicePackageVersion: string;
}

/**
 * Service Health Report Created event.
 */
export interface ServiceNewHealthReportEvent extends ServiceEvent {
  /**
   * Id of Service instance.
   */
  instanceId: number;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Service Health Report Expired event.
 */
export interface ServiceHealthReportExpiredEvent extends ServiceEvent {
  /**
   * Id of Service instance.
   */
  instanceId: number;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Deployed Service Health Report Created event.
 */
export interface DeployedServicePackageNewHealthReportEvent extends ApplicationEvent {
  /**
   * Service manifest name.
   */
  serviceManifestName: string;
  /**
   * Id of Service package instance.
   */
  servicePackageInstanceId: number;
  /**
   * Id of Service package activation.
   */
  servicePackageActivationId: string;
  /**
   * The name of a Service Fabric node.
   */
  nodeName: string;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Deployed Service Health Report Expired event.
 */
export interface DeployedServicePackageHealthReportExpiredEvent extends ApplicationEvent {
  /**
   * Service manifest name.
   */
  serviceManifest: string;
  /**
   * Id of Service package instance.
   */
  servicePackageInstanceId: number;
  /**
   * Id of Service package activation.
   */
  servicePackageActivationId: string;
  /**
   * The name of a Service Fabric node.
   */
  nodeName: string;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Stateful Replica Health Report Created event.
 */
export interface StatefulReplicaNewHealthReportEvent extends ReplicaEvent {
  /**
   * Id of Replica instance.
   */
  replicaInstanceId: number;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Stateful Replica Health Report Expired event.
 */
export interface StatefulReplicaHealthReportExpiredEvent extends ReplicaEvent {
  /**
   * Id of Replica instance.
   */
  replicaInstanceId: number;
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Stateless Replica Health Report Created event.
 */
export interface StatelessReplicaNewHealthReportEvent extends ReplicaEvent {
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Stateless Replica Health Report Expired event.
 */
export interface StatelessReplicaHealthReportExpiredEvent extends ReplicaEvent {
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Cluster Health Report Created event.
 */
export interface ClusterNewHealthReportEvent extends ClusterEvent {
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Cluster Health Report Expired event.
 */
export interface ClusterHealthReportExpiredEvent extends ClusterEvent {
  /**
   * Id of report source.
   */
  sourceId: string;
  /**
   * Describes the property.
   */
  property: string;
  /**
   * Describes the property health state.
   */
  healthState: string;
  /**
   * Time to live in milli-seconds.
   */
  timeToLiveMs: number;
  /**
   * Sequence number of report.
   */
  sequenceNumber: number;
  /**
   * Description of report.
   */
  description: string;
  /**
   * Indicates the removal when it expires.
   */
  removeWhenExpired: boolean;
  /**
   * Source time.
   */
  sourceUtcTimestamp: Date;
}

/**
 * Cluster Upgrade Completed event.
 */
export interface ClusterUpgradeCompletedEvent extends ClusterEvent {
  /**
   * Target Cluster version.
   */
  targetClusterVersion: string;
  /**
   * Overall duration of upgrade in milli-seconds.
   */
  overallUpgradeElapsedTimeInMs: number;
}

/**
 * Cluster Upgrade Domain Completed event.
 */
export interface ClusterUpgradeDomainCompletedEvent extends ClusterEvent {
  /**
   * Target Cluster version.
   */
  targetClusterVersion: string;
  /**
   * State of upgrade.
   */
  upgradeState: string;
  /**
   * Upgrade domains.
   */
  upgradeDomains: string;
  /**
   * Duration of domain upgrade in milli-seconds.
   */
  upgradeDomainElapsedTimeInMs: number;
}

/**
 * Cluster Upgrade Rollback Completed event.
 */
export interface ClusterUpgradeRollbackCompletedEvent extends ClusterEvent {
  /**
   * Target Cluster version.
   */
  targetClusterVersion: string;
  /**
   * Describes failure.
   */
  failureReason: string;
  /**
   * Overall duration of upgrade in milli-seconds.
   */
  overallUpgradeElapsedTimeInMs: number;
}

/**
 * Cluster Upgrade Rollback Started event.
 */
export interface ClusterUpgradeRollbackStartedEvent extends ClusterEvent {
  /**
   * Target Cluster version.
   */
  targetClusterVersion: string;
  /**
   * Describes failure.
   */
  failureReason: string;
  /**
   * Overall duration of upgrade in milli-seconds.
   */
  overallUpgradeElapsedTimeInMs: number;
}

/**
 * Cluster Upgrade Started event.
 */
export interface ClusterUpgradeStartedEvent extends ClusterEvent {
  /**
   * Current Cluster version.
   */
  currentClusterVersion: string;
  /**
   * Target Cluster version.
   */
  targetClusterVersion: string;
  /**
   * Type of upgrade.
   */
  upgradeType: string;
  /**
   * Mode of upgrade.
   */
  rollingUpgradeMode: string;
  /**
   * Action if failed.
   */
  failureAction: string;
}

/**
 * Chaos Stopped event.
 */
export interface ChaosStoppedEvent extends ClusterEvent {
  /**
   * Describes reason.
   */
  reason: string;
}

/**
 * Chaos Started event.
 */
export interface ChaosStartedEvent extends ClusterEvent {
  /**
   * Maximum number of concurrent faults.
   */
  maxConcurrentFaults: number;
  /**
   * Time to run in seconds.
   */
  timeToRunInSeconds: number;
  /**
   * Maximum timeout for cluster stabilization in seconds.
   */
  maxClusterStabilizationTimeoutInSeconds: number;
  /**
   * Wait time between iterations in seconds.
   */
  waitTimeBetweenIterationsInSeconds: number;
  /**
   * Wait time between faults in seconds.
   */
  waitTimeBetweenFautlsInSeconds: number;
  /**
   * Indicates MoveReplica fault is enabled.
   */
  moveReplicaFaultEnabled: boolean;
  /**
   * List of included Node types.
   */
  includedNodeTypeList: string;
  /**
   * List of included Applications.
   */
  includedApplicationList: string;
  /**
   * Health policy.
   */
  clusterHealthPolicy: string;
  /**
   * Chaos Context.
   */
  chaosContext: string;
}

/**
 * Chaos Restart Code Package Fault Scheduled event.
 */
export interface ChaosCodePackageRestartScheduledEvent extends ApplicationEvent {
  /**
   * Id of fault group.
   */
  faultGroupId: string;
  /**
   * Id of fault.
   */
  faultId: string;
  /**
   * The name of a Service Fabric node.
   */
  nodeName: string;
  /**
   * Service manifest name.
   */
  serviceManifestName: string;
  /**
   * Code package name.
   */
  codePackageName: string;
  /**
   * Id of Service package activation.
   */
  servicePackageActivationId: string;
}

/**
 * Chaos Remove Replica Fault Scheduled event.
 */
export interface ChaosReplicaRemovalScheduledEvent extends ReplicaEvent {
  /**
   * Id of fault group.
   */
  faultGroupId: string;
  /**
   * Id of fault.
   */
  faultId: string;
  /**
   * Service name.
   */
  serviceUri: string;
}

/**
 * Chaos Move Secondary Fault Scheduled event.
 */
export interface ChaosPartitionSecondaryMoveScheduledEvent extends PartitionEvent {
  /**
   * Id of fault group.
   */
  faultGroupId: string;
  /**
   * Id of fault.
   */
  faultId: string;
  /**
   * Service name.
   */
  serviceName: string;
  /**
   * The name of a Service Fabric node.
   */
  sourceNode: string;
  /**
   * The name of a Service Fabric node.
   */
  destinationNode: string;
  /**
   * Indicates a forced move.
   */
  forcedMove: boolean;
}

/**
 * Chaos Move Primary Fault Scheduled event.
 */
export interface ChaosPartitionPrimaryMoveScheduledEvent extends PartitionEvent {
  /**
   * Id of fault group.
   */
  faultGroupId: string;
  /**
   * Id of fault.
   */
  faultId: string;
  /**
   * Service name.
   */
  serviceName: string;
  /**
   * The name of a Service Fabric node.
   */
  nodeTo: string;
  /**
   * Indicates a forced move.
   */
  forcedMove: boolean;
}

/**
 * Chaos Restart Replica Fault Scheduled event.
 */
export interface ChaosReplicaRestartScheduledEvent extends ReplicaEvent {
  /**
   * Id of fault group.
   */
  faultGroupId: string;
  /**
   * Id of fault.
   */
  faultId: string;
  /**
   * Service name.
   */
  serviceUri: string;
}

/**
 * Chaos Restart Node Fault Scheduled event.
 */
export interface ChaosNodeRestartScheduledEvent extends NodeEvent {
  /**
   * Id of Node instance.
   */
  nodeInstanceId: number;
  /**
   * Id of fault group.
   */
  faultGroupId: string;
  /**
   * Id of fault.
   */
  faultId: string;
}

/**
 * This type describes the properties of a secret resource, including its kind.
 */
export interface SecretResourcePropertiesBase {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes the properties of a secret resource.
 */
export interface SecretResourceProperties extends SecretResourcePropertiesBase {
  /**
   * User readable description of the secret.
   */
  description?: string;
  /**
   * Status of the resource. Possible values include: 'Unknown', 'Ready', 'Upgrading', 'Creating',
   * 'Deleting', 'Failed'
   */
  readonly status?: string;
  /**
   * Gives additional information about the current status of the secret.
   */
  readonly statusDetails?: string;
  /**
   * The type of the content stored in the secret value. The value of this property is opaque to
   * Service Fabric. Once set, the value of this property cannot be changed.
   */
  contentType?: string;
}

/**
 * Describes the properties of a secret resource whose value is provided explicitly as plaintext.
 * The secret resource may have multiple values, each being uniquely versioned. The secret value of
 * each version is stored encrypted, and delivered as plaintext into the context of applications
 * referencing it.
 */
export interface InlinedValueSecretResourceProperties extends SecretResourceProperties {
}

/**
 * This type describes a secret resource.
 */
export interface SecretResourceDescription {
  /**
   * Describes the properties of a secret resource.
   */
  properties: SecretResourceProperties;
  /**
   * Name of the Secret resource.
   */
  name: string;
}

/**
 * The list of secret resources. The list is paged when all of the results cannot fit in a single
 * message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in this list.
 */
export interface PagedSecretResourceDescriptionList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * One page of the list.
   */
  items?: SecretResourceDescription[];
}

/**
 * This type represents the unencrypted value of the secret.
 */
export interface SecretValue {
  /**
   * The actual value of the secret.
   */
  value?: string;
}

/**
 * This type describes properties of secret value resource.
 */
export interface SecretValueProperties {
  /**
   * The actual value of the secret.
   */
  value?: string;
}

/**
 * This type describes a value of a secret resource. The name of this resource is the version
 * identifier corresponding to this secret value.
 */
export interface SecretValueResourceDescription {
  /**
   * Version identifier of the secret value.
   */
  name: string;
  /**
   * The actual value of the secret.
   */
  value?: string;
}

/**
 * The list of values of a secret resource, paged if the number of results exceeds the limits of a
 * single message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in the previous page.
 */
export interface PagedSecretValueResourceDescriptionList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * One page of the list.
   */
  items?: SecretValueResourceDescription[];
}

/**
 * This type describes a volume provided by an Azure Files file share.
 */
export interface VolumeProviderParametersAzureFile {
  /**
   * Name of the Azure storage account for the File Share.
   */
  accountName: string;
  /**
   * Access key of the Azure storage account for the File Share.
   */
  accountKey?: string;
  /**
   * Name of the Azure Files file share that provides storage for the volume.
   */
  shareName: string;
}

/**
 * Describes a reference to a volume resource.
 */
export interface VolumeReference {
  /**
   * Name of the volume being referenced.
   */
  name: string;
  /**
   * The flag indicating whether the volume is read only. Default is 'false'.
   */
  readOnly?: boolean;
  /**
   * The path within the container at which the volume should be mounted. Only valid path
   * characters are allowed.
   */
  destinationPath: string;
}

/**
 * Describes parameters for creating application-scoped volumes.
 */
export interface ApplicationScopedVolumeCreationParameters {
  /**
   * User readable description of the volume.
   */
  description?: string;
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes a volume whose lifetime is scoped to the application's lifetime.
 */
export interface ApplicationScopedVolume extends VolumeReference {
  /**
   * Describes parameters for creating application-scoped volumes.
   */
  creationParameters: ApplicationScopedVolumeCreationParameters;
}

/**
 * Describes parameters for creating application-scoped volumes provided by Service Fabric Volume
 * Disks
 */
export interface ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk extends ApplicationScopedVolumeCreationParameters {
  /**
   * Volume size. Possible values include: 'Small', 'Medium', 'Large'
   */
  sizeDisk: string;
}

/**
 * This type describes a volume resource.
 */
export interface VolumeResourceDescription {
  /**
   * Name of the Volume resource.
   */
  name: string;
  /**
   * User readable description of the volume.
   */
  description?: string;
  /**
   * Status of the volume. Possible values include: 'Unknown', 'Ready', 'Upgrading', 'Creating',
   * 'Deleting', 'Failed'
   */
  readonly status?: string;
  /**
   * Gives additional information about the current status of the volume.
   */
  readonly statusDetails?: string;
  /**
   * This type describes a volume provided by an Azure Files file share.
   */
  azureFileParameters?: VolumeProviderParametersAzureFile;
}

/**
 * The list of volume resources. The list is paged when all of the results cannot fit in a single
 * message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in this list.
 */
export interface PagedVolumeResourceDescriptionList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * One page of the list.
   */
  items?: VolumeResourceDescription[];
}

/**
 * This type describes the properties of a network resource, including its kind.
 */
export interface NetworkResourcePropertiesBase {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes properties of a network resource.
 */
export interface NetworkResourceProperties extends NetworkResourcePropertiesBase {
  /**
   * User readable description of the network.
   */
  description?: string;
  /**
   * Status of the network. Possible values include: 'Unknown', 'Ready', 'Upgrading', 'Creating',
   * 'Deleting', 'Failed'
   */
  readonly status?: string;
  /**
   * Gives additional information about the current status of the network.
   */
  readonly statusDetails?: string;
}

/**
 * Information about a Service Fabric container network local to a single Service Fabric cluster.
 */
export interface LocalNetworkResourceProperties extends NetworkResourceProperties {
  /**
   * Address space for the local container network.
   */
  networkAddressPrefix?: string;
}

/**
 * Describes a reference to a service endpoint.
 */
export interface EndpointRef {
  /**
   * Name of the endpoint.
   */
  name?: string;
}

/**
 * Describes a network reference in a service.
 */
export interface NetworkRef {
  /**
   * Name of the network
   */
  name?: string;
  /**
   * A list of endpoints that are exposed on this network.
   */
  endpointRefs?: EndpointRef[];
}

/**
 * This type describes a network resource.
 */
export interface NetworkResourceDescription {
  /**
   * Name of the Network resource.
   */
  name: string;
  /**
   * Describes properties of a network resource.
   */
  properties: NetworkResourceProperties;
}

/**
 * The list of network resources. The list is paged when all of the results cannot fit in a single
 * message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in this list.
 */
export interface PagedNetworkResourceDescriptionList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * One page of the list.
   */
  items?: NetworkResourceDescription[];
}

/**
 * Describes destination endpoint for routing traffic.
 */
export interface GatewayDestination {
  /**
   * Name of the service fabric Mesh application.
   */
  applicationName: string;
  /**
   * service that contains the endpoint.
   */
  serviceName: string;
  /**
   * name of the endpoint in the service.
   */
  endpointName: string;
}

/**
 * Describes the tcp configuration for external connectivity for this network.
 */
export interface TcpConfig {
  /**
   * tcp gateway config name.
   */
  name: string;
  /**
   * Specifies the port at which the service endpoint below needs to be exposed.
   */
  port: number;
  /**
   * Describes destination endpoint for routing traffic.
   */
  destination: GatewayDestination;
}

/**
 * Path to match for routing.
 */
export interface HttpRouteMatchPath {
  /**
   * Uri path to match for request.
   */
  value: string;
  /**
   * replacement string for matched part of the Uri.
   */
  rewrite?: string;
}

/**
 * Describes header information for http route matching.
 */
export interface HttpRouteMatchHeader {
  /**
   * Name of header to match in request.
   */
  name: string;
  /**
   * Value of header to match in request.
   */
  value?: string;
  /**
   * how to match header value. Possible values include: 'exact'
   */
  type?: string;
}

/**
 * Describes a rule for http route matching.
 */
export interface HttpRouteMatchRule {
  /**
   * Path to match for routing.
   */
  path: HttpRouteMatchPath;
  /**
   * headers and their values to match in request.
   */
  headers?: HttpRouteMatchHeader[];
}

/**
 * Describes the hostname properties for http routing.
 */
export interface HttpRouteConfig {
  /**
   * http route name.
   */
  name: string;
  /**
   * Describes a rule for http route matching.
   */
  match: HttpRouteMatchRule;
  /**
   * Describes destination endpoint for routing traffic.
   */
  destination: GatewayDestination;
}

/**
 * Describes the hostname properties for http routing.
 */
export interface HttpHostConfig {
  /**
   * http hostname config name.
   */
  name: string;
  /**
   * Route information to use for routing. Routes are processed in the order they are specified.
   * Specify routes that are more specific before routes that can hamdle general cases.
   */
  routes: HttpRouteConfig[];
}

/**
 * Describes the http configuration for external connectivity for this network.
 */
export interface HttpConfig {
  /**
   * http gateway config name.
   */
  name: string;
  /**
   * Specifies the port at which the service endpoint below needs to be exposed.
   */
  port: number;
  /**
   * description for routing.
   */
  hosts: HttpHostConfig[];
}

/**
 * This type describes a gateway resource.
 */
export interface GatewayResourceDescription {
  /**
   * Name of the Gateway resource.
   */
  name: string;
  /**
   * User readable description of the gateway.
   */
  description?: string;
  /**
   * Network the gateway should listen on for requests.
   */
  sourceNetwork: NetworkRef;
  /**
   * Network that the Application is using.
   */
  destinationNetwork: NetworkRef;
  /**
   * Configuration for tcp connectivity for this gateway.
   */
  tcp?: TcpConfig[];
  /**
   * Configuration for http connectivity for this gateway.
   */
  http?: HttpConfig[];
  /**
   * Status of the resource. Possible values include: 'Unknown', 'Ready', 'Upgrading', 'Creating',
   * 'Deleting', 'Failed'
   */
  readonly status?: string;
  /**
   * Gives additional information about the current status of the gateway.
   */
  readonly statusDetails?: string;
  /**
   * IP address of the gateway. This is populated in the response and is ignored for incoming
   * requests.
   */
  readonly ipAddress?: string;
}

/**
 * The list of gateway resources. The list is paged when all of the results cannot fit in a single
 * message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in this list.
 */
export interface PagedGatewayResourceDescriptionList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * One page of the list.
   */
  items?: GatewayResourceDescription[];
}

/**
 * Image registry credential.
 */
export interface ImageRegistryCredential {
  /**
   * Docker image registry server, without protocol such as `http` and `https`.
   */
  server: string;
  /**
   * The username for the private registry.
   */
  username: string;
  /**
   * The password for the private registry. The password is required for create or update
   * operations, however it is not returned in the get or list operations.
   */
  password?: string;
}

/**
 * Describes an environment variable for the container.
 */
export interface EnvironmentVariable {
  /**
   * The name of the environment variable.
   */
  name?: string;
  /**
   * The value of the environment variable.
   */
  value?: string;
}

/**
 * Describes a setting for the container. The setting file path can be fetched from environment
 * variable "Fabric_SettingPath". The path for Windows container is "C:\\secrets". The path for
 * Linux container is "/var/secrets".
 */
export interface Setting {
  /**
   * The name of the setting.
   */
  name?: string;
  /**
   * The value of the setting.
   */
  value?: string;
}

/**
 * Describes a container label.
 */
export interface ContainerLabel {
  /**
   * The name of the container label.
   */
  name: string;
  /**
   * The value of the container label.
   */
  value: string;
}

/**
 * Describes a container endpoint.
 */
export interface EndpointProperties {
  /**
   * The name of the endpoint.
   */
  name: string;
  /**
   * Port used by the container.
   */
  port?: number;
}

/**
 * This type describes the requested resources for a given container. It describes the least amount
 * of resources required for the container. A container can consume more than requested resources
 * up to the specified limits before being restarted. Currently, the requested resources are
 * treated as limits.
 */
export interface ResourceRequests {
  /**
   * The memory request in GB for this container.
   */
  memoryInGB: number;
  /**
   * Requested number of CPU cores. At present, only full cores are supported.
   */
  cpu: number;
}

/**
 * This type describes the resource limits for a given container. It describes the most amount of
 * resources a container is allowed to use before being restarted.
 */
export interface ResourceLimits {
  /**
   * The memory limit in GB.
   */
  memoryInGB?: number;
  /**
   * CPU limits in cores. At present, only full cores are supported.
   */
  cpu?: number;
}

/**
 * This type describes the resource requirements for a container or a service.
 */
export interface ResourceRequirements {
  /**
   * Describes the requested resources for a given container.
   */
  requests: ResourceRequests;
  /**
   * Describes the maximum limits on the resources for a given container.
   */
  limits?: ResourceLimits;
}

/**
 * Reference to sinks in DiagnosticsDescription.
 */
export interface DiagnosticsRef {
  /**
   * Status of whether or not sinks are enabled.
   */
  enabled?: boolean;
  /**
   * List of sinks to be used if enabled. References the list of sinks in DiagnosticsDescription.
   */
  sinkRefs?: string[];
}

/**
 * Specifying this parameter adds support for reliable collections
 */
export interface ReliableCollectionsRef {
  /**
   * Name of ReliableCollection resource. Right now it's not used and you can use any string.
   */
  name: string;
  /**
   * False (the default) if ReliableCollections state is persisted to disk as usual. True if you do
   * not want to persist state, in which case replication is still enabled and you can use
   * ReliableCollections as distributed cache.
   */
  doNotPersistState?: boolean;
}

/**
 * The container state.
 */
export interface ContainerState {
  /**
   * The state of this container
   */
  state?: string;
  /**
   * Date/time when the container state started.
   */
  startTime?: Date;
  /**
   * The container exit code.
   */
  exitCode?: string;
  /**
   * Date/time when the container state finished.
   */
  finishTime?: Date;
  /**
   * Human-readable status of this state.
   */
  detailStatus?: string;
}

/**
 * A container event.
 */
export interface ContainerEvent {
  /**
   * The name of the container event.
   */
  name?: string;
  /**
   * The count of the event.
   */
  count?: number;
  /**
   * Date/time of the first event.
   */
  firstTimestamp?: string;
  /**
   * Date/time of the last event.
   */
  lastTimestamp?: string;
  /**
   * The event message
   */
  message?: string;
  /**
   * The event type.
   */
  type?: string;
}

/**
 * Runtime information of a container instance.
 */
export interface ContainerInstanceView {
  /**
   * The number of times the container has been restarted.
   */
  restartCount?: number;
  /**
   * Current container instance state.
   */
  currentState?: ContainerState;
  /**
   * Previous container instance state.
   */
  previousState?: ContainerState;
  /**
   * The events of this container instance.
   */
  events?: ContainerEvent[];
}

/**
 * Describes a container and its runtime properties.
 */
export interface ContainerCodePackageProperties {
  /**
   * The name of the code package.
   */
  name: string;
  /**
   * The Container image to use.
   */
  image: string;
  /**
   * Image registry credential.
   */
  imageRegistryCredential?: ImageRegistryCredential;
  /**
   * Override for the default entry point in the container.
   */
  entrypoint?: string;
  /**
   * Command array to execute within the container in exec form.
   */
  commands?: string[];
  /**
   * The environment variables to set in this container
   */
  environmentVariables?: EnvironmentVariable[];
  /**
   * The settings to set in this container. The setting file path can be fetched from environment
   * variable "Fabric_SettingPath". The path for Windows container is "C:\\secrets". The path for
   * Linux container is "/var/secrets".
   */
  settings?: Setting[];
  /**
   * The labels to set in this container.
   */
  labels?: ContainerLabel[];
  /**
   * The endpoints exposed by this container.
   */
  endpoints?: EndpointProperties[];
  /**
   * The resources required by this container.
   */
  resources: ResourceRequirements;
  /**
   * Volumes to be attached to the container. The lifetime of these volumes is independent of the
   * application's lifetime.
   */
  volumeRefs?: VolumeReference[];
  /**
   * Volumes to be attached to the container. The lifetime of these volumes is scoped to the
   * application's lifetime.
   */
  volumes?: ApplicationScopedVolume[];
  /**
   * Reference to sinks in DiagnosticsDescription.
   */
  diagnostics?: DiagnosticsRef;
  /**
   * A list of ReliableCollection resources used by this particular code package. Please refer to
   * ReliablecollectionsRef for more details.
   */
  reliableCollectionsRefs?: ReliableCollectionsRef[];
  /**
   * Runtime information of a container instance.
   */
  readonly instanceView?: ContainerInstanceView;
}

/**
 * Describes the trigger for performing auto scaling operation.
 */
export interface AutoScalingTrigger {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes the mechanism for performing auto scaling operation. Derived classes will describe the
 * actual mechanism.
 */
export interface AutoScalingMechanism {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes the auto scaling policy
 */
export interface AutoScalingPolicy {
  /**
   * The name of the auto scaling policy.
   */
  name: string;
  /**
   * Determines when auto scaling operation will be invoked.
   */
  trigger: AutoScalingTrigger;
  /**
   * The mechanism that is used to scale when auto scaling operation is invoked.
   */
  mechanism: AutoScalingMechanism;
}

/**
 * This type describes a service resource.
 */
export interface ServiceResourceDescription {
  /**
   * Name of the Service resource.
   */
  name: string;
  /**
   * The operation system required by the code in service. Possible values include: 'Linux',
   * 'Windows'
   */
  osType: string;
  /**
   * Describes the set of code packages that forms the service. A code package describes the
   * container and the properties for running it. All the code packages are started together on the
   * same host and share the same context (network, process etc.).
   */
  codePackages: ContainerCodePackageProperties[];
  /**
   * The names of the private networks that this service needs to be part of.
   */
  networkRefs?: NetworkRef[];
  /**
   * Reference to sinks in DiagnosticsDescription.
   */
  diagnostics?: DiagnosticsRef;
  /**
   * User readable description of the service.
   */
  description?: string;
  /**
   * The number of replicas of the service to create. Defaults to 1 if not specified.
   */
  replicaCount?: number;
  /**
   * Auto scaling policies
   */
  autoScalingPolicies?: AutoScalingPolicy[];
  /**
   * Status of the service. Possible values include: 'Unknown', 'Ready', 'Upgrading', 'Creating',
   * 'Deleting', 'Failed'
   */
  readonly status?: string;
  /**
   * Gives additional information about the current status of the service.
   */
  readonly statusDetails?: string;
  /**
   * Describes the health state of an application resource. Possible values include: 'Invalid',
   * 'Ok', 'Warning', 'Error', 'Unknown'
   */
  readonly healthState?: string;
  /**
   * When the service's health state is not 'Ok', this additional details from service fabric
   * Health Manager for the user to know why the service is marked unhealthy.
   */
  readonly unhealthyEvaluation?: string;
}

/**
 * Properties of a DiagnosticsSink.
 */
export interface DiagnosticsSinkProperties {
  /**
   * Name of the sink. This value is referenced by DiagnosticsReferenceDescription
   */
  name?: string;
  /**
   * A description of the sink.
   */
  description?: string;
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes the diagnostics options available
 */
export interface DiagnosticsDescription {
  /**
   * List of supported sinks that can be referenced.
   */
  sinks?: DiagnosticsSinkProperties[];
  /**
   * Status of whether or not sinks are enabled.
   */
  enabled?: boolean;
  /**
   * The sinks to be used if diagnostics is enabled. Sink choices can be overridden at the service
   * and code package level.
   */
  defaultSinkRefs?: string[];
}

/**
 * Diagnostics settings for Geneva.
 */
export interface AzureInternalMonitoringPipelineSinkDescription extends DiagnosticsSinkProperties {
  /**
   * Azure Internal monitoring pipeline account.
   */
  accountName?: string;
  /**
   * Azure Internal monitoring pipeline account namespace.
   */
  namespace?: string;
  /**
   * Azure Internal monitoring agent configuration.
   */
  maConfigUrl?: string;
  /**
   * Azure Internal monitoring agent fluentd configuration.
   */
  fluentdConfigUrl?: any;
  /**
   * Azure Internal monitoring pipeline autokey associated with the certificate.
   */
  autoKeyConfigUrl?: string;
}

/**
 * Describes the horizontal auto scaling mechanism that adds or removes replicas (containers or
 * container groups).
 */
export interface AddRemoveReplicaScalingMechanism extends AutoScalingMechanism {
  /**
   * Minimum number of containers (scale down won't be performed below this number).
   */
  minCount: number;
  /**
   * Maximum number of containers (scale up won't be performed above this number).
   */
  maxCount: number;
  /**
   * Each time auto scaling is performed, this number of containers will be added or removed.
   */
  scaleIncrement: number;
}

/**
 * Describes the metric that is used for triggering auto scaling operation. Derived classes will
 * describe resources or metrics.
 */
export interface AutoScalingMetric {
  /**
   * Polymorphic Discriminator
   */
  kind: string;
}

/**
 * Describes the resource that is used for triggering auto scaling.
 */
export interface AutoScalingResourceMetric extends AutoScalingMetric {
  /**
   * Name of the resource. Possible values include: 'cpu', 'memoryInGB'
   */
  name: string;
}

/**
 * Describes properties of a service resource.
 */
export interface ServiceProperties {
  /**
   * User readable description of the service.
   */
  description?: string;
  /**
   * The number of replicas of the service to create. Defaults to 1 if not specified.
   */
  replicaCount?: number;
  /**
   * Auto scaling policies
   */
  autoScalingPolicies?: AutoScalingPolicy[];
  /**
   * Status of the service. Possible values include: 'Unknown', 'Ready', 'Upgrading', 'Creating',
   * 'Deleting', 'Failed'
   */
  readonly status?: string;
  /**
   * Gives additional information about the current status of the service.
   */
  readonly statusDetails?: string;
  /**
   * Describes the health state of an application resource. Possible values include: 'Invalid',
   * 'Ok', 'Warning', 'Error', 'Unknown'
   */
  readonly healthState?: string;
  /**
   * When the service's health state is not 'Ok', this additional details from service fabric
   * Health Manager for the user to know why the service is marked unhealthy.
   */
  readonly unhealthyEvaluation?: string;
}

/**
 * Describes the properties of a service replica.
 */
export interface ServiceReplicaProperties {
  /**
   * The operation system required by the code in service. Possible values include: 'Linux',
   * 'Windows'
   */
  osType: string;
  /**
   * Describes the set of code packages that forms the service. A code package describes the
   * container and the properties for running it. All the code packages are started together on the
   * same host and share the same context (network, process etc.).
   */
  codePackages: ContainerCodePackageProperties[];
  /**
   * The names of the private networks that this service needs to be part of.
   */
  networkRefs?: NetworkRef[];
  /**
   * Reference to sinks in DiagnosticsDescription.
   */
  diagnostics?: DiagnosticsRef;
}

/**
 * Describes a replica of a service resource.
 */
export interface ServiceReplicaDescription extends ServiceReplicaProperties {
  /**
   * Name of the replica.
   */
  replicaName: string;
}

/**
 * Describes the average load trigger used for auto scaling.
 */
export interface AverageLoadScalingTrigger extends AutoScalingTrigger {
  /**
   * Description of the metric that is used for scaling.
   */
  metric: AutoScalingMetric;
  /**
   * Lower load threshold (if average load is below this threshold, service will scale down).
   */
  lowerLoadThreshold: number;
  /**
   * Upper load threshold (if average load is above this threshold, service will scale up).
   */
  upperLoadThreshold: number;
  /**
   * Scale interval that indicates how often will this trigger be checked.
   */
  scaleIntervalInSeconds: number;
}

/**
 * The list of service resources. The list is paged when all of the results cannot fit in a single
 * message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in this list.
 */
export interface PagedServiceResourceDescriptionList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * One page of the list.
   */
  items?: ServiceResourceDescription[];
}

/**
 * The list of service resource replicas in the cluster. The list is paged when all of the results
 * cannot fit in a single message. The next set of results can be obtained by executing the same
 * query with the continuation token provided in this list.
 */
export interface PagedServiceReplicaDescriptionList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * List of service resource replica description.
   */
  items?: ServiceReplicaDescription[];
}

/**
 * This type describes a application resource.
 */
export interface ApplicationResourceDescription {
  /**
   * Name of the Application resource.
   */
  name: string;
  /**
   * User readable description of the application.
   */
  description?: string;
  /**
   * Describes the services in the application. This property is used to create or modify services
   * of the application. On get only the name of the service is returned. The service description
   * can be obtained by querying for the service resource.
   */
  services?: ServiceResourceDescription[];
  /**
   * Describes the diagnostics definition and usage for an application resource.
   */
  diagnostics?: DiagnosticsDescription;
  /**
   * Internal - used by Visual Studio to setup the debugging session on the local development
   * environment.
   */
  debugParams?: string;
  /**
   * Names of the services in the application.
   */
  readonly serviceNames?: string[];
  /**
   * Status of the application. Possible values include: 'Unknown', 'Ready', 'Upgrading',
   * 'Creating', 'Deleting', 'Failed'
   */
  readonly status?: string;
  /**
   * Gives additional information about the current status of the application.
   */
  readonly statusDetails?: string;
  /**
   * Describes the health state of an application resource. Possible values include: 'Invalid',
   * 'Ok', 'Warning', 'Error', 'Unknown'
   */
  readonly healthState?: string;
  /**
   * When the application's health state is not 'Ok', this additional details from service fabric
   * Health Manager for the user to know why the application is marked unhealthy.
   */
  readonly unhealthyEvaluation?: string;
}

/**
 * The list of application resources. The list is paged when all of the results cannot fit in a
 * single message. The next set of results can be obtained by executing the same query with the
 * continuation token provided in this list.
 */
export interface PagedApplicationResourceDescriptionList {
  /**
   * The continuation token parameter is used to obtain next set of results. The continuation token
   * is included in the response of the API when the results from the system do not fit in a single
   * response. When this value is passed to the next API call, the API returns next set of results.
   * If there are no further results, then the continuation token is not included in the response.
   */
  continuationToken?: string;
  /**
   * One page of the list.
   */
  items?: ApplicationResourceDescription[];
}
